package com.grendelscan.GUI.http;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import com.grendelscan.requester.TransactionSource;

public class TransactionTableSettings
{
	public int maxRecords;
	public boolean tail;
	public int updateFrequency;
	private List<TransactionSource> sources;
	private List<String> stringSources;
	private String hostPattern;
	private String pathPattern;
	private String queryPattern;
	private String responseCodePattern;
	
	private Pattern hostRegex;
	private Pattern pathRegex;
	private Pattern queryRegex;
	private Pattern responseCodeRegex;
	
	public TransactionTableSettings()
	{
		maxRecords = 1000;
		tail = false;
		updateFrequency = 5000;
		setHostPattern("");
		setPathPattern("");
		setQueryPattern("");
		setResponseCodePattern("");
		sources = new ArrayList<TransactionSource>(1);
		stringSources = new ArrayList<String>(1);
		addSource(TransactionSource.PROXY);
		addSource(TransactionSource.MANUAL_REQUEST);
	};
	
	
	public boolean checkTransactionCriteria(TransactionSummary transactionSummary)
	{
		boolean good = true;

		if (!(
				getHostRegex().matcher(transactionSummary.host).find()
				&& getPathRegex().matcher(transactionSummary.path).find()
				&& getQueryRegex().matcher(transactionSummary.query).find()
				&& getResponseCodeRegex().matcher(String.valueOf(transactionSummary.responseCode)).find()
				&& stringSources.contains(transactionSummary.source)
			))
		{
			good = false;
		}
		
		return good;
	}

	public void addSource(TransactionSource source)
	{
		sources.add(source);
		stringSources.add(source.getText());
	}
	
	public String getHostPattern()
	{
		return hostPattern;
	}
	
	public Pattern getHostRegex()
	{
		return hostRegex;
	}
	
	public String getPathPattern()
	{
		return pathPattern;
	}
	
	public Pattern getPathRegex()
	{
		return pathRegex;
	}
	
	public String getQueryPattern()
	{
		return queryPattern;
	}
	
	public Pattern getQueryRegex()
	{
		return queryRegex;
	}
	
	public String getResponseCodePattern()
	{
		return responseCodePattern;
	}
	
	public Pattern getResponseCodeRegex()
	{
		return responseCodeRegex;
	}
	
	public void setHostPattern(String hostPattern)
	{
		this.hostPattern = hostPattern;
		hostRegex = Pattern.compile(hostPattern, Pattern.CASE_INSENSITIVE);
	}
	
	public void setPathPattern(String pathPattern)
	{
		this.pathPattern = pathPattern;
		pathRegex = Pattern.compile(pathPattern, Pattern.CASE_INSENSITIVE);
	}
	
	public void setQueryPattern(String queryPattern)
	{
		this.queryPattern = queryPattern;
		queryRegex = Pattern.compile(queryPattern, Pattern.CASE_INSENSITIVE);
	}
	
	public void setResponseCodePattern(String responseCodePattern)
	{
		this.responseCodePattern = responseCodePattern;
		responseCodeRegex = Pattern.compile(responseCodePattern, Pattern.CASE_INSENSITIVE);
	}

	public TransactionSource[] getSources()
    {
    	return sources.toArray(new TransactionSource[0]);
    }

	public List<String> getStringSources()
    {
    	return stringSources;
    }
	
	public void clearSources()
	{
		sources.clear();
		stringSources.clear();
	}

	public boolean containsSource(TransactionSource source)
    {
	    return sources.contains(source);
    }
}
