package com.grendelscan.GUI.http.interception;
import org.apache.http.*;

import java.util.List;
import java.util.regex.*;
import com.grendelscan.requester.http.transactions.*;
public class InterceptFilter
{
	private InterceptFilterLocation location;
	public InterceptFilterLocation getLocation()
    {
    	return location;
    }

	public Pattern getPattern()
    {
    	return pattern;
    }

	public boolean isMatches()
    {
    	return matches;
    }

	private Pattern pattern;
	private boolean matches;
	private boolean intercept;
	public InterceptFilter(InterceptFilterLocation location, String patternString, boolean matches,
            boolean intercept)
    {
	    this.location = location;
	    pattern = Pattern.compile(patternString, Pattern.DOTALL | Pattern.CASE_INSENSITIVE);
	    this.matches = matches;
	    this.intercept = intercept;
    }
	
	public boolean performAction(AbstractHttpTransaction transaction)
	{
		return isHit(transaction) == matches;
	}
	
	private boolean isHit(AbstractHttpTransaction transaction)
	{
		switch(location)
		{
			case METHOD: return simpleMatches(transaction.getMethod());
			case HOST: return simpleMatches(transaction.getHost());
			case SCHEME: return simpleMatches(transaction.getScheme());
			case PATH: return simpleMatches(transaction.getPath() + transaction.getFilename());
			case QUERY: return simpleMatches(transaction.getUriQueryString());
			case URL: return simpleMatches(transaction.getAbsoluteUriString());
			case REQUEST_BODY: return simpleMatches(new String(transaction.getRequestBody()));
			case RESPONSE_MIME_TYPE: return simpleMatches(transaction.getResponseMimeType());
			case RESPONSE_BODY: return simpleMatches(transaction.getResponseBodyAsString());
			case RESPONSE_CODE: return simpleMatches(String.valueOf(transaction.getResponseStatusCode()));
			case COOKIE_HEADER: return simpleMatches(transaction.getFirstRequestHeader("Cookie").getValue());
			case REQUEST_HEADER_NAME: return checkHeaderName(transaction.getAllRequestHeaders()); 
			case REQUEST_HEADER_VALUE: return checkHeaderValue(transaction.getAllRequestHeaders());
			case RESPONSE_HEADER_NAME: return checkHeaderName(transaction.getAllResponseHeaders()); 
			case RESPONSE_HEADER_VALUE: return checkHeaderValue(transaction.getAllResponseHeaders());
		}
		return false;
	}
	
	private boolean checkHeaderName(Header [] headers)
	{
		for(Header header: headers)
		{
			if (simpleMatches(header.getName()))
			{
				return true;
			}
		}
		return false;
	}
	
	private boolean checkHeaderValue(Header [] headers)
	{
		for(Header header: headers)
		{
			if (simpleMatches(header.getValue()))
			{
				return true;
			}
		}
		return false;
	}
	
	private boolean simpleMatches(String string)
	{
		return pattern.matcher(string).find();
	}


	public boolean isIntercept()
    {
    	return intercept;
    }
	
	public static boolean matchesFilters(List<InterceptFilter> filters, AbstractHttpTransaction transaction)
	{
		boolean match = false;
		
		for(InterceptFilter filter: filters)
		{
			if (filter.performAction(transaction))
			{
				if (filter.isIntercept())
				{
					match = true;
				}
				else
				{
					match = false;
					break;
				}
			}
			else if (filter.isIntercept())
			{
				match = false;
				break;
			}
		}
		return match;	
	}

	
}
