package com.grendelscan.categorizers;


import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.utils.StringUtils;

/**
 * This categorizer is for a category tracked by tokens, for example, a file name.
 * The tests are only performed once per unique token.
 * @author David Byrne
 */
public abstract class ByTokenCategorizer extends SingleSetCategorizer implements TransactionCategorizer
{
	private Set<String> tokens;

	public ByTokenCategorizer(String categoryTestClass)
	{
		super(categoryTestClass);
		tokens = Collections.synchronizedSet(new HashSet<String>());
		this.categoryTestClass = categoryTestClass;
	}

	public Map<TestModule, Set<TestJob>> analyzeTransaction(AbstractHttpTransaction transaction)
	{
		Map<TestModule, Set<TestJob>> tests = new HashMap<TestModule, Set<TestJob>>();

		for (String token: getTokens(transaction))
		{
			if (token == null)
			{
				continue;
			}
			
			String tokenHash = StringUtils.md5Hash(token);
			if (!tokens.contains(tokenHash))
			{
				tokens.add(tokenHash);
				for (TestModule module: testModules)
				{
					for (TestJob testJob: makeTestJobs(transaction, module, token))
					{
						addJobToCollection(testJob, module, tests);
					}
				}
			}
		}
		return tests;
	}

	
	protected abstract String[] getTokens(AbstractHttpTransaction transaction);
	protected abstract Set<TestJob> makeTestJobs(AbstractHttpTransaction transaction, TestModule module, String token);
}
