package com.grendelscan.categorizers;


import java.util.ArrayList;

import com.grendelscan.categorizers.impl.AllTransactionsCategorizer;
import com.grendelscan.categorizers.impl.ByAuthenticationPackageCategorizer;
import com.grendelscan.categorizers.impl.ByDirectoryCategorizer;
import com.grendelscan.categorizers.impl.ByFileCategorizer;
import com.grendelscan.categorizers.impl.ByHostCategorizer;
import com.grendelscan.categorizers.impl.ByHtmlElementCategorizer;
import com.grendelscan.categorizers.impl.ByHttpQueryCategorizer;
import com.grendelscan.categorizers.impl.ByHttpQueryParameterCategorizer;
import com.grendelscan.categorizers.impl.*;
import com.grendelscan.categorizers.impl.ByMimeTypeCategorizer;
import com.grendelscan.categorizers.impl.ByOutputContextCategorizer;
import com.grendelscan.categorizers.impl.ByRepeatableOutputContextCategorizer;
import com.grendelscan.categorizers.impl.ByResponseHeaderCategorizer;
import com.grendelscan.categorizers.impl.FinalAnalysisCategorizer;
import com.grendelscan.categorizers.impl.InitialAuthenticationCategorizer;
import com.grendelscan.scan.Scan;

public class Categorizers
{
	private ArrayList<Categorizer> allCategorizers;
	private Scan scan;
	private ArrayList<TransactionCategorizer> transactionCategorizers;

	private AllTransactionsCategorizer allTransactionsCategorizerCategorizer;
	private ByAuthenticationPackageCategorizer byAuthenticationPackageCategorizerCategorizer;
	private ByHtmlElementCategorizer byHtmlElementCategorizerCategorizer;
	private ByHttpResponseCodeCategorizer byHttpResponseCodeCategorizerCategorizer;
	private ByMimeTypeCategorizer byMimeTypeCategorizerCategorizer;
	private ByResponseHeaderCategorizer byResponseHeaderCategorizerCategorizer;
	private ByHostCategorizer byHostCategorizerCategorizer;
	private ByHttpQueryCategorizer byHttpQueryCategorizer; 
	private ByHttpQueryParameterCategorizer byHttpQueryParameterCategorizer; 
	private ByFileCategorizer byFileCategorizer;
	private ByHttpMethodCategorizer byHttpMethodCategorizer;
	private ByDirectoryCategorizer byDirectoryCategorizer;
	private InitialAuthenticationCategorizer initialAuthenticationCategorizer;
	private FinalAnalysisCategorizer finalAnalysisCategorizer;
	private ByOutputContextCategorizer byOutputContextCategorizer;
	private ByRepeatableOutputContextCategorizer byRepeatableOutputContextCategorizer;
	private BySetCookieCategorizer bySetCookieCategorizer;
	
	public Categorizers(Scan scan)
	{
		this.scan = scan;
		initCategorizers();
	}

	public ByOutputContextCategorizer getByOutputContextCategorizer()
    {
    	return byOutputContextCategorizer;
    }

	public ArrayList<Categorizer> getAllCategorizers()
	{
		return allCategorizers;
	}

	public AllTransactionsCategorizer getAllTransactionsCategorizerCategorizer()
	{
		return allTransactionsCategorizerCategorizer;
	}

	public ByAuthenticationPackageCategorizer getByAuthenticationPackageCategorizerCategorizer()
	{
		return byAuthenticationPackageCategorizerCategorizer;
	}

	public ByHtmlElementCategorizer getByHtmlElementCategorizerCategorizer()
	{
		return byHtmlElementCategorizerCategorizer;
	}

	public ByHttpResponseCodeCategorizer getByHttpResponseCodeCategorizerCategorizer()
	{
		return byHttpResponseCodeCategorizerCategorizer;
	}

	public ByMimeTypeCategorizer getByMimeTypeCategorizerCategorizer()
	{
		return byMimeTypeCategorizerCategorizer;
	}

	public ByResponseHeaderCategorizer getByResponseHeaderCategorizerCategorizer()
	{
		return byResponseHeaderCategorizerCategorizer;
	}

	public ByHostCategorizer getByHostCategorizerCategorizer()
	{
		return byHostCategorizerCategorizer;
	}

	public ArrayList<TransactionCategorizer> getTransactionCategorizers()
	{
		return transactionCategorizers;
	}

	private void initCategorizers()
	{
		// Change the starting element count in the HashSet
		// to match the number of categorizers being
		// initialized
		transactionCategorizers = new ArrayList<TransactionCategorizer>(8);
		allCategorizers = new ArrayList<Categorizer>(10);

		// Repeat for all categorizers used during the main
		// testing period. By host should always be first.

		byHostCategorizerCategorizer = new ByHostCategorizer();
		byHostCategorizerCategorizer.setScan(scan);
		transactionCategorizers.add(byHostCategorizerCategorizer);
		allCategorizers.add(byHostCategorizerCategorizer);

		byFileCategorizer = new ByFileCategorizer();
		byFileCategorizer.setScan(scan);
		transactionCategorizers.add(byFileCategorizer);
		allCategorizers.add(byFileCategorizer);

		byDirectoryCategorizer = new ByDirectoryCategorizer();
		byDirectoryCategorizer.setScan(scan);
		transactionCategorizers.add(byDirectoryCategorizer);
		allCategorizers.add(byDirectoryCategorizer);

		allTransactionsCategorizerCategorizer = new AllTransactionsCategorizer();
		allTransactionsCategorizerCategorizer.setScan(scan);
		transactionCategorizers.add(allTransactionsCategorizerCategorizer);
		allCategorizers.add(allTransactionsCategorizerCategorizer);

		byHtmlElementCategorizerCategorizer = new ByHtmlElementCategorizer();
		byHtmlElementCategorizerCategorizer.setScan(scan);
		transactionCategorizers.add(byHtmlElementCategorizerCategorizer);
		allCategorizers.add(byHtmlElementCategorizerCategorizer);

		byHttpResponseCodeCategorizerCategorizer = new ByHttpResponseCodeCategorizer();
		byHttpResponseCodeCategorizerCategorizer.setScan(scan);
		transactionCategorizers.add(byHttpResponseCodeCategorizerCategorizer);
		allCategorizers.add(byHttpResponseCodeCategorizerCategorizer);

		byMimeTypeCategorizerCategorizer = new ByMimeTypeCategorizer();
		byMimeTypeCategorizerCategorizer.setScan(scan);
		transactionCategorizers.add(byMimeTypeCategorizerCategorizer);
		allCategorizers.add(byMimeTypeCategorizerCategorizer);

		byResponseHeaderCategorizerCategorizer = new ByResponseHeaderCategorizer();
		byResponseHeaderCategorizerCategorizer.setScan(scan);
		transactionCategorizers.add(byResponseHeaderCategorizerCategorizer);
		allCategorizers.add(byResponseHeaderCategorizerCategorizer);

		byHttpQueryCategorizer = new ByHttpQueryCategorizer();
		byHttpQueryCategorizer.setScan(scan);
		transactionCategorizers.add(byHttpQueryCategorizer);
		allCategorizers.add(byHttpQueryCategorizer);

		byHttpQueryParameterCategorizer = new ByHttpQueryParameterCategorizer();
		byHttpQueryParameterCategorizer.setScan(scan);
		allCategorizers.add(byHttpQueryParameterCategorizer);
		transactionCategorizers.add(byHttpQueryParameterCategorizer);
		
		byOutputContextCategorizer = new ByOutputContextCategorizer();
		byOutputContextCategorizer.setScan(scan);
		transactionCategorizers.add(byOutputContextCategorizer);
		allCategorizers.add(byOutputContextCategorizer);

		byHttpMethodCategorizer = new ByHttpMethodCategorizer();
		byHttpMethodCategorizer.setScan(scan);
		transactionCategorizers.add(byHttpMethodCategorizer);
		allCategorizers.add(byHttpMethodCategorizer);
		
		bySetCookieCategorizer = new BySetCookieCategorizer();
		bySetCookieCategorizer.setScan(scan);
		transactionCategorizers.add(bySetCookieCategorizer);
		allCategorizers.add(bySetCookieCategorizer);
		
		
		
		// Technically, this is a transaction categorizer, but it isn't called for all transactions
		initialAuthenticationCategorizer = new InitialAuthenticationCategorizer();
		initialAuthenticationCategorizer.setScan(scan);
		allCategorizers.add(initialAuthenticationCategorizer);
		
		
		// Non transaction categorizers go below here
		byAuthenticationPackageCategorizerCategorizer = new ByAuthenticationPackageCategorizer();
		byAuthenticationPackageCategorizerCategorizer.setScan(scan);
		allCategorizers.add(byAuthenticationPackageCategorizerCategorizer);
		
		finalAnalysisCategorizer = new FinalAnalysisCategorizer();
		finalAnalysisCategorizer.setScan(scan);
		allCategorizers.add(finalAnalysisCategorizer);

		byRepeatableOutputContextCategorizer = new ByRepeatableOutputContextCategorizer();
		byRepeatableOutputContextCategorizer.setScan(scan);
		allCategorizers.add(byRepeatableOutputContextCategorizer);

	}

	public ByHttpQueryCategorizer getByHttpQueryCategorizer()
    {
    	return byHttpQueryCategorizer;
    }

	public ByFileCategorizer getByFileCategorizer()
    {
    	return byFileCategorizer;
    }

	public ByDirectoryCategorizer getByDirectoryCategorizer()
    {
    	return byDirectoryCategorizer;
    }

	public InitialAuthenticationCategorizer getInitialAuthenticationCategorizer()
    {
    	return initialAuthenticationCategorizer;
    }

	public FinalAnalysisCategorizer getFinalAnalysisCategorizer()
    {
    	return finalAnalysisCategorizer;
    }

	public ByRepeatableOutputContextCategorizer getByRepeatableOutputContextCategorizer()
    {
    	return byRepeatableOutputContextCategorizer;
    }

	public ByHttpQueryParameterCategorizer getByHttpQueryParameterCategorizer()
    {
    	return byHttpQueryParameterCategorizer;
    }

	public ByHttpMethodCategorizer getByHttpMethodCategorizer()
    {
    	return byHttpMethodCategorizer;
    }

	public BySetCookieCategorizer getBySetCookieCategorizer()
    {
    	return bySetCookieCategorizer;
    }

}
