package com.grendelscan.categorizers;


import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.tests.testModules.collections.CategoryTestModuleCollection;
import com.grendelscan.utils.Debug;

/**
 * The type strings are not case-sensitive.
 * 
 * @author David Byrne
 * 
 */
public abstract class MultiSetCategorizer extends Categorizer
{
	private String categoryTestClass;
	protected Map<Object, CategoryTestModuleCollection> modulesByType;

	@Override
    public boolean hasModules()
    {
	    return modulesByType.size() > 0 ? true : false;
    }

	public MultiSetCategorizer(String categoryTestClass)
	{
		modulesByType = Collections.synchronizedMap(new HashMap<Object, CategoryTestModuleCollection>());
		this.categoryTestClass = categoryTestClass;
	}

	@Override
	public void addModule(TestModule module)
	{
		try
		{
			if (Class.forName(categoryTestClass).isInstance(module))
			{
				for (Object type: getModuleTypes(module))
				{
					if (type instanceof String)
					{
						type = ((String)type).toUpperCase();
					}

					if (!modulesByType.containsKey(type))
					{
						CategoryTestModuleCollection testModules = new CategoryTestModuleCollection();
						modulesByType.put(type, testModules);
					}
					modulesByType.get(type).add(module);
				}
			}
		}
		catch (ClassNotFoundException e)
		{
			Debug.errDebug("Class " + categoryTestClass + " not found in ByTypeAbstractCategorizer.addModule. Someone screwed up: " + e.toString());
			System.exit(1);
		}
	}

	@Override
	public void resolveDependencies()
	{
		for (CategoryTestModuleCollection modules: modulesByType.values())
		{
			modules.resolveDependencies();
		}
	}

	public abstract Object[] getModuleTypes(TestModule module);
}
