package com.grendelscan.categorizers.impl;

import com.grendelscan.requester.*;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import com.grendelscan.requester.http.factories.*;
import com.grendelscan.categorizers.ByTokenCategorizer;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.tests.testJobs.ByDirectoryTestJob;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.*;

/**
 * 
 * @author Administrator
 */
public class ByDirectoryCategorizer extends ByTokenCategorizer
{
	private Set<String> testedDirs;
	
	public ByDirectoryCategorizer()
	{
		super("com.grendelscan.tests.testTypes.ByDirectoryTest");
		testedDirs = new HashSet<String>();
	}

	protected Set<TestJob> makeTestJobs(AbstractHttpTransaction transaction, TestModule module, String token)
	{
		Set<TestJob> jobs = new HashSet<TestJob>();
		jobs.add(new ByDirectoryTestJob(module.getModuleNumber(), transaction.getId(), token, scan));
		
		return jobs;
	}
	
	protected String[] getTokens(AbstractHttpTransaction transaction)
	{
		String tokens [] = {};
		int responseCode = transaction.getLogicalResponseCode();
		String path = transaction.getDirectory();
		if (HttpUtils.fileExists(responseCode) || path.equals("/"))
		{
			ArrayList<String> dirs = new ArrayList<String>();
			int lastSlashIndex = -1;
			int slashIndex;
			while ((slashIndex = path.indexOf("/", lastSlashIndex + 1)) >= 0)
			{
				if (!(slashIndex - lastSlashIndex == 1 && lastSlashIndex > 0))
				{
					String directory = path.substring(0, slashIndex + 1);
					try
                    {
						/*
						 * Check to see that the directory really exists. This is mostly due to 
						 * some bugs that throw invalid directories here
						 */
						String testUri = transaction.getHostUriWithoutTrailingSlash() + directory;
						if (!testedDirs.contains(testUri))
						{
							AbstractHttpTransaction dirTest = HttpTransactionFactory.createTransaction(scan, "GET", testUri, 0, TransactionSource.CATEGORIZER);
							if (!scan.isRunning())
							{
								return tokens;
							}
							dirTest.execute("ByDirectoryCategorizer");
							if (HttpUtils.fileExists(dirTest.getLogicalResponseCode()))
							{
								testedDirs.add(testUri);
			                    dirs.add(directory);
							}
							else
							{
								Debug.debug("Invalid directory passed to categorizer: " + testUri);
							}
						}
							
                    }
                    catch (URISyntaxException e)
                    {
                    	Debug.errDebug("Problem with URI syntax: " + e.toString(), e);
                    }
				}
				else
				{
					Debug.debug("odd URL");
				}
				lastSlashIndex = slashIndex;
			}

			tokens = new String[dirs.size()];
			int index = 0;
			for (String string: dirs)
			{
				tokens[index++] = string;
			}
		}
		return tokens;
	}

}
