package com.grendelscan.categorizers.impl;


import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.cobra_grendel.html.domimpl.HTMLElementImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.grendelscan.categorizers.MultiSetTransactionCategorizer;
import com.grendelscan.html.HtmlNodeUtilities;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.tests.testJobs.ByHtmlElementTestJob;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.tests.testTypes.ByHtmlElementTest;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.MimeUtils;

public class ByHtmlElementCategorizer extends MultiSetTransactionCategorizer
{
	private Set<String> elementHashes;

	public ByHtmlElementCategorizer()
	{
		super("com.grendelscan.tests.testTypes.ByHtmlElementTest");
		elementHashes = Collections.synchronizedSet(new HashSet<String>(500));
	}

	@Override
	public Map<TestModule, Set<TestJob>> analyzeTransaction(AbstractHttpTransaction transaction)
	{
		Map<TestModule, Set<TestJob>> tests = new HashMap<TestModule, Set<TestJob>>();

		if (MimeUtils.isHtmlMimeType(transaction.getResponseMimeType()))
		{
			Document doc = transaction.getResponseDOM();

			for (Object tagnameObject: modulesByType.keySet())
			{
				String tagName = (String) tagnameObject;
				NodeList nodes = doc.getElementsByTagName(tagName);
				for (int index = 0; index < nodes.getLength(); index++)
				{
					if (!scan.isRunning())
					{
						return tests;
					}
					Node node = nodes.item(index);
					if (node instanceof HTMLElementImpl)
					{
						HTMLElementImpl element = (HTMLElementImpl) node.cloneNode(true);
						HtmlUtils.StripAllChildrenButInput(element);
						element.clearParent();
						String hash = HtmlNodeUtilities.getNodeHash(element);
						if (!elementHashes.contains(hash))
						{
							elementHashes.add(hash);
							for (TestModule module: modulesByType.get(tagName))
							{
								if (!scan.isRunning())
								{
									return tests;
								}
								ByHtmlElementTestJob testJob = new ByHtmlElementTestJob(module.getModuleNumber(), transaction.getId(), element, tagName, scan);
								addJobToCollection(testJob, module, tests);
							}
						}
					}
				}
			}
		}
		return tests;
	}

	@Override
	public String[] getModuleTypes(TestModule module)
	{
		ByHtmlElementTest test = (ByHtmlElementTest) module;
		return test.getHtmlElements();
	}

	@Override
	/**
	 * This isn't used by ByHtmlElementCategorizer because it implements its own analyze transaction
	 */
	protected Set<String> getTransactionTypeStrings(AbstractHttpTransaction transaction)
	{
		return null;
	}

	@Override
	/**
	 * This isn't used by ByHtmlElementCategorizer because it implements its own analyze transaction
	 */
	protected TestJob makeTestJob(AbstractHttpTransaction transaction, TestModule module, String type)
	{
		return null;
	}
}
