package com.grendelscan.categorizers.impl;



import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.grendelscan.categorizers.MultiSetCategorizer;
import com.grendelscan.categorizers.TransactionCategorizer;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.tests.libraries.TokenTesting.TokenContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextType;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.testJobs.ByOutputContextTestJob;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.tests.testTypes.ByOutputContextTest;

public class ByOutputContextCategorizer extends MultiSetCategorizer implements TransactionCategorizer
{
    protected Set<String> getTestTokens(AbstractHttpTransaction transaction)
    {
    	Set<String> tokens = new HashSet<String>();
    	String body = transaction.getResponseBodyAsString();
    	Matcher m = tokenPattern.matcher(body);
    	while (m.find())
    	{
    		tokens.add(m.group(1));
    	}
    	return tokens;
    }


	private Pattern tokenPattern;

	public ByOutputContextCategorizer()
	{
		super("com.grendelscan.tests.testTypes.ByOutputContextTest");
		
		String tokenPrefix = TokenTesting.getTokenPrefix();
		int tokenSuffixLength = TokenTesting.getTokenSuffixLength();
		tokenPattern = Pattern.compile("(" + tokenPrefix + "[A-Z]{" + tokenSuffixLength + "})", Pattern.CASE_INSENSITIVE);
	}

	@Override
	public TokenContextType[] getModuleTypes(TestModule module)
	{
		return ((ByOutputContextTest) module).getDesiredContexts();
	}

	@Override
    public boolean hasModules()
    {
	    return super.hasModules() || scan.getCategorizers().getByRepeatableOutputContextCategorizer().hasModules();
    }

	public Map<TestModule, Set<TestJob>> analyzeTransaction(AbstractHttpTransaction transaction)
	{
		Map<TestModule, Set<TestJob>> tests = new HashMap<TestModule, Set<TestJob>>();
		Set<TokenContext> contexts = TokenTesting.findTokenContexts(transaction);

		for (TokenContext context: contexts)
		{
			if (!scan.isRunning())
			{
				break;
			}
			/*
			 * If any modules have registered for this type of context and the process is 
			 * repeatable
			 */ 
			if (modulesByType.containsKey(context.getContextType()))
			{
				for (TestModule module: modulesByType.get(context.getContextType()))
				{
					TestJob testJob = new ByOutputContextTestJob(module.getModuleNumber(), transaction.getId(), context, scan);
					addJobToCollection(testJob, module, tests);
				}
			}

			
			if (TokenTesting.verifyTokenRepeatability(context, "ByOutputContextCategorizer", TransactionSource.CATEGORIZER))
			{
				scan.getCategorizers().getByRepeatableOutputContextCategorizer().analyzeRepeatableOutputContext(context, transaction.getId());
			}
		}
		return tests;
	}

	
}
