package com.grendelscan.categorizers.queue;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.scan.ScanPhase;
import com.grendelscan.scan.abstractQueue.AbstractQueueThread;
import com.grendelscan.scan.abstractQueue.QueueThreadGroup;
import com.grendelscan.scan.abstractQueue.QueueThreadState;
import com.grendelscan.utils.Debug;

public class CategorizerThread implements Runnable, AbstractQueueThread

{
	private CategorizerQueue queue;
	private Thread thread;
	private Scan scan;

    private QueueThreadState threadState;
    
    public QueueThreadState getThreadState()
    {
    	return threadState;
    }

    
	/** Creates a new instance of requesterThread */
	public CategorizerThread(int ID, Scan scan, QueueThreadGroup threadGroup)
	{
    	threadState = QueueThreadState.CREATING;
		thread = new Thread(threadGroup, this, "CategorizerThread-" + ID);
		queue = scan.getCategorizerQueue();
		this.scan = scan;
	}


	public void run()
	{
		while (queue.isRunning() && scan.isRunning())
		{
	    	threadState = QueueThreadState.POLLING;
	    	try
	    	{
		    	AbstractHttpTransaction transaction = getNextTransaction();
		    	if ((transaction != null) && queue.isRunning() && scan.isRunning())
				{
			    	threadState = QueueThreadState.PROCESSING;
			    	Debug.debug("Categorizer thread processing " + transaction.getUriString());
					queue.categorizeTransaction(transaction);
				}
			}
			catch (Exception e) 
			{
				Debug.errDebug("Unhandled exception in " + thread.getName() + ": " + e.toString(), e);
			}

		}
    	threadState = QueueThreadState.TERMINATING;
	}

	/**
	 * This will properly interrupt the thread and tell it to process no other tasks. 
	 * It does not currently have the ability to abort a task already started. 
	 */
	public void interupt()
	{
		thread.interrupt();
	}
	
	public void start()
	{
		thread.start();
	}

	private AbstractHttpTransaction getNextTransaction()
	{
		AbstractHttpTransaction request = null;
		while ((request == null) && queue.isRunning() && scan.isRunning())
		{
			if (scan.isRunning())
			{
				request = queue.getNext();
			}
			if ((request == null) && queue.isRunning() && scan.isRunning())
			{
				synchronized(this)
				{
					try
					{
						this.wait(250);
					}
					catch (InterruptedException e)
					{
					}
				}
			}
		}
		return request;
	}


	public String getName()
    {
	    return thread.getName();
    }

}
