package com.grendelscan.data;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.grendelscan.utils.Debug;

public class DataStore
{
	
	private Map<String, Object> sharedData;
	
	Connection dbConnection;
	
	public DataStore(Connection dbConnection)
	{
		this.dbConnection = dbConnection;
		sharedData = Collections.synchronizedMap(new HashMap<String, Object>(50));
	}
	
	public static void createTableSpace(Connection connection)
	{
		String query =
		        "CREATE TABLE PUBLIC.GenericDataStore (\n" + "Key LONGVARCHAR,\n" + "Value Other,\n"
		                + "PRIMARY KEY (Key));";
		
		if (connection != null)
		{
			try
			{
				connection.createStatement().execute(query);
			}
			catch (SQLException e)
			{
				Debug.errDebug("SQL error when creating DataStore table: " + e.toString(), e);
			}
		}
	}
	
	public void addToStringMap(String mapName, String name, String value)
	{
		if (!containsKey(mapName))
		{
			createStringSet(mapName);
		}
		Map<String, String> s = (Map<String, String>) getData(mapName);
		s.put(name, value);
	}
	
	public void addToStringSet(String setName, String value)
	{
		if (!containsKey(setName))
		{
			createStringSet(setName);
		}
		Set<String> s = (Set<String>) getData(setName);
		s.add(value);
	}
	
	public boolean containsKey(String key)
	{
		if (sharedData.containsKey(key))
		{
			return true;
		}
		return dbContainsKey(key);
	}
	
	public Object getData(String key)
	{
		if (sharedData.containsKey(key))
		{
			return sharedData.get(key);
		}
		return getDBData(key);
	}
	
	public Map<String, String> getStringMap(String mapName)
	{
		if (!containsKey(mapName))
		{
			createStringMap(mapName);
		}
		Map<String, String> s = (Map<String, String>) getData(mapName);
		return s;
	}
	
	public Set<String> getStringSet(String setName)
	{
		if (!containsKey(setName))
		{
			createStringSet(setName);
		}
		Set<String> s = (Set<String>) getData(setName);
		return s;
	}
	
	/**
	 * @return the connection
	 */
	
	public boolean moveDataToDisk(String key)
	{
		if (sharedData.containsKey(key) && (sharedData.get(key) instanceof Serializable))
		{
			putDBData(key, sharedData.remove(key));
			return true;
		}
		return false;
	}
	
	public boolean moveDataToMemory(String key)
	{
		if (dbContainsKey(key))
		{
			sharedData.put(key, removeDBData(key));
			return true;
		}
		return false;
	}
	
	public Object putData(String key, Object value, boolean storeInDatabase)
	{
		Object oldValue;
		if (sharedData.containsKey(key))
		{
			oldValue = sharedData.remove(key);
		}
		else if (dbContainsKey(key))
		{
			oldValue = removeDBData(key);
		}
		else
		{
			oldValue = null;
		}
		
		if (storeInDatabase && (value instanceof Serializable))
		{
			putDBData(key, value);
		}
		else
		{
			sharedData.put(key, value);
		}
		
		return oldValue;
	}
	
	public Object removeData(String key)
	{
		Object oldValue;
		if (sharedData.containsKey(key))
		{
			return sharedData.remove(key);
		}
		else if (dbContainsKey(key))
		{
			oldValue = getDBData(key);
			removeDBData(key);
		}
		else
		{
			oldValue = null;
		}
		return oldValue;
	}
	
	private void createStringMap(String setName)
	{
		Map<String, String> s = Collections.synchronizedMap(new HashMap<String, String>());
		putData(setName, s, false);
	}
	
	private void createStringSet(String setName)
	{
		Set<String> s = Collections.synchronizedSet(new HashSet<String>());
		putData(setName, s, false);
	}
	
	private boolean dbContainsKey(String key)
	{
		return false;
	}
	
	private Object getDBData(String key)
	{
		
		return null;
	}
	
	private void putDBData(String key, Object value)
	{
		
	}
	
	private Object removeDBData(String key)
	{
		return null;
	}
	
}
