package com.grendelscan.data;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import com.grendelscan.scan.*;

import javax.sql.rowset.serial.SerialBlob;

import com.grendelscan.utils.Debug;

public class Database
{
	private int connectionTransactionCount = 0;
	private int totalTransactionCount = 0;
	private String databaseDirectory;
	private Connection connection;
//	private Scan scan;

//	public Database(Scan scan) throws ClassNotFoundException, SQLException
//    {
//		Debug.debug("Initializing database connection/instance");
//        Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
////        Class.forName("org.apache.derby.jdbc.ClientDriver");
////        this.scan = scan;
//		databaseDirectory = scan.getScanSettings().getOutputDirectory() + File.separator + "transaction-record";
//		checkConnection();
//	}

	public Database(String path, boolean create) throws ClassNotFoundException, SQLException
    {
		Debug.debug("Initializing database connection/instance");
        Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
        databaseDirectory = path;
		String connectionString = "jdbc:derby:" + path;
		if (create)
		{
			connectionString += ";create=true";
		}
        connection = DriverManager.getConnection(connectionString);
        connection.setAutoCommit(true);
	}

	public void execute(String query) throws SQLException
	{
    	checkConnection();
        Statement statement = connection.createStatement();
        statement.execute(query);
        statement.close();
	}

	private synchronized void checkConnection() throws SQLException
	{
		connectionTransactionCount++;
		totalTransactionCount++;
		if (connection == null || connection.isClosed())
		{
			connectionTransactionCount = 0;
			if (connection != null && !connection.isClosed())
			{
				connection.close();
			}
			String connectionString = "jdbc:derby:" + databaseDirectory;
	        connection = DriverManager.getConnection(connectionString);
	        connection.setAutoCommit(true);
		}
	}

	
	public PreparedStatement prepareStatement(String sql) throws SQLException
    {
    	checkConnection();
	    return connection.prepareStatement(sql);
    }

	public static Blob serializableToBlob(Serializable object) throws IOException, SQLException
	{
		Blob blob = null;
		ByteArrayOutputStream bytes = new ByteArrayOutputStream();
		ObjectOutputStream objstream = null;
		boolean written = false;
		while (!written)
		{
			try
			{	
				objstream = new ObjectOutputStream(bytes);
				objstream.writeObject(object);
				written = true;
			}
			catch (ConcurrentModificationException e) 
			{
				Debug.errDebug("Problem while serializing object: " + e.toString(), e);
			}
		}
		blob = new SerialBlob(bytes.toByteArray());
		objstream.close();

		return blob;
	}
	
	public static Object blobToObject(Blob blob) throws IOException, SQLException, ClassNotFoundException
	{
		ObjectInputStream objstream = new ObjectInputStream(blob.getBinaryStream());
//		blob.free();
		return objstream.readObject();
	}

	public Connection getConnection()
    {
//    	checkConnection();
    	return connection;
    }

	
}
