package com.grendelscan.proxy;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.URI;
import java.net.URISyntaxException;
import com.grendelscan.GUI.http.interception.*;

import org.apache.http.*;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicHttpEntityEnclosingRequest;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpRequestHandler;
import org.apache.http.protocol.HttpService;

import com.grendelscan.proxy.ssl.TunneledSSLConnection;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.client.CustomHttpClient;
import com.grendelscan.requester.http.factories.HttpRequestFactory;
import com.grendelscan.requester.http.*;
import com.grendelscan.requester.http.factories.HttpTransactionFactory;
import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.transactions.*;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.*;

import java.util.List;
import java.util.regex.*;

public class HttpProxyRequestHandler implements HttpRequestHandler
{
	
	private Proxy proxy;
	private Scan scan;
	private Socket socket;
	private boolean ssl = false;
	private int sslPort;
	private CustomHttpClient proxyHttpClient;
	

	public HttpProxyRequestHandler(Scan scan, Socket socket, Proxy proxy, CustomHttpClient proxyHttpClient, boolean ssl, int sslPort)
	{
		this.proxyHttpClient = proxyHttpClient;
		this.socket = socket;
		this.scan = scan;
		this.proxy = proxy;
		this.ssl = ssl;
		this.sslPort = sslPort;
	}
	
	public void handle(HttpRequest request, HttpResponse response, HttpContext context) throws HttpException,
	        IOException
	{
		Debug.debug("Proxy received request for: " + request.getRequestLine().getUri());
		if (request.getRequestLine().getMethod().equals("CONNECT"))
		{
			handleConnectMethod(request, response, context);
		}
		else
		{
			Header hostHeaders[] = request.getHeaders("Host");
			String host;
			int port = -1;
			String uri = null;
			if (hostHeaders.length > 0)
			{
				host = hostHeaders[0].getValue();
				int colonIndex = host.indexOf(":");
				if (colonIndex > 0)
				{
					host = host.substring(0, colonIndex);
					port = Integer.valueOf(hostHeaders[0].getValue().substring(colonIndex + 1));
				}
				else
				{
					if (!ssl)
					{
						port = 80;
					}
				}
			}
			else
			{
				host = URIStringUtils.getHost(request.getRequestLine().getUri());
				port = URIStringUtils.getPort(request.getRequestLine().getUri());
			}
			
			
			if (ssl)
			{
				String oldUri = request.getRequestLine().getUri();
				// if the URI isn't absolute, we need to make it so
				if (!oldUri.toLowerCase().startsWith("https://"))
				{
					uri = "https://" + host + ":" + sslPort + oldUri;
				}
				port = sslPort;
			}
			else
			{
				uri = request.getRequestLine().getUri();
				if (!uri.toLowerCase().startsWith("http://"))
				{
					uri = "http://" + host + ":" + port + uri;
				}
			}
			UnvalidatedHttpRequest requestToUse;
			byte body[] = null;
			if (request instanceof BasicHttpEntityEnclosingRequest)
			{
				HttpEntity entity = ((BasicHttpEntityEnclosingRequest) request).getEntity();
				body = HttpUtils.entityToByteArray(entity, 0);
				requestToUse = new UnvalidatedHttpEntityRequest(request.getRequestLine().getMethod(), uri, host, port, ssl, body);
			}
			else
			{
				 requestToUse = new UnvalidatedHttpRequest(request.getRequestLine().getMethod(), uri, host, port, ssl);
			}
			requestToUse.addHeaders(request.getAllHeaders());
			
			boolean testUninterceptedTransaction = scan.getScanSettings().isTestProxyRequests();
			
			if (!scan.getScanSettings().getUrlFilters().isUriAllowed(uri))
			{
				if (scan.getScanSettings().isAllowAllProxyRequests())
				{
					testUninterceptedTransaction = false;
				}
				else
				{
					forbiddenMessage(request, response);
					return;
				}
			}

			// Get rid of proxy and connection-related headers
			requestToUse.removeHeaders("Proxy-Connection");
			requestToUse.removeHeaders("Connection");
			requestToUse.removeHeaders("Keep-Alive");
			
			// An easy way to get rid of compression. Need a more elegant solution to handle it later
			requestToUse.removeHeaders("Accept-Encoding");
			
//			requestToUse.removeHeaders("Proxy-Authorization");
			
			UnvalidatedHttpTransaction transaction = new UnvalidatedHttpTransaction(scan, 0, TransactionSource.PROXY, body, requestToUse); 
			transaction.setFollowRedirects(false);
			UnvalidatedHttpTransaction postInterceptTransaction = checkRequestIntercept(transaction);
			boolean intercepted = false;
			if (postInterceptTransaction != transaction)
			{
				transaction = postInterceptTransaction;
				intercepted = true;
			}
			if (!transaction.isSuccessfullExecution())
			{
				scan.getRequesterQueue().sendRequest(transaction, "Internal proxy", proxyHttpClient, false, true);
			}
			else
			{
				System.err.println("Why is this here?");
			}
			intercepted = checkResponseIntercept(transaction) || intercepted;
			if (testUninterceptedTransaction || (intercepted && scan.getScanSettings().isTestInterceptedRequests()))
			{
				scan.getCategorizerQueue().addTransaction(transaction);
			}
				
			if (transaction.getResponseWrapper() == null)
			{
				errorMessage(request, response, 500, "Proxy error", "Unknown problem with Grendel-Scan proxy. The remote host may be unreachable");
			}
			else
			{
				HttpEntity entity = new ByteArrayEntity(getResponseBody(transaction));
				response.setEntity(entity);
				for (Header header: transaction.getAllResponseHeaders())
				{
					response.addHeader(HttpCloner.clone(header));
				}
				response.setStatusLine(HttpCloner.clone(transaction.getResponse().getStatusLine()));
			}
		}
	}
	
	
	private UnvalidatedHttpTransaction checkRequestIntercept(UnvalidatedHttpTransaction transaction)
	{
		if (scan.getScanSettings().isInterceptRequests())
		{
			if (InterceptFilter.matchesFilters(scan.getScanSettings().getRequestInterceptFilters(), transaction))
			{
				return (UnvalidatedHttpTransaction) InterceptionComposite.showRequestIntercept(scan, transaction);
			}
		}
		return transaction;
	}
	
	/**
	 * This doesn't need to return a transaction because the response intercept
	 * will never create a new transaction
	 * @param transaction
	 * @return
	 */
	private boolean checkResponseIntercept(AbstractHttpTransaction transaction)
	{
		if (scan.getScanSettings().isInterceptResponses())
		{
			if (InterceptFilter.matchesFilters(scan.getScanSettings().getResponseInterceptFilters(), transaction))
			{
				return InterceptionComposite.showResponseIntercept(scan, transaction);
			}
		}
		return false;
	}
	
	protected HttpService makeHttpServiceProxy(Scan scan, Socket socket, Proxy proxy, CustomHttpClient proxyHttpClient, boolean ssl, int sslPort)
	{
		return MiscHttpFactory.createHttpServiceProxy(scan, socket, proxy, proxyHttpClient, true, sslPort);
	}
	
	protected void handleConnectMethod(HttpRequest request, HttpResponse response, HttpContext context)
	{
		try
		{
			if (!scan.getScanSettings().getUrlFilters().isUriAllowed("https://" + request.getRequestLine().getUri()) && ! scan.getScanSettings().isAllowAllProxyRequests())
			{
				forbiddenMessage(request, response);
				return;
			}
			
			/*
			 * This is a little dirty. Normally, the response is sent back
			 * when this method returns. However, since this method is
			 * handling the SSL connection, we need to manually send an OK.
			 * I could create a response object, but it's so simple, it's
			 * easier just to write it manually.
			 */

			socket.getOutputStream().write("HTTP/1.0 200 Ok\n\n".getBytes());
			
			
			TunneledSSLConnection tunneledSSLConnection = new TunneledSSLConnection(socket);
			
			String uri = request.getRequestLine().getUri();
			int colonPosition = uri.indexOf(':');
			int sslPort;
			if (colonPosition > 0)
			{
				sslPort = Integer.valueOf(uri.substring(colonPosition + 1, uri.length()));
				
			}
			else
			{
				sslPort = 443;
			}
			
			/*
			 * This will create a sub-HttpProxyRequestHandler to handle the
			 * requests that come through the SSL connection. In theory,
			 * those requests could even be other CONNECTs
			 */
			HttpService httpService = makeHttpServiceProxy(scan, socket, proxy, proxyHttpClient, true, sslPort);
			
			/*
			 * Normally, this would be in the worker thread, but we don't
			 * need to create a new thread since the worker thread created
			 * for the CONNECT transaction will work just fine. Eventually,
			 * I should consolidate the two code areas.
			 */
			try
			{
				while (!Thread.interrupted() && tunneledSSLConnection.isOpen() && proxy.isRunning()
				        && !socket.isClosed())
				{
					httpService.handleRequest(tunneledSSLConnection, context);
				}
			}
			catch (ConnectionClosedException e)
			{
				// not a problem
			}
			catch (Exception e)
			{
				Debug.errDebug("Exception handling proxy request: " + e.toString(), e);
			}
			finally
			{
				/*
				 * In theory, I think that another CONNECT could be
				 * processed on the same socket, but I'm shutting it down
				 * anyway.
				 */
				try
				{
					tunneledSSLConnection.shutdown();
				}
				catch (IOException e)
				{
				}
			}
		}
		catch (Exception e)
		{
			Debug.errDebug("Problem with SSL proxy: " + e.toString(), e);
			// String errorText = "Problem with "
			// return some kind of error page
		}
	}

	static Pattern hiddenFieldsPattern = Pattern.compile("(<input [^>]*)\\s*type\\s*=\\s*[\"']*hidden[\"']*([^>]*>)", Pattern.DOTALL | Pattern.CASE_INSENSITIVE);
	static Pattern fieldNamePattern = Pattern.compile("name\\s*=[\"'](\\w+)", Pattern.DOTALL | Pattern.CASE_INSENSITIVE);
	private byte[] getResponseBody(AbstractHttpTransaction transaction)
	{
		byte body[];
		if (scan.getScanSettings().isRevealHiddenFields() && MimeUtils.isHtmlMimeType(transaction.getResponseMimeType()))
		{
			String responseBody = new String(transaction.getResponseBody());
			while (true)
			{
				Matcher hiddenFieldMatcher = hiddenFieldsPattern.matcher(responseBody);
				if (hiddenFieldMatcher.find())
				{
					String first = hiddenFieldMatcher.group(1);
					String second = hiddenFieldMatcher.group(2);
					Matcher m1 = fieldNamePattern.matcher(first);
					String name = "Unknown name";
					if (m1.find())
					{
						name = m1.group(1);
					}
					else
					{
						Matcher m2 = fieldNamePattern.matcher(second);
						if (m2.find())
						{
							name = m2.group(1);
						}
					}
					responseBody = hiddenFieldMatcher.replaceFirst("<br><span style=\"font-weight: bold; color: black;background-color: white\">" +
							"Hidden field \"" + name + "\"$1 type=\"text\" $2</span><br>");
				}
				else
				{
					break;
				}
			}
			body = responseBody.getBytes();
		}
		else
		{
			body =  transaction.getResponseBody();
		}
		
		return body;
	}
	
	private void forbiddenMessage(HttpRequest httpRequest, HttpResponse response)
	{
		String body =
		        "The scanner is not allowed to access the requested URL (" + httpRequest.getRequestLine().getUri()
		                + ").";
		errorMessage(httpRequest, response, 403, "Forbidden", body);
	}
	
	private void errorMessage(HttpRequest httpRequest, HttpResponse response, int code, String reason, String body)
	{
		response.setStatusCode(code);
		response.setReasonPhrase(reason);
		StringEntity entity = null;
		try
		{
			entity = new StringEntity(body);
		}
		catch (UnsupportedEncodingException e)
		{
		}
		response.setEntity(entity);
		response.addHeader("Server", "Grendel-Scan proxy");
		response.addHeader("Content-Length", String.valueOf(body.getBytes().length));
	}

}
