package com.grendelscan.proxy;


import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.apache.http.client.HttpClient;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HttpService;

import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;

public class RequestListenerThread extends Thread
{
	private final HttpParams params;
	private Proxy proxy;
	private Scan scan;
	private final ServerSocket serverSocket;
	
	public RequestListenerThread(Scan scan, ServerSocket serverSocket, Proxy proxy)
	{
		this.proxy = proxy;
		this.scan = scan;
		this.serverSocket = serverSocket;
		params = MiscHttpFactory.createDefaultHttpParams();
	}
	
	public void shutdown()
	{
        this.interrupt();
	}
	
	@Override
	public void run()
	{
		Socket socket;
		Debug.debug("Listening on port " + serverSocket.getLocalPort());
		while (!Thread.interrupted() && proxy.isRunning())
		{
			synchronized (serverSocket)
			{
				try
				{
					socket = serverSocket.accept();
				}
				catch (IOException e)
				{
					//Debug.errDebug("Problem with accepting a proxy socket: " + e.toString(), e);
					continue;
				}
			}
			
			try
			{
				if (!proxy.isRunning())
				{
					break;
				}
				// Set up HTTP connection
				DefaultHttpProxyConnection connection = new DefaultHttpProxyConnection();
				Debug.debug("Incoming connection from " + socket.getInetAddress());
				connection.bind(socket, params);
				
				HttpService httpService = proxy.createHttpService(socket, params);
				
				if (!proxy.isRunning())
				{
					break;
				}
				// Start worker thread
				Thread workerThread = new WorkerThread(httpService, connection, proxy);
				workerThread.setDaemon(true);
				workerThread.start();
			}
			catch (InterruptedIOException ex)
			{
				break;
			}
			catch (IOException e)
			{
				Debug.errDebug("I/O error initializing connection thread: " + e.getMessage(), e);
				break;
			}
			
		}
	}
}
