package com.grendelscan.proxy.authWizardProxy;

import java.io.IOException;
import java.net.Socket;

import org.apache.http.*;
import org.apache.http.impl.DefaultConnectionReuseStrategy;
import org.apache.http.impl.DefaultHttpResponseFactory;
import org.apache.http.message.BasicHttpEntityEnclosingRequest;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.*;

import com.grendelscan.GUI.*;
import com.grendelscan.utils.*;
import com.grendelscan.proxy.*;
import com.grendelscan.requester.http.client.CustomHttpClient;
import com.grendelscan.scan.Scan;

public class AuthWizardRequestHandler extends HttpProxyRequestHandler
{
	AuthWizardDialog wizardDialog;
	AuthWizardProxy wizardProxy;
	
	public AuthWizardRequestHandler(Scan scan, Socket socket, Proxy proxy, CustomHttpClient proxyHttpClient, 
			boolean ssl, int sslPort, AuthWizardDialog wizardDialog, AuthWizardProxy wizardProxy)
    {
		super(scan, socket, proxy, proxyHttpClient, ssl, sslPort);
	    this.wizardDialog = wizardDialog;
	    this.wizardProxy = wizardProxy;
    }

	@Override
    protected HttpService makeHttpServiceProxy(Scan scan, Socket socket, Proxy proxy, CustomHttpClient proxyHttpClient,
            boolean ssl, int sslPort)
    {
		HttpParams params = MiscHttpFactory.createDefaultHttpParams();
		BasicHttpProcessor httpproc = new BasicHttpProcessor();
		httpproc.addInterceptor(new ResponseConnControl());
		
		// Set up request handlers
		HttpRequestHandlerRegistry reqistry = new HttpRequestHandlerRegistry();
		reqistry.register("*", new AuthWizardRequestHandler(scan, socket, proxy, proxyHttpClient, ssl, sslPort, wizardDialog, wizardProxy));
		
		// Set up the HTTP service
		HttpService httpService =
		        new HttpService(httpproc, new DefaultConnectionReuseStrategy(), new DefaultHttpResponseFactory());
		httpService.setParams(params);
		httpService.setHandlerResolver(reqistry);
		
		return httpService;
    }

	public void handle(HttpRequest request, HttpResponse response, HttpContext context) throws HttpException,
	        IOException
	{
		String method = request.getRequestLine().getMethod();
		if (method.equalsIgnoreCase("CONNECT"))
		{
			handleConnectMethod(request, response, context);
		}
		else
		{
			wizardProxy.stopProxy();
			wizardDialog.setHttpMethod(method);
			wizardDialog.setUri(request.getRequestLine().getUri());
			if (method.equalsIgnoreCase("POST"))
			{
				if (request instanceof BasicHttpEntityEnclosingRequest)
				{
					HttpEntity entity = ((BasicHttpEntityEnclosingRequest) request).getEntity();
					String postQuery = new String(HttpUtils.entityToByteArray(entity, 0));
					wizardDialog.setPostQuery(postQuery);
				}
				else
				{
					Debug.errDebug("Problem with post in wizard");
				}
			}
			wizardDialog.requestComplete();
		}
	}
	
}
