package com.grendelscan.proxy.ssl;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLEngineResult;
import javax.net.ssl.SSLException;

public class SSLInputStream extends InputStream
{
	private int bufferSize = 2048;
	
	private ByteBuffer inEncrypted;
	private ByteBuffer inPlain;
	private InputStream inputStream;
	private SSLEngine sslEngine;
	
	public SSLInputStream(SSLEngine sslEngine, InputStream inputStream)
	{
		this.sslEngine = sslEngine;
		this.inputStream = inputStream;
		
		int netBuffersize = sslEngine.getSession().getPacketBufferSize();
		inEncrypted = ByteBuffer.allocateDirect(netBuffersize);
		
		int appBuffersize = sslEngine.getSession().getApplicationBufferSize();
		inPlain = ByteBuffer.allocateDirect(appBuffersize);
	}
	
	@Override
	public int available() throws IOException
	{
		// I'm not sure this is the right way of doing it
		return inPlain.position();
	}
	
	@Override
	public int read() throws IOException
	{
		inPlain.flip();
		byte read = inPlain.get();
		inPlain.compact();
		return read;
	}
	
	public void readFromSSL() throws SSLException, IOException
	{
		readEncryptedData();
		decryptData();
	}
	
	private boolean decryptData() throws SSLException, IOException
	{
		boolean decrypted = false;
		
		// if the encrypted buffer has been written to
		if (inEncrypted.position() > 0)
		{
			SSLEngineResult result = unwrapData();
			switch (result.getStatus())
			{
				case CLOSED:
				{
					// need some way of reporting a closed connection
					break;
				}
				case OK:
				{
					decrypted = true;
					break;
				}
				default:
				{
					// problem
				}
			}
		}
		return decrypted;
	}
	
	private int readEncryptedData() throws IOException
	{
		byte bytes[] = new byte[bufferSize];
		int count = inputStream.read(bytes);
		inEncrypted.put(bytes, 0, count);
		return count;
	}
	
	public SSLEngineResult unwrapData() throws IOException
	{
//		inPlain.flip();
		SSLEngineResult result = sslEngine.unwrap(inEncrypted, inPlain);
		inPlain.compact();
		return result;
	}

}
