package com.grendelscan.proxy.ssl;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;

import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLEngineResult;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLEngineResult.Status;

public class SSLOutputStream extends OutputStream
{

	private ByteBuffer outEncrypted;
	private ByteBuffer outPlain;
	private SSLEngine sslEngine;
	private OutputStream outputStream;
	private int sslBufferSize;
	
	public SSLOutputStream(SSLEngine sslEngine, OutputStream outputStream)
    {
	    this.sslEngine = sslEngine;
	    this.outputStream = outputStream;

	    sslBufferSize = sslEngine.getSession().getPacketBufferSize();
		outEncrypted = ByteBuffer.allocateDirect(sslBufferSize);
		
		int appBuffersize = sslEngine.getSession().getApplicationBufferSize();
		outPlain = ByteBuffer.allocateDirect(appBuffersize);
    }


	@Override
	public void write(int i) throws IOException
	{
		byte b = (byte) i;
		outPlain.put(b);
	}
	
	/**
	 * 
	 * outPlain.flip();
	 * SSLEngineResult result = sslEngine.wrap(outPlain, outEncrypted);
	 * outPlain.compact();
	 */
	public SSLEngineResult wrapData() throws IOException
	{
		outPlain.flip();
		SSLEngineResult result = sslEngine.wrap(outPlain, outEncrypted);
		outPlain.compact();
		return result;
	}
	
	
	@Override
	public void flush() throws SSLException, IOException
	{
		if (outPlain.position() > 0)
		{
			SSLEngineResult result = wrapData();
			if (result.getStatus() == Status.CLOSED ||
					(sslEngine.isInboundDone() && sslEngine.isOutboundDone()))
			{
				// handle closed session
			}

			int count = outEncrypted.position();
			byte buffer[] = new byte[count];
			
			outEncrypted.get(buffer, 0, count);
			outEncrypted.compact();
			
			outputStream.write(buffer);
		}

	}
}
