package com.grendelscan.proxy.ssl;

/*
 * $HeadURL:
 * https://svn.apache.org/repos/asf/httpcomponents/httpcore/trunk/module-nio/src/main/java/org/apache/http/impl/nio/reactor/SSLIOSession.java $
 * $Revision: 600151 $ $Date: 2007-12-01 08:57:51 -0700 (Sat, 01 Dec 2007) $
 * 
 * ==================================================================== Licensed
 * to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership. The ASF licenses this file to you
 * under the Apache License, Version 2.0 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain a copy of the
 * License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many individuals on
 * behalf of the Apache Software Foundation. For more information on the Apache
 * Software Foundation, please see <http://www.apache.org/>.
 * 
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

import org.apache.http.impl.AbstractHttpServerConnection;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import com.grendelscan.utils.Debug;

public class TunneledSSLConnection extends AbstractHttpServerConnection
{
	
	private volatile boolean closed;
	
	private boolean open;
	
	private Socket socket;
	
	private SSLEngine sslEngine;
	
	private InputStream sslInputStream;
	
	private OutputStream sslOutputStream;
	private SSLTunnelInputBuffer sslTunnelInputBuffer;
	private SSLTunnelOutputBuffer sslTunnelOutputBuffer;
	private HttpParams params;
	private SSLSocketFactory sslSocketFactory;
	private SSLSocket sslSocket;
	public TunneledSSLConnection(Socket socket) 
			throws 
				SSLException, 
				IOException, 
				UnrecoverableKeyException, 
				KeyManagementException, 
				KeyStoreException, 
				NoSuchAlgorithmException, 
				CertificateException
	{
		
		initializeSSL();

		
		this.socket = socket;
		int buffersize = HttpConnectionParams.getSocketBufferSize(params);
		//sslsock=(SSLSocket)_factory.createSocket(sock,sock.getInetAddress().getHostName(),sock.getPort(),true);
		sslSocket = (SSLSocket) sslSocketFactory.createSocket(socket, socket.getInetAddress().getHostName(), socket.getPort(), true);
		sslSocket.setUseClientMode(false);
		
//		sslInputStream = new SSLInputStream(sslEngine, socket.getInputStream());
		sslInputStream = sslSocket.getInputStream();
		sslTunnelInputBuffer = new SSLTunnelInputBuffer(sslInputStream, buffersize, params);
		
		//sslOutputStream = new SSLOutputStream(sslEngine, socket.getOutputStream());
		sslOutputStream = sslSocket.getOutputStream();
		sslTunnelOutputBuffer = new SSLTunnelOutputBuffer(sslOutputStream, buffersize, params);
		
//		sslEngine.setUseClientMode(false);
//		sslEngine.beginHandshake();
//		doHandshake();
		init(sslTunnelInputBuffer, sslTunnelOutputBuffer, params);
		open = true;
	}
	
	private void initializeSSL() throws KeyStoreException, 
									NoSuchAlgorithmException, 
									CertificateException, 
									IOException, 
									UnrecoverableKeyException, 
									KeyManagementException
	{
		
//        KeyStore ks = null;
//        String _keystore = "server.p12";
//        char[] _keystorepass = "password".toCharArray();
//        char[] _keypassword = "password".toCharArray();
//        KeyManagerFactory kmf = null;
//        SSLContext sslcontext = null;
//        try {
//            ks = KeyStore.getInstance("PKCS12");
//            InputStream is = getClass().getClassLoader().getResourceAsStream("server.p12");
//            if (is == null) throw new NullPointerException("No keystore found!!");
//            ks.load(is, _keystorepass);
//            kmf = KeyManagerFactory.getInstance("SunX509");
//            kmf.init(ks, _keypassword);
//            sslcontext = SSLContext.getInstance("SSLv3");
//            sslcontext.init(kmf.getKeyManagers(), null, null);
//            sslSocketFactory = sslcontext.getSocketFactory();
//        } catch (Exception e) {
//        }

        
		KeyManagerFactory kmfactory;
        try
        {
	        File kfile = new File("conf\\server.pkcs12"); //conf\\scanner.keystore
	        FileInputStream keystoreStream = new FileInputStream(kfile);
	        KeyStore keystore = KeyStore.getInstance("PKCS12"); // jks
	        keystore.load(keystoreStream, "password".toCharArray());
	        String s = KeyManagerFactory.getDefaultAlgorithm();
	        kmfactory = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
	        kmfactory.init(keystore, "password".toCharArray());
			SSLContext sslContext = SSLContext.getInstance("TLS");
			sslContext.init(kmfactory.getKeyManagers(), null, null);
			sslSocketFactory = sslContext.getSocketFactory();
        }
        catch (Exception e)
        {
	        // TODO Auto-generated catch block
	        Debug.errDebug("problem: " + e.toString(), e);
        }

		
		params = new BasicHttpParams();
		params.setIntParameter(CoreConnectionPNames.SO_TIMEOUT, 5000).setIntParameter(
		        CoreConnectionPNames.SOCKET_BUFFER_SIZE, 8 * 1024).setBooleanParameter(
		        CoreConnectionPNames.STALE_CONNECTION_CHECK, false).setBooleanParameter(
		        CoreConnectionPNames.TCP_NODELAY, true).setParameter(CoreProtocolPNames.ORIGIN_SERVER,
		        "Jakarta-HttpComponents-NIO/1.1");
//		sslEngine = sslContext.createSSLEngine();
	}

	public void close()
	{
		if (closed)
		{
			return;
		}
		closed = true;
		synchronized (this)
		{
			sslEngine.closeOutbound();
		}
	}
	
	public int getSocketTimeout()
	{
		if (socket != null)
		{
			try
			{
				return socket.getSoTimeout();
			}
			catch (SocketException ignore)
			{
				return -1;
			}
		}
		else
		{
			return -1;
		}
	}
	
	public boolean isOpen()
	{
		return open;
	}
	
	public void setSocketTimeout(int timeout)
	{
		assertOpen();
		if (socket != null)
		{
			try
			{
				socket.setSoTimeout(timeout);
			}
			catch (SocketException ignore)
			{
			}
		}
	}
	
	public void shutdown() throws IOException
	{
		open = false;
		Socket tmpsocket = socket;
		if (tmpsocket != null)
		{
			tmpsocket.close();
		}
	}
	
//	private void doHandshake() throws SSLException, IOException
//	{
//		boolean handshaking = true;
//		
//		SSLEngineResult result = null;
//		while (handshaking)
//		{
//			sslOutputStream.flush();
//			sslInputStream.readFromSSL();
//
//			switch (sslEngine.getHandshakeStatus())
//			{
//				case NEED_WRAP:
//					// Generate outgoing handshake data
//					result = sslOutputStream.wrapData();
//					if (result.getStatus() != Status.OK)
//					{
//						handshaking = false;
//					}
//					if (result.getStatus() == Status.CLOSED)
//					{
//						closed = true;
//					}
//					break;
//				case NEED_UNWRAP:
//					// Process incoming handshake data
//					result = sslInputStream.unwrapData();
//					if (result.getStatus() != Status.OK)
//					{
//						handshaking = false;
//					}
//					if (result.getStatus() == Status.CLOSED)
//					{
//						closed = true;
//					}
//					break;
//				case NEED_TASK:
//					Runnable r = sslEngine.getDelegatedTask();
//					r.run();
//					break;
//				case NOT_HANDSHAKING:
//					handshaking = false;
//					break;
//			}
//		}
//	}
	
	@Override
	protected void assertOpen() throws IllegalStateException
	{
		if (!open)
		{
			throw new IllegalStateException("Connection is not open");
		}
	}
}
