package com.grendelscan.requester;


import java.awt.Image;
import java.io.ByteArrayInputStream;
import java.net.URISyntaxException;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;

import org.cobra_grendel.html.HttpRequest;
import org.cobra_grendel.html.ReadyStateChangeListener;
import org.w3c.dom.Document;

import com.grendelscan.requester.http.factories.HttpTransactionFactory;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;

/**
 * This is used by HttpTransaction and should not be used by
 * itself unless you really know what you are doing. It may
 * go away in the future.
 * 
 * @author David Byrne
 */
public class CobraHttpRequest implements HttpRequest
{
	private AbstractHttpTransaction rawTransaction;
	private Scan scan;

	public CobraHttpRequest(Scan scan)
	{
		this.scan = scan;
	}

	public void abort()
	{
		// TODO Auto-generated method stub

	}

	public void addReadyStateChangeListener(ReadyStateChangeListener arg0)
	{
		// TODO Auto-generated method stub

	}

	public String getAllResponseHeaders()
	{
		if (rawTransaction != null)
		{
			return rawTransaction.getAllResponseHeadersAsString();
		}
		return null;
	}

	public int getReadyState()
	{
		if (rawTransaction == null)
		{
			return HttpRequest.STATE_UNINITIALIZED;
		}

		if (rawTransaction.isResponsePresent())
		{
			return HttpRequest.STATE_COMPLETE;
		}

		return HttpRequest.STATE_LOADING;
	}

	public byte[] getResponseBytes()
	{
		if (rawTransaction != null)
		{
			return rawTransaction.getResponseBody();
		}
		return null;
	}

	public String getResponseHeader(String headerName)
	{
		if (rawTransaction != null)
		{
			return rawTransaction.getFirstResponseHeader(headerName).getValue();
		}
		return null;
	}

	public Image getResponseImage()
	{
		// TODO Auto-generated method stub
		return null;
	}

	public String getResponseText()
	{
		if (rawTransaction != null)
		{
			return rawTransaction.getResponseBodyAsString();
		}
		return null;
	}

	public Document getResponseXML()
	{
		if (rawTransaction != null)
		{
			try
			{
				java.io.InputStream in = new ByteArrayInputStream(rawTransaction.getResponseBody());
				return DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(in);
			}
			catch (Exception err)
			{
				Debug.errDebug("Unable to parse response as XML." + err.toString());
				return null;
			}
		}
		return null;
	}

	public int getStatus()
	{
		if (rawTransaction != null)
		{
			return rawTransaction.getResponseStatusCode();
		}
		return 0;
	}

	public String getStatusText()
	{
		if (rawTransaction != null)
		{
			return rawTransaction.getStatusText();
		}
		return null;
	}

	public void open(String method, String url)
	{
		try
		{
			rawTransaction = HttpTransactionFactory.createTransaction(scan, method, url, 0, TransactionSource.COBRA);
			// We don't want to test stuff that Cobra does because we have other ways of finding it
			rawTransaction.execute("CobraHttpRequest", false);
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Problem with URI syntax at CobraHttpRequest.open: " + e.toString(), e);
		}
	}

	public void open(String method, String url, boolean asyncFlag)
	{
		open(method, url);
	}

	public void open(String method, String url, boolean asyncFlag, String userName)
	{
		open(method, url);
	}

	public void open(String method, String url, boolean asyncFlag, String userName, String password)
	{
		open(method, url);
	}

	public void open(String method, URL url)
	{
		try
		{
			rawTransaction = HttpTransactionFactory.createTransaction(scan, method, url.toURI(), 0, TransactionSource.COBRA);
			rawTransaction.execute("CobraHttpRequest", false);
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Problem with URI syntax at CobraHttpRequest.open: " + e.toString());
			e.printStackTrace();
		}
	}

	public void open(String method, URL url, boolean asyncFlag)
	{
		open(method, url);
	}

}
