/*
 * TransactionRecord.java
 * 
 * Created on September 16, 2007, 1:15 PM
 * 
 * To change this template, choose Tools | Template Manager and open the
 * template in the editor.
 */

package com.grendelscan.requester;

import java.util.regex.*;
import java.io.IOException;
import java.lang.ref.Reference;
import java.lang.ref.SoftReference;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

import org.apache.http.NameValuePair;

import com.grendelscan.data.Database;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.UnvalidatedHttpTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.StringUtils;
import com.grendelscan.utils.URIStringUtils;
import com.grendelscan.GUI.http.*;
/**
 * 
 * @author Administrator
 */
public class TransactionRecord
{
	private Scan scan;
	private Map<Integer, Reference<AbstractHttpTransaction>> transactions;
	private Map<String, Set<String>> uris;
	private Database database;
	
	private int transactionCommandCount = 0;
	
	/** Creates a new instance of TransactionRecord 
	 * @throws SQLException */
	public TransactionRecord(Scan scan) throws SQLException
	{
		this.scan = scan;
		database = scan.getDatabase();
		uris = Collections.synchronizedMap(new HashMap<String, Set<String>>(500));
		transactions = Collections.synchronizedMap(new HashMap<Integer, Reference<AbstractHttpTransaction>>(200));
		if (scan.getScanSettings().isLoadDatabase())
		{
			setNextTransactionID();
		}
		else
		{
			initializeDatabase();
		}
	}
	
	private void setNextTransactionID() throws SQLException
	{
		synchronized (database)
		{
			PreparedStatement transactionSelectStatement =
			        database
			                .prepareStatement("SELECT MAX(id) FROM http_transactions");
			transactionSelectStatement.execute();
			
			ResultSet results = transactionSelectStatement.getResultSet();
			results.next();
			int id = results.getInt(1);
			results.close();
			transactionSelectStatement.close();
			AbstractHttpTransaction.setLastID(id);
		}
	}
	
	public void addOrRefreshTransactionReference(AbstractHttpTransaction transaction)
	{
		Reference<AbstractHttpTransaction> ref = null;
		if (transaction instanceof UnvalidatedHttpTransaction)
		{
			/*
			 * As long as we're only using unvalidated transactions for Nikto,
			 * there's no reason to keep them in memory. If they are used for
			 * something else, it won't break anything, it will just require a
			 * database lookup.
			 */
			// ref = new WeakReference<AbstractHttpTransaction>(transaction);
		}
		else
		{
			ref = new SoftReference<AbstractHttpTransaction>(transaction);
		}
		if (ref != null)
		{
			cleanMap();
			synchronized (transactions)
			{
				transactions.put(transaction.getId(), ref);
			}
		}
	}
	
	public AbstractHttpTransaction getTransaction(int id)
	{
		if (id <= 0)
		{
			return null;
		}
		
		AbstractHttpTransaction transaction = null;
		
		cleanMap();
		synchronized (transactions)
		{
			if (transactions.containsKey(id))
			{
				Reference<AbstractHttpTransaction> ref = transactions.get(id);
				transaction = ref.get();
			}
		}
		
		if (transaction == null)
		{
			transaction = loadTransaction(id);
		}
		return transaction;
	}
	
	public synchronized boolean hasUriBeenRequested(String uri, String username, boolean ignoreUser)
	{
		if (username == null)
		{
			username = "";
		}
		Boolean oldRequest = false;
		String hash = StringUtils.md5Hash(normalizeUniqueURI(uri));
		// Has the URI been sent before
		if (uris.keySet().contains(hash))
		{
			// If the user doesn't matter, or the user has requested the uri
			if (ignoreUser || uris.get(hash).contains(username))
			{
				oldRequest = true;
			}
			else
			{
				// Add user to the list of users that have requested the uri
				uris.get(hash).add(username);
			}
		}
		else
		// The URI has never been requested before
		{
			Set<String> userList = new HashSet<String>(1);
			uris.put(hash, userList);
			userList.add(username);
		}
		return oldRequest;
	}
	
	public void initializeDatabase()
	{
		Debug.debug("Initializing database for transaction storage");
		try
		{
			String tableQuery =
			        "CREATE TABLE http_transactions (\n" + "id INT,\n" + "execution_time BIGINT, executed SMALLINT,\n"
			                + "redirect_composite SMALLINT, method varchar(15),\n" + "host varchar(100),\n" + "path varchar(2000),\n"
			                + "query varchar(2000),\n" + "response_code INT, response_line varchar(200),\n" + "serialized_transaction BLOB,\n"
			                + "source varchar(20), PRIMARY KEY (id))";
			String indexQuery0 = "CREATE INDEX IDX_DEFAULT ON http_transactions (id, executed, redirect_composite)";
//			String indexQuery1 = "CREATE INDEX IDX_METHOD ON http_transactions (id, executed, method)";
//			String indexQuery2 = "CREATE INDEX IDX_HOST ON http_transactions (id, executed, host)";
//			String indexQuery3 = "CREATE INDEX IDX_PATH ON http_transactions (id, executed, path)";
//			String indexQuery4 = "CREATE INDEX IDX_QUERY ON http_transactions (id, executed, query)";
//			String indexQuery5 = "CREATE INDEX IDX_RESPONSE ON http_transactions (id, executed, response)";
//			String indexQuery6 = "CREATE INDEX IDX_SOURCE ON http_transactions (id, executed, source)";
			synchronized (database)
			{
				database.execute(tableQuery);
				database.execute(indexQuery0);
//				database.execute(indexQuery1);
//				database.execute(indexQuery2);
//				database.execute(indexQuery3);
//				database.execute(indexQuery4);
//				database.execute(indexQuery5);
//				database.execute(indexQuery5);
//				database.execute(indexQuery6);
			}
		}
		catch (SQLException e)
		{
			Debug.errDebug("Problem with creating database: " + e.toString(), e);
			System.exit(1);
		}
	}
	
	public boolean saveOrUpdateTransaction(AbstractHttpTransaction transaction)
	{
		boolean saved = false;
		cleanMap();
		if (transaction.isSaved())
		{
			saved = updateTransaction(transaction);
		}
		else
		{
			saved = saveTransaction(transaction);
		}
		addOrRefreshTransactionReference(transaction);
		return saved;
	}
	
	/**
	 * This will purge the null references from the map. This isn't critical, so
	 * it will only run every 1000
	 */
	private synchronized void cleanMap()
	{
		if (transactionCommandCount++ > 1000)
		{
			transactionCommandCount = 0;
			synchronized (transactions)
			{
				Set<Integer> keys = new HashSet<Integer>(transactions.keySet());
				for (int key: keys)
				{
					Reference<AbstractHttpTransaction> ref = transactions.get(key);
					if ((ref != null) && (ref.get() == null))
					{
						transactions.remove(key);
					}
				}
			}
		}
	}
	
	private AbstractHttpTransaction loadTransaction(int id)
	{
		AbstractHttpTransaction transaction = null;
		try
		{
			synchronized (database)
			{
				PreparedStatement transactionSelectStatement =
				        database
				                .prepareStatement("SELECT serialized_transaction FROM http_transactions WHERE id = ?");
				transactionSelectStatement.setInt(1, id);
				transactionSelectStatement.execute();
				
				ResultSet results = transactionSelectStatement.getResultSet();
				results.next();
				transaction = (AbstractHttpTransaction) Database.blobToObject(results.getBlob(1));
				results.close();
				transactionSelectStatement.close();
			}
			transaction.setScan(scan);
			addOrRefreshTransactionReference(transaction);
			transaction.setSaved(true);
			transaction.setLoaded(true);
		}
		catch (SQLException e)
		{
			Debug.errDebug("Problem loading HTTP transaction (" + id + ") from database: " + e.toString(), e);
		}
		catch (ClassNotFoundException e)
		{
			Debug.errDebug("Problem loading HTTP transaction (" + id + ") from database: " + e.toString(), e);
		}
		catch (IOException e)
		{
			Debug.errDebug("Problem loading HTTP transaction (" + id + ") from database: " + e.toString(), e);
		}
		
		return transaction;
	}
	
	private String normalizeUniqueURI(String uri)
	{
		String newUri = uri;
		
		// Ignore the irrelevant parameters
		for (String parameterName: scan.getScanSettings().getIrrelevantQueryParameters())
		{
			newUri = URIStringUtils.removeQueryParameter(newUri, parameterName);
		}
		List<NameValuePair> paramSet = URIStringUtils.getQueryParametersFromUri(newUri);
		String params[] = new String[paramSet.size()];
		int index = 0;
		for (NameValuePair param: paramSet)
		{
			params[index++] = param.getName() + "=" + param.getValue() + "&";
		}
		Arrays.sort(params);
		
		String base = URIStringUtils.getBaseUri(newUri) + "?";
		for (String param: params)
		{
			base += param;
		}
		
		return URIStringUtils.normalizeUri(base);
	}
	
	private boolean saveTransaction(AbstractHttpTransaction transaction)
	{
		boolean saved = false;
		try
		{
			short executed = (short) (transaction.isSuccessfullExecution() ? 1 : 0);
			short redirect = (short) (transaction.getOriginalResponseTransactionID() > 0 ? 1 : 0);
			transaction.setSaved(true);
			synchronized (database)
			{
				PreparedStatement transactionInsertStatement =
				        database.prepareStatement("INSERT INTO http_transactions "
				                + "(id, executed, execution_time, method, host, path, query, response_code, response_line, source, serialized_transaction, redirect_composite) "
				                + "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				transactionInsertStatement.setInt(1, transaction.getId());
				transactionInsertStatement.setShort(2, executed);
				transactionInsertStatement.setLong(3, transaction.getExecutionTime());
				transactionInsertStatement.setString(4, transaction.getMethod());
				transactionInsertStatement.setString(5, transaction.getHostUri());
				transactionInsertStatement.setString(6, transaction.getPath() + transaction.getFilename());
				transactionInsertStatement.setString(7, transaction.getUriQueryString());
				transactionInsertStatement.setInt(8, transaction.getResponseStatusCode());
				transactionInsertStatement.setString(9, transaction.getStatusLineString());
				transactionInsertStatement.setString(10, transaction.getSource().getText());
				transactionInsertStatement.setBlob(11, Database.serializableToBlob(transaction));
				transactionInsertStatement.setShort(12, redirect);
				transactionInsertStatement.execute();
				transactionInsertStatement.close();
				saved = true;
			}
		}
		catch (SQLException e)
		{
			Debug.errDebug("Problem saving HTTP transaction to database: " + e.toString(), e);
			transaction.setSaved(false);
		}
		catch (IOException e)
		{
			Debug.errDebug("Problem saving HTTP transaction to database: " + e.toString(), e);
			transaction.setSaved(false);
		}
		if (!saved)
		{
			Debug.errDebug("Error saving transaction");
		}
		return saved;
	}
	
	private boolean updateTransaction(AbstractHttpTransaction transaction)
	{
		boolean saved = false;

		try
		{
			short executed = (short) (transaction.isSuccessfullExecution() ? 1 : 0);
			short redirect = (short) (transaction.getOriginalResponseTransactionID() > 0 ? 1 : 0);
			synchronized (database)
			{
				PreparedStatement transactionUpdateStatement =
					database.prepareStatement("UPDATE http_transactions " + 
				        			"SET executed = ?, " + 
				        				"execution_time = ?, " +
				        				"method = ?, " + 
				        				"host = ?, " + 
				        				"path = ?, " + 
				        				"query = ?, " + 
				        				"response_code = ?, " + 
				        				"response_line = ?, " + 
				        				"source = ?, " + 
				        				"serialized_transaction = ?, " + 
				        				"redirect_composite = ? " +
				        			"WHERE id = ?");
				
				transactionUpdateStatement.setShort(1, executed);
				transactionUpdateStatement.setLong(2, transaction.getExecutionTime());
				transactionUpdateStatement.setString(3, transaction.getMethod());
				transactionUpdateStatement.setString(4, transaction.getHostUri());
				transactionUpdateStatement.setString(5, transaction.getPath() + transaction.getFilename());
				transactionUpdateStatement.setString(6, transaction.getUriQueryString());
				transactionUpdateStatement.setInt(7, transaction.getResponseStatusCode());
				transactionUpdateStatement.setString(8, transaction.getStatusLineString());
				transactionUpdateStatement.setString(9, transaction.getSource().getText());
				transactionUpdateStatement.setBlob(10, Database.serializableToBlob(transaction));
				transactionUpdateStatement.setShort(11, redirect);
				transactionUpdateStatement.setInt(12, transaction.getId());

				transactionUpdateStatement.execute();
				transactionUpdateStatement.close();
				saved = true;
			}
		}
		catch (SQLException e)
		{
			Debug.errDebug("Problem updating HTTP transaction in database: " + e.toString(), e);
		}
		catch (IOException e)
		{
			Debug.errDebug("Problem updating HTTP transaction in database: " + e.toString(), e);
		}
		if (!saved)
		{
			Debug.errDebug("Error updating transaction");
		}
		return saved;
	}
	
	public List<TransactionSummary> getTransactions(TransactionTableSettings settings, int startingID, boolean topToBottom, boolean getRedirectComposites)
	{
		List<TransactionSummary> transactionIDs = new ArrayList<TransactionSummary>(settings.maxRecords);
		Connection connection = database.getConnection();
		int startRecord = 0;
		int endRecord = 0;
		if (topToBottom)
		{
			endRecord = startingID - 1; 
		}
		else
		{
			startRecord = startingID + 1; 
		}
//		long start = new Date().getTime();
		while (transactionIDs.size() < settings.maxRecords)
		{
			int increment = (int) ((settings.maxRecords - transactionIDs.size()) * 1.5);
			if (topToBottom)
			{
				startRecord = endRecord + 1;
				endRecord = startRecord + increment;
			}
			else
			{
				endRecord = startRecord - 1;
				startRecord = endRecord - increment;
			}
			
			if (endRecord < 1 || startRecord > AbstractHttpTransaction.getLastID())
			{
				break;
			}
			
			try
			{
//				synchronized (database)
				{
					PreparedStatement transactionSelectStatement = connection.prepareStatement(
					                	"SELECT id, execution_time, method, host, path, query, response_code, response_line, source " +
					                	"FROM http_transactions " +
					                	"WHERE " +
					                		"id >= ? AND id <= ? " +
					                		"AND executed = 1 " +
					                		(getRedirectComposites ? "" : "AND redirect_composite = 0 ") +
					                	"ORDER BY id ASC");
					transactionSelectStatement.setInt(1, startRecord);
					transactionSelectStatement.setInt(2, endRecord);
					transactionSelectStatement.execute();
					ResultSet results = transactionSelectStatement.getResultSet();
					while (results.next() && transactionIDs.size() < settings.maxRecords)
					{
						TransactionSummary transactionSummary = new TransactionSummary(results.getInt(1), results.getLong(2), results.getString(3),
								results.getString(4), results.getString(5), results.getString(6), results.getInt(7), results.getString(8), 
								results.getString(9));
						if (settings.checkTransactionCriteria(transactionSummary))
						{
							transactionIDs.add(transactionSummary);
						}
					}
					results.close();
					transactionSelectStatement.close();
				}
			}
			catch (SQLException e)
			{
				Debug.errDebug("Problem when searching for transactions: " + e.toString(), e);
			}
		}
//		long end = new Date().getTime();
//		System.out.println(end-start);
		return transactionIDs;
	}
	
	public void writeAllExecutedToDisk()
	{
		Thread t = new Thread(new Runnable()
		{
			public void run()
            {
				Connection connection = database.getConnection();
				PreparedStatement transactionSelectStatement;
                try
                {
	                transactionSelectStatement = connection.prepareStatement(
	                    	"SELECT id " +
	                    	"FROM http_transactions " +
	                    	"WHERE executed = 1 ");
					transactionSelectStatement.execute();
					ResultSet results = transactionSelectStatement.getResultSet();
					while (results.next())
					{
						int id = results.getInt(1);
						AbstractHttpTransaction transaction = getTransaction(id);
						transaction.writeToDisk(true);
					}
					results.close();
					transactionSelectStatement.close();
                }
                catch (SQLException e)
                {
                	Debug.errDebug("Error exporting transactions: " + e.toString(), e);
                }
            }
		});
		t.start();
	}
	
}
