/*
 * FormBased.java
 * 
 * Created on September 13, 2007, 9:29 PM
 * 
 * To change this template, choose Tools | Template Manager and open the
 * template in the editor.
 */

package com.grendelscan.requester.authentication;


import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.factories.HttpTransactionFactory;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.AbstractStandardHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpPostTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;

/**
 * 
 * @author Administrator
 */
public class FormBasedAuthentication extends AuthenticationPackage
{

	protected AbstractStandardHttpTransaction loginTemplate;
	protected String userParameterName, passwordParameterName, method, postQuery;
	protected URI uri;

	/**
	 * 
	 * @param sharedComponents
	 * @param loginTemplate
	 * @param userParameterName
	 * @param passwordParameterName
	 */
	public FormBasedAuthentication(Scan scan, AbstractStandardHttpTransaction loginTemplate, String userParameterName,
	        String passwordParameterName)
	{
		super(scan);
		
		this.loginTemplate = loginTemplate;
		this.userParameterName = userParameterName;
		this.passwordParameterName = passwordParameterName;

		credentials = new HashMap<String, String>();
	}

	public FormBasedAuthentication(Scan scan, URI uri, String method, String postQuery, String userParameterName,
	        String passwordParameterName)
	{
		super(scan);
		this.uri = uri;
		this.method = method;
		this.postQuery = postQuery;
		this.userParameterName = userParameterName;
		this.passwordParameterName = passwordParameterName;

		credentials = new HashMap<String, String>();
	}

	
	@Override
	public AbstractHttpTransaction createAnyLoginTransaction()
	{
		return createLoginTransaction(credentials.keySet().iterator().next());
	}

	/**
	 * This returns an HttpTransaction that represents a login
	 */
	@Override
	public AbstractHttpTransaction createLoginTransaction(String username)
	{

		AbstractHttpTransaction transaction = getLoginTemplate().clone(TransactionSource.AUTHENTICATION);
		transaction.setAuthenticated(true);
		transaction.replaceQueryParameter(userParameterName, username);
		transaction.replaceQueryParameter(passwordParameterName, credentials.get(username));
		transaction.setLoginTransaction(true);
		return transaction;
	}

	@Override
	public List<AbstractHttpTransaction> createLoginTransactions()
	{
		ArrayList<AbstractHttpTransaction> transactions = new ArrayList<AbstractHttpTransaction>(credentials.size());
		for (String username: credentials.keySet())
		{
			transactions.add(createLoginTransaction(username));
		}

		return transactions;
	}

	public AbstractStandardHttpTransaction getLoginTemplate()
	{
		if (loginTemplate == null)
		{
			try
            {
	            loginTemplate = HttpTransactionFactory.createTransaction(scan, method, uri, 0, TransactionSource.AUTHENTICATION);
				if (method.equals("POST"))
				{
					((HttpPostTransaction) loginTemplate).setPayloadQueryString(postQuery);
				}
            }
            catch (URISyntaxException e)
            {
	            Debug.errDebug("Odd problem with the uri when creating a login template: " + e.toString(), e);
            }
            loginTemplate.updatePayload();
		}
		return loginTemplate;
	}

	public String getPasswordParameterName()
	{
		return passwordParameterName;
	}

	public String getUserParameterName()
	{
		return userParameterName;
	}

	public String getMethod()
    {
    	return method;
    }

	public String getPostQuery()
    {
    	return postQuery;
    }

	public URI getUri()
    {
    	return uri;
    }

}
