/**
 * CustomSSLVerifier.java Created by: David Byrne Created on: Jan 30, 2008
 */
package com.grendelscan.requester.http.SSL;


import java.io.IOException;
import java.security.cert.X509Certificate;

import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;

import org.apache.http.conn.ssl.BrowserCompatHostnameVerifier;
import org.apache.http.conn.ssl.X509HostnameVerifier;

import com.grendelscan.scan.Scan;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;

public class CustomSSLVerifier implements X509HostnameVerifier
{
	Scan scan;
	BrowserCompatHostnameVerifier verifier;

	public CustomSSLVerifier(Scan scan)
	{
		verifier = new BrowserCompatHostnameVerifier();
		this.scan = scan;
	}

	public final boolean verify(String host, SSLSession session)
	{
		if (!verifier.verify(host, session))
		{
			String briefDescription =
			        "A problem was found with the SSL certificate for " + host + ":" + session.getPeerPort();
			String longDescription =
			        "The SSL certificate for " + host + ":" + session.getPeerPort() + " had an unknown error";
			String impact =
			        "Problems with SSL certificates can lead to man-in-the-middle attacks, browser errors and more.";
			String recomendation = "Use current certificates from recognized Certificate Authorities.";
			String references = "";
			ReportableEvent event =
			        new ReportableEvent(null, "SSL Certificate check", EventSeverity.INFO, "https://" + host + ":"
			                + session.getPeerPort(), "SSL Certificate Error", briefDescription, longDescription,
			                impact, recomendation, references);
			scan.getReportCollector().addEvent(event);
		}

		return true;
	}

	public final void verify(String host, SSLSocket ssl) throws IOException
	{
		try
		{
	        SSLSession session = ssl.getSession();
//			verifier.verify(host, ssl);
		}
		catch (Exception e)
		{
			int port = ssl.getPort();
			String briefDescription = "A problem was found with the SSL certificate for " + host + ":" + port;
			String longDescription = "The SSL certificate for " + host + ":" + port + " had an error: " + e.toString();
			String impact =
			        "Problems with SSL certificates can lead to man-in-the-middle attacks, browser errors and more.";
			String recomendation = "Use current certificates from recognized Certificate Authorities.";
			String references = "";
			ReportableEvent event =
			        new ReportableEvent(null, "SSL Certificate check", EventSeverity.INFO, "https://" + host + ":"
			                + port, "SSL Certificate Error", briefDescription, longDescription, impact, recomendation,
			                references);
			scan.getReportCollector().addEvent(event);
		}
	}

	public final void verify(String host, String[] cns, String[] subjectAlts) throws SSLException
	{
		try
		{
			verifier.verify(host, cns, subjectAlts);
		}
		catch (SSLException e)
		{
			String briefDescription = "A problem was found with the SSL certificate for " + host;
			String longDescription = "The SSL certificate for " + host + " had an error: " + e.toString();
			String impact =
			        "Problems with SSL certificates can lead to man-in-the-middle attacks, browser errors and more.";
			String recomendation = "Use current certificates from recognized Certificate Authorities.";
			String references = "";
			ReportableEvent event =
			        new ReportableEvent(null, "SSL Certificate check", EventSeverity.INFO, "https://" + host,
			                "SSL Certificate Error", briefDescription, longDescription, impact, recomendation,
			                references);
			scan.getReportCollector().addEvent(event);
		}

	}

	public final void verify(String host, String[] cns, String[] subjectAlts, boolean strictWithSubDomains)
	        throws SSLException
	{
		try
		{
			verifier.verify(host, cns, subjectAlts, strictWithSubDomains);
		}
		catch (SSLException e)
		{
			String briefDescription = "A problem was found with the SSL certificate for " + host;
			String longDescription = "The SSL certificate for " + host + " had an error: " + e.toString();
			String impact =
			        "Problems with SSL certificates can lead to man-in-the-middle attacks, browser errors and more.";
			String recomendation = "Use current certificates from recognized Certificate Authorities.";
			String references = "";
			ReportableEvent event =
			        new ReportableEvent(null, "SSL Certificate check", EventSeverity.INFO, "https://" + host,
			                "SSL Certificate Error", briefDescription, longDescription, impact, recomendation,
			                references);
			scan.getReportCollector().addEvent(event);
		}

	}

	public final void verify(String host, X509Certificate cert) throws SSLException
	{
		try
		{
			verifier.verify(host, cert);
		}
		catch (SSLException e)
		{
			String briefDescription = "A problem was found with the SSL certificate for " + host;
			String longDescription = "The SSL certificate for " + host + " had an error: " + e.toString();
			String impact =
			        "Problems with SSL certificates can lead to man-in-the-middle attacks, browser errors and more.";
			String recomendation = "Use current certificates from recognized Certificate Authorities.";
			String references = "";
			ReportableEvent event =
			        new ReportableEvent(null, "SSL Certificate check", EventSeverity.INFO, "https://" + host,
			                "SSL Certificate Error", briefDescription, longDescription, impact, recomendation,
			                references);
			scan.getReportCollector().addEvent(event);
		}

	}

}
