package com.grendelscan.requester.http.client;

import com.grendelscan.scan.*;
import org.apache.http.HttpHost;
import org.apache.http.HttpVersion;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.AuthState;
import org.apache.http.client.params.AllClientPNames;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnManagerPNames;
import org.apache.http.conn.params.ConnPerRouteBean;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.scheme.SocketFactory;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.auth.DigestScheme;
import org.apache.http.impl.auth.RFC2617Scheme;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;

import com.grendelscan.requester.http.SSL.CustomSSLSocketFactory;
import com.grendelscan.utils.Debug;

public class ClientUtilities
{
	public static ClientConnectionManager createClientConnectionManager(HttpParams params)
	{
		SchemeRegistry supportedSchemes;
		
		supportedSchemes = new SchemeRegistry();
		
		// Register the "http" and "https" protocol schemes, they are
		// required by the default operator to look up socket factories.
		SocketFactory sf = PlainSocketFactory.getSocketFactory();
		supportedSchemes.register(new Scheme("http", sf, 80));
		
		CustomSSLSocketFactory sslsf;
        try
        {
	        sslsf = new CustomSSLSocketFactory();
			supportedSchemes.register(new Scheme("https", sslsf, 443));
        }
        catch (Exception e)
        {
        	Debug.errDebug("Problem creating SSLSocketFactory: " + e.toString(), e);
        	System.exit(0);
        }
		
		
		return new ThreadSafeClientConnManager(params, supportedSchemes);
	}
	
	public static HttpParams createHttpParams(Scan scan)
	{
		ScanSettings scanSettings = scan.getScanSettings();
		return createHttpParams(scan, scanSettings.getMaxTotalConnections(), scanSettings.getMaxConnectionsPerServer());
	}
	public static HttpParams createHttpParams(Scan scan, int maxTotalConnections, int maxConnectionsPerServer)
	{
		ScanSettings scanSettings = scan.getScanSettings();

		// prepare parameters
		HttpParams params = new BasicHttpParams();

		ConnPerRouteBean maxConnectionsPerRoute = new ConnPerRouteBean();
		params.setParameter(ConnManagerPNames.MAX_CONNECTIONS_PER_ROUTE, maxConnectionsPerRoute);
		params.setIntParameter(ConnManagerPNames.MAX_TOTAL_CONNECTIONS, maxTotalConnections);
		HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
		HttpProtocolParams.setContentCharset(params, "UTF-8");
		params.setParameter(CoreProtocolPNames.USER_AGENT, scanSettings.getUserAgentString());
		params.setBooleanParameter(ClientPNames.HANDLE_REDIRECTS, false);
		params.setIntParameter(CoreConnectionPNames.SO_TIMEOUT, scanSettings.getSocketReadTimeout() * 1000);
		params.setBooleanParameter(AllClientPNames.USE_EXPECT_CONTINUE, false);
		if (scanSettings.getUseUpstreamProxy())
		{
			HttpHost proxy = new HttpHost(scanSettings.getUpstreamProxyAddress(), scanSettings.getUpstreamProxyPort());
			params.setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);
		}
		return params;
	}
	
	public static HttpContext createHttpContext(Scan scan)
	{
		ScanSettings scanSettings = scan.getScanSettings();
		HttpContext context = new BasicHttpContext(null);
		if (scanSettings.getUseUpstreamProxy())
		{
			if (!scanSettings.getUpstreamProxyUsername().equals(""))
			{
				AuthState authState = new AuthState();
				UsernamePasswordCredentials credentials = new UsernamePasswordCredentials(scanSettings.getUpstreamProxyUsername(), scanSettings.getUpstreamProxyPassword());
				authState.setCredentials(credentials);
				RFC2617Scheme authScheme = null;
				switch (scanSettings.getUpstreamProxyAuthenticationType())
				{
					case BASIC: authScheme = new BasicScheme(); break;
					case DIGEST: authScheme = new DigestScheme(); break;
				}
				authState.setAuthScheme(authScheme);
				context.setAttribute(ClientContext.PROXY_AUTH_STATE, authState);
			}
		}
		return context;
	}
}
