package com.grendelscan.requester.http.factories;


import java.net.URI;
import java.net.URISyntaxException;

import org.apache.http.HttpRequest;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.message.BasicHttpEntityEnclosingRequest;
import org.apache.http.message.BasicHttpRequest;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractStandardHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.requester.http.transactions.HttpPostTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;

public class HttpTransactionFactory
{
	/**
	 * 
	 * @param sharedComponents
	 * @param method
	 * @param uri
	 * @param referer
	 * @param source
	 * @return
	 * @throws URISyntaxException
	 */
	public static AbstractStandardHttpTransaction createTransaction(Scan scan, String method, String uri, int referer, TransactionSource source) throws URISyntaxException
	{
		URI u = UriFactory.makeUri(uri, true);
		return createTransaction(scan, method, u, referer, source);
	}
	
	/**
	 * 
	 * @param scan
	 * @param method
	 * @param uri
	 * @param referer
	 * @param source
	 * @return
	 */
	public static AbstractStandardHttpTransaction createTransaction(Scan scan, String method, URI uri, int referer, TransactionSource source) throws URISyntaxException
	{
		AbstractStandardHttpTransaction transaction = null;
		
		if (method.equalsIgnoreCase("get"))
		{
			transaction = new HttpGetTransaction(scan, uri, referer, source);
		}
		else if (method.equalsIgnoreCase("post"))
		{
			transaction = new HttpPostTransaction(scan, uri, referer, source);
		}
		
		return transaction;
	}
	
	public static HttpGet convertGenericGet(HttpRequest request)
	{
		HttpGet get = null;
		try
		{
			get = new HttpGet(request.getRequestLine().getUri());
			get.setHeaders(request.getAllHeaders());
			get.setParams(request.getParams());
		}
		catch (URISyntaxException e) 
		{
			Debug.errDebug("Problem with URI \"" + request.getRequestLine().getUri() + "\": ", e);
		}
		return get;
	}
	
	
	public static HttpPost convertGenericPost(BasicHttpEntityEnclosingRequest request)
	{
		HttpPost post = null;
		try
		{
			post = new HttpPost(request.getRequestLine().getUri());
			post.setHeaders(request.getAllHeaders());
			post.setEntity(request.getEntity());
			post.setParams(request.getParams());
		}
		catch (URISyntaxException e) 
		{
			Debug.errDebug("Problem with URI \"" + request.getRequestLine().getUri() + "\": ", e);
		}
		return post;
	}
	
	
	public static AbstractStandardHttpTransaction createTransaction(Scan scan, HttpRequest request, int referer, TransactionSource source)
	{
		return createTransaction(scan, request, referer, source, false, 0);
	}
	
	public static AbstractStandardHttpTransaction createTransaction(Scan scan, HttpRequest request, int referer, TransactionSource source, boolean ssl, int sslPort)
	{
		AbstractStandardHttpTransaction transaction = null;
		if (request.getRequestLine().getMethod().equals("GET"))
		{
			HttpGet get = null;
			if (request instanceof BasicHttpRequest)
			{
				get = convertGenericGet(request);
			}
			else if (request instanceof HttpGet)
			{
				get = (HttpGet) request;
			}
			
			try
            {
	            transaction = new HttpGetTransaction(scan, get, referer, source);
            }
            catch (URISyntaxException e)
            {
	            Debug.errDebug("Problem creating transaction due to uri: " + e.toString(), e);
            }
		}
		else if (request.getRequestLine().getMethod().equals("POST"))
		{
			HttpPost post = null;
			if (request instanceof BasicHttpEntityEnclosingRequest)
			{
				post = convertGenericPost((BasicHttpEntityEnclosingRequest) request);
			}
			else if (request instanceof HttpPost)
			{
				post = (HttpPost) request;
			}
			
			try
            {
	            transaction = new HttpPostTransaction(scan, post, referer, source);
            }
            catch (URISyntaxException e)
            {
	            Debug.errDebug("Problem creating transaction due to uri: " + e.toString(), e);
            }
		}
		return transaction;
	}
}
