package com.grendelscan.requester.http.serializable;

import java.io.Serializable;

import org.apache.http.ProtocolVersion;
import org.apache.http.RequestLine;
import org.apache.http.message.BasicLineFormatter;

public class SerializableRequestLine implements RequestLine, Cloneable, Serializable
{
	
	private String method;
	private ProtocolVersion protoversion;
	private String uri;
	
	public SerializableRequestLine(RequestLine requestline)
    {
		this(requestline.getMethod(), requestline.getUri(), requestline.getProtocolVersion());
    }

	public SerializableRequestLine(final String method, final String uri, final ProtocolVersion version)
	{
		if (method == null)
		{
			throw new IllegalArgumentException("Method must not be null.");
		}
		if (uri == null)
		{
			throw new IllegalArgumentException("URI must not be null.");
		}
		if (version == null)
		{
			throw new IllegalArgumentException("Protocol version must not be null.");
		}
		this.method = method;
		this.uri = uri;
		protoversion = version;
	}
	
	@Override
	public Object clone() throws CloneNotSupportedException
	{
		return super.clone();
	}
	
	public String getMethod()
	{
		return method;
	}
	
	public ProtocolVersion getProtocolVersion()
	{
		return protoversion;
	}
	
	public String getUri()
	{
		return uri;
	}
	
	@Override
	public String toString()
	{
		// no need for non-default formatting in toString()
		return BasicLineFormatter.DEFAULT.formatRequestLine(null, this).toString();
	}
	
}
