/*
 * HttpRequest.java
 * 
 * Created on September 15, 2007, 2:34 PM
 * 
 * To change this template, choose Tools | Template Manager and open the
 * template in the editor.
 */

package com.grendelscan.requester.http.transactions;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.*;
import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.http.client.methods.HttpUriRequest;

import org.apache.http.Header;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.RequestLine;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.cookie.Cookie;
import org.apache.http.cookie.CookieOrigin;
import org.apache.http.cookie.MalformedCookieException;
import org.apache.http.impl.cookie.BrowserCompatSpec;
import org.apache.http.protocol.HttpContext;
import org.apache.http.util.EntityUtils;
import org.cobra_grendel.html.domimpl.HTMLDocumentImpl;
import org.cobra_grendel.html.parser.DocumentBuilderImpl;
import org.cobra_grendel.html.parser.InputSourceImpl;
import org.cobra_grendel.html.test.SimpleHtmlRendererContext;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.grendelscan.html.HtmlNodeWriter;
import com.grendelscan.requester.CobraUserAgent;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.authentication.AuthenticationPackage;
import com.grendelscan.requester.http.CookieJar;
import com.grendelscan.requester.http.client.CustomHttpClient;
import com.grendelscan.requester.http.serializable.SerializableBasicCookie;
import com.grendelscan.requester.http.wrappers.HttpResponseWrapper;
import com.grendelscan.requester.sessionState.SessionState;
import com.grendelscan.requester.sessionState.SessionStateUtils;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.*;
import com.grendelscan.utils.MimeUtils;
import com.grendelscan.utils.URIStringUtils;

/**
 * 
 * This class represents a complete HTTP transaction including the request and
 * the response.
 * 
 * @author David Byrne
 * 
 * 
 */
public abstract class AbstractHttpTransaction implements Serializable
{
	private transient static Integer lastID = 0;
	private transient static final Integer idLock = 1;
	protected transient static Pattern charSetPattern = Pattern.compile("charset=(.+)", Pattern.CASE_INSENSITIVE);
	protected transient static Pattern queryParameterPattern = Pattern.compile("([^=&]++)=([^&]*+)");
	protected transient static final long serialVersionUID = -3126496659368918711L;
	protected transient static Integer totalExecutions = 0;
	static long domParsingTime = 0;
	private transient boolean stringResponseBodyInitialized = false;
	protected boolean authenticated;
	protected AuthenticationPackage authenticationPackage = null;
	protected transient CookieJar childrensCookieJar;
	protected CookieJar cookieJar;
	protected transient CookieOrigin cookieOrigin;
	protected transient BrowserCompatSpec cookieSpec;
	protected long executionTime;
	
	protected transient boolean loaded = false;
	protected int id;
	protected int logicalResponseCode = 0;
	protected boolean loginTransaction;
	protected int redirectCount = 0;
	protected int referer;
	protected int requestDepth;
	protected transient String responseBody = "";
	protected String responseText;
	protected boolean successfullExecution;
	
	private HttpResponseWrapper responseWrapper;
	protected transient boolean saved = false;
	// protected transient WeakReference<Document> responseDOM;
	protected transient Scan scan;
	protected transient Map<String, SessionState> sessionStates;
	protected TransactionSource source;
	private static long currentSecond;
	private static int perSecondTransactionCount;
	protected boolean followRedirects = true;
	private String executionSource;
	protected String username;
	protected boolean writtenToDisk = false;
	protected String sentRequestHead;

	protected int redirectRequestTransactionID;
	protected int originalResponseTransactionID;
	
	public int getOriginalResponseTransactionID()
    {
    	return originalResponseTransactionID;
    }

	protected AbstractHttpTransaction(Scan scan, int referer, TransactionSource source)
	{
		synchronized (idLock)
		{
			id = ++lastID;
		}
		this.scan = scan;
		this.source = source;
		setReferer(referer);
	}
	
	public static Integer getLastID()
	{
		return lastID;
	}
	
	public static void setLastID(int lastID)
	{
		synchronized(idLock)
		{
			AbstractHttpTransaction.lastID = lastID;
		}
	}
	
	
	public static long getSerialVersionUID()
	{
		return serialVersionUID;
	}
	
	public static Integer getTotalExecutions()
	{
		return totalExecutions;
	}
	
	/**
	 * If there is already a cookie with the same name, it will be replaced
	 * 
	 * @param cookie
	 */
	public void addCookie(Cookie cookie, int transaction)
	{
		cookieJar.addCookie(cookie, 0);
	}
	
	public void addRawCookieHeader(Header setCookie, int transaction)
	{
		
		try
		{
			for (Cookie cookie: getCookieSpec().parse(setCookie, getCookieOrigin()))
			{
				addCookie(cookie, transaction);
			}
		}
		catch (MalformedCookieException e)
		{
			Debug.errDebug("Problem with set-cookie header in AbstractHttpTransaction.addRawCookieHeader(): "
			        + e.toString(), e);
		}
		
	}
	
	/**
	 * Adds a request header. This will only work if the request has not been
	 * sent yet. If it has been sent, no exception is thrown. To check if the
	 * transaction has been sent, use {@link #hasBeenUsed() hasBeenUsed}.
	 * 
	 * @param header
	 *            The header to add
	 * @see org.apache.commons.httpclient.HttpMethodBase#addRequestHeader(org.apache.commons.httpclient.Header)
	 */
	public abstract void addRequestHeader(Header header);
	
	/**
	 * This is used if something went wrong the first time the request was sent.
	 * A good example is if the first request resulted in a loggedout condition,
	 * so the request is being sent again.
	 */
	public void clearResponse()
	{
		responseWrapper = null;
		responseBody = null;
	}
	
	/**
	 * This will copy the transaction with an identical request, but not
	 * response
	 * 
	 * @return
	 */
	public abstract AbstractHttpTransaction clone(TransactionSource newSource);
	
	/**
	 * This will NOT cause the result to be tested. If it needs to be, use
	 * execute(true)
	 */
	public void execute(String source)
	{
		execute(source, false, false);
	}
	
	public boolean execute(String source, boolean testResponse)
	{
		return execute(source, testResponse, false);
	}
	
	/**
	 * 
	 * @param testResponse
	 * @param ignoreRestrictions
	 *            This can be very dangerous. It allows you to go anywhere, even
	 *            if it's been declared out-of-bounds. Be careful!
	 */
	public boolean execute(String source, boolean testResponse, boolean ignoreRestrictions)
	{
		return scan.getRequesterQueue().processRequestNow(this, source, testResponse, false, ignoreRestrictions, false);
	}
	
	/**
	 * Executes the transaction using the provided {@link HttpClient} object.
	 * The {@link HttpState} is taken from the client. Nothing is returned since
	 * the HTTP response is also stored inside of this class. There is some
	 * internal logic here, so this should be the only way that a request is
	 * executed.
	 * 
	 * @param client
	 */
	public final void execute(CustomHttpClient client, HttpContext context, boolean testTransaction, String executionSource)
	{
		if (totalExecutions > scan.getScanSettings().getMaxRequestCount())
		{
			scan.terminateScan("Maximum request count reached (" + scan.getScanSettings().getMaxRequestCount() + ")");
			return;
		}
		this.executionSource = executionSource;
		while (source != TransactionSource.PROXY && source != TransactionSource.MANUAL_REQUEST && !checkRequestThrottle())
		{
			try
            {
                synchronized (this)
                {
	                this.wait(250);
                }
            }
            catch (InterruptedException e)
            {
            }
		}
		
		if (source != TransactionSource.PROXY && source != TransactionSource.MANUAL_REQUEST && source != TransactionSource.FUZZER && !scan.isRunning(false))
		{
			return;
		}
		
		privateExecute(client, context, testTransaction, executionSource);
		save();
		synchronized (totalExecutions)
		{
			totalExecutions++;
		}
	}
	
	private synchronized boolean checkRequestThrottle()
	{
		boolean run = false;
		long second = (new Date().getTime()) / 1000;
		
		if (second > currentSecond)
		{
			currentSecond = second;
			perSecondTransactionCount = 1;
			run = true;
		}
		else if (perSecondTransactionCount < scan.getScanSettings().getMaxRequestsPerSecond())
		{
			perSecondTransactionCount++;
			run = true;
		}
		return run;
	}
	
	public NameValuePair findFirstParameterByValue(String value)
	{
		NameValuePair match = null;
		for (NameValuePair parameter: getQueryParameters())
		{
			if (parameter.getValue().equalsIgnoreCase(value))
			{
				match = parameter;
				break;
			}
		}
		return match;
	}
	
	/**
	 * Returns the absolute URI, even if the transaction used a relative URI
	 * 
	 * @return
	 */
	public String getAbsoluteUriString()
	{
		String uri = getHostUriWithoutTrailingSlash() + getPath() + getFilename();
		String query = getUriQueryString();
		if ((query != null) && !query.equals(""))
		{
			uri += "?" + query;
		}
		return uri;
	}
	
	public String[] getAllRequestHeaderNames()
	{
		if (responseWrapper != null)
		{
			Header headers[] = getAllRequestHeaders();
			String headerNames[] = new String[headers.length];
			for (int index = 0; index < headers.length; index++)
			{
				headerNames[index] = headers[index].getName();
			}
			return headerNames;
		}
		return null;
	}
	
	public abstract Header[] getAllRequestHeaders();
	
	public String[] getAllResponseHeaderNames()
	{
		if (responseWrapper != null)
		{
			Header headers[] = responseWrapper.getCopyOfAllHeaders();
			String headerNames[] = new String[headers.length];
			for (int index = 0; index < headers.length; index++)
			{
				headerNames[index] = headers[index].getName();
			}
			return headerNames;
		}
		return null;
	}
	
	public Header[] getAllResponseHeaders()
	{
		if (responseWrapper != null)
		{
			return responseWrapper.getCopyOfAllHeaders();
		}
		return null;
	}
	
	/**
	 * 
	 * @return All response headers formated as they would be found in the raw
	 *         HTTP response.
	 */
	public String getAllResponseHeadersAsString()
	{
		String headers = "";
		if (responseWrapper != null)
		{
			for (Header header: getAllResponseHeaders())
			{
				headers = headers.concat(header.toString()).concat("\r\n");
			}
			return headers;
		}
		return null;
	}
	
	/**
	 * 
	 * @return The authentication package associated with the transaction's
	 *         request. By default, this is the same as the referer's
	 *         authentication package.
	 */
	public AuthenticationPackage getAuthenticationPackage()
	{
		return authenticationPackage;
	}
	
	/**
	 * Returns the URI scheme, host, port, path and filename. No query or fragment is
	 * included.
	 */
	public String getBaseUri()
	{
		return URIStringUtils.getBaseUri(getUriString());
	}
	
	public CookieJar getChildrensCookieJar()
	{
		if (childrensCookieJar == null)
		{
			childrensCookieJar = new CookieJar();
			childrensCookieJar.addCookies(cookieJar);
			if (responseWrapper != null)
			{
				for (Cookie cookie: HttpUtils.getSetCookies(this))
				{
					childrensCookieJar.addCookie(cookie, id);
				}
			}
		}
		
		return childrensCookieJar;
	}
	
	/**
	 * Only returns cookies that were actually used in this transaction
	 * 
	 * @param cookieName
	 * @return
	 */
	public Cookie getCookie(String cookieName)
	{
		for (Cookie cookie: getUsedCookies())
		{
			if (cookie.getName().equals(cookieName))
			{
				return cookie;
			}
		}
		return null;
	}
	
	public CookieJar getCookieJar()
	{
		return cookieJar;
	}
	
	public CookieOrigin getCookieOrigin()
	{
		if (cookieOrigin == null)
		{
			cookieOrigin = new CookieOrigin(getHost(), getPort(), getPath(), isSecure());
		}
		return cookieOrigin;
	}
	
	public BrowserCompatSpec getCookieSpec()
	{
		if (cookieSpec == null)
		{
			cookieSpec = new BrowserCompatSpec();
		}
		return cookieSpec;
	}
	
	/**
	 * 
	 * @return The directory of the URI, without file name. Always has a
	 *         trailing forward slash (/).
	 */
	public abstract String getDirectory();
	
	/**
	 * 
	 * @return The filename of the URI, or an empty string if there isn't one
	 */
	public abstract String getFilename();
	
	public abstract Header getFirstRequestHeader(String headerName);
	
	public Header getFirstResponseHeader(String headerName)
	{
		if (responseWrapper != null)
		{
			return responseWrapper.getFirstHeader(headerName);
		}
		return null;
	}
	
	public abstract String getHost();
	
	/**
	 * Returns the URI scheme, host, port, and trailing slash. No path, query or
	 * fragment is included.
	 */
	public String getHostUri()
	{
		return getHostUriWithoutTrailingSlash() + "/";
	}
	
	/**
	 * Yes, it's a long method name, but it's intuitive. This is often handy
	 * when You need to create a new URI, with a path that already has a leading
	 * slash
	 * 
	 * @return
	 */
	public String getHostUriWithoutTrailingSlash()
	{
		String base = getScheme() + "://" + getHost();
		int port = getPort();
		if (port > 0)
		{
			base += ":" + port;
		}
		return base;
	}
	
	public int getId()
	{
		return id;
	}
	
	public abstract Header getLastRequestHeader(String headerName);
	
	public Header getLastResponseHeader(String headerName)
	{
		if (responseWrapper != null)
		{
			return responseWrapper.getLastHeader(headerName);
		}
		return null;
	}
	
	/**
	 * This will take the ResponseCodeOverrides into account and return what the
	 * web server meant, not what the web server said.
	 * 
	 * @return
	 */
	public int getLogicalResponseCode()
	{
		if ((logicalResponseCode == 0) && (responseWrapper != null))
		{
			logicalResponseCode = scan.getResponseCodeOverrides().getLogicalResponseCode(this);
		}
		return logicalResponseCode;
	}
	
	public abstract String getMethod();
	
	/**
	 * Returns the URI's path but NOT the file name
	 * @return
	 */
	public abstract String getPath();
	
	public abstract String getPayloadQueryString();
	
	public abstract int getPort();
	
	public abstract Set<String> getQueryParameterNames();
	
	public abstract Set<NameValuePair> getQueryParameters();
	
	public abstract String getQueryParameterValue(String name);
	
	
	/**
	 * This will return the query string, but no values. If It this is a POST
	 * transaction, it will return the string as if it were a GET. If the URL in
	 * a POST also has a query string, the two will be concactenated. This is
	 * mostly for finger printing the transaction, it isn't very useful on its
	 * own.
	 */
	public String getQuerySummary(boolean removeForbiddenParameters)
	{
		String payloadQuery = getPayloadQueryString();
		String uriQuery = getUriQueryString();
		String query;
		if (!payloadQuery.equals(uriQuery))
		{
			if (payloadQuery.equals(""))
			{
				query = uriQuery;
			}
			else if (uriQuery.equals(""))
			{
				query = payloadQuery;
			}
			else
			{
				query = uriQuery + "&" + payloadQuery;
			}
		}
		else
		{
			query = uriQuery;
		}
		query = query.replaceAll("=[^&]+", "");
		if (removeForbiddenParameters)
		{
			String tmpQuery = query;
			query = "";
			for (String name: tmpQuery.split("&"))
			{
				if (!name.equals("") && !scan.isQueryParameterForbidden(name.replace("=", "")))
				{
					query += name + "&";
				}
			}
		}
		return query;
	}
	
	public int getRedirectCount()
	{
		return redirectCount;
	}
	
	
	public AbstractHttpTransaction getReferer()
	{
		return scan.getTransactionRecord().getTransaction(referer);
	}
	
	/**
	 * @return the referer
	 */
	public int getRefererID()
	{
		return referer;
	}
	
	public abstract HttpRequest getRequest();
	
	/**
	 * Returns an empty array if no body is present
	 * @return
	 */
	public abstract byte[] getRequestBody();
	
	public String getRequestCharacterSet()
	{
		return getCharacterSet(getFirstRequestHeader("Content-Type"));
	}
	
	public int getRequestDepth()
	{
		return requestDepth;
	}
	
	public abstract Header[] getRequestHeaders(String headerName);
	
	public HttpResponse getResponse()
	{
		HttpResponse response = null;
		if (responseWrapper != null)
		{
			response = responseWrapper.getResponse();
		}
		return response;
	}
	
	public byte[] getResponseBody()
	{
		byte[] responseBody = null;
		if (responseWrapper != null)
		{
			responseBody = responseWrapper.getResponseBody();
		}
		return responseBody;
	}
	
	public synchronized String getResponseBodyAsString()
	{
		if ((getResponseBody() != null) && !stringResponseBodyInitialized)
		{
			stringResponseBodyInitialized = true;
			responseBody = new String(getResponseBody());
		}
		return responseBody;
	}
	
	public String getResponseCharacterSet()
	{
		Header h = null;
		if (responseWrapper != null)
		{
			h = responseWrapper.getContentType();
		}
		
		return getCharacterSet(h);
	}
	
	
	public long getResponseContentLength()
	{
		long length = -1;
		if (getResponseBody() != null)
		{
			length = getResponseBody().length;
		}
		return length;
	}
	
	public synchronized Document getResponseDOM()
	{
		// Document document = null;
		// if ((responseDOM == null || (document = responseDOM.get()) == null)
		// &&
		// isHtmlMimeType(transaction.getResponseMimeType())
		// {
		//			
		// document = parseIntoDOM();
		// responseDOM = new WeakReference<Document>(document);
		// }
		// return document;
		return parseIntoDOM();
	}
	
	public Header[] getResponseHeaders(String headerName)
	{
		Header[] headers = null;
		if (responseWrapper != null)
		{
			headers = responseWrapper.getHeaders(headerName);
		}
		return headers;
	}
	
	/**
	 * 
	 * @return The response MIME type as reported by the Content-Type response
	 *         header
	 */
	public String getResponseMimeType()
	{
		String mimeType = "";
		if (responseWrapper != null)
		{
			
			Header contentTypeHeader = responseWrapper.getContentType();
			if (contentTypeHeader != null)
			{
				String contentType = contentTypeHeader.getValue();
				
				if (contentType != null)
				{
					mimeType = MimeUtils.getMimeFromContentType(contentType);
				}
			}
			else
			{
				mimeType = "text/html";
			}
		}
		return mimeType;
	}
	
	/**
	 * Returns the text elements of the response (if it is an HTML response), 
	 * if it is some other kind of text response, the response body is 
	 * returned.
	 * 
	 * @return
	 */
	public String getResponseTextNodes()
	{
		return responseText;
	}
	
	public HttpResponseWrapper getResponseWrapper()
	{
		return responseWrapper;
	}
	
	public Scan getScan()
	{
		return scan;
	}
	
	public abstract String getScheme();
	
	public Map<String, SessionState> getSessionStates()
	{
		if (sessionStates == null)
		{
			initializeSessionStates();
		}
		return sessionStates;
	}
	
	public TransactionSource getSource()
	{
		return source;
	}
	
	/**
	 * @return The HTTP response code
	 * @see org.apache.commons.httpclient.HttpMethodBase#getResponseStatusCode()
	 */
	public int getResponseStatusCode()
	{
		if (responseWrapper == null)
		{
			return 0;
		}
		return responseWrapper.getStatusLine().getStatusCode();
	}
	
	/**
	 * @return
	 * @see org.apache.commons.httpclient.HttpMethodBase#getStatusLine()
	 */
	public StatusLine getStatusLine()
	{
		if (responseWrapper != null)
		{
			return responseWrapper.getStatusLine();
		}
		return null;
	}
	
	/**
	 * @return
	 * @see org.apache.commons.httpclient.HttpMethodBase#getStatusText()
	 */
	public String getStatusText()
	{
		if (responseWrapper == null)
		{
			return "";
		}
		return responseWrapper.getStatusLine().getReasonPhrase();
	}

	public String getStatusLineString()
	{
		if (responseWrapper == null)
		{
			return "";
		}
		return responseWrapper.getStatusLine().toString();
	}

	public abstract String getUriQueryString();
	
	public abstract String getUriString();
	
	/**
	 * This will return a set of cookies that were/would be used in the request
	 * 
	 * @return
	 */
	public Set<String> getUsedCookieNames()
	{
		Set<String> cookies = new HashSet<String>();
		for (Cookie cookie: getUsedCookies())
		{
			cookies.add(cookie.getName());
		}
		return cookies;
	}
	
	/**
	 * This will return a set of cookies that were/would be used in the request
	 * 
	 * @return
	 */
	public Set<SerializableBasicCookie> getUsedCookies()
	{
		Set<SerializableBasicCookie> cookies = new HashSet<SerializableBasicCookie>();
		for (SerializableBasicCookie cookie: cookieJar.getCookies())
		{
			if (getCookieSpec().match(cookie, getCookieOrigin()))
			{
				cookies.add(cookie);
			}
		}
		return cookies;
	}
	
	public String getUsername()
	{
		return username;
	}
	
	public boolean isAuthenticated()
	{
		return authenticated;
	}
	
	public boolean isLoginTransaction()
	{
		return loginTransaction;
	}
	
	public boolean isResponsePresent()
	{
		if (responseWrapper == null)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	public boolean isSaved()
	{
		return saved;
	}
	
	public abstract boolean isSecure();
	
	/**
	 * Removes all known session identification and cookies
	 */
	public void removeAllAuthentication()
	{
		for (SessionState state: getSessionStates().values())
		{
			removeSessionState(state);
		}
	}
	
	/**
	 * This will only remove cookies that match the cookie name and the host of
	 * the request
	 * 
	 * @param cookieName
	 */
	public void removeCookie(String cookieName)
	{
		for (Cookie cookie: getUsedCookies())
		{
			if (cookie.getName().equals(cookieName))
			{
				cookieJar.removeCookie(cookie);
			}
		}
	}
	
	public abstract void removeQueryParameters(String name);
	
	public boolean removeSessionState(SessionState sessionState)
	{
		boolean removed = false;
		String name = sessionState.getSessionIDName();
		if (getSessionStates().containsKey(name))
		{
			getSessionStates().remove(name);
			sessionState.removeFromTransaction(this);
			removed = true;
		}
		return removed;
	}
	
	public abstract void replaceQueryParameter(NameValuePair parameter);
	
	public abstract void replaceQueryParameter(String name, String newValue);
	
	/**
	 * This finds all parameters with a specific value and replaces it with a
	 * new value. It is primarily used for token testing. It is case insensitive
	 * when searching.
	 * 
	 * @param value
	 */
	public void replaceQueryParameterValue(String oldValue, String newValue)
	{
		for (NameValuePair parameter: getQueryParameters())
		{
			if (parameter.getValue().equalsIgnoreCase(oldValue))
			{
				replaceQueryParameter(parameter.getName(), newValue);
			}
		}
	}
	
	/**
	 * Looks for an existing session state with the same name and replaces it
	 * with the supplied state, but only if an old one exists. If the state is
	 * tracked with a cookie, the old cookie will be removed and the new one
	 * added. If the state is tracked with a query parameter, the value will be
	 * replaced. The username associated with the transaction will also be
	 * updated.
	 * 
	 * This is useful for repeating the same request, but with a different
	 * sessionID.
	 * 
	 * @param newSessionState
	 *            The new SessionState
	 * @return False if there was no match, otherwise true
	 */
	public boolean replaceSessionState(SessionState newSessionState)
	{
		boolean replaced = false;
		String name = newSessionState.getSessionIDName();
		if (getSessionStates().containsKey(name))
		{
			getSessionStates().remove(name);
			getSessionStates().put(name, newSessionState);
			username = newSessionState.getUsername();
			newSessionState.addIntoTransaction(this);
			replaced = true;
		}
		return replaced;
	}
	
	/**
	 * This will parse the DOM with a renderer context and return the document.
	 * Mostly for XSS testing.
	 */
	public Document runDOM()
	{
		if (getResponseBody() == null)
		{
			return null;
		}
		
		CobraUserAgent ua = new CobraUserAgent(scan, true);
		SimpleHtmlRendererContext rcontext = new SimpleHtmlRendererContext();
		DocumentBuilderImpl dbi = new DocumentBuilderImpl(ua, rcontext);
		ByteArrayInputStream inputStream = new ByteArrayInputStream(getResponseBody());
		InputSource inputSource = new InputSourceImpl(inputStream, getAbsoluteUriString(), getRequestCharacterSet());
		Document tmpDocument = null;
		try
		{
			tmpDocument = dbi.parse(inputSource);
		}
		catch (SAXException e)
		{
			Debug.errDebug(e.toString());
			e.printStackTrace();
		}
		catch (FileNotFoundException e)
		{
			Debug.errDebug(e.toString());
			e.printStackTrace();
		}
		catch (IOException e)
		{
			Debug.errDebug(e.toString());
			e.printStackTrace();
		}
		return tmpDocument;
	}
	
	public void setAuthenticated(boolean authenticated)
	{
		this.authenticated = authenticated;
	}
	
	/**
	 * @param authenticationPackage
	 *            the authenticationPackage to set
	 */
	public void setAuthenticationPackage(AuthenticationPackage authenticationPackage)
	{
		this.authenticationPackage = authenticationPackage;
	}
	
	public void setCookieJar(CookieJar cookieJar)
	{
		this.cookieJar = cookieJar;
	}
	
	public void setCookieOrigin(CookieOrigin cookieOrigin)
	{
		this.cookieOrigin = cookieOrigin;
	}
	
	public void setCookieSpec(BrowserCompatSpec cookieSpec)
	{
		this.cookieSpec = cookieSpec;
	}
	
	public void setId(int id)
	{
		this.id = id;
	}
	
	public void setLogicalResponseCode(int logicalResponseCode)
	{
		this.logicalResponseCode = logicalResponseCode;
	}
	
	public void setLoginTransaction(boolean loginTransaction)
	{
		this.loginTransaction = loginTransaction;
	}
	
	public void setRedirectCount(int redirectCount)
	{
		this.redirectCount = redirectCount;
	}
	
	/**
	 * @param referer
	 *            the referer to set
	 */
	public void setReferer(AbstractHttpTransaction referer)
	{
		if (referer != null)
		{
			requestDepth = referer.requestDepth + 1;
			this.referer = referer.getId();
		}
		else
		{
			requestDepth = 0;
		}
	}
	
	public void setReferer(int referer)
	{
		this.referer = referer;
	}
	
	public void setRequestDepth(int requestDepth)
	{
		this.requestDepth = requestDepth;
	}
	
	public void setResponseWrapper(HttpResponseWrapper wrapper)
	{
		this.responseWrapper = wrapper;
		successfullExecution = true;
		updateResponseText();
	}
	
	public void setResponse(HttpResponse response)
	{
		if (response != null)
		{
			responseWrapper = new HttpResponseWrapper(response, scan);
			successfullExecution = true;
			updateResponseText();
		}
		else
		{
			responseWrapper = null;
			successfullExecution = false;
			responseText = "";
		}
	}
	
	protected void updateResponseText()
	{
		if (MimeUtils.isHtmlMimeType(getResponseMimeType()))
		{
			responseText = HtmlNodeWriter.writeTextOnly(getResponseDOM(), false);
		}
		else if (MimeUtils.isWebMimeType(getResponseMimeType()))
		{
			responseText = getResponseBodyAsString();
		}
		if (responseText == null)
		{
			responseText = "";
		}
	}
	
	
	public void setSaved(boolean saved)
	{
		this.saved = saved;
	}
	
	public void setScan(Scan scan)
	{
		this.scan = scan;
	}
	
	public void setSource(TransactionSource source)
	{
		this.source = source;
	}
	
	// @Override
	// protected void finalize() throws Throwable
	// {
	// Exception e1 = null;
	// try
	// {
	// save();
	// }
	// catch (Exception e)
	// {
	// e1 = e;
	// }
	// finally
	// {
	// if (e1 == null)
	// {
	// // Debug.debug("It worked");
	// }
	// }
	// super.finalize();
	// }
	
	abstract public void setURI(String uri);
	
	public void setUsername(String username)
	{
		this.username = username;
	}
	
	private Document parseIntoDOM()
	{
		Document dom = null;
		if (responseWrapper != null && MimeUtils.isHtmlMimeType(getResponseMimeType()))
		{
			Date t = new Date();
			CobraUserAgent ua = new CobraUserAgent(scan, false);
			DocumentBuilderImpl dbi = new DocumentBuilderImpl(ua);
			
			ByteArrayInputStream inputStream = new ByteArrayInputStream(getResponseBody());
			try
			{
				InputSource inputSource =
				        new InputSourceImpl(inputStream, getAbsoluteUriString(), getResponseCharacterSet());
				HTMLDocumentImpl document = (HTMLDocumentImpl) dbi.createDocument(inputSource);
				document.load();
				dom = document;
			}
			catch (SAXException e)
			{
				Debug.errDebug(e.toString());
				e.printStackTrace();
			}
			catch (FileNotFoundException e)
			{
				Debug.errDebug(e.toString());
				e.printStackTrace();
			}
			catch (IOException e)
			{
				Debug.errDebug(e.toString());
				e.printStackTrace();
			}
			Date t2 = new Date();
			domParsingTime += t2.getTime() - t.getTime();
		}
		return dom;
	}
	
	// Is there a better way of doing this?
	/**
	 * Doesn't copy the response or the request
	 */
	protected void copyFields(AbstractHttpTransaction target)
	{
		target.setAuthenticationPackage(authenticationPackage);
		target.setCookieOrigin(getCookieOrigin());
		target.setCookieJar(cookieJar.clone());
		target.setCookieSpec(getCookieSpec());
		target.setLoginTransaction(loginTransaction);
		target.setAuthenticated(authenticated);
		target.setReferer(referer);
		target.setRequestDepth(requestDepth);
		target.setUsername(username);
	}
	
	/**
	 * This needs to be called at the end of non abstract contstuctors
	 */
	protected void generalObjectInit()
	{
		AbstractHttpTransaction refererTransaction = scan.getTransactionRecord().getTransaction(referer);
		if (refererTransaction != null)
		{
			cookieJar = refererTransaction.getChildrensCookieJar();
			authenticated = refererTransaction.isAuthenticated();
			username = refererTransaction.getUsername();
		}
		else
		{
			cookieJar = new CookieJar();
		}
		initializeSessionStates();
	}
	
	protected String getCharacterSet(Header contentTypeHeader)
	{
		String charset = "ISO-8859-1";
		if (contentTypeHeader != null)
		{
			String contentType;
			if ((contentType = contentTypeHeader.getValue()) != null)
			{
				Matcher matcher = charSetPattern.matcher(contentType);
				
				if (matcher.find())
				{
					charset = matcher.group(1);
				}
			}
		}
		return charset;
	}
	
	protected void initializeSessionStates()
	{
		sessionStates = SessionStateUtils.findSessionStates(this);
	}
	
	protected abstract void privateExecute(CustomHttpClient client, HttpContext context, boolean testTransaction, String executionSource);
	
	public synchronized boolean save()
	{
		return scan.getTransactionRecord().saveOrUpdateTransaction(this);
	}
	
	protected void updateCookies()
	{
		removeRequestHeaders("Cookie");
		if (cookieJar.getCookies().size() > 0)
		{
			String cookieString = "";
			for (Cookie cookie: cookieJar.getCookies())
			{
				if (getCookieSpec().match(cookie, getCookieOrigin()))
				{
					cookieString += cookie.getName() + "=" + cookie.getValue() + "; ";
				}
			}
			addRequestHeader("Cookie", cookieString);
		}
	}

	public abstract void removeRequestHeaders(String name);
	public abstract void addRequestHeader(String name, String value);
	public boolean isLoaded()
    {
    	return loaded;
    }

	public void setLoaded(boolean loaded)
    {
    	this.loaded = loaded;
    }

	public boolean isFollowRedirects()
    {
    	return followRedirects;
    }

	public void setFollowRedirects(boolean followRedirects)
    {
    	this.followRedirects = followRedirects;
    }

	public boolean isSuccessfullExecution()
    {
    	return successfullExecution;
    }

	public String getExecutionSource()
    {
    	return executionSource;
    }

	public void setExecutionSource(String executionSource)
    {
    	this.executionSource = executionSource;
    }

	public String requestToString()
	{
		String requestString = "";
		if (sentRequestHead == null)
		{
			requestString += this.getRequestLine().toString() + "\n";
			for (Header header: getAllRequestHeaders())
			{
				requestString += header.toString() + "\n";
			}
		}
		else
		{
			requestString += sentRequestHead;
		}
		
		requestString += "\n";
		
		return requestString;
	}
	
	public String responseToString()
	{
		String response = "";
		if (responseWrapper != null)
		{
			response = responseWrapper.toString();
		}
		return response;
	}

	
	@Override
    public String toString()
    {
		return requestToString() + "\r\n\r\n" + responseToString();
    }

	public synchronized void writeToDisk()
	{
		writeToDisk(false);
	}
	
	public synchronized void writeToDisk(boolean force)
	{
		if (successfullExecution && (!writtenToDisk || force))
		{
			writtenToDisk = true;
			String filename = scan.getScanSettings().getOutputDirectory() + File.separator + scan.getScanSettings().getSavedTextTransactionsDirectory() + File.separator + getId() + ".txt";
			String text = toString();
			FileUtils.writeToFile(filename, text);
		}
	}

	public String getSavedUrl()
	{
		return "./" + scan.getScanSettings().getSavedTextTransactionsDirectory() + "/" + getId() + ".txt";
	}
	
	public abstract RequestLine getRequestLine();

	/**
	 * This will return the transaction used to request the redirect URL
	 * 
	 * @return
	 */
	public int getRedirectRequestTransactionID()
	{
		return redirectRequestTransactionID;
	}

	public long getExecutionTime()
    {
    	return executionTime;
    }

	public void setExecutionTime(long executionTime)
    {
    	this.executionTime = executionTime;
    }

	public void setSentRequestHead(String sentRequestHead)
    {
    	this.sentRequestHead = sentRequestHead;
    }
}
