package com.grendelscan.requester.http.transactions;

import java.net.URI;
import java.net.URISyntaxException;

import org.apache.http.HttpRequest;
import org.apache.http.client.methods.HttpGet;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.payloads.URLEncodedRequestPayload;
import com.grendelscan.requester.http.wrappers.StandardHttpRequestWrapper;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.*;

public class HttpGetTransaction extends AbstractStandardHttpTransaction
{
	
	// /**
	// * This is usually just for Hibernate. Don't use it unless you know what
	// you're doing.
	// */
	// protected HttpGetTransaction()
	// {
	//		
	// }
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -2723754068257268573L;
	
	/**
	 * 
	 * @param sharedComponents
	 * @param request
	 * @param referer
	 * @param source
	 */
	public HttpGetTransaction(Scan scan, HttpGet request, int referer, TransactionSource source)
	        throws URISyntaxException
	{
		super(scan, request, referer, source);
		generalObjectInit();
		save();
	}
	
	/**
	 * 
	 * @param sharedComponents
	 * @param uri
	 * @param referer
	 * @param source
	 * @throws URISyntaxException
	 */
	public HttpGetTransaction(Scan scan, String uri, int referer, TransactionSource source) throws URISyntaxException
	{
		this(scan, new HttpGet(UriFactory.makeUri(uri, true)), referer, source);
	}
	
	/**
	 * 
	 * @param sharedComponents
	 * @param uri
	 * @param referer
	 * @param source
	 */
	public HttpGetTransaction(Scan scan, URI uri, int referer, TransactionSource source) throws URISyntaxException
	{
		this(scan, new HttpGet(uri), referer, source);
	}
	
	public HttpGetTransaction(Scan scan, StandardHttpRequestWrapper requestWrapper, int referer, TransactionSource source)
	{
		super(scan, requestWrapper, referer, source);
		generalObjectInit();
		save();
	}

	
	@Override
	public AbstractHttpTransaction clone(TransactionSource source)
	{
		HttpGetTransaction clone = new HttpGetTransaction(scan, requestWrapper.clone(), referer, source);
		super.copyFields(clone);
		clone.save();
		return clone;
	}
	
	@Override
	public synchronized void updatePayload()
	{
		if (payloadChanged)
		{
			// If getBaseUri() or getUriString() is called, there is an endless
			// loop
			String baseUri = URIStringUtils.getBaseUri(requestWrapper.getURI().toASCIIString());
			String query = getRequestPayload().toUrlQueryString();
			String uriString = baseUri + "?" + query;
			URI uri;
            try
            {
	            uri = new URI(uriString);
				setURI(uri);
            }
            catch (URISyntaxException e)
            {
            	Debug.errDebug("Problem updating URI (" + uriString + "): " + e.toString(), e);
            }
		}
	}

	@Override
    protected void initRequestWrapper(HttpRequest request)
    {
		URLEncodedRequestPayload newPayload =
	        new URLEncodedRequestPayload(getCharacterSet(request.getFirstHeader("Content-Type")), URIStringUtils
	                .getQuery(request.getRequestLine().getUri()));
		initRequestWrapper(new StandardHttpRequestWrapper(request, newPayload));
    }

	@Override
    public byte[] getRequestBody()
    {
	    return new byte[0];
    }
	
//	@Override
//	protected String extractPayload()
//	{
//		return getUriQueryString();
//	}
	
}
