package com.grendelscan.requester.http.transactions;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import org.apache.http.HttpRequest;
import org.apache.http.ParseException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.message.BasicHeader;
import org.apache.http.util.EntityUtils;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.factories.HttpRequestFactory;
import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.payloads.URLEncodedRequestPayload;
import com.grendelscan.requester.http.wrappers.StandardHttpRequestWrapper;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.*;

public class HttpPostTransaction extends HttpEntityEnclosingRequestTransaction
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 8808200948565783586L;
	
	/**
	 * 
	 * @param sharedComponents
	 * @param request
	 * @param referer
	 * @param source
	 */
	public HttpPostTransaction(Scan scan, HttpPost request, int referer, TransactionSource source)
	        throws URISyntaxException
	{
		super(scan, request, referer, source);
		generalObjectInit();
		
//		This will need to change once we start supporting mime encoded requests
		if (this.getRequestHeaders("Content-Type").length == 0)
		{
			addRequestHeader(new BasicHeader("Content-Type", "application/x-www-form-urlencoded"));
		}
		save();
	}
	
	public HttpPostTransaction(Scan scan, String uri, int referer, TransactionSource source) throws URISyntaxException
	{
		this(scan, 
				(HttpPost) HttpRequestFactory.makeRequest("POST", UriFactory.makeUri(uri, true), null, null), 
				referer, source);
	}
	
	/**
	 * 
	 * @param scan
	 * @param uri
	 * @param referer
	 * @param source
	 */
	public HttpPostTransaction(Scan scan, URI uri, int referer, TransactionSource source) throws URISyntaxException
	{
		this(scan, (HttpPost) HttpRequestFactory.makeRequest("POST", uri, null, null), referer, source);
	}
	
	public HttpPostTransaction(Scan scan, StandardHttpRequestWrapper requestWrapper, int referer, TransactionSource source)
	{
		super(scan, requestWrapper, referer, source);
		generalObjectInit();
		addRequestHeader(new BasicHeader("Content-Type", "application/x-www-form-urlencoded"));
		save();
	}
	
	@Override
	public AbstractHttpTransaction clone(TransactionSource source)
	{
		HttpPostTransaction clone = new HttpPostTransaction(scan, requestWrapper.clone(), referer, source);
		super.copyFields(clone);
		clone.save();
		return clone;
	}

	@Override
	protected void initRequestWrapper(HttpRequest request)
	{
		String body = "";
		if (((HttpPost)request).getEntity() != null)
		{
			try
	        {
		        body = EntityUtils.toString(((HttpPost)request).getEntity());
	        }
	        catch (ParseException e)
	        {
		        Debug.errDebug("Problem decoding POST entity: " + e.toString(), e);
	        }
	        catch (IOException e)
	        {
		        Debug.errDebug("Problem decoding POST entity: " + e.toString(), e);
	        }
		}
		URLEncodedRequestPayload newPayload =
		        new URLEncodedRequestPayload(getCharacterSet(request.getFirstHeader("Content-Type")), body);
		initRequestWrapper(new StandardHttpRequestWrapper(request, newPayload));
	}
	
}
