package com.grendelscan.requester.http.wrappers;

import java.io.Serializable;

import org.apache.http.Header;

import com.grendelscan.requester.http.serializable.SerializableHttpHeader;

public class HttpMessageWrapper implements Serializable
{
	protected SerializableHttpHeader headers[];
	protected final Boolean headerLock = true; 

	public HttpMessageWrapper()
    {
		headers = new SerializableHttpHeader[0];
    }

	public void addHeader(String name, String value)
	{
		addHeaders(new Header[] {new SerializableHttpHeader(name, value)});
	}
	
	public void addHeader(Header header)
	{
		addHeaders(new Header[] {header});
	}
	
	protected void addHeaders(Header addedHeaders[])
	{
		synchronized (headerLock)
        {
	        SerializableHttpHeader newHeaders[] = new SerializableHttpHeader[headers.length + addedHeaders.length];
	        for (int index = 0; index < headers.length; index++)
	        {
		        newHeaders[index] = headers[index];
	        }
	        for (int index = 0; index < addedHeaders.length; index++)
	        {
		        newHeaders[index + headers.length] = SerializableHttpHeader.convertToSerializableHeader(addedHeaders[index]);
	        }
	        headers = newHeaders;
        }
	}
	

	/**
	 * Returns a copy of the array of headers
	 * @return
	 */
	public Header[] getCopyOfAllHeaders()
    {
		Header[] copy;
		synchronized (headerLock)
        {
	        copy = new Header[headers.length];
	        for (int i = 0; i < headers.length; i++)
	        {
		        copy[i] = headers[i];
	        }
        }
		return copy;
    }

	protected void setHeaders(Header[] headers)
    {
		synchronized (headerLock)
        {
	        headers = new SerializableHttpHeader[0];
        }
		addHeaders(headers);
    }

	
    public Header getFirstHeader(String headerName)
    {
    	Header header = null;
    	synchronized (headerLock)
        {
	        for (int i = 0; i < headers.length; i++)
	        {
		        if (headers[i].getName().equals(headerName))
		        {
			        header = headers[i];
			        break;
		        }
	        }
        }
		return header;
    }

    private int getHeaderCount(String headerName)
    {
    	int count = 0;
    	for (int i = 0; i < headers.length; i++)
        {
	        if (headers[i].getName().equals(headerName))
	        {
		        count++;
	        }
        }
    	return count;
    }
    
    public Header getLastHeader(String headerName)
    {
    	Header header = null;
    	synchronized (headerLock)
        {
	        for (int i = headers.length; i >= 0; i--)
	        {
		        if (headers[i].getName().equals(headerName))
		        {
			        header = headers[i];
			        break;
		        }
	        }
        }
		return header;
    }

    public Header[] getHeaders(String headerName)
    {
    	Header[] tmpHeaders;
    	synchronized (headerLock)
        {
    		int count = getHeaderCount(headerName);
	        tmpHeaders = new Header[count];
	        int index = 0;
	        for (int i = 0; i < headers.length; i++)
	        {
		        if (headers[i].getName().equals(headerName))
		        {
			        tmpHeaders[index++] = headers[i];
		        }
	        }
        }
		return tmpHeaders;
    }
    
    public void removeHeaders(String headerName)
    {
    	synchronized (headerLock)
        {
    		int count = getHeaderCount(headerName);
    		SerializableHttpHeader[] newHeaders = new SerializableHttpHeader[headers.length - count];
	        if (newHeaders.length > 0)
	        {
		        int index = 0;
		        for (int i = 0; i < headers.length; i++)
		        {
			        if (!headers[i].getName().equals(headerName))
			        {
				        newHeaders[index++] = headers[i];
			        }
		        }
	        }
	        headers = newHeaders;
        }
    }

    protected String headersToString()
    {
    	String headerString = "";
		synchronized (headerLock)
        {
	        for (int i = 0; i < headers.length; i++)
	        {
	        	headerString += headers[i].toString() + "\r\n";
	        }
        }
		return headerString;
    }
}
