package com.grendelscan.requester.http.wrappers;
import java.io.IOException;
import java.io.Serializable;
import java.lang.ref.WeakReference;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.message.BasicHttpResponse;
import org.apache.http.util.EntityUtils;

import com.grendelscan.requester.http.serializable.SerializableHttpHeader;
import com.grendelscan.requester.http.serializable.*;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HttpUtils;
import com.grendelscan.scan.*;
import org.apache.http.ProtocolVersion;

public class HttpResponseWrapper extends HttpMessageWrapper implements Serializable
{
	private transient WeakReference<HttpResponse> response;
	private SerializableStatusLine statusLine;
	private byte responseBody[];
	private SerializableHttpHeader contentType;
	private transient Scan scan; 
	
	
	public HttpResponseWrapper(HttpResponse response, Scan scan)
    {
		this.scan = scan;
	    setResponse(response);
    }
	
	public HttpResponseWrapper(Scan scan, int statusCode, String reasonPhrase, String protocol, int major, int minor, Header[] headers, byte[] body)
	{
	   	this.addHeaders(headers);
	   	ProtocolVersion version = new ProtocolVersion(protocol.toUpperCase(), major, minor);
	   	statusLine = new SerializableStatusLine(version, statusCode, reasonPhrase);
	   	this.responseBody = body;
	   	for(Header header: headers)
	   	{
	   		if(header.getName().equalsIgnoreCase("Content-Type"))
	   		{
	   			contentType = SerializableHttpHeader.convertToSerializableHeader(header);
	   		}
	   	}
	}

	public Header getFirstHeader(String name)
    {
	    return getResponse().getFirstHeader(name);
    }

	public HttpResponse getResponse()
    {
		HttpResponse actualResponse;
		// I'm not sure why, but somehow response keeps popping up null here
		if (response == null || (actualResponse = response.get()) == null)
		{
			actualResponse = new BasicHttpResponse(statusLine);
			ByteArrayEntity entity = new ByteArrayEntity(responseBody);
			actualResponse.setEntity(entity);
			actualResponse.setHeaders(getCopyOfAllHeaders());
			response = new WeakReference<HttpResponse>(actualResponse);
		}
    	return actualResponse;
    }

	private void setResponse(HttpResponse response)
    {
    	this.response = new WeakReference<HttpResponse>(response);
    	HttpEntity entity = response.getEntity();
    	
    	// preps an empty response body, incase it was empty
    	responseBody = new byte[0];
    	
    	if (entity != null)
    	{
	    	contentType = SerializableHttpHeader.convertToSerializableHeader(entity.getContentType());
	    	try
	    	{
	    		
	    		responseBody = HttpUtils.entityToByteArray(entity, scan.getScanSettings().getMaxFileSizeBytes());
	    	}
	    	catch (IOException e) 
			{
				Debug.errDebug("Problem processing HTTP response: " + e.toString(), e);
			}
    	}
    	
    	this.addHeaders(response.getAllHeaders());
    	statusLine = new SerializableStatusLine(response.getStatusLine());
    }


	public StatusLine getStatusLine()
    {
    	return statusLine;
    }

	public byte[] getResponseBody()
    {
    	return responseBody;
    }


	public Header getContentType()
    {
    	return contentType;
    }

	public void setResponseBody(byte[] responseBody)
    {
    	this.responseBody = responseBody;
    }
	
	@Override
	public String toString()
	{
		String responseString = statusLine.toString() + "\r\n";
		responseString += headersToString();
		responseString += "\r\n";
		responseString += new String(responseBody);
		return responseString; 
	}
	
}
