package com.grendelscan.requester.http.wrappers;

import java.io.Serializable;
import java.net.URI;

import org.apache.http.HttpRequest;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.client.methods.HttpOptions;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpTrace;
import org.apache.http.client.methods.HttpUriRequest;

import com.grendelscan.requester.http.factories.HttpRequestFactory;
import com.grendelscan.requester.http.payloads.RequestPayload;

// Need to take a closer look at synchronization here
public class StandardHttpRequestWrapper extends HttpMessageWrapper implements Serializable
{
	private static final long serialVersionUID = -4950627342682724871L;
	private String method;
	private RequestPayload requestPayload;
	private URI uri;
	
	
	/**
	 * 
	 * @param request
	 *            The request to build from
	 * @param transactionId
	 *            The transaction using this wrapper. This is important for
	 *            updating the request payload.
	 */
	public StandardHttpRequestWrapper(HttpRequest request, RequestPayload requestPayload)
	{
		setRequest(request, requestPayload);
	}


	public StandardHttpRequestWrapper(String method, RequestPayload requestPayload, URI uri)
    {
	    this.method = method;
	    this.requestPayload = requestPayload;
	    this.uri = uri;
    }

	public String getMethod()
	{
		return method;
	}
	
	public synchronized HttpRequest getRequest()
	{
		return HttpRequestFactory.makeRequest(method, uri, getCopyOfAllHeaders(), requestPayload.toEntity());
	}
	
	public RequestPayload getRequestPayload()
	{
		return requestPayload;
	}
	
	public URI getURI()
	{
		return uri;
	}
	
	public StandardHttpRequestWrapper clone()
	{
		return new StandardHttpRequestWrapper(method, requestPayload.clone(), URI.create(uri.toASCIIString()));
	}
	
	public void setRequest(HttpRequest request, RequestPayload requestPayload)
	{
		if (!((request instanceof HttpTrace) || (request instanceof HttpOptions) || 
				(request instanceof HttpHead) || (request instanceof HttpGet) || 
				(request instanceof HttpPut) || (request instanceof HttpPost)))
		{
			RuntimeException e = new RuntimeException("Unknown HttpRequest type");
			throw e;
		}
		this.requestPayload = requestPayload;
		super.addHeaders(request.getAllHeaders());
		uri = ((HttpUriRequest) request).getURI();
		method = request.getRequestLine().getMethod();
	}
	
	public void setRequestPayload(RequestPayload requestPayload)
	{
		this.requestPayload = requestPayload;
	}
	
	public void setURI(URI uri)
	{
		this.uri = uri;
	}
}
