package com.grendelscan.requester.sessionState;

import org.apache.http.cookie.Cookie;

import com.grendelscan.requester.authentication.AuthenticationPackage;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;

public class CookieSessionState extends SessionState
{
	@Override
    public void removeFromTransaction(AbstractHttpTransaction transaction)
    {
		transaction.getCookieJar().removeCookie(cookie);
    }

	@Override
    public void addIntoTransaction(AbstractHttpTransaction transaction)
    {
		// Don't need to remove the old one because addCookie will
		transaction.addCookie(cookie, initialTransaction);
    }

	private Cookie cookie;

	/**
     * @param authPackage
     * @param username
     * @param cookie
     */
    public CookieSessionState(AuthenticationPackage authPackage, String username, Cookie cookie, int initialTransaction)
    {
	    super(authPackage, username, initialTransaction);
	    this.cookie = cookie;
    }

	public Cookie getCookie()
    {
    	return cookie;
    }

	@Override
    public boolean equals(Object obj)
    {
		if (!(obj instanceof CookieSessionState))
		{
			return false;
		}
		CookieSessionState o = (CookieSessionState) obj;
		return cookie.getName().equals(o.getCookie().getName()) && cookie.getValue().equals(o.getCookie().getValue()) ;
    }

	@Override
    public String getSessionIDName()
    {
	    return cookie.getName();
    }

	@Override
    public String getSessionIDValue()
    {
	    return cookie.getValue();
    }
	
}
