package com.grendelscan.requester.sessionState;

import com.grendelscan.requester.authentication.AuthenticationPackage;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.utils.URIStringUtils;

public class ParameterSessionState extends SessionState
{
	/**
	 * Use the call in AbstractHttpTransaction, NOT this
	 */
	@Override
    public void removeFromTransaction(AbstractHttpTransaction transaction)
    {
		if (!inUrl || transaction.getMethod().equalsIgnoreCase("GET"))
		{
			transaction.removeQueryParameters(name);
		}
		else
		{
			String newUri = URIStringUtils.removeQueryParameter(transaction.getUriString(), name);
			transaction.setURI(newUri);
		}
    }

	private String name;
	private String value;
	private boolean inUrl;

	/**
	 * 
	 * @param authPackage
	 * @param username
	 * @param name
	 * @param value
	 * @param inUrl
	 */
	public ParameterSessionState(AuthenticationPackage authPackage, String username, String name, String value, boolean inUrl, int initialTransaction)
    {
	    super(authPackage, username, initialTransaction);
	    this.name = name;
	    this.value = value;
	    this.inUrl = inUrl;
    }

	/**
	 * Use the call in AbstractHttpTransaction, NOT this
	 */
	@Override
    public void addIntoTransaction(AbstractHttpTransaction transaction)
    {
	    if (transaction.getMethod().equalsIgnoreCase("GET") || !inUrl)
	    {
	    	transaction.replaceQueryParameter(name, value);
	    }
	    else
	    {
	    	transaction.setURI(URIStringUtils.replaceQueryParameter(transaction.getUriString(), name, value));
	    }
    }

	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public String getValue()
	{
		return value;
	}

	public void setValue(String value)
	{
		this.value = value;
	}

	@Override
    public boolean equals(Object obj)
    {
		if (!(obj instanceof ParameterSessionState))
		{
			return false;
		}
		ParameterSessionState o = (ParameterSessionState) obj;
		
	    return o.getName().equals(name) && o.getValue().equals(value);
    }

	@Override
    public String getSessionIDName()
    {
	    return name;
    }

	public boolean isInUrl()
    {
    	return inUrl;
    }

	@Override
    public String getSessionIDValue()
    {
	    return value;
    }

}
