package com.grendelscan.requester.sessionState;

import com.grendelscan.requester.authentication.AuthenticationPackage;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;

/**
 * We have to track session state so that logouts, etc can be handled.
 * 
 * @author David Byrne
 *
 */
public abstract class SessionState
{
	protected AuthenticationPackage authPackage;
	protected String username;
	protected int initialTransaction;
	
	public AuthenticationPackage getAuthPackage()
    {
    	return authPackage;
    }

	public SessionState(AuthenticationPackage authPackage, String username, int initialTransaction)
    {
	    this.authPackage = authPackage;
	    this.username = username;
	    this.initialTransaction = initialTransaction;
    }

	public String getUsername()
    {
    	return username;
    }

	@Override
    abstract public boolean equals(Object obj);
	abstract public String getSessionIDName();
	abstract public String getSessionIDValue();
	
	/**
	 * Force this session state onto the request in the specified transaction. 
	 * @param transaction
	 */
	abstract public void addIntoTransaction(AbstractHttpTransaction transaction);

	abstract public void removeFromTransaction(AbstractHttpTransaction transaction);

	public int getInitialTransaction()
    {
    	return initialTransaction;
    }
}
