package com.grendelscan.requester.sessionState;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.NameValuePair;
import org.apache.http.cookie.Cookie;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.serializable.SerializableBasicCookie;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionID;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.URIStringUtils;
import com.grendelscan.scan.*;
public class SessionStateUtils
{
	private static boolean initialized = false;
	private static Scan scan;
	public static void initializeLibrary(Scan scan)
	{
		if (!initialized)
		{
			initialized = true;
			SessionStateUtils.scan = scan;
		}
	}
	
	public static Map<String, SessionState> findSessionStates(AbstractHttpTransaction transaction)
	{
		Map<String, SessionState> states = Collections.synchronizedMap(new HashMap<String, SessionState>(1));
		findParameterSessionStates(transaction, states);
		findCookieSessionStates(transaction, states);
		return states;
	}
	
	private static void findParameterSessionStates(AbstractHttpTransaction transaction, Map<String, SessionState> states)
	{
		boolean get = true; 
		if (transaction.getMethod().equalsIgnoreCase("POST"))
		{
			get = false;
			checkParameters(URIStringUtils.getQueryParametersFromUri(transaction.getUriString()), states, transaction, true);
		}

		checkParameters(transaction.getQueryParameters(), states, transaction, get);
	}

	private static void checkParameters(Iterable<NameValuePair> params, Map<String, SessionState> states, AbstractHttpTransaction transaction, boolean inUrl)
	{
		for (NameValuePair param: params)
		{
			if (isSessionIDName(param.getName()) && !states.containsKey(param.getName()))
			{
				AbstractHttpTransaction referer = transaction.getReferer();
				if (referer != null)
				{
					SessionState oldState = referer.getSessionStates().get(param.getName());
					int initialTransaction;
					if (oldState != null && true)
					{
						initialTransaction = oldState.getInitialTransaction();
					}
					else
					{
						initialTransaction = transaction.getRefererID();
					}
					states.put(param.getName(), new ParameterSessionState(
													transaction.getAuthenticationPackage(), 
													transaction.getUsername(), 
													param.getName(), 
													param.getValue(), 
													inUrl,
													initialTransaction));
				}
			}
		}
	}
	
	
	private static void findCookieSessionStates(AbstractHttpTransaction transaction, Map<String, SessionState> states)
	{
		for (SerializableBasicCookie cookie: transaction.getUsedCookies())
		{
			if (isSessionIDName(cookie.getName()) && !states.containsKey(cookie.getName()))
			{
				states.put(cookie.getName(), new CookieSessionState(
													transaction.getAuthenticationPackage(), 
													transaction.getUsername(), 
													cookie,
													cookie.getOriginatingTransaction()));
			}
		}
	}

	private static boolean isSessionIDName(String name)
	{
		boolean isname = false;
		for (Pattern pattern: scan.getScanSettings().getKnownSessionIDRegexs())
		{
			if (pattern.matcher(name).matches())
			{
				isname = true;
				break;
			}
		}
		return isname;
	}

	public static SessionState findNewSessionIDInResponse(String sessionIDName, AbstractHttpTransaction transaction)
	{
		SessionState state = null;
		// Look for the new ID in a cookie
		for (Cookie cookie: transaction.getChildrensCookieJar().getCookies())
		{
			if (cookie.getName().equalsIgnoreCase(sessionIDName))
			{
				state = new CookieSessionState(transaction.getAuthenticationPackage(), transaction.getUsername(), cookie, transaction.getId());
				break;
			}
		}
		
		// Look for the new ID in a URL query parameter in the location header
		if (state == null)
		{
			String value = URIStringUtils.getFirstQueryParameter(transaction.getResponseHeaders("Location")[0].getValue(), "sessionIDName");
			if (value != null && !value.equals(""))
			{
				state = new ParameterSessionState(transaction.getAuthenticationPackage(), transaction.getUsername(), sessionIDName, value, true, transaction.getId());
			}
		}

		// Look for the new ID in a URL query parameter
		if (state == null)
		{
			Pattern p = Pattern.compile(sessionIDName + "=([^'\"&\\s>#]+)", Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(transaction.getResponseBodyAsString());
			if (m.find())
			{
				state = new ParameterSessionState(transaction.getAuthenticationPackage(), transaction.getUsername(), sessionIDName, m.group(1), true, transaction.getId());
			}
		}

		// Look for the new ID in a input parameter, after the parameter name
		if (state == null)
		{
			Pattern p = Pattern.compile("name\\s*=\\s*['\"]?" + sessionIDName + "[^>]+value\\s*=\\s*['\"]?([^'\"\\s>]+)", Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(transaction.getResponseBodyAsString());
			if (m.find())
			{
				state = new ParameterSessionState(transaction.getAuthenticationPackage(), transaction.getUsername(), sessionIDName, m.group(1), true, transaction.getId());
			}
		}

		// Look for the new ID in a input parameter, before the parameter name
		if (state == null)
		{
			Pattern p = Pattern.compile("value\\s*=\\s*['\"]?([^'\"\\s>]++)[^>]+name\\s*=\\s*['\"]?" + sessionIDName, Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(transaction.getResponseBodyAsString());
			if (m.find())
			{
				state = new ParameterSessionState(transaction.getAuthenticationPackage(), transaction.getUsername(), sessionIDName, m.group(1), true, transaction.getId());
			}
		}

		return state;
	}
}
