package com.grendelscan.scan;


import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;

import com.grendelscan.utils.Debug;
import com.grendelscan.utils.StringUtils;

public class ConfigurationManager 
{
	private static Configuration scannerConfiguration;

	public static void initializeConfiguration(String path)
	{
		try
		{
			scannerConfiguration = new PropertiesConfiguration(path);
		}
		catch (ConfigurationException e)
		{
			Debug.errDebug("Failed to open scanner.conf. Program will now exit: " + e.toString());
			System.exit(1);
		}

	}

	public static boolean containsKey(String key)
    {
	    return scannerConfiguration.containsKey(key);
    }

	public static BigDecimal getBigDecimal(String key, BigDecimal defaultValue)
    {
	    return scannerConfiguration.getBigDecimal(key, defaultValue);
    }

	public static BigDecimal getBigDecimal(String key)
    {
	    return scannerConfiguration.getBigDecimal(key);
    }

	public static BigInteger getBigInteger(String key, BigInteger defaultValue)
    {
	    return scannerConfiguration.getBigInteger(key, defaultValue);
    }

	public static BigInteger getBigInteger(String key)
    {
	    return scannerConfiguration.getBigInteger(key);
    }

	public static boolean getBoolean(String key, boolean defaultValue)
    {
	    return scannerConfiguration.getBoolean(key, defaultValue);
    }

	public static Boolean getBoolean(String key, Boolean defaultValue)
    {
	    return scannerConfiguration.getBoolean(key, defaultValue);
    }

	public static boolean getBoolean(String key)
    {
	    return scannerConfiguration.getBoolean(key);
    }

	public static byte getByte(String key, byte defaultValue)
    {
	    return scannerConfiguration.getByte(key, defaultValue);
    }

	public static Byte getByte(String key, Byte defaultValue)
    {
	    return scannerConfiguration.getByte(key, defaultValue);
    }

	public static byte getByte(String key)
    {
	    return scannerConfiguration.getByte(key);
    }

	public static double getDouble(String key, double defaultValue)
    {
	    return scannerConfiguration.getDouble(key, defaultValue);
    }

	public static Double getDouble(String key, Double defaultValue)
    {
	    return scannerConfiguration.getDouble(key, defaultValue);
    }

	public static double getDouble(String key)
    {
	    return scannerConfiguration.getDouble(key);
    }

	public static float getFloat(String key, float defaultValue)
    {
	    return scannerConfiguration.getFloat(key, defaultValue);
    }

	public static Float getFloat(String key, Float defaultValue)
    {
	    return scannerConfiguration.getFloat(key, defaultValue);
    }

	public static float getFloat(String key)
    {
	    return scannerConfiguration.getFloat(key);
    }

	public static int getInt(String key, int defaultValue)
    {
	    return scannerConfiguration.getInt(key, defaultValue);
    }

	public static int getInt(String key)
    {
	    return scannerConfiguration.getInt(key);
    }

	public static Integer getInteger(String key, Integer defaultValue)
    {
	    return scannerConfiguration.getInteger(key, defaultValue);
    }

	public static Iterator getKeys()
    {
	    return scannerConfiguration.getKeys();
    }

	public static Iterator getKeys(String prefix)
    {
	    return scannerConfiguration.getKeys(prefix);
    }

	public static List getList(String key, List defaultValue)
    {
	    return scannerConfiguration.getList(key, defaultValue);
    }

	public static List getList(String key)
    {
	    return scannerConfiguration.getList(key);
    }

	public static long getLong(String key, long defaultValue)
    {
	    return scannerConfiguration.getLong(key, defaultValue);
    }

	public static Long getLong(String key, Long defaultValue)
    {
	    return scannerConfiguration.getLong(key, defaultValue);
    }

	public static long getLong(String key)
    {
	    return scannerConfiguration.getLong(key);
    }

	public static Properties getProperties(String key)
    {
	    return scannerConfiguration.getProperties(key);
    }

	public static Object getProperty(String key)
    {
	    return scannerConfiguration.getProperty(key);
    }

	public static short getShort(String key, short defaultValue)
    {
	    return scannerConfiguration.getShort(key, defaultValue);
    }

	public static Short getShort(String key, Short defaultValue)
    {
	    return scannerConfiguration.getShort(key, defaultValue);
    }

	public static short getShort(String key)
    {
	    return scannerConfiguration.getShort(key);
    }

	public static String getString(String key, String defaultValue)
    {
	    return StringUtils.unquote(scannerConfiguration.getString(key, defaultValue));
    }

	public static String getString(String key)
    {
	    return StringUtils.unquote(scannerConfiguration.getString(key));
    }

	public static String[] getStringArray(String key)
    {
		String a[] = scannerConfiguration.getStringArray(key);
		for (int index = 0; index < a.length; index++)
		{
			a[index] = StringUtils.unquote(a[index]);
		}
	    return a;
    }

	public static Configuration subset(String prefix)
    {
	    return scannerConfiguration.subset(prefix);
    }

	public static void addProperty(String key, Object value)
    {
	    scannerConfiguration.addProperty(key, value);
    }

	public static void clear()
    {
	    scannerConfiguration.clear();
    }

	public static void clearProperty(String key)
    {
	    scannerConfiguration.clearProperty(key);
    }

	public static boolean isEmpty()
    {
	    return scannerConfiguration.equals("");
    }

	public static void setProperty(String key, Object value)
    {
	    scannerConfiguration.setProperty(key, value);
    }
}
