/*
 * Scan.java
 * 
 * Created on August 19, 2007, 1:25 PM
 * 
 * To change this template, choose Tools | Template Manager and open the
 * template in the editor.
 */

package com.grendelscan.scan;
import com.grendelscan.requester.sessionState.*;
import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.http.HttpHost;
import org.apache.http.HttpVersion;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.AuthState;
import org.apache.http.client.params.AllClientPNames;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnManagerPNames;
import org.apache.http.conn.params.ConnPerRouteBean;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.scheme.SocketFactory;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.auth.DigestScheme;
import org.apache.http.impl.auth.RFC2617Scheme;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.*;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;

import com.grendelscan.GUI.ScanSettingsWindow;
import com.grendelscan.GUI.ScanStatusWindow;
import com.grendelscan.categorizers.Categorizer;
import com.grendelscan.categorizers.Categorizers;
import com.grendelscan.categorizers.queue.CategorizerQueue;
import com.grendelscan.categorizers.queue.CategorizerThread;
import com.grendelscan.data.Database;
import com.grendelscan.proxy.Proxy;
import com.grendelscan.requester.ResponseCodeOverrides;
import com.grendelscan.requester.TransactionRecord;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.SSL.CustomSSLSocketFactory;
import com.grendelscan.requester.http.client.*;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.requester.queue.RequesterQueue;
import com.grendelscan.requester.queue.RequesterThread;
import com.grendelscan.scan.abstractQueue.AbstractQueueThread;
import com.grendelscan.scan.abstractQueue.QueueThreadGroup;
import com.grendelscan.scan.abstractQueue.QueueThreadState;
import com.grendelscan.scan.reporting.EventCollector;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionID;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.queue.TesterQueue;
import com.grendelscan.tests.queue.TesterThread;
import com.grendelscan.tests.testJobs.ByDirectoryTestJob;
import com.grendelscan.tests.testJobs.CompletedScanJob;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testModules.*;
import com.grendelscan.utils.*;

/**
 * 
 * @author David Byrne
 */
public class Scan implements Runnable
{
	private CategorizerQueue categorizerQueue;
	private Categorizers categorizers;
	private CategorizerThread[] categorizerThreads;
//	private Connection databaseConnection;
//	private DataStore dataStore;
	private CustomHttpClient httpClient;
	private RawHttpClient rawHttpClient;
	private Thread masterScanThread;
	private EventCollector reportCollector;
	private RequesterQueue requesterQueue;
	private RequesterThread[] requesterThreads;
	private ResponseCodeOverrides responseCodeOverrides;
	private boolean running;
	private boolean paused = false;
	private boolean terminated = false;
	private ScanPhase scanPhase;
	private ScanSettings scanSettings;
	private TesterQueue testerQueue;
	private TesterThread[] testerThreads;
	private Proxy proxy;
	private QueueThreadGroup threadGroup;
	private TransactionRecord transactionRecord;
	private ScanStatusWindow scanStatusWindow;
	private ScanSettingsWindow scanSettingsWindow;

	private Database database;
	
	/** Creates a new instance of Scan */
	public Scan(ScanSettings scanSettings)
	{
		this.scanSettings = scanSettings;
		scanPhase = ScanPhase.INITIALIZATION;
	}
	

	private void createHttpClients()
	{
		HttpParams params = ClientUtilities.createHttpParams(this);
		HttpContext context = ClientUtilities.createHttpContext(this);
		ClientConnectionManager ccm = ClientUtilities.createClientConnectionManager(params);
		// The proxy won't use the normal client
		if (!scanSettings.isProxyOnly())
		{
			httpClient = new CustomHttpClient(ccm, params, context, this);
		}
		rawHttpClient = new RawHttpClient(ccm, params, context, this);
	}
	
	public CategorizerQueue getCategorizerQueue()
	{
		return categorizerQueue;
	}
	
	public Categorizers getCategorizers()
	{
		return categorizers;
	}
	
	
	public CustomHttpClient getHttpClient()
	{
		return httpClient;
	}
	
	public EventCollector getReportCollector()
	{
		return reportCollector;
	}
	
	public RequesterQueue getRequesterQueue()
	{
		return requesterQueue;
	}
	
	public ResponseCodeOverrides getResponseCodeOverrides()
	{
		return responseCodeOverrides;
	}
	
	public ScanPhase getScanPhase()
	{
		return scanPhase;
	}
	
	public ScanSettings getScanSettings()
	{
		return scanSettings;
	}
	
	public TesterQueue getTesterQueue()
	{
		return testerQueue;
	}
	
	public TransactionRecord getTransactionRecord()
	{
		return transactionRecord;
	}
	
	private void initResponseCodeOverrides()
	{
		responseCodeOverrides =
	        new ResponseCodeOverrides(this, scanSettings.getManualResponseCodeOverrides(), 
	        								scanSettings.getUseAutomaticResponseCodeOverrides(), 
	        								scanSettings.getAcceptableAutomaticResponseCodeOverrideThreshold());
		if (scanSettings.getUseAutomaticResponseCodeOverrides())
		{
			Set<TestJob> jobs = new HashSet<TestJob>(1);
			for (URI uri: scanSettings.getBaseURIs())
			{
                try
                {
                	AbstractHttpTransaction request = new HttpGetTransaction(this, uri, 0, TransactionSource.TEST);
					jobs.add(new ByDirectoryTestJob(19, request.getId(), "/", this));
                }
                catch (URISyntaxException e)
                {
                	Debug.errDebug("Problem creating request due to uri: " + e.toString(), e);
                }
				
			}
			testerQueue.addTests(jobs);
		}
		monitorScanStatus(250, false, false, false);
	}
	
	
	public boolean isQueryParameterForbidden(String parameterName)
	{
		return CollectionUtils.containsStringIgnoreCase(scanSettings.getForbiddenQueryParameters(), parameterName);
	}
	
	/**
	 * Returns true if the parameter is forbidden, or irrelevant
	 * 
	 * @param parameterName
	 * @return
	 */
	public boolean isQueryParameterIgnored(String parameterName)
	{
		return CollectionUtils.containsStringIgnoreCase(scanSettings.getIrrelevantQueryParameters(), parameterName)
		        || CollectionUtils.containsStringIgnoreCase(scanSettings.getForbiddenQueryParameters(), parameterName);
	}
	
	public boolean isRunning()
	{
		return isRunning(false);
	}
	
	public boolean isRunning(boolean proxyOrCategorizerRequest)
	{
		while (running && paused && !proxyOrCategorizerRequest)
		{
			/*
			 * This will block, but that's okay. It basically means one wait
			 * governs all that call isRunning().
			 */
			synchronized(this)
			{
				try
                {
	                this.wait(250);
                }
                catch (InterruptedException e)
                {
                }
			}
		}
		return running;
	}
	
	private class ScanCompleteException extends Exception
	{
        private static final long serialVersionUID = 1L;
	}
	
	public void startProxyOnly()
	{
		universalInit();

		// The requester queue manages some aspects of requests, but no threads are needed
		requesterQueue = new RequesterQueue(this);

		scanPhase = ScanPhase.COMPLETE;
		scanSettings.setAllowAllProxyRequests(true);
		scanSettings.setTestProxyRequests(false);
		scanSettings.setTestInterceptedRequests(false);
		scanSettings.setTestManualRequests(false);
		startProxy();

	}

	public boolean initDatabase(String path, boolean create)
	{
		boolean good = true;
		try
		{
			database = new Database(path, create);
			transactionRecord = new TransactionRecord(this);
		}
		catch (Exception e) 
		{
			Debug.errDebug("Problem with initializing database: " + e.toString(), e);
			good = false;
		}

		return good;
	}
	
//	private void createDatabase()
//	{
//		if (database == null)
//		{
//			try
//			{
//				database = new Database(scanSettings.getOutputDirectory() + File.separator + "transaction-record", true);
//			}
//			catch (Exception e) 
//			{
//				Debug.errDebug("Problem with creating database: " + e.toString(), e);
//				System.exit(1);
//			}
//		}
//		
//		if (transactionRecord == null)
//		{
//			transactionRecord = new TransactionRecord(this);
//		}
//	}

	
	public void startScan()
	{
		universalInit();
//		initDatabase(scanSettings.getOutputDirectory() + File.separator + "transaction-record", true);
		running = true;
		masterScanThread = new Thread(this, "MasterScanThread");
		masterScanThread.start();
	}

	private void universalInit()
	{
		// This is stuff that happens even for proxy-only mode
		FileUtils.createDirectories(scanSettings.getOutputDirectory() + File.separator + scanSettings.getSavedTextTransactionsDirectory());
		createHttpClients();
		if (scanSettings.getMaxRequestCount() <= 0)
		{
			scanSettings.setMaxRequestCount(Integer.MAX_VALUE);
		}
		if (scanSettings.getMaxFailedRequestsPerHost() <= 0)
		{
			scanSettings.setMaxFailedRequestsPerHost(Integer.MAX_VALUE);
		}
		
		// init static libraries must come after init database
		initStaticLibraries();
	}
	
	public void run()
	{
		threadGroup = new QueueThreadGroup();
		preScanInit();
		
		try
		{
			scanPhase = ScanPhase.PRIMARY_SCAN;
			
			// This is what really kicks off the main scan
			addBaseURIs();
			
			// handle the auth packages
			if (scanSettings.getAuthenticationPackage() != null)
			{
				categorizers.getByAuthenticationPackageCategorizerCategorizer().analyzeAuthenticationPackage(
				        scanSettings.getAuthenticationPackage());
			}
			
			monitorScanStatus(250, false, true, false);
			scanPhase = ScanPhase.FINAL_REPORTING;
			
			if (!running)
			{
				throw new ScanCompleteException();
			}
			
			categorizers.getFinalAnalysisCategorizer().runAnalysis();
			
			if (!running)
			{
				throw new ScanCompleteException();
			}
			
			// recrawl
			scanPhase = ScanPhase.RECRAWL;
			
		}
		catch (ScanCompleteException e)
		{
		}
		finally
		{
			if (!scanStatusWindow.isExiting())
			{
				displayMessage("Reporting", "Report being generated");
			}
			
			scanPhase = ScanPhase.FINAL_REPORTING;
			paused = false;
			
			/*
			 * Don't need to do this for the custom client because it uses the 
			 * same connection manager
			 */
			httpClient.getConnectionManager().shutdown();
			stopNonTesterThreads();
			emptyQueues(true);
			interuptTesterThreads();
			monitorScanStatus(250, true, false, true);
			unlockQueues();
			running = true;
			
			Set<TestJob> jobs = new HashSet<TestJob>(1);
			for (TestModule module: scanSettings.getMasterTestModuleCollection().getAllTestModules())
			{
				jobs.add(new CompletedScanJob(module.getModuleNumber(), this));
			}
			getTesterQueue().addTests(jobs);
			monitorScanStatus(250, false, false, true);
			
			scanPhase = ScanPhase.STOPPING;
			running = false;
			testerQueue.setRunning(false);
			interuptTesterThreads();

			scanSettings.getReportGenerator().generateReport();
			scanStatusWindow.scanComplete();
			scanPhase = ScanPhase.COMPLETE;
		}
	}
	
	
	private void preScanInit()
	{
		scanSettings.checkCompleteness();
		scanSettings.convertBaseUris2WhiteLists();

		if (scanSettings.getUseAutomaticResponseCodeOverrides())
		{
			scanSettings.enableTestModule(19);
		}
		scanSettings.getMasterTestModuleCollection().finishModules();
		initializeTesterQueue();
		initializeCategorizers();
		initializeCategorizerQueue();
		
		// Requester init has to come after categorizer init and transaction
		// record init (which isn't here yet)
		initializeRequesterQueue();
		scanSettings.getMasterTestModuleCollection().initializeModules();
		
		initThreads();
		
		reportCollector = new EventCollector(this);
		
		// Keep the resposneCodeOverride at the bottom
		initResponseCodeOverrides();
	}
	

	private String generateStatus()
	{
		String s = "";
		
		s += "Enabled modules:\n";
		for (TestModule module: scanSettings.getMasterTestModuleCollection().getAllTestModules())
		{
			s += "\t" + module.getName() + " (" + module.getModuleNumber() + ")\n";
		}
		s += "\n\n";
		
		s += "Processing times:\n";
		for (int moduleId: testerQueue.getTimes().keySet())
		{
			s += "\t" + scanSettings.getMasterTestModuleCollection().getTestModule(moduleId).getName() + " (" + moduleId + "): " 
			+ Math.round(testerQueue.getTimes().get(moduleId)/1000) + "\n";
		}
		s+= "\n\n";
		
		s += "\nTester queue stats:\n";
		s += "\tPending in memory:\n";
		for (int module: testerQueue.getPendingCount().keySet())
		{
			s += "\t\t" + scanSettings.getMasterTestModuleCollection().getTestModule(module).getName() + " (" + module + "): "  + testerQueue.getPendingCount().get(module) + "\n";
		}
		
		s += "\tPending on disk:\n";
		for (int module: testerQueue.getQueueStorage().getPendingCount().keySet())
		{
			s += "\t\t" + scanSettings.getMasterTestModuleCollection().getTestModule(module).getName() + " (" + module + "): " + testerQueue.getQueueStorage().getPendingCount().get(module) + "\n";
		}
		
		s += "\tCompleted tests:\n";
		for (int module: testerQueue.getProcessedCount().keySet())
		{
			s += "\t\t" + scanSettings.getMasterTestModuleCollection().getTestModule(module).getName() + " (" + module + "): " + testerQueue.getProcessedCount().get(module) + "\n";
		}
		s += "\n\n";
		
		
		s += scanSettings.getMaxTesterThreads() + " tester threads originally created\n";
		for (TesterThread thread: testerThreads)
		{
			s += "\t" + thread.getName() + ": " + thread.getThreadState().getText() + "  --  " + thread.getCurrentModule() + "\n";
		}
		s += "\n\n";
		
		s += scanSettings.getMaxCategorizerThreads() + " categorizer threads originally created\n";
		for (CategorizerThread thread: categorizerThreads)
		{
			s += "\t" + thread.getName() + ": " + thread.getThreadState().getText() + "\n";
		}
		s += "\n\n";
		
		s += scanSettings.getMaxRequesterThreads() + " requester threads originally created\n";
		for (RequesterThread thread: requesterThreads)
		{
			s += "\t" + thread.getName() + ": " + thread.getThreadState().getText() + "\n";
		}
		s += "\n\n";
		
		s += "Completed HTTP transactions: " + AbstractHttpTransaction.getTotalExecutions() + "\n";
		
		return s;
	}
	
	
	public synchronized void terminateScan(String message)
	{
		if (!terminated && !scanSettings.isProxyOnly())
		{
			terminated = true;
			if (message != null && !message.equals(""))
			{
				scanStatusWindow.displayMessage("Scan termination:", "The scan is being terminated. This is the reason: " + message);
			}
			running = false;
			String s = "Terminating scan";
			if (!message.equals(""))
			{
				s += ". The reason is: " + message;
			}
			s += generateStatus();
			Debug.errDebug(s);
		}
	}
	
	private void emptyQueues(boolean keepLocked)
	{
		requesterQueue.lockQueue();
		testerQueue.lockQueue();
		categorizerQueue.lockQueue();
		requesterQueue.emptyQueue();
		testerQueue.emptyQueue();
		categorizerQueue.emptyQueue();
		if (!keepLocked)
		{
			unlockQueues();
		}
	}
	
	private void unlockQueues()
	{
		requesterQueue.unlockQueue();
		testerQueue.unlockQueue();
		categorizerQueue.unlockQueue();
	}
	
	private synchronized void stopNonTesterThreads()
	{
		Debug.debug("Stopping non testing threads");
		
		emptyQueues(true);
		
		requesterQueue.setRunning(false);
		categorizerQueue.setRunning(false);

		Debug.debug("Pausing for 5 seconds for threads to shut down gracefully");
		synchronized(this)
		{
			// Give the threads a second to shut down on their own
			try
            {
	            this.wait(5000);
            }
            catch (InterruptedException e)
            {
            }
		}
		
		Debug.debug("Forcing thread shutdown");
		for (CategorizerThread categorizerThread: categorizerThreads)
		{
			categorizerThread.interupt();
		}

		for (RequesterThread requesterThread: requesterThreads)
		{
			requesterThread.interupt();
		}
}
	
	private synchronized void interuptTesterThreads()
	{
		for (TesterThread testerThread: testerThreads)
		{
			testerThread.interupt();
		}
	}
	
	private void addBaseURIs()
	{
		for (URI uri: scanSettings.getBaseURIs())
		{
            try
            {
    			HttpGetTransaction get = new HttpGetTransaction(this, uri, 0, TransactionSource.BASE);
    			requesterQueue.addRequest(get, "base url processing");
            }
            catch (URISyntaxException e)
            {
            	Debug.errDebug("Problem with base uri format: " + e.toString(), e);
            }
		}
	}
	
	private void initializeCategorizerQueue()
	{
		categorizerQueue = new CategorizerQueue(this);
		
		categorizerThreads = new CategorizerThread[scanSettings.getMaxCategorizerThreads()];
		for (int loop = 0; loop < categorizerThreads.length; loop++)
		{
			categorizerThreads[loop] = new CategorizerThread(loop, this, threadGroup);
		}
	}
	
	private void initializeCategorizers()
	{
		categorizers = new Categorizers(this);
		
		for (Categorizer categorizer: categorizers.getAllCategorizers())
		{
			for (TestModule module: scanSettings.getMasterTestModuleCollection().getAllTestModules())
			{
				categorizer.addModule(module);
			}
		}
	}
	
	
	private void initializeRequesterQueue()
	{
		requesterQueue = new RequesterQueue(this);
		
		requesterThreads = new RequesterThread[scanSettings.getMaxRequesterThreads()];
		for (int loop = 0; loop < requesterThreads.length; loop++)
		{
			requesterThreads[loop] = new RequesterThread(loop, this, threadGroup);
		}
	}
	
	private void initializeTesterQueue()
	{
		testerQueue = new TesterQueue(this);
		
		testerThreads = new TesterThread[scanSettings.getMaxTesterThreads()];
		for (int loop = 0; loop < testerThreads.length; loop++)
		{
			testerThreads[loop] = new TesterThread(loop, testerQueue, threadGroup, this);
		}
	}
	
	private void initStaticLibraries()
	{
		Debug.initializeLibrary(this);
		TokenTesting.initializeLibrary(this);
		SessionID.initializeLibrary(this);
		SessionStateUtils.initializeLibrary(this);
	}
	
	private void initThreads()
	{
		for (RequesterThread requesterThread: requesterThreads)
		{
			requesterThread.start();
		}
		
		for (TesterThread testerThread: testerThreads)
		{
			testerThread.start();
		}
		
		for (CategorizerThread categorizerThread: categorizerThreads)
		{
			categorizerThread.start();
		}
		
		proxy = new Proxy(this);
		if (scanSettings.isProxyEnabled())
		{
			startProxy();
		}
		scanStatusWindow.updateProxyStatus();
	}
	
	public void startProxy()
	{
		if (proxy == null)
		{
			proxy = new Proxy(this);
		}
		proxy.startProxy();
		getScanStatusWindow().updateProxyStatus();
	}
	
	public void restartProxy()
	{
		stopProxy();
		startProxy();
	}
	
	public void stopProxy()
	{
		if (proxy != null)
		{
			proxy.stopProxy();
		}
		getScanStatusWindow().updateProxyStatus();
	}
	
	private void monitorScanStatus(long delay, boolean monitorQueuesAndThreadsOnly, boolean monitorProxy, boolean onlyCheckTestQueue)
	{
		boolean messageDisplayed = false;
		while (running || monitorQueuesAndThreadsOnly)
		{
			// continue while the queues have content or a thread is busy
			if (
					testerQueue.queueLength() == 0 
			        && !threadsProcessing(testerThreads) 
			        && 
			        (
			        	onlyCheckTestQueue 
			        	|| 
				        (
					        requesterQueue.queueLength() == 0
					        && categorizerQueue.queueLength() == 0 
					        && !threadsProcessing(requesterThreads)
					        && !threadsProcessing(categorizerThreads)
					    )
					)
				)
			{
				// Queues are known to be empty at this point
				if (
						monitorProxy && scanSettings.isWaitOnProxy()
						&&
						proxy != null && proxy.isRunning()
					)
				{
					if (!messageDisplayed)
					{
						messageDisplayed = true;
						scanStatusWindow.displayMessage("Notice:", "The scan queues are empty, but more content can\n" +
														"still be supplied through the internal proxy\n" +
														"server. If you are done, stop the proxy, or\n" +
														"uncheck the \"Wait on proxy\" check box.");
					}	
				}
				else
				{	
					break;
				}
			}
			try
			{
				synchronized (masterScanThread)
				{
					masterScanThread.wait(delay);
				}
			}
			catch (InterruptedException e)
			{
			}
		}
	}
	
	
	private boolean threadsProcessing(AbstractQueueThread[] threads)
	{
		boolean testing = false;
		for (AbstractQueueThread thread: threads)
		{
			if (thread.getThreadState() == QueueThreadState.PROCESSING)
			{
				testing = true;
				break;
			}
		}
		return testing;
	}

	public void setScanStatusWindow(ScanStatusWindow scanStatusWindow)
    {
    	this.scanStatusWindow = scanStatusWindow;
    }

	/**
	 * Will try to generate a report
	 * @param text
	 */
	public synchronized void fatalError(String text)
	{
		if (running)
		{
			running = false;
			Debug.errDebug("FATAL ERROR: " + text);
			terminateScan("FATAL ERROR: " + text);
		}
	}
	

	public Proxy getProxy()
    {
    	return proxy;
    }

	public Database getDatabase()
    {
    	return database;
    }

	public boolean isPaused()
    {
    	return paused;
    }

	public void setPaused(boolean paused)
    {
    	this.paused = paused;
    }

	public RawHttpClient getRawHttpClient()
    {
    	return rawHttpClient;
    }
	
	public ScanStatusWindow getScanStatusWindow()
    {
    	return scanStatusWindow;
    }
	
	public void displayMessage(final String title, final String message)
	{
		if (scanStatusWindow != null)
		{
			scanStatusWindow.displayMessage(title, message);
		} 
		else if (scanSettingsWindow != null)
		{
			scanSettingsWindow.displayMessage(title, message);
		}
	}


	public ScanSettingsWindow getScanSettingsWindow()
    {
    	return scanSettingsWindow;
    }


	public void setScanSettingsWindow(ScanSettingsWindow scanSettingsWindow)
    {
    	this.scanSettingsWindow = scanSettingsWindow;
    }
	
}
