/*
 * ScanSettings.java
 * 
 * Created on August 19, 2007, 1:26 PM
 * 
 * To change this template, choose Tools | Template Manager and open the
 * template in the editor.
 */

package com.grendelscan.scan;
import com.grendelscan.tests.testModules.settings.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;

import org.apache.commons.configuration.*;

import com.grendelscan.GrendelScan;
import com.grendelscan.GUI.http.interception.*;
import com.grendelscan.requester.UrlFilters;
import com.grendelscan.requester.authentication.*;
import com.grendelscan.scan.reporting.*;
import com.grendelscan.tests.testModules.collections.MasterTestModuleCollection;
import com.grendelscan.utils.Debug;
import com.grendelscan.tests.testModules.*;
/**
 * @author David Byrne
 * 
 */
public class ScanSettings
{
	private int maxRedirects;
//	private boolean useLoggedOutDetection;
	private int acceptableAutomaticResponseCodeOverrideThreshold;
	private AuthenticationPackage authenticationPackage;
	private ArrayList<URI> baseURIs;
	private List<String> irrelevantQueryParameters;
	private List<String> forbiddenQueryParameters;
	private HashMap<String, Integer> manualResponseCodeOverrides;
	private MasterTestModuleCollection masterTestModuleCollection;
	private int maxCategorizerThreads;
	private int maxFileSizeBytes;
	private int maxRequesterThreads;
	private int maxTesterThreads;

	private boolean proxyEnabled;
	private int maxProxyThreads;
	private String proxyIPAddress;
	private int proxyPort;

	// client limits
	private int maxRequestsPerSecond;
	private int maxTotalConnections;
	private int maxConnectionsPerServer;
	private int maxRequestRetries;
	
	private ReportGenerator reportGenerator;
	private UrlFilters urlFilters;
	private boolean useAutomaticResponseCodeOverrides;
	private boolean testAllDirectoriesForResponseCodeOverrides;
	private String userAgentString;
	private int maxRequestDepth;
	private int maxRequestCount;

	private String upstreamProxyAddress;
	private int upstreamProxyPort;
	private boolean useUpstreamProxy;
	private String upstreamProxyUsername;
	private String upstreamProxyPassword;
	private HttpAuthenticationType upstreamProxyAuthenticationType;
	
	private String outputDirectory;
	private int maxFailedRequestsPerHost;
	private int socketReadTimeout; // in seconds
	private String savedTextTransactionsDirectory;
	private int maxConsecutiveFailedRequests;
	
	private boolean allowAllProxyRequests = true;
	private boolean revealHiddenFields = true;
	private boolean testProxyRequests = true;
	private boolean testInterceptedRequests = false;
	
	private boolean interceptRequests = false;
	private boolean interceptResponses = false;

	private List<InterceptFilter> requestInterceptFilters;
	private List<InterceptFilter> responseInterceptFilters;
	
	private List<Pattern> knownSessionIDRegexs;
	private boolean useAuthentication;
	private boolean waitOnProxy = true;
	private boolean proxyOnly = false;
	private boolean testManualRequests;
	private boolean loadDatabase = false;
	private final String transactionRecordDirectory = "transaction-record";
	private final String scanConfigDir = "scan-configs";
	
	public String getScanConfigDir()
    {
    	return scanConfigDir;
    }

	public void loadDefaultSettings(Scan scan)
    {
    	String filename = GrendelScan.defaultConfigDirectory + File.separator + scanConfigDir + File.separator + ConfigurationManager.getString("default_scan_config");
		XMLConfiguration config;
        try
        {
            config = new XMLConfiguration(filename);
        }
        catch (ConfigurationException e)
        {
        	config = new XMLConfiguration();
        }
        
        loadScanSettings(scan, config);
    }
    
	public List<Pattern> getKnownSessionIDRegexs()
    {
    	return knownSessionIDRegexs;
    }

	public void convertBaseUris2WhiteLists()
	{

		String regex = "";

		for (URI baseURI: baseURIs)
		{
			regex = baseURI.getScheme() + "://" + baseURI.getHost();
			if (baseURI.getPort() > 0)
			{
				regex.concat(":" + baseURI.getPort());
			}
			Pattern p = Pattern.compile("^" + Pattern.quote(regex) + ".*$", Pattern.CASE_INSENSITIVE);
			urlFilters.addUrlWhitelist(p);
		}
	}
		

	public boolean checkCompleteness() throws IllegalStateException
	{
		if (masterTestModuleCollection == null)
		{
			throw new IllegalStateException("Test modules not loaded");
		}
		if (baseURIs.size() == 0)
		{
			throw new IllegalStateException("No base URIs provided");
		}
		return true;
	}

	public boolean enableTestModule(int moduleNumber)
	{
		return getMasterTestModuleCollection().enableTestModule(moduleNumber);
	}

	public boolean disableTestModule(int moduleNumber)
	{
		return getMasterTestModuleCollection().disableTestModule(moduleNumber);
	}

	public int getAcceptableAutomaticResponseCodeOverrideThreshold()
	{
		return acceptableAutomaticResponseCodeOverrideThreshold;
	}

	public AuthenticationPackage getAuthenticationPackage()
	{
		return authenticationPackage;
	}

	/**
	 * @return the baseURIs
	 */
	public ArrayList<URI> getBaseURIs()
	{
		return baseURIs;
	}


	/**
	 * Irrelevent parameters shouldn't be viewed as significant to application logic 
	 * (e.g. sessionIDs), but can still be tested for things like XSS, or even SQL 
	 * injection. 
	 * @return
	 */
	public List<String> getIrrelevantQueryParameters()
	{
		return irrelevantQueryParameters;
	}

	/**
	 * Forbidden parameters are considered dangerous and should not be tested at all. 
	 * They may be used for crawling, but only if the value is supplied by the server 
	 * (i.e. a hidden field, or part of a URL).
	 * @return
	 */
	public List<String> getForbiddenQueryParameters()
	{
		return forbiddenQueryParameters;
	}

	public HashMap<String, Integer> getManualResponseCodeOverrides()
	{
		return manualResponseCodeOverrides;
	}

	public MasterTestModuleCollection getMasterTestModuleCollection()
	{
		return masterTestModuleCollection;
	}

	public int getMaxCategorizerThreads()
	{
		return maxCategorizerThreads;
	}

	public int getMaxConnectionsPerServer()
	{
		return maxConnectionsPerServer;
	}

	public int getMaxFileSizeBytes()
	{
		return maxFileSizeBytes;
	}

	public int getMaxProxyThreads()
	{
		return maxProxyThreads;
	}

	/**
	 * @return the maxRequesterThreads
	 */
	public int getMaxRequesterThreads()
	{
		return maxRequesterThreads;
	}

	/**
	 * @return the maxTesterThreads
	 */
	public int getMaxTesterThreads()
	{
		return maxTesterThreads;
	}

	public int getMaxTotalConnections()
	{
		return maxTotalConnections;
	}

	public String getProxyIPAddress()
	{
		return proxyIPAddress;
	}

	public int getProxyPort()
	{
		return proxyPort;
	}

	public ReportGenerator getReportGenerator()
	{
		return reportGenerator;
	}

	public UrlFilters getUrlFilters()
	{
		return urlFilters;
	}

	public boolean getUseAutomaticResponseCodeOverrides()
	{
		return useAutomaticResponseCodeOverrides;
	}

	/**
	 * @return the userAgentString
	 */
	public String getUserAgentString()
	{
		return userAgentString;
	}

	public void loadModules(Scan scan) throws FileNotFoundException
	{
		masterTestModuleCollection = new MasterTestModuleCollection(scan);
	}

	public void setAcceptableAutomaticResponseCodeOverrideThreshold(int acceptableAutomaticResponseCodeOverrideThreshold)
	{
		this.acceptableAutomaticResponseCodeOverrideThreshold = acceptableAutomaticResponseCodeOverrideThreshold;
	}

	public void setAuthenticationPackage(AuthenticationPackage authenticationPackage)
	{
		this.authenticationPackage = authenticationPackage;
	}

	public void setMasterTestModuleCollection(MasterTestModuleCollection masterTestModuleCollection)
	{
		this.masterTestModuleCollection = masterTestModuleCollection;
	}

	public void setMaxCategorizerThreads(int maxCategorizerThreads)
	{
		this.maxCategorizerThreads = maxCategorizerThreads;
	}

	public void setMaxConnectionsPerServer(int maxConnectionsPerServer)
	{
		this.maxConnectionsPerServer = maxConnectionsPerServer;
	}

	public void setMaxFileSizeKiloBytes(int maxFileSizeBytes)
	{
		this.maxFileSizeBytes = maxFileSizeBytes;
	}

	public void setMaxProxyThreads(int maxProxyThreads)
	{
		this.maxProxyThreads = maxProxyThreads;
	}

	/**
	 * @param maxRequesterThreads
	 *            the maxRequesterThreads to set
	 */
	public void setMaxRequesterThreads(int maxRequesterThreads)
	{
		if (maxRequesterThreads > 0)
		{
			this.maxRequesterThreads = maxRequesterThreads;
		}
	}

	/**
	 * @param maxTesterThreads
	 *            the maxTesterThreads to set
	 */
	public void setMaxTesterThreads(int maxTesterThreads)
	{
		if (maxTesterThreads > 0)
		{
			this.maxTesterThreads = maxTesterThreads;
		}
	}

	public void setMaxTotalConnections(int maxTotalConnections)
	{
		this.maxTotalConnections = maxTotalConnections;
	}

	public void setProxyIPAddress(String proxyIPAddress)
	{
		this.proxyIPAddress = proxyIPAddress;
	}

	public void setProxyPort(int proxyPort)
	{
		this.proxyPort = proxyPort;
	}

	public void setReportGenerator(ReportGenerator reportGenerator)
	{
		this.reportGenerator = reportGenerator;
	}


	public void setUrlFilters(UrlFilters urlFilters)
	{
		this.urlFilters = urlFilters;
	}

	public void setUseAutomaticResponseCodeOverrides(boolean useAutomaticResponseCodeOverrides)
	{
		this.useAutomaticResponseCodeOverrides = useAutomaticResponseCodeOverrides;
	}

	/**
	 * @param userAgentString
	 *            the userAgentString to set
	 */
	public void setUserAgentString(String userAgentString)
	{
		this.userAgentString = userAgentString;
	}

	public int getMaxRequestDepth()
    {
    	return maxRequestDepth;
    }

	public void setMaxRequestDepth(int maxRequestDepth)
    {
    	this.maxRequestDepth = maxRequestDepth;
    }

	public boolean isProxyEnabled()
    {
    	return proxyEnabled;
    }

	public void setProxyEnabled(boolean proxyEnabled)
    {
    	this.proxyEnabled = proxyEnabled;
    }

	public int getMaxRequestsPerSecond()
    {
    	return maxRequestsPerSecond;
    }

	public void setMaxRequestsPerSecond(int maxRequestsPerSecond)
    {
    	this.maxRequestsPerSecond = maxRequestsPerSecond;
    }

	public int getMaxFailedRequestsPerHost()
    {
    	return maxFailedRequestsPerHost;
    }

	public void setMaxFailedRequestsPerHost(int maxFailedRequestsPerHost)
    {
    	this.maxFailedRequestsPerHost = maxFailedRequestsPerHost;
    }

//	public boolean isUseLoggedOutDetection()
//    {
//    	return useLoggedOutDetection;
//    }
//
//	public void setUseLoggedOutDetection(boolean useLoggedOutDetection)
//    {
//    	this.useLoggedOutDetection = useLoggedOutDetection;
//    }

	public int getMaxRedirects()
    {
    	return maxRedirects;
    }

	public void setMaxRedirects(int maxRedirects)
    {
    	this.maxRedirects = maxRedirects;
    }

	public boolean getTestAllDirectoriesForResponseCodeOverrides()
    {
    	return testAllDirectoriesForResponseCodeOverrides;
    }

	public void setTestAllDirectoriesForResponseCodeOverrides(boolean checkAllDirectoriesForResponseCodeOverrides)
    {
    	this.testAllDirectoriesForResponseCodeOverrides = checkAllDirectoriesForResponseCodeOverrides;
    }

	public String getUpstreamProxyAddress()
    {
    	return upstreamProxyAddress;
    }

	public void setUpstreamProxyAddress(String upstreamProxyHost)
    {
    	this.upstreamProxyAddress = upstreamProxyHost;
    }

	public int getUpstreamProxyPort()
    {
    	return upstreamProxyPort;
    }

	public void setUpstreamProxyPort(int upstreamProxyPort)
    {
    	this.upstreamProxyPort = upstreamProxyPort;
    }

	public boolean getUseUpstreamProxy()
    {
    	return useUpstreamProxy;
    }

	public void setUseUpstreamProxy(boolean useUpstreamProxy)
    {
    	this.useUpstreamProxy = useUpstreamProxy;
    }

	public String getUpstreamProxyUsername()
    {
    	return upstreamProxyUsername;
    }

	public void setUpstreamProxyUsername(String upstreamProxyUsername)
    {
    	this.upstreamProxyUsername = upstreamProxyUsername;
    }

	public String getUpstreamProxyPassword()
    {
    	return upstreamProxyPassword;
    }

	public void setUpstreamProxyPassword(String upstreamProxyPassword)
    {
    	this.upstreamProxyPassword = upstreamProxyPassword;
    }

	public HttpAuthenticationType getUpstreamProxyAuthenticationType()
    {
    	return upstreamProxyAuthenticationType;
    }

	public void setUpstreamProxyAuthenticationType(HttpAuthenticationType upstreamProxyAuthenticationType)
    {
    	this.upstreamProxyAuthenticationType = upstreamProxyAuthenticationType;
    }

	public int getMaxRequestCount()
    {
    	return maxRequestCount;
    }

	public void setMaxRequestCount(int maxRequestCount)
    {
    	this.maxRequestCount = maxRequestCount;
    }

	public String getOutputDirectory()
    {
    	return outputDirectory;
    }

	public void setOutputDirectory(String outputDirectory)
    {
    	this.outputDirectory = outputDirectory;
    }

	public String getSavedTextTransactionsDirectory()
    {
    	return savedTextTransactionsDirectory;
    }

	public void setSavedTextTransactionsDirectory(String transactionDirectory)
    {
    	this.savedTextTransactionsDirectory = transactionDirectory;
    }

	public int getMaxRequestRetries()
    {
    	return maxRequestRetries;
    }

	public void setMaxRequestRetries(int maxRequestRetries)
    {
    	this.maxRequestRetries = maxRequestRetries;
    }

	public int getMaxConsecutiveFailedRequests()
    {
    	return maxConsecutiveFailedRequests;
    }

	public void setMaxConsecutiveFailedRequests(int maxConsecutiveFailedRequests)
    {
    	this.maxConsecutiveFailedRequests = maxConsecutiveFailedRequests;
    }

	public int getSocketReadTimeout()
    {
    	return socketReadTimeout;
    }

	public void setSocketReadTimeout(int connectionTimeout)
    {
    	this.socketReadTimeout = connectionTimeout;
    }

	public boolean isAllowAllProxyRequests()
    {
    	return allowAllProxyRequests;
    }

	public void setAllowAllProxyRequests(boolean allowAllProxyRequests)
    {
    	this.allowAllProxyRequests = allowAllProxyRequests;
    }

	public boolean isRevealHiddenFields()
    {
    	return revealHiddenFields;
    }

	public void setRevealHiddenFields(boolean revealHiddenFields)
    {
    	this.revealHiddenFields = revealHiddenFields;
    }

	public boolean isInterceptRequests()
    {
    	return interceptRequests;
    }

	public void setInterceptRequests(boolean interceptRequests)
    {
    	this.interceptRequests = interceptRequests;
    }

	public boolean isInterceptResponses()
    {
    	return interceptResponses;
    }

	public void setInterceptResponses(boolean interceptResponses)
    {
    	this.interceptResponses = interceptResponses;
    }

	public List<InterceptFilter> getRequestInterceptFilters()
    {
    	return requestInterceptFilters;
    }

	public List<InterceptFilter> getResponseInterceptFilters()
    {
    	return responseInterceptFilters;
    }

	public boolean isTestProxyRequests()
    {
    	return testProxyRequests;
    }

	public void setTestProxyRequests(boolean testProxyRequests)
    {
    	this.testProxyRequests = testProxyRequests;
    }

	public boolean isTestInterceptedRequests()
    {
    	return testInterceptedRequests;
    }

	public void setTestInterceptedRequests(boolean testInterceptedRequests)
    {
    	this.testInterceptedRequests = testInterceptedRequests;
    }

	
	private static final String AUTHENTICATION__CREDENTIALS = "authentication.credentials(";
	private static final String AUTHENTICATION__CREDENTIALS_PASSWORD = ").password";
	private static final String AUTHENTICATION__CREDENTIALS_USERNAME = ").username";
	private static final String AUTHENTICATION__METHOD = "authentication.form_based.method";
	private static final String AUTHENTICATION__PASSWORD_PARAMETER = "authentication.form_based.password_parameter_name";
	private static final String AUTHENTICATION__POST_QUERY = "authentication.form_based.post_query";
	private static final String AUTHENTICATION__URI = "authentication.form_based.uri";
	private static final String AUTHENTICATION__USER_PARAMETER = "authentication.form_based.user_parameter_name";
	private static final String AUTHENTICATION__TYPE = "authentication.form_based.type";
	private static final String AUTHENTICATION__TYPE__FORM = "form";
	private static final String AUTHENTICATION__TYPE__HTTP = "http";
	private static final String AUTHENTICATION__USE_AUTHENTICATION = "authentication.use_authentication";
	private static final String HTTP_CLIENT__MAX_CONNECTIONS_PER_SERVER = "http_client.max_connections_per_server";
	private static final String HTTP_CLIENT__MAX_CONSECUTIVE_FAILED_REQUESTS = "http_client.max_consecutive_failed_requests";
	private static final String HTTP_CLIENT__MAX_FAILED_REQUESTS_PER_SERVER = "http_client.max_failed_requests_per_server";
	private static final String HTTP_CLIENT__MAX_FILE_SIZE = "http_client.max_file_size";
	private static final String HTTP_CLIENT__MAX_REDIRECTS = "http_client.defaults.max_redirects";
	private static final String HTTP_CLIENT__MAX_REQUESTS_PER_SECOND = "http_client.max_requests_per_second";
	private static final String HTTP_CLIENT__MAX_REQUEST_COUNT = "http_client.max_request_count";
	private static final String HTTP_CLIENT__MAX_REQUEST_DEPTH = "http_client.max_request_depth";
	private static final String HTTP_CLIENT__MAX_REQUEST_RETRIES = "http_client.max_request_retries";
	private static final String HTTP_CLIENT__MAX_TOTAL_CONNECTIONS = "http_client.max_total_connections";
	private static final String HTTP_CLIENT__SOCKET_READ_TIMEOUT = "http_client.socket_read_timeout";
	private static final String HTTP_CLIENT__UPSTREAM_PROXY__ADDRESS = "http_client.upstream_proxy.address";
	private static final String HTTP_CLIENT__UPSTREAM_PROXY__PASSWORD = "http_client.upstream_proxy.password";
	private static final String HTTP_CLIENT__UPSTREAM_PROXY__PORT = "http_client.upstream_proxy.port";
	private static final String HTTP_CLIENT__UPSTREAM_PROXY__USERNAME = "http_client.upstream_proxy.username";
	private static final String HTTP_CLIENT__UPSTREAM_PROXY__USE_UPSTREAM_PROXY = "http_client.upstream_proxy.use_upstream_proxy";
	private static final String HTTP_CLIENT__USER_AGENT_STRING = "http_client.user_agent_string";
	private static final String PROXY_SETTINGS__ALLOW_ALL_PROXY_REQUESTS = "proxy_settings.allow_all_proxy_requests";
	private static final String PROXY_SETTINGS__INTERCEPT_REQUESTS = "proxy_settings.intercept_requests";
	private static final String PROXY_SETTINGS__INTERCEPT_RESPONSES = "proxy_settings.intercept_responses";
	private static final String PROXY_SETTINGS__MAX_PROXY_THREADS = "proxy_settings.max_proxy_threads";
	private static final String PROXY_SETTINGS__PROXY_BIND_ADDRESS = "proxy_settings.proxy_bind_address";
	private static final String PROXY_SETTINGS__PROXY_BIND_PORT = "proxy_settings.proxy_bind_port";
	private static final String PROXY_SETTINGS__PROXY_ENABLED = "proxy_settings.proxy_enabled";
	private static final String PROXY_SETTINGS__REVEAL_HIDDEN_FIELDS = "proxy_settings.reveal_hidden_fields";
	private static final String PROXY_SETTINGS__TEST_INTERCEPTED_REQUESTS = "proxy_settings.test_intercepted_requests";
	private static final String PROXY_SETTINGS__TEST_PROXY_REQUESTS = "proxy_settings.test_proxy_requests";
	private static final String SCAN_SETTINGS__BASE_URIS = "scan_settings.base_uris(";
	private static final String SCAN_SETTINGS__OUTPUT_DIRECTORY = "scan_settings.output_directory";
	private static final String SCAN_SETTINGS__QUEUES__MAX_CATEGORIZER_THREADS = "scan_settings.queues.max_categorizer_threads";
	private static final String SCAN_SETTINGS__QUEUES__MAX_REQUESTER_THREADS = "scan_settings.queues.max_requester_threads";
	private static final String SCAN_SETTINGS__QUEUES__MAX_TESTER_THREADS = "scan_settings.queues.max_tester_threads";
	private static final String SCAN_SETTINGS__REPORTING__FORMAT = "scan_settings.reporting.format";
	private static final String SCAN_SETTINGS__REPORTING__FILENAME = "scan_settings.reporting.filename";
	private static final String SCAN_SETTINGS__REPORTING__MIN_SEVERITY = "scan_settings.reporting.min_severity";
	private static final String REPORT_FORMAT__TEXT = "text";
	private static final String REPORT_FORMAT__HTML = "html";
	private static final String SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__BASE = "scan_settings.response_code_overrides.manual_overrides(";
	private static final String SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__PATTERN = ").pattern";
	private static final String SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__STATUS_CODE = ").status_code";
	private static final String SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__TEST_ALL_DIRECTORIES = "scan_settings.response_code_overrides.test_all_directories";
	private static final String SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__THRESHOLD = "scan_settings.response_code_overrides.override_threshold";
	private static final String SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__USE_AUTOMATIC_OVERRIDES = "scan_settings.response_code_overrides.use_automatic_overrides";
	private static final String SCAN_SETTINGS__RESTRICTIONS__FORBIDDEN_PARAMETERS = "scan_settings.restrictions.forbidden_parameter_names(";
	private static final String SCAN_SETTINGS__RESTRICTIONS__IRRELEVANT_PARAMETERS = "scan_settings.restrictions.irrelevant_parameters(";
	private static final String SCAN_SETTINGS__RESTRICTIONS__URL_BLACKLIST = "scan_settings.restrictions.url_blacklist(";
	private static final String SCAN_SETTINGS__RESTRICTIONS__URL_WHITELIST = "scan_settings.restrictions.url_whitelist(";
	private static final String SCAN_SETTINGS__TRANSACTION_DIRECTORY = "scan_settings.transaction_directory";
	private static final String SESSION_TRACKING__KNOWN_SESSION_ID_PATTERNS = "session_tracking.known_session_id_patterns(";
	private static final String SESSION_TRACKING__USE_LOGGED_OUT_DETECTION = "session_tracking.use_logged_out_detection";
	private static final String TEST_MODULES = "test_modules(";
	private static final String TEST_MODULES__MODULE_NUMBER = ").module_number";
	private static final String TEST_MODULES__MODULE_NAME = ").module_name";
	private static final String TEST_MODULES__ENABLED = ").enabled";
	private static final String TEST_MODULES__OPTIONS = ").options(";
	private static final String TEST_MODULES__OPTION_NAME = ").option_name";
	private static final String TEST_MODULES__OPTION_VALUE = ").option_value";

	
	
	public void saveScanSettings(String filename) throws ConfigurationException
	{
		XMLConfiguration config = new XMLConfiguration();
		config.setProperty(HTTP_CLIENT__MAX_REDIRECTS, maxRedirects);
//		config.setProperty(SESSION_TRACKING__USE_LOGGED_OUT_DETECTION, useLoggedOutDetection);
		config.setProperty(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__THRESHOLD, acceptableAutomaticResponseCodeOverrideThreshold);
		
		if (authenticationPackage != null)
		{
			config.setProperty(AUTHENTICATION__USE_AUTHENTICATION, true);
			if (authenticationPackage instanceof FormBasedAuthentication)
			{
				config.setProperty(AUTHENTICATION__TYPE, AUTHENTICATION__TYPE__FORM);
				FormBasedAuthentication auth = (FormBasedAuthentication) authenticationPackage;
				config.setProperty(AUTHENTICATION__USER_PARAMETER, auth.getUserParameterName());
				config.setProperty(AUTHENTICATION__PASSWORD_PARAMETER, auth.getPasswordParameterName());
				config.setProperty(AUTHENTICATION__POST_QUERY, auth.getPostQuery());
				config.setProperty(AUTHENTICATION__URI, auth.getUri());
				config.setProperty(AUTHENTICATION__METHOD, auth.getMethod());
			}
			else
			{
				config.setProperty(AUTHENTICATION__TYPE, AUTHENTICATION__TYPE__HTTP);
			}
			
			int index = 0;
			for (String username: authenticationPackage.getCredentials().keySet())
			{
				config.addProperty(AUTHENTICATION__CREDENTIALS + index + AUTHENTICATION__CREDENTIALS_USERNAME, username);
				config.addProperty(AUTHENTICATION__CREDENTIALS + index + AUTHENTICATION__CREDENTIALS_PASSWORD, authenticationPackage.getCredentials().get(username));
				index++;
			}
		}
		else
		{
			config.setProperty(AUTHENTICATION__USE_AUTHENTICATION, false);
		}
		
		{
			int index = 0;
			for(URI baseUri: baseURIs)
			{
				config.addProperty(SCAN_SETTINGS__BASE_URIS + index++ + ")", baseUri.toASCIIString());
			}
		}
		
		{
			int index = 0;
			for(String param: irrelevantQueryParameters)
			{
				config.addProperty(SCAN_SETTINGS__RESTRICTIONS__IRRELEVANT_PARAMETERS + index++ + ")", param);
			}
		}
		{
			int index = 0;
			for(String param: forbiddenQueryParameters)
			{
				config.addProperty(SCAN_SETTINGS__RESTRICTIONS__FORBIDDEN_PARAMETERS + index++ + ")", param);
			}
		}
		{
			int index = 0;
			for (String pattern: manualResponseCodeOverrides.keySet())
			{
				config.addProperty(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__BASE + index + SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__PATTERN, pattern);
				config.addProperty(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__BASE + index + SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__STATUS_CODE, manualResponseCodeOverrides.get(pattern));
				index++;
			}
		}
		
		
		config.setProperty(HTTP_CLIENT__MAX_CONNECTIONS_PER_SERVER, maxConnectionsPerServer);
		config.setProperty(HTTP_CLIENT__MAX_CONSECUTIVE_FAILED_REQUESTS, maxConsecutiveFailedRequests);
		config.setProperty(HTTP_CLIENT__MAX_FAILED_REQUESTS_PER_SERVER, maxFailedRequestsPerHost);
		config.setProperty(HTTP_CLIENT__MAX_FILE_SIZE, maxFileSizeBytes);
		config.setProperty(HTTP_CLIENT__MAX_REQUESTS_PER_SECOND, maxRequestsPerSecond);
		config.setProperty(HTTP_CLIENT__MAX_REQUEST_COUNT, maxRequestCount);
		config.setProperty(HTTP_CLIENT__MAX_REQUEST_DEPTH, maxRequestDepth);
		config.setProperty(HTTP_CLIENT__MAX_REQUEST_RETRIES, maxRequestRetries);
		config.setProperty(HTTP_CLIENT__MAX_TOTAL_CONNECTIONS, maxTotalConnections);
		config.setProperty(HTTP_CLIENT__SOCKET_READ_TIMEOUT, socketReadTimeout);
		config.setProperty(HTTP_CLIENT__UPSTREAM_PROXY__ADDRESS, upstreamProxyAddress);
		config.setProperty(HTTP_CLIENT__UPSTREAM_PROXY__PASSWORD, upstreamProxyPassword);
		config.setProperty(HTTP_CLIENT__UPSTREAM_PROXY__PORT, upstreamProxyPort);
		config.setProperty(HTTP_CLIENT__UPSTREAM_PROXY__USERNAME, upstreamProxyUsername);
		config.setProperty(HTTP_CLIENT__UPSTREAM_PROXY__USE_UPSTREAM_PROXY, useUpstreamProxy);
		config.setProperty(HTTP_CLIENT__USER_AGENT_STRING, userAgentString);
		config.setProperty(PROXY_SETTINGS__ALLOW_ALL_PROXY_REQUESTS, allowAllProxyRequests);
		config.setProperty(PROXY_SETTINGS__INTERCEPT_REQUESTS, interceptRequests);
		config.setProperty(PROXY_SETTINGS__INTERCEPT_RESPONSES, interceptResponses);
		config.setProperty(PROXY_SETTINGS__MAX_PROXY_THREADS, maxProxyThreads);
		config.setProperty(PROXY_SETTINGS__PROXY_BIND_ADDRESS, proxyIPAddress);
		config.setProperty(PROXY_SETTINGS__PROXY_BIND_PORT, proxyPort);
		config.setProperty(PROXY_SETTINGS__PROXY_ENABLED, proxyEnabled);
		config.setProperty(PROXY_SETTINGS__REVEAL_HIDDEN_FIELDS, revealHiddenFields);
		config.setProperty(PROXY_SETTINGS__TEST_INTERCEPTED_REQUESTS, testInterceptedRequests);
		config.setProperty(PROXY_SETTINGS__TEST_PROXY_REQUESTS, testProxyRequests);
		config.setProperty(SCAN_SETTINGS__OUTPUT_DIRECTORY, outputDirectory);
		config.setProperty(SCAN_SETTINGS__QUEUES__MAX_CATEGORIZER_THREADS, maxCategorizerThreads);
		config.setProperty(SCAN_SETTINGS__QUEUES__MAX_REQUESTER_THREADS, maxRequesterThreads);
		config.setProperty(SCAN_SETTINGS__QUEUES__MAX_TESTER_THREADS, maxTesterThreads);
		config.setProperty(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__TEST_ALL_DIRECTORIES, testAllDirectoriesForResponseCodeOverrides);
		config.setProperty(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__USE_AUTOMATIC_OVERRIDES, useAutomaticResponseCodeOverrides);
		config.setProperty(SCAN_SETTINGS__TRANSACTION_DIRECTORY, savedTextTransactionsDirectory);

		
		
		{
			int index = 0;
			for(String filter: urlFilters.getBlacklistsAsString())
			{
				config.addProperty(SCAN_SETTINGS__RESTRICTIONS__URL_BLACKLIST + index++ + ")", filter);
			}
		}
		{
			int index = 0;
			for(String filter: urlFilters.getWhitelistsAsString())
			{
				config.addProperty(SCAN_SETTINGS__RESTRICTIONS__URL_WHITELIST + index++ + ")", filter);
			}
		}
		{
			int index = 0;
			for(Pattern pattern: knownSessionIDRegexs)
			{
				config.addProperty(SESSION_TRACKING__KNOWN_SESSION_ID_PATTERNS + index++ + ")", pattern.pattern());
			}
		}
		
		config.setProperty(SCAN_SETTINGS__REPORTING__FILENAME, reportGenerator.getReportFilename());
		config.setProperty(SCAN_SETTINGS__REPORTING__MIN_SEVERITY, reportGenerator.getMinSeverity().getFullName());
		
		if (reportGenerator instanceof TextReportGenerator)
		{
			config.setProperty(SCAN_SETTINGS__REPORTING__FORMAT, REPORT_FORMAT__TEXT);
		}
		else if (reportGenerator instanceof HtmlReportGenerator)
		{
			config.setProperty(SCAN_SETTINGS__REPORTING__FORMAT, REPORT_FORMAT__HTML);
		}
		
		saveTestModuleSettings(config);
		
		config.save(filename);
	}

	
	private void saveTestModuleSettings(XMLConfiguration config)
	{
		int index = 0;
		for(TestModule module: masterTestModuleCollection.getAllTestModules())
		{
			config.addProperty(TEST_MODULES + index + TEST_MODULES__MODULE_NUMBER, module.getModuleNumber());
			config.addProperty(TEST_MODULES + index + TEST_MODULES__MODULE_NAME, module.getName());
			config.addProperty(TEST_MODULES + index + TEST_MODULES__ENABLED, masterTestModuleCollection.isModuleEnabled(module));
			int optionIndex = 0;
			for (ConfigurationOption option: module.getConfigurationOptions())
			{
				optionIndex = saveTestModuleOption(config, index, optionIndex, option);
			}
			index++;
		}
	}
	
	private int saveTestModuleOption(XMLConfiguration config, int moduleIndex, int optionIndex, ConfigurationOption option)
	{
		
		if (option instanceof OptionGroup)
		{
			for (ConfigurationOption subOption: ((OptionGroup) option).getAllOptions())
			{
				optionIndex = saveTestModuleOption(config, moduleIndex, optionIndex, subOption);
			}
			return optionIndex;
		}
		else
		{
			config.addProperty(TEST_MODULES + moduleIndex + TEST_MODULES__OPTIONS + optionIndex + TEST_MODULES__OPTION_NAME, option.getName());
			if (option instanceof TextOption)
			{
				config.addProperty(TEST_MODULES + moduleIndex + TEST_MODULES__OPTIONS + optionIndex + TEST_MODULES__OPTION_VALUE, ((TextOption) option).getValue());
				return optionIndex + 1;
			}
			else if (option instanceof IntegerOption)
			{
				config.addProperty(TEST_MODULES + moduleIndex + TEST_MODULES__OPTIONS + optionIndex + TEST_MODULES__OPTION_VALUE, ((IntegerOption) option).getValue());
				return optionIndex + 1;
			}
			else if (option instanceof SelectableOption)
			{
				config.addProperty(TEST_MODULES + moduleIndex + TEST_MODULES__OPTIONS + optionIndex + TEST_MODULES__OPTION_VALUE, ((SelectableOption) option).isSelected());
				return optionIndex + 1;
			}
			else
			{
				Debug.errDebug("Very odd problem with saving module options. This probably needs to be updated.", new Throwable());
				return optionIndex;
			}
		}
	}
	
	
	private void loadTestModuleSettings(XMLConfiguration config)
	{
		int index = 0;
		while(true)
		{
			int moduleNumber = config.getInt(TEST_MODULES + index + TEST_MODULES__MODULE_NUMBER, -1);
			if (moduleNumber > 0)
			{
				if(config.getBoolean(TEST_MODULES + index + TEST_MODULES__ENABLED))
				{
					masterTestModuleCollection.enableTestModule(moduleNumber);
				}
				else
				{
					masterTestModuleCollection.disableTestModule(moduleNumber);
				}
				TestModule module = masterTestModuleCollection.getTestModule(moduleNumber);
				Map<String, String> options = loadAllOptionValues(config, index);

				for (ConfigurationOption option: module.getConfigurationOptions())
				{
					populateOptionSettings(option, options);
				}
				index++;
			}
			else
			{
				break;
			}
		}
	}
	
	private void populateOptionSettings(ConfigurationOption option, Map<String, String> options)
	{
		if (option instanceof OptionGroup)
		{
			for (ConfigurationOption subOption: ((OptionGroup) option).getAllOptions())
			{
				populateOptionSettings(subOption, options);
			}
		}
		else
		{
			String name = option.getName();
			if (options.containsKey(name))
			{
				String value = options.get(name);
				if (option instanceof TextOption)
				{
					((TextOption) option).setValue(value);
				}
				else if (option instanceof IntegerOption)
				{
					((IntegerOption) option).setValue(Integer.valueOf(value));
				}
				else if (option instanceof SelectableOption)
				{
					((SelectableOption) option).setSelected(Boolean.valueOf(value));
				}
			}
		}
	}
	
	private Map<String, String> loadAllOptionValues(XMLConfiguration config, int moduleIndex)
	{
		HashMap<String, String> options = new HashMap<String, String>(1);
		int optionIndex = 0;
		while(true)
		{
			String name = config.getString(TEST_MODULES + moduleIndex + TEST_MODULES__OPTIONS + optionIndex + TEST_MODULES__OPTION_NAME, null);
			if (name != null)
			{
				options.put(name, config.getString(TEST_MODULES + moduleIndex + TEST_MODULES__OPTIONS + optionIndex + TEST_MODULES__OPTION_VALUE));
				optionIndex++;
			}
			else
			{
				break;
			}
		}
		return options;
	}
	
	

	public void loadScanSettings(Scan scan, String filename) throws ConfigurationException
	{
		XMLConfiguration config = new XMLConfiguration(filename);
		loadScanSettings(scan, config);
	}
	
	public String generateDefaultOutputDirectory()
	{
		Date date = new Date();
		SimpleDateFormat s = new SimpleDateFormat("yyyyMMddHHmmss");
		s.format(date);
		return "scans" + File.separator + "grendel-scan-" + s.format(date);
	}
	
	private void loadScanSettings(Scan scan, XMLConfiguration config)
	{
		maxRedirects = config.getInt(HTTP_CLIENT__MAX_REDIRECTS, 3);
//		useLoggedOutDetection = config.getBoolean(SESSION_TRACKING__USE_LOGGED_OUT_DETECTION, false);
		acceptableAutomaticResponseCodeOverrideThreshold = config.getInt(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__THRESHOLD, 80);
		useAuthentication = config.getBoolean(AUTHENTICATION__USE_AUTHENTICATION, false);
		String authType = config.getString(AUTHENTICATION__TYPE, null);
		authenticationPackage = null;
		if (authType != null)
		{
			if (authType.equalsIgnoreCase(AUTHENTICATION__TYPE__FORM))
			{
				String rawUri = config.getString(AUTHENTICATION__URI);
				URI uri = null;
	            try
	            {
		            uri = new URI(rawUri);
	            }
	            catch (URISyntaxException e)
	            {
	            }
				String method = config.getString(AUTHENTICATION__METHOD, "GET");
				String postQuery = config.getString(AUTHENTICATION__POST_QUERY, "");
				String userParam = config.getString(AUTHENTICATION__USER_PARAMETER, "");
				String passwordParam = config.getString(AUTHENTICATION__PASSWORD_PARAMETER, "");
				authenticationPackage = new FormBasedAuthentication(scan, uri, method, postQuery, userParam, passwordParam);
			}
			else if (authType.equalsIgnoreCase(AUTHENTICATION__TYPE__HTTP))
			{
				authenticationPackage = new HttpBasedAuthentication(scan);
			}
		}
		if (authenticationPackage != null)
		{
			int index = 0;
			while (true)
			{
				String username = config.getString(AUTHENTICATION__CREDENTIALS + index + AUTHENTICATION__CREDENTIALS_USERNAME, null);
				String password = config.getString(AUTHENTICATION__CREDENTIALS + index + AUTHENTICATION__CREDENTIALS_PASSWORD, "");
				index++;
				if (username != null)
				{
					authenticationPackage.addCredentialSet(username, password);
				}
				else
				{
					break;
				}
			}
		}
				
		
		{
			irrelevantQueryParameters = new ArrayList<String>(1);
			int index = 0;
			while(true)
			{
				String param = config.getString(SCAN_SETTINGS__RESTRICTIONS__IRRELEVANT_PARAMETERS + index++ + ")", null);
				if (param != null)
				{
					irrelevantQueryParameters.add(param);
				}
				else
				{
					break;
				}
			}
		}

		{
			forbiddenQueryParameters = new ArrayList<String>(1);
			int index = 0;
			while(true)
			{
				String param = config.getString(SCAN_SETTINGS__RESTRICTIONS__FORBIDDEN_PARAMETERS + index++ + ")", null);
				if (param != null)
				{
					forbiddenQueryParameters.add(param);
				}
				else
				{
					break;
				}
			}
		}

		{
			baseURIs = new ArrayList<URI>(1);
			int index = 0;
			while(true)
			{
				String param = config.getString(SCAN_SETTINGS__BASE_URIS + index++ + ")", null);
				if (param != null)
				{
					URI uri;
                    try
                    {
	                    uri = new URI(param);
						baseURIs.add(uri);
                    }
                    catch (URISyntaxException e)
                    {
                    	Debug.debug("Invalid URI format when loading config: " + e.toString());
                    }
				}
				else
				{
					break;
				}
			}
		}

		
		{
			
			manualResponseCodeOverrides = new HashMap<String, Integer>(1);
			int index = 0;
			while(true)
			{
				String pattern = config.getString(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__BASE + index + SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__PATTERN, null);
				int code = config.getInt(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__BASE + index + SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__STATUS_CODE, 0);
				index++;
				if (pattern != null)
				{
					manualResponseCodeOverrides.put(pattern, code);
				}
				else
				{
					break;
				}
			}
		}

		
		maxConnectionsPerServer = config.getInt(HTTP_CLIENT__MAX_CONNECTIONS_PER_SERVER, 5);
		maxConsecutiveFailedRequests = config.getInt(HTTP_CLIENT__MAX_CONSECUTIVE_FAILED_REQUESTS, 10);
		maxFailedRequestsPerHost = config.getInt(HTTP_CLIENT__MAX_FAILED_REQUESTS_PER_SERVER, 500);
		maxFileSizeBytes = Math.round(config.getInt(HTTP_CLIENT__MAX_FILE_SIZE, 1048576) / 1024);
		maxRequestsPerSecond = config.getInt(HTTP_CLIENT__MAX_REQUESTS_PER_SECOND, 30);
		maxRequestCount = config.getInt(HTTP_CLIENT__MAX_REQUEST_COUNT, 0);
		maxRequestDepth = config.getInt(HTTP_CLIENT__MAX_REQUEST_DEPTH, 10);
		maxRequestRetries = config.getInt(HTTP_CLIENT__MAX_REQUEST_RETRIES, 3);
		maxTotalConnections = config.getInt(HTTP_CLIENT__MAX_TOTAL_CONNECTIONS, 10);
		socketReadTimeout = config.getInt(HTTP_CLIENT__SOCKET_READ_TIMEOUT, 60);
		upstreamProxyAddress = config.getString(HTTP_CLIENT__UPSTREAM_PROXY__ADDRESS, "");
		upstreamProxyPassword = config.getString(HTTP_CLIENT__UPSTREAM_PROXY__PASSWORD, "");
		upstreamProxyPort = config.getInt(HTTP_CLIENT__UPSTREAM_PROXY__PORT, 8080);
		upstreamProxyUsername = config.getString(HTTP_CLIENT__UPSTREAM_PROXY__USERNAME, "");
		useUpstreamProxy = config.getBoolean(HTTP_CLIENT__UPSTREAM_PROXY__USE_UPSTREAM_PROXY, false);
		userAgentString = config.getString(HTTP_CLIENT__USER_AGENT_STRING, "");
		allowAllProxyRequests = config.getBoolean(PROXY_SETTINGS__ALLOW_ALL_PROXY_REQUESTS, true);
		interceptRequests = config.getBoolean(PROXY_SETTINGS__INTERCEPT_REQUESTS, false);
		interceptResponses = config.getBoolean(PROXY_SETTINGS__INTERCEPT_RESPONSES, false);
		maxProxyThreads = config.getInt(PROXY_SETTINGS__MAX_PROXY_THREADS, 2);
		proxyIPAddress = config.getString(PROXY_SETTINGS__PROXY_BIND_ADDRESS, "127.0.0.1");
		proxyPort = config.getInt(PROXY_SETTINGS__PROXY_BIND_PORT, 8008);
		proxyEnabled = config.getBoolean(PROXY_SETTINGS__PROXY_ENABLED, true);
		revealHiddenFields = config.getBoolean(PROXY_SETTINGS__REVEAL_HIDDEN_FIELDS, true);
		testInterceptedRequests = config.getBoolean(PROXY_SETTINGS__TEST_INTERCEPTED_REQUESTS, true);
		testProxyRequests = config.getBoolean(PROXY_SETTINGS__TEST_PROXY_REQUESTS, true);

		outputDirectory = config.getString(SCAN_SETTINGS__OUTPUT_DIRECTORY, generateDefaultOutputDirectory());

		maxCategorizerThreads = config.getInt(SCAN_SETTINGS__QUEUES__MAX_CATEGORIZER_THREADS, 5);
		maxRequesterThreads = config.getInt(SCAN_SETTINGS__QUEUES__MAX_REQUESTER_THREADS, 10);
		maxTesterThreads = config.getInt(SCAN_SETTINGS__QUEUES__MAX_TESTER_THREADS, 15);
		testAllDirectoriesForResponseCodeOverrides = config.getBoolean(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__TEST_ALL_DIRECTORIES, true);
		useAutomaticResponseCodeOverrides = config.getBoolean(SCAN_SETTINGS__RESPONSE_CODE_OVERRIDES__USE_AUTOMATIC_OVERRIDES, true);
		savedTextTransactionsDirectory = config.getString(SCAN_SETTINGS__TRANSACTION_DIRECTORY, "http-transactions");

		
		urlFilters = new UrlFilters();
		{
			int index = 0;
			while(true)
			{
				String pattern = config.getString(SCAN_SETTINGS__RESTRICTIONS__URL_BLACKLIST + index++ + ")", null);
				if (pattern != null)
				{
					urlFilters.addUrlBlacklist(pattern);
				}
				else
				{
					break;
				}
			}
		}
		{
			int index = 0;
			while(true)
			{
				String pattern = config.getString(SCAN_SETTINGS__RESTRICTIONS__URL_WHITELIST + index++ + ")", null);
				if (pattern != null)
				{
					urlFilters.addUrlWhitelist(pattern);
				}
				else
				{
					break;
				}
			}
		}

		{
			knownSessionIDRegexs = Collections.synchronizedList(new ArrayList<Pattern>(1));
			int index = 0;
			while(true)
			{
				String pattern = config.getString(SESSION_TRACKING__KNOWN_SESSION_ID_PATTERNS + index++ + ")", null);
				if (pattern != null)
				{
					try
                    {
	                    knownSessionIDRegexs.add(Pattern.compile(pattern));
                    }
                    catch (Exception e)
                    {
	                    Debug.errDebug("Error in session ID pattern from config file: " + e.toString());
                    }
				}
				else
				{
					break;
				}
			}
		}

		String reportFormat = config.getString(SCAN_SETTINGS__REPORTING__FORMAT, "");
		String reportFilename = config.getString(SCAN_SETTINGS__REPORTING__FILENAME, "report.html");
		EventSeverity minSeverity = EventSeverity.getSeverityByName(config.getString(SCAN_SETTINGS__REPORTING__MIN_SEVERITY, EventSeverity.INFO.getFullName()));
		if (reportFormat.equalsIgnoreCase(REPORT_FORMAT__TEXT))
		{
			reportGenerator = new TextReportGenerator(scan, reportFilename, minSeverity);
		}
		else
		{
			reportGenerator = new HtmlReportGenerator(scan, reportFilename, minSeverity);
		}
		
			
		requestInterceptFilters = new ArrayList<InterceptFilter>(1);
		responseInterceptFilters = new ArrayList<InterceptFilter>(1);
		loadTestModuleSettings(config);
	}
	
	
	
	
	public boolean isUseAuthentication()
    {
    	return useAuthentication;
    }

	public void setUseAuthentication(boolean useAuthentication)
    {
    	this.useAuthentication = useAuthentication;
    }

    public boolean isWaitOnProxy()
    {
    	return waitOnProxy;
    }

	public void setWaitOnProxy(boolean waitOnProxy)
    {
    	this.waitOnProxy = waitOnProxy;
    }

	public boolean isProxyOnly()
    {
    	return proxyOnly;
    }

	public void setProxyOnly(boolean proxyOnly)
    {
    	this.proxyOnly = proxyOnly;
    }

	public boolean isTestManualRequests()
    {
    	return testManualRequests;
    }

	public void setTestManualRequests(boolean testManualRequests)
    {
    	this.testManualRequests = testManualRequests;
    }

	public boolean isLoadDatabase()
    {
    	return loadDatabase;
    }

	public void setLoadDatabase(boolean loadDatabase)
    {
    	this.loadDatabase = loadDatabase;
    }

	public String getTransactionRecordDirectory()
    {
    	return transactionRecordDirectory;
    }

}
