package com.grendelscan.scan.reporting;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.*;
import java.util.Iterator;

import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;
public abstract class ReportGenerator
{
	protected Scan scan;
	protected String reportFilename;

	protected EventSeverity minSeverity;
	private Iterator<ReportableEvent> iterator;
	
	
	public ReportGenerator(Scan scan, String reportFilename, EventSeverity minSeverity)
    {
	    this.scan = scan;
	    this.reportFilename = reportFilename;
	    this.minSeverity = minSeverity;
    }

	protected boolean isMoreEvents()
	{
		return getIterator().hasNext();
	}
	
	private Iterator<ReportableEvent> getIterator()
	{
		if (iterator == null)
		{
			iterator = scan.getReportCollector().iterator();
		}
		return iterator;
	}
	/**
	 * Grabs the next event 
	 * @return Null if no more records
	 */
	protected ReportableEvent getNextEvent()
	{
		ReportableEvent event = null;

		if (getIterator().hasNext())
		{
			event = getIterator().next();
		}
		
		return event;
	}
	 
	
	public abstract void generateReport();
	
	protected void writeReport(String reportString)
	{
		OutputStream outfile;

		try
		{
			String filename;
			if (reportFilename.contains(File.separator))
			{
				filename = reportFilename;
			}
			else
			{
				filename = scan.getScanSettings().getOutputDirectory() + File.separator + reportFilename;
			}
			outfile = new FileOutputStream(filename);
			outfile.write(reportString.getBytes());
			outfile.close();
		}
		catch (FileNotFoundException e)
		{
			Debug.errDebug("Problem opening file for writing in SimpleDebugReport.writeReport: " + e.toString());
		}
		catch (IOException e)
		{
			Debug.errDebug("IOException in SimpleDebugReportGenerator.generateReport: " + e.toString());

		}
	}
	public String getReportFilename()
    {
    	return reportFilename;
    }

	public EventSeverity getMinSeverity()
    {
    	return minSeverity;
    }

}
