package com.grendelscan.scan.reporting;




/**
 * A basic event that should show up in a scan report.
 * 
 * @author David Byrne
 */
public class ReportableEvent 
{
	private int id;
	private EventSeverity severity;
	private String source, title, briefDescription, longDescription, recomendations, references, url, impact;
	private java.util.Date time;


	public ReportableEvent()
	{
		
	}
	
	/**
	 * It's best to use all of these fields. If you don't
	 * any nulls will default to empty strings.
	 * 
	 * @param time
	 *            Probably easiest to send something like
	 *            "new Date()". If null, defaults to the
	 *            current time when the object is
	 *            instantiated.
	 * @param url
	 *            The URL associated with the event.
	 * @param source
	 *            What generated the event. Probably the
	 *            module name, but could be something else,
	 *            especially if the event is an error.
	 * @param severity
	 * @param title
	 * @param briefDescription
	 * @param longDescription
	 * @param recomendations
	 * @param references
	 */
	public ReportableEvent(java.util.Date time, String source, EventSeverity severity, String url, String title, 
						String briefDescription, String longDescription, String impact, String recomendations, String references)
	{
		this.source = source;
		this.severity = severity;
		this.title = title;
		this.briefDescription = briefDescription;
		this.longDescription = longDescription;
		this.impact = impact;
		this.recomendations = recomendations;
		this.references = references;
		this.time = time;
		this.url = url;
		initDefaults();

	}


	public String getBriefDescription()
	{
		return briefDescription;
	}

	public String getLongDescription()
	{
		return longDescription;
	}

	public String getReferences()
	{
		return references;
	}

	public EventSeverity getSeverity()
	{
		return severity;
	}

	public String getSource()
	{
		return source;
	}

	public java.util.Date getTime()
	{
		return time;
	}

	public String getTitle()
	{
		return title;
	}

	public String getUrl()
	{
		return url;
	}



	private void initDefaults()
	{
		if (time == null)
		{
			time = new java.util.Date();
		}

		if (source == null)
		{
			source = "Unknown";
		}

		/*
		if (severity < 0)
		{
			severity = ReportableEvent.UNKNOWN_SEVERITY;
		}
*/
		if (title == null)
		{
			title = "";
		}

		if (briefDescription == null)
		{
			briefDescription = "";
		}

		if (longDescription == null)
		{
			longDescription = "";
		}

		if (recomendations == null)
		{
			recomendations = "";
		}

		if (references == null)
		{
			references = "";
		}
	}
	


	public int getId()
    {
    	return id;
    }


	public void setId(int id)
    {
    	this.id = id;
    }


	public String getRecomendations()
    {
    	return recomendations;
    }


	public void setRecomendations(String recomendations)
    {
    	this.recomendations = recomendations;
    }


	public String getImpact()
    {
    	return impact;
    }


	public void setImpact(String impact)
    {
    	this.impact = impact;
    }


	public void setSeverity(EventSeverity severity)
    {
    	this.severity = severity;
    }


	public void setSource(String source)
    {
    	this.source = source;
    }


	public void setTitle(String title)
    {
    	this.title = title;
    }


	public void setBriefDescription(String briefDescription)
    {
    	this.briefDescription = briefDescription;
    }


	public void setLongDescription(String longDescription)
    {
    	this.longDescription = longDescription;
    }


	public void setReferences(String references)
    {
    	this.references = references;
    }


	public void setUrl(String url)
    {
    	this.url = url;
    }


	public void setTime(java.util.Date time)
    {
    	this.time = time;
    }
}
