package com.grendelscan.tests.libraries;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.*;

import org.cobra_grendel.html.domimpl.HTMLDocumentImpl;
import org.w3c.dom.Node;

import com.grendelscan.html.EventExecutorTerminator;
import com.grendelscan.html.EventHandlerExecutor;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.tests.libraries.TokenTesting.HtmlContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextType;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.utils.StringUtils;
import com.grendelscan.utils.URIStringUtils;

public class XSS
{
	
	static private List<String> advancedBaseAttacks;
	
	static private String fakeHostname;
	
	static private String fakeIPAddress;
	
	static private boolean initialized = false;
	
	static private String javascriptMethod;
	static private Set<String> knownXSS;
	static private List<String> mediumBaseAttacks;
	static private List<String> simpleBaseAttacks;
	
	public synchronized static void addXSS(AbstractHttpTransaction transaction, String parameterName)
	{
		initializeLibrary();
		knownXSS.add(parameterName + "-----" + transaction.getBaseUri() + transaction.getQuerySummary(true));
	}
	
	public synchronized static boolean checkForKnownXSS(AbstractHttpTransaction transaction, String parameterName)
	{
		initializeLibrary();
		return knownXSS.contains(parameterName + "-----" + transaction.getBaseUri() + transaction.getQuerySummary(true));
	}
	
	public static void checkForBadChars(List<String> attackStrings, Character badChars[])
	{
		List<String> tmpAttacks = new ArrayList<String>(attackStrings);
		for(String attackString: tmpAttacks)
		{
			for (char badChar: badChars)
			{
				if (attackString.indexOf(badChar) >= 0)
				{
					attackStrings.remove(attackString);
					break;
				}
			}
		}
	}
	
	public static ArrayList<String> craftAdvancedXSSAttacks(TokenContextType htmlContext)
	{
		initializeLibrary();
		ArrayList<String> attacks = new ArrayList<String>();
		// We add in all of the simple & medium ones because we will try some evasion with them
		attacks.addAll(craftSimpleEscapeAttacks(htmlContext, advancedBaseAttacks));
		attacks.addAll(craftMediumEscapeAttacks(htmlContext, advancedBaseAttacks));
		attacks.addAll(advancedBaseAttacks);
		
		ArrayList<String> tmpAttacks = new ArrayList<String>(attacks);
		for (String attack: tmpAttacks)
		{
			if (attack.contains(" "))
			{
				attacks.add(attack.replace(' ', '\t'));
				attacks.add(attack.replace(' ', '\b'));
				attacks.add(attack.replace(' ', '\n'));
				attacks.add(attack.replace(' ', '\r'));
			}
		}
		return attacks;
	}
	
	public static ArrayList<String> craftMediumXSSAttacks(TokenContextType htmlContext)
	{
		initializeLibrary();
		return craftMediumEscapeAttacks(htmlContext, mediumBaseAttacks);
	}
	
	public static ArrayList<String> craftSimpleXSSAttacks(TokenContextType htmlContext)
	{
		initializeLibrary();
		return craftSimpleEscapeAttacks(htmlContext, simpleBaseAttacks);
	}
	
	
//	/**
//	 * This is supposed to look for successfull attacks, but it doesn't really work right. It's too complicated. 
//	 * @param token
//	 * @param node
//	 * @param context
//	 * @return
//	 */
//	@SuppressWarnings("unused")
//    private static boolean findSuccesfulXSSAttack(String token, Node node, HtmlContext context)
//	{
//		initializeLibrary();
//		ArrayList<String> attackPatterns = new ArrayList<String>();
//		String baseRegex = javascriptMethod + "\\s*(" + token + ")";
//		attackPatterns.addAll(XSS.updateAttackStringTokens("%%html_encoded_method_token%%", token, context));
//		attackPatterns.addAll(XSS.updateAttackStringTokens(javascriptMethod
//		        + "(String.fromCharCode(%%decimal_comma_token%%))", token, context));
//		for (String regex: attackPatterns)
//		{
//			baseRegex = baseRegex.concat("|").concat(Pattern.quote(regex));
//		}
//		
//		Pattern executableRegex = Pattern.compile(baseRegex, Pattern.CASE_INSENSITIVE);
//		
//		// This isn't a typo. The scheme and colon aren't required for some
//		// browsers (FF)
//		baseRegex = "//" + fakeHostname + "/" + token;
//		attackPatterns.clear();
//		attackPatterns.addAll(XSS.updateAttackStringTokens("%%html_encoded_script_path_token%%", token, context));
//		for (String regex: attackPatterns)
//		{
//			baseRegex = baseRegex.concat("|").concat(Pattern.quote(regex));
//		}
//		Pattern srcRegex = Pattern.compile(baseRegex, Pattern.CASE_INSENSITIVE);
//		
//		return recursiveXSSSearch(node, executableRegex, srcRegex);
//	}
//	
	
//	public static boolean findXSSAttackString(String attackString, Node node)
//	{
//		return findXSSAttackString(attackString, HtmlNodeWriter.write(node));
//	}
	
//	public static boolean findXSSAttackString(String attackString, String body)
//	{
//		initializeLibrary();
//		boolean found = false;
//		
//		String firstChar = attackString.substring(0, 1);
//		String regex = "";
//		if (firstChar.equals("/") || firstChar.equals("\\"))
//		{
//			regex = "[^\\\\]";
//		}
//		regex += attackString;
//		Pattern p = Pattern.compile(regex, Pattern.CASE_INSENSITIVE);
//		Matcher m = p.matcher(body);
//		if (m.find())
//		{
//			found = true;
//		}
//		
//		return found;
//	}
	
	public static String getFakeHostname()
	{
		initializeLibrary();
		return fakeHostname;
	}
	
	public static String getFakeIPAddress()
	{
		initializeLibrary();
		return fakeIPAddress;
	}
	
	public static String getJavascriptMethod()
	{
		initializeLibrary();
		return javascriptMethod;
	}
	
	public static String getXSSImpact()
	{
		return "Cross site scripting can (XSS) is a vulnerability that allows an attacker to insert arbitrary web content "
		        + "(HTML, JavaScript, etc) into an otherwise legitimate page. Because the URL is still on the targeted website, a user "
		        + "may consider it to be a trusted link. This can be leveraged to perform many attacks, such as: <br>"
		        + "<ul><li>Reformatting the page to appear as a login page, but with the credentials sent to the attacker</li>"
		        + "<li>Sending the users session key to the attacker, allowing for session hijacking</li>"
		        + "<li>Forcing the users browser to send attacks to other websites</li>"
		        + "<li>Logging the users keystrokes</li></ul>";
	}
	
	public static String getXSSRecomendations()
	{
		return "Before using any data (stored or user-supplied) to generate web page content intended to be "
		        + "simple text, escape all HTML meta characters. Usually, the characters listed below are the only "
		        + "ones that need to be escaped. This can be done by converting them to &amp;#nn; (ignore the quotes), "
		        + "where nn is the hexadecimal ASCII character number. <br>" + "<center><table border=0>" + "<tr>"
		        + "<td><b>Character</b></td>" + "<td><b>Encoding</b></td>" + "<td> </td>" + "<td><b>Character</b></td>"
		        + "<td><b>Encoding</b></td>" + "</tr>" + "<tr>" + "<td>&lt;</td>" + "<td>&amp;lt; or &amp;#60;</td>"
		        + "<td> </td>" + "<td>)</td>" + "<td>&amp;#41;</td>" + "</tr>" +

		        "<tr>" + "<td>&gt;</td>" + "<td>&amp;gt; or &amp;#60;</td>" + "<td> </td>" + "<td>#</td>"
		        + "<td>&amp;#35;</td>" + "</tr>" +

		        "<tr>" + "<td>&amp;</td>" + "<td>&amp;amp; or &amp;#38;</td>" + "<td> </td>" + "<td>%</td>"
		        + "<td>&amp;#37;</td>" + "</tr>" +

		        "<tr>" + "<td>&quot;</td>" + "<td>&amp;quot; or &amp;#34;</td>" + "<td> </td>" + "<td>;</td>"
		        + "<td>&amp;#59;</td>" + "</tr>" +

		        "<tr>" + "<td>'</td>" + "<td>&amp;#39;</td>" + "<td> </td>" + "<td>+</td>" + "<td>&amp;#43;</td>"
		        + "</tr>" +

		        "<tr>" + "<td>(</td>" + "<td>&amp;#40;</td>" + "<td> </td>" + "<td>-</td>" + "<td>&amp;#45;</td>"
		        + "</tr></table>";
	}
	
	public static String getXSSReferences()
	{
		return "http://www.owasp.org/index.php/Cross_Site_Scripting";
	}
	
	/**
	 * Replaces various tokens in an XSS attack string with actual data
	 * @param attack
	 * @param token
	 * @param context
	 * @return
	 */
	public static ArrayList<String> updateAttackStringTokens(String attack, String token, HtmlContext context, Character badChars[])
	{
		initializeLibrary();
		boolean added = false;
		ArrayList<String> attacks = new ArrayList<String>();
		
		String newAttack = attack.replaceAll("%%token%%", token);
		newAttack =
		        newAttack.replaceAll("%%decimal_comma_token%%", StringUtils.completeEncode(token,
		                StringUtils.FORMAT_DECIMAL_COMMA_SEPERATED));
		
		if (newAttack.contains("%%html_encoded_method_token%%"))
		{
			String htmlEncodedBaseAttack =
			        newAttack.replaceAll("%%html_encoded_method_token%%", StringUtils.completeEncode(
			                javascriptMethod + "(" + token + ")", StringUtils.FORMAT_HTML_HEX_ENCODED));
			attacks.add(htmlEncodedBaseAttack);
			
			htmlEncodedBaseAttack =
			        newAttack.replaceAll("%%html_encoded_method_token%%", StringUtils.completeEncode(
			                javascriptMethod + "(" + token + ")", StringUtils.FORMAT_HTML_DECIMAL_ENCODED));
			attacks.add(htmlEncodedBaseAttack);
			
			htmlEncodedBaseAttack =
			        newAttack.replaceAll("%%html_encoded_method_token%%", StringUtils
			                .completeEncode(javascriptMethod + "(" + token + ")",
			                        StringUtils.FORMAT_HTML_DOUBLE_PADDED_HEX_ENCODED));
			attacks.add(htmlEncodedBaseAttack);
			added = true;
		}
		
		if (newAttack.contains("%%html_encoded_script_path_token%%"))
		{
			String htmlEncodedBaseAttack =
			        newAttack.replaceAll("%%html_encoded_script_path_token%%", StringUtils.completeEncode("http://"
			                + fakeHostname + "/" + token, StringUtils.FORMAT_HTML_HEX_ENCODED));
			attacks.add(htmlEncodedBaseAttack);
			
			htmlEncodedBaseAttack =
			        newAttack.replaceAll("%%html_encoded_script_path_token%%", StringUtils.completeEncode("http://"
			                + fakeHostname + "/" + token, StringUtils.FORMAT_HTML_DECIMAL_ENCODED));
			attacks.add(htmlEncodedBaseAttack);
			
			htmlEncodedBaseAttack =
			        newAttack.replaceAll("%%html_encoded_script_path_token%%", StringUtils.completeEncode("http://"
			                + fakeHostname + "/" + token, StringUtils.FORMAT_HTML_DOUBLE_PADDED_HEX_ENCODED));
			attacks.add(htmlEncodedBaseAttack);
			added = true;
		}
		
		if (!added)
		{
			attacks.add(newAttack);
		}
		
		ArrayList<String> tempAttacks = new ArrayList<String>(attacks);
		for (String tempAttack: tempAttacks)
		{
			if (tempAttack.contains("%%quote%%"))
			{
				attacks.remove(tempAttack);
				for(String quote: context.getPossibleQuoteCharacters())
				{
					attacks.add(tempAttack.replace("%%quote%%", quote));
				}
			}
		}
		
		checkForBadChars(attacks, badChars);
		
		return attacks;
	}
	
//	/**
//	 * Generates test strings to determine if the required characters are escaped. For example, if
//	 * a double quote is required to get out of a tag attribute, then there's no point in 
//	 * more testing if double quotes are escaped. 
//	 * @param htmlContext
//	 * @param attacksTemplates
//	 * @return
//	 */
//	public static ArrayList<String> craftXSSCharacterTestStrings(TokenContextType htmlContext)
//    {
//		ArrayList<String> testStrings = new ArrayList<String>();
//		String token = TokenTesting.generateToken();
//
//		switch (htmlContext)
//		{
//			case HTML_COMMENT:
//				testStrings.add("-->" + token);
//				break;
//			case HTML_TITLE:
//				testStrings.add("</title>" + token);
//				break;
//			case HTML_SCRIPT:
//				testStrings.add("</script>" + token);
//				// There is intentionally no "break" here
//			case HTML_EVENT_HANDLER:
//				testStrings.add(";" + javascriptMethod + "(%%token%%);");
//				testStrings.add("%%quote%%);" + javascriptMethod + "(%%token%%);");
//				testStrings.add("%%quote%%;" + javascriptMethod + "(%%token%%);");
//				testStrings.add("%%quote%%onclick=" + javascriptMethod + "(%%token%%) ");
//				testStrings.add(" onclick=" + javascriptMethod + "(%%token%%) ");
//				// There is intentionally no "break" here
//				if (htmlContext == TokenContextType.HTML_SCRIPT)
//				{
//					break;
//				}
//			case HTML_TAG_ATTRIBUTE_VALUE:
//				testStrings.add("%%quote%%>" + token);
//				break;
//			case HTML_STYLE:
//				testStrings.add("</style>" + token);
//				break;
//			case HTML_TAG_ATTRIBUTE_NAME:
//				testStrings.add(" onclick=" + javascriptMethod + "(%%token%%) ");
//				// There is intentionally no "break" here
//			case HTML_TAG_NAME:
//				testStrings.add(">" + token);
//				break;
//			case HTML_TEXTAREA:
//				testStrings.add("</textarea>" + token);
//				break;
//			case HTML_PRE:
//				testStrings.add("</pre>" + token);
//				break;
//			default:
//				testStrings.add(token);
//		}
//		
//		return null;
//    }

	private static ArrayList<String> craftMediumEscapeAttacks(TokenContextType htmlContext,
	       List<String> attacksTemplates)
	{
		ArrayList<String> attacks = new ArrayList<String>();
		for (String baseAttack: attacksTemplates)
		{
			switch (htmlContext)
			{
				case HTML_COMMENT:
					attacks.add("-->" + baseAttack);
					break;
				case HTML_TITLE:
					attacks.add("</title>" + baseAttack);
					break;
				case HTML_SCRIPT:
					attacks.add("</script>" + baseAttack);
					// There is intentionally no "break" here
				case HTML_EVENT_HANDLER:
					attacks.add(";" + javascriptMethod + "(%%token%%);");
					attacks.add("%%quote%%);" + javascriptMethod + "(%%token%%);");
					attacks.add("%%quote%%;" + javascriptMethod + "(%%token%%);");
					attacks.add("%%quote%%onclick=" + javascriptMethod + "(%%token%%) ");
					attacks.add(" onclick=" + javascriptMethod + "(%%token%%) ");
					// There is intentionally no "break" here
					if (htmlContext == TokenContextType.HTML_SCRIPT)
					{
						break;
					}
				case HTML_TAG_ATTRIBUTE_VALUE:
					attacks.add("%%quote%%>" + baseAttack);
					break;
				case HTML_STYLE:
					attacks.add("</style>" + baseAttack);
					break;
				case HTML_TAG_ATTRIBUTE_NAME:
					attacks.add(" onclick=" + javascriptMethod + "(%%token%%) ");
					// There is intentionally no "break" here
				case HTML_TAG_NAME:
					attacks.add(">" + baseAttack);
					break;
				case HTML_TEXTAREA:
					attacks.add("</textarea>" + baseAttack);
					break;
				case HTML_PRE:
					attacks.add("</pre>" + baseAttack);
					break;
				default:
					attacks.add(baseAttack);
			}
		}
		return attacks;
	}
	
	private static ArrayList<String> craftSimpleEscapeAttacks(TokenContextType htmlContext,
	        List<String> attacksTemplates)
	{
		ArrayList<String> attacks = new ArrayList<String>();
		for (String baseAttack: attacksTemplates)
		{
			switch (htmlContext)
			{
				case HTML_COMMENT:
					attacks.add("-->" + baseAttack);
					break;
				case HTML_TITLE:
					attacks.add("</title>" + baseAttack);
					break;
				case HTML_TAG_ATTRIBUTE_VALUE:
				case HTML_EVENT_HANDLER:
					attacks.add("%%quote%%>" + baseAttack);
					break;
				case HTML_SCRIPT:
					attacks.add("</script>" + baseAttack);
					break;
				case HTML_STYLE:
					attacks.add("</style>" + baseAttack);
					break;
				case HTML_TAG_ATTRIBUTE_NAME:
				case HTML_TAG_NAME:
					attacks.add(">" + baseAttack);
					break;
				case HTML_TEXTAREA:
					attacks.add("</textarea>" + baseAttack);
					break;
				case HTML_PRE:
					attacks.add("</pre>" + baseAttack);
					break;
				default:
					attacks.add(baseAttack);
			}
		}
		return attacks;
	}
	
	@SuppressWarnings("unchecked")
    private synchronized static void initializeLibrary()
	{
		if (!initialized)
		{
			initialized = true;
			simpleBaseAttacks = ConfigurationManager.getList("xss.simple_base_attacks");
			mediumBaseAttacks = ConfigurationManager.getList("xss.medium_base_attacks");
			advancedBaseAttacks = ConfigurationManager.getList("xss.advanced_base_attacks");
			fakeIPAddress = ConfigurationManager.getString("xss.fake_ip_address");
			fakeHostname = ConfigurationManager.getString("xss.fake_hostname");
			javascriptMethod = ConfigurationManager.getString("xss.javascript_method");
			knownXSS = new HashSet<String>(1);
		}
	}
	
//	private static boolean recursiveXSSSearch(Node node, Pattern executableRegex, Pattern srcRegex)
//	{
//		String nodeValue = node.getNodeValue();
//		if ((nodeValue != null) && node.getParentNode().getNodeName().equalsIgnoreCase("SCRIPT")
//		        && executableRegex.matcher(nodeValue).find())
//		{
//			return true;
//		}
//		
//		short type = node.getNodeType();
//		switch (type)
//		{
//			case Node.DOCUMENT_NODE:
//			{
//				Document document = (Document) node;
//				if (recursiveXSSSearch(document.getDocumentElement(), executableRegex, srcRegex))
//				{
//					return true;
//				}
//				break;
//			}
//				
//			case Node.ELEMENT_NODE:
//			{
//				NamedNodeMap attrs = node.getAttributes();
//				for (int index = 0; index < attrs.getLength(); index++)
//				{
//					Attr attr = (Attr) attrs.item(index);
//					String nodeName = attr.getName();
//					if (HtmlUtils.isDOMEvent(nodeName) || nodeName.equalsIgnoreCase("src")
//					        || nodeName.equalsIgnoreCase("href"))
//					{
//						String value = attr.getValue();
//						value = value.replaceAll("\\s+", " ");
//						if (executableRegex.matcher(value).matches())
//						{
//							return true;
//						}
//						if ((nodeName.equalsIgnoreCase("src") || nodeName.equalsIgnoreCase("href"))
//						        && srcRegex.matcher(value).matches())
//						{
//							return true;
//						}
//					}
//				}
//				
//				Node child = node.getFirstChild();
//				while (child != null)
//				{
//					if (recursiveXSSSearch(child, executableRegex, srcRegex))
//					{
//						return true;
//					}
//					child = child.getNextSibling();
//				}
//				break;
//			}
//				
//			case Node.ENTITY_REFERENCE_NODE:
//			{
//				Node child = node.getFirstChild();
//				while (child != null)
//				{
//					if (recursiveXSSSearch(child, executableRegex, srcRegex))
//					{
//						return true;
//					}
//					child = child.getNextSibling();
//				}
//				break;
//			}
//		}
//		return false;
//	}

	public static Set<String> getKnownXSS()
    {
		initializeLibrary();
    	return knownXSS;
    }
	
	public static boolean checkSuccessfulXSSExecution(AbstractHttpTransaction transaction)
	{
		boolean success = false;
		HTMLDocumentImpl document = (HTMLDocumentImpl) transaction.runDOM();
		if (document != null)
		{
			if (!(success = checkDOMXSS(document)))
			{
				EventExecutorTerminator test = new EventExecutorTerminator()
				{
					public boolean stopExecution(Node node)
					{
						return checkDOMXSS((HTMLDocumentImpl)node.getOwnerDocument());
					}
				};
				
				try
				{
					EventHandlerExecutor.executeEvents(document, test);
				}
				catch (Exception e) 
				{
					//Debug.errDebug("Problem executing JavaScript for XSS detection: " + e.toString());
				}
				success = checkDOMXSS(document);
			}
		}
		return success;
	}
	
	private static boolean checkDOMXSS(HTMLDocumentImpl document)
	{
		boolean success = false;
		if (document.getXssToken() != null && !document.getXssToken().equals(""))
		{
			success = true;
		}
		return success;
	}
	
	
	public static Character[] getBlockedCharacters(HtmlContext htmlContext, String source)
	{
		List<Character> badChars = new ArrayList<Character>(1);
		char testChars[][] = new char[][]{
									new char[]{'&'}, 
									new char[]{';'}, 
									new char[]{'<'}, 
									new char[]{'('}, 
									new char[]{'"'}, 
									new char[]{'\''}, 
									new char[]{','}, 
									new char[]{'-'}, 
									new char[]{'>'}, 
									new char[]{')'},
//									new char[]{'<', '(', '"', '\'', ',', '-'}, 
//									new char[]{'>', ')'
											};
		for(char[] test: testChars)
		{
			String startToken = TokenTesting.generateToken();
			String endToken = TokenTesting.generateToken();
			AbstractHttpTransaction outputTransaction =
		        TokenTesting.duplicateTokenTest(htmlContext, startToken + URIStringUtils.escapeQueryParameterComponent(String.copyValueOf(test)) + endToken, "Module " + source, TransactionSource.TEST)[1];
			Pattern blockedCharactersPattern = Pattern.compile(startToken + "(.*?)" + endToken, Pattern.CASE_INSENSITIVE);
			Matcher m = blockedCharactersPattern.matcher(outputTransaction.getResponseBodyAsString());
			Set<Character> goodChars = new HashSet<Character>(1);
			while (m.find())
			{
				String middle = m.group(1);
				if (!middle.equals(""))
				{
					if (test.length == 1)
					{
						if (middle.length() == 1 && middle.charAt(0) == test[0])
						{
							goodChars.add(test[0]);
						}
					}
					else
					{
						for (char character: test)
						{
							if (middle.indexOf(character) >= 0)
							{
								goodChars.add(character);
							}
						}
					}
				}
			}
			for(char c: test)
			{
				if (!goodChars.contains(c))
				{
					badChars.add(c);
				}
			}
		}

		return badChars.toArray(new Character[0]);
	}
}
