package com.grendelscan.tests.libraries.nikto;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.Header;

import com.csvreader.CsvReader;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.*;
import com.grendelscan.scan.*;
import com.grendelscan.utils.*;


public class Nikto
{
	private static Set<CurrentSoftwareVersion> currentSoftwareVersions;
	private static boolean initialized = false;
	private static Pattern linePattern = Pattern.compile("(@\\w+)=(.*)$");
	private static int majorHttpVersion;
	private static int minorHttpVersion;
	private static String niktoDatabasePath;
	private static Set<NiktoTest> niktoTests;
	private static Map<String, Set<String>> cgiDirs;
	private static Map<String, Set<String>> niktoVariables;
	private static Map<String, Set<String>> serverHeaders;
	private static Map<String, Map<String, Integer>> serverHeaderLocations;
	private static Set<WebServerNote> serverNotes;
	private static int indexPhpThreshold;

	public static void addServerHeader(AbstractHttpTransaction transaction)
	{
		initialize();
		String hostUri = transaction.getHostUri();
		if (!serverHeaders.containsKey(hostUri))
		{
			Set<String> headers = Collections.synchronizedSet(new HashSet<String>(1));
			serverHeaders.put(hostUri, headers);
			Map<String, Integer> locations = Collections.synchronizedMap(new HashMap<String, Integer>(1));
			serverHeaderLocations.put(hostUri, locations);
		}
		for (Header header: transaction.getResponseHeaders("Server"))
		{
			if (! serverHeaders.get(hostUri).contains(header.getValue()))
			{
				serverHeaders.get(hostUri).add(header.getValue());
				serverHeaderLocations.get(hostUri).put(header.getValue(), transaction.getId());
			}
		}
		for (Header header: transaction.getResponseHeaders("X-Powered-By"))
		{
			if (! serverHeaders.get(hostUri).contains(header.getValue()))
			{
				serverHeaders.get(hostUri).add(header.getValue());
				serverHeaderLocations.get(hostUri).put(header.getValue(), transaction.getId());
			}
		}
	}

	public static Set<CurrentSoftwareVersion> getCurrentSoftwareVersions()
	{
		initialize();
		return currentSoftwareVersions;
	}

	public static int getMajorHttpVersion()
	{
		return majorHttpVersion;
	}

	public static int getMinorHttpVersion()
	{
		return minorHttpVersion;
	}

	public static String getNiktoCredits()
	{
		return "This module uses databases distributed with Nikto 2.x. Permission granted "
		        + "in writing by Sullo. Please see the database files for licensing information. "
		        + "Thank you to Sullo and the other contributors to Nikto.";
	}

	public static Set<NiktoTest> getNiktoTests()
	{
		initialize();
		return niktoTests;
	}

	public static Map<String, Set<String>> getNiktoVariables()
	{
		return niktoVariables;
	}

	public static Map<String, Set<String>> getServerHeaders()
	{
		return serverHeaders;
	}

	public static Set<String> getServerHeaders(String scheme, String host, int port)
	{
		initialize();
		String key = scheme + "://" + host + ":" + port;
		return serverHeaders.get(key);
	}

	public static Set<WebServerNote> getServerNotes()
	{
		return serverNotes;
	}

	public static Set<String> replaceNiktoVariables(String template)
	{
		Set<String> newStrings = new HashSet<String>();
		newStrings.add(template);

		for (String variableName: niktoVariables.keySet())
		{
			for (String target: newStrings)
			{
				if (target.contains(variableName))
				{
					newStrings.remove(target);
					for (String value: niktoVariables.get(variableName))
					{
						newStrings.add(target.replaceAll(Matcher.quoteReplacement(variableName), value));
					}
				}
			}
		}
		return newStrings;
	}

	private static void getHttpVersion()
	{
		String version[] = ConfigurationManager.getString("nikto.default_http_version").split("\\.");
		majorHttpVersion = Integer.valueOf(version[0]);
		minorHttpVersion = Integer.valueOf(version[1]);
	}

	private static void handleFatalNiktoError(Exception e)
	{
		Debug.errDebug("Fatal problem loading Nikto database: " + e.toString(), e);
		System.exit(1);
	}

	private synchronized static void initialize()
	{
		if (!initialized)
		{
			initialized = true;
			niktoDatabasePath = ConfigurationManager.getString("nikto.database_directory");
			serverHeaders = Collections.synchronizedMap(new HashMap<String, Set<String>>(1));
			niktoTests = loadNiktoTests("nikto.db_tests");
			currentSoftwareVersions = loadCurrentVersionDatabase("nikto.web_server_version_file");
			serverNotes = loadWebServerNoteDatabase("nikto.web_server_notes_file");
			loadNiktoVariables();
			getHttpVersion();
			serverHeaderLocations = Collections.synchronizedMap(new HashMap<String, Map<String, Integer>>(1));
			indexPhpThreshold = ConfigurationManager.getInt("nikto.index_php_threshold");
			cgiDirs = new HashMap<String, Set<String>>(1);
		}
	}
	
	private static void findCgiDirs(Scan scan, String hostUriWithoutTrailingSlash)
	{
		Set<String> dirs = new HashSet<String>(1);
		for (String dir: niktoVariables.get("@CGIDIRS"))
		{
			cgiDirs.put(hostUriWithoutTrailingSlash, dirs);
			try
            {
	            HttpGetTransaction get = new HttpGetTransaction(scan, hostUriWithoutTrailingSlash + dir, 0, TransactionSource.NIKTO);
	            get.execute("Nikto cgi dir search");
	            if (HttpUtils.fileExists(get.getLogicalResponseCode()))
	            {
	            	dirs.add(dir);
	            }
            }
            catch (URISyntaxException e)
            {
            	Debug.errDebug("Error in URI format (" + hostUriWithoutTrailingSlash + dir + "): " + e.toString(), e);
            }
		}
	}

	public static Set<String> getCgiDirs(Scan scan, String hostUriWithoutTrailingSlash)
	{
		if (!cgiDirs.containsKey(hostUriWithoutTrailingSlash))
		{
			findCgiDirs(scan, hostUriWithoutTrailingSlash);
		}
		return cgiDirs.get(hostUriWithoutTrailingSlash);
	}
	
	private static Set<CurrentSoftwareVersion> loadCurrentVersionDatabase(String configString)
	{
		initialize();
		List<String[]> lines = loadNiktoDatabase(configString);
		Set<CurrentSoftwareVersion> versions = new HashSet<CurrentSoftwareVersion>(lines.size());

		for (String line[]: lines)
		{
			CurrentSoftwareVersion currentVersion = null;
			try
			{
				currentVersion = new CurrentSoftwareVersion(line);
				versions.add(currentVersion);
			}
			catch (java.text.ParseException e)
			{
				// System.err.println("Problem parsing Nikto database line from
				// " + configString + "(" + StringUtilities.join(line, ",") +
				// "): " + e.toString());
			}
		}
		return versions;
	}

	private static List<String[]> loadNiktoDatabase(String configString)
	{
		initialize();
		List<String[]> lines = new ArrayList<String[]>(100);
		String fileName =
		        FileUtils.correctFilePathFormat(niktoDatabasePath + ConfigurationManager.getString(configString));
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new FileReader(fileName));
			CsvReader csv = new CsvReader(reader);
			csv.setEscapeMode(CsvReader.ESCAPE_MODE_BACKSLASH);
			while(csv.readRecord())
			{
				String line[] = csv.getValues();
				lines.add(line);
			}
		}
		catch (IOException e)
		{
			handleFatalNiktoError(e);
		}
		return lines;
	}

	private static Set<NiktoTest> loadNiktoTests(String configString)
	{
		initialize();
		List<String[]> lines = loadNiktoDatabase(configString);
		Set<NiktoTest> tests = new HashSet<NiktoTest>(lines.size());

		for (String line[]: lines)
		{
			NiktoTest test;
			try
			{
				test = new NiktoTest(line);
				tests.add(test);
			}
			catch (java.text.ParseException e)
			{
				// System.err.println("Problem parsing Nikto database line from
				// " + configString + "(" + StringUtilities.join(line, ",") +
				// "): " + e.toString());
			}
		}

		return tests;
	}

	private static void loadNiktoVariables()
	{
		String fileName =
		        FileUtils.correctFilePathFormat(niktoDatabasePath
		                + ConfigurationManager.getString("nikto.db_variables"));
		List<String> lines = null;
		try
		{
			lines = FileUtils.getFileAsLines(fileName);
		}
		catch (IOException e)
		{
			handleFatalNiktoError(e);
		}

		niktoVariables = new HashMap<String, Set<String>>();
		for (String line: lines)
		{
			Matcher matcher = linePattern.matcher(line);
			if (matcher.matches())
			{
				Set<String> valueSet = new HashSet<String>();
				String name = matcher.group(1);
				String values = matcher.group(2);
				niktoVariables.put(name, valueSet);
				for (String value: values.split(" "))
				{
					valueSet.add(value);
				}
			}
		}
	}

	private static Set<WebServerNote> loadWebServerNoteDatabase(String configString)
	{
		initialize();
		List<String[]> lines = loadNiktoDatabase(configString);
		Set<WebServerNote> notes = new HashSet<WebServerNote>(lines.size());

		for (String line[]: lines)
		{
			WebServerNote serverNote;
			try
			{
				serverNote = new WebServerNote(line);
				notes.add(serverNote);
			}
			catch (java.text.ParseException e)
			{
				// System.err.println("Problem parsing Nikto database line from
				// " + configString + "(" + StringUtilities.join(line, ",") +
				// "): " + e.toString());
			}
		}
		return notes;
	}

	public static Map<String, Map<String, Integer>> getServerHeaderLocations()
    {
    	return serverHeaderLocations;
    }
	
	private static Map<String, AbstractHttpTransaction> indexPHPTests;
	/**
	 * Gets /index.php for the host and stores the result.
	 * @param transaction
	 */
	public static void initializeIndexPHP(AbstractHttpTransaction transaction)
	{
		if (indexPHPTests == null)
		{
			indexPHPTests = Collections.synchronizedMap(new HashMap<String, AbstractHttpTransaction>(1));
		}
		if (!indexPHPTests.containsKey(transaction.getHostUri()))
		{
			try
			{
				HttpGetTransaction testTransaction = new HttpGetTransaction(transaction.getScan(), 
						transaction.getHostUri() + "index.php", 0, TransactionSource.NIKTO);
				testTransaction.execute("Nikto index.php test");
				indexPHPTests.put(transaction.getHostUri(), testTransaction);
			}
			catch (URISyntaxException e) 
			{
				Debug.errDebug("Odd problem with Nikto: " + e.toString(), e);
			}
		}
	}
	
	/**
	 * Checks to see if this looks the same as a normal /index.php. Currently,
	 * it checks text nodes only; basically, just what someone could read in a
	 * browser. That may change in the future.
	 * @param transaction
	 * @return
	 */
	public static boolean isNormalIndexPHP(AbstractHttpTransaction transaction)
	{
		boolean normal = true;
		initializeIndexPHP(transaction);
		if (HttpTransactionUtils.scoreResponseMatch(indexPHPTests.get(transaction.getHostUri()), transaction, 100) < indexPhpThreshold)
		{
			normal = false;
		}
		
		return normal;
	}
}
