package com.grendelscan.tests.libraries.platformErrorMessages;
import com.grendelscan.scan.*;
import com.grendelscan.requester.http.transactions.*;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.*;
import com.grendelscan.tests.libraries.*;
import com.grendelscan.html.*;
import com.grendelscan.utils.*;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import java.io.*;
import com.grendelscan.tests.testModules.settings.*;
import java.util.regex.*;
public class PlatformErrorMessages
{
	private static boolean initialized = false;
	private static List<PlatformErrorMessageMatchPattern> htmlErrorMatchPatterns;
	private static Map<Integer, String> messageHits;
	private static TextOption regexErrorPatternsOption;
	private static List<Pattern> regexPatterns;
	
	private synchronized static void initialize()
	{
		if (!initialized)
		{
			initialized = true;
			messageHits = new HashMap<Integer, String>();
			createRegexPatterns();
			htmlErrorMatchPatterns = new ArrayList<PlatformErrorMessageMatchPattern>();
			loadFuzzPatterns();
		}
	}
	
	private static void createRegexPatterns()
	{
		regexPatterns = new ArrayList<Pattern>();
		for (String rawPattern: regexErrorPatternsOption.getValue().split("[\r\n]+"))
		{
			// skip any "pattern" that is empty, or only white space
			if (rawPattern.matches("\\s*"))
			{
				continue;
			}
			try
			{
				regexPatterns.add(Pattern.compile(rawPattern, Pattern.CASE_INSENSITIVE | Pattern.MULTILINE));
			}
			catch (Exception e)
			{
				Debug.errDebug("Invalid regex (" + rawPattern + ")in platform error messages: " + e.toString(), e);
			}
		}
	}
	
	
	/**
	 * Will return true if a fuzz pattern or SQL injection error pattern is matched
	 * @param transaction
	 * @return
	 */
	public static String isErrorMatch(AbstractHttpTransaction transaction, boolean recordHit)
	{
		initialize();
		String fuzzPattern = "";
		boolean match = false;
		if (
				!(fuzzPattern = isTextErrorMatchPattern(transaction.getResponseBodyAsString())).equals("")
				|| !(fuzzPattern = isTextErrorMatchPattern(transaction.getResponseTextNodes())).equals("")
				|| !(fuzzPattern = isTitleErrorMatchPattern(HtmlNodeUtilities.getTitleText(transaction.getResponseDOM()))).equals("")
				
			)
		{
			match = true;
		}
//		if (!match)
//		{
//			String sqlType;
//			if ((sqlType = SQLInjection.findSQLErrorMessages(transaction.getResponseTextNodes())) != null)
//			{
//				fuzzPattern = "SQL error (" + sqlType + ")";
//				match = true;
//			}
//		}
		if (!match && !(fuzzPattern = testRegexPatterns(transaction)).equals(""))
		{
			match = true;
		}
		
		if (match && recordHit)
		{
			messageHits.put(transaction.getId(), fuzzPattern);
		}
		return fuzzPattern;
	}
	
	private static String testRegexPatterns(AbstractHttpTransaction transaction)
	{
		String patternText = "";
		String rawBody = normalizeText(transaction.getResponseBodyAsString());
		String textResponse = normalizeText(transaction.getResponseTextNodes());
		
		for (Pattern pattern: regexPatterns)
		{
			if (pattern.matcher(textResponse).find() || pattern.matcher(rawBody).find())
			{
				patternText = pattern.pattern();
				break;
			}
		}
		return patternText;
	}
	
	private static String normalizeText(String text)
	{
		if (text == null)
		{
			return "";
		}
		return text.toLowerCase().replaceAll("\\s+", " ");
	}
	
	private static String isTextErrorMatchPattern(String text)
	{
		String fuzzString = "";
		
		String lText = normalizeText(text);

		for (PlatformErrorMessageMatchPattern fuzzPattern: htmlErrorMatchPatterns)
		{
			if (!(fuzzString = fuzzPattern.isTextMatch(lText)).equals(""))
			{
				break;
			}
		}
		
		return fuzzString;
	}

	private static String isTitleErrorMatchPattern(String text)
	{
		String fuzzString = "";
		
		String lText = normalizeText(text);

		for (PlatformErrorMessageMatchPattern fuzzPattern: htmlErrorMatchPatterns)
		{
			if (!(fuzzString = fuzzPattern.isTitleMatch(lText)).equals(""))
			{
				break;
			}
		}
		
		return fuzzString;
	}

	
	private static void loadFuzzPatterns()
	{
		XMLReader xmlReader;
        try
        {
	        xmlReader = XMLReaderFactory.createXMLReader();
	    	FuzzPatternXMLParser handler = new FuzzPatternXMLParser(htmlErrorMatchPatterns);
	    	xmlReader.setContentHandler(handler);
	    	xmlReader.setErrorHandler(handler);
	    	FileReader reader = new FileReader("conf" + File.separator + ConfigurationManager.getString("fuzzer.html_error_match_pattern_file"));
	    	xmlReader.parse(new InputSource(reader));
        }
        catch (SAXException e)
        {
        	Debug.errDebug("Problem loading fuzz match patterns: " + e.toString(), e);
        }
        catch (FileNotFoundException e) 
		{
        	Debug.errDebug("Problem loading fuzz match patterns: " + e.toString(), e);
		}
        catch (IOException e) 
		{
        	Debug.errDebug("Problem loading fuzz match patterns: " + e.toString(), e);
		}
	}
	
	private static class FuzzPatternXMLParser extends DefaultHandler
	{
		PlatformErrorMessageMatchPattern currentPattern;
		List<PlatformErrorMessageMatchPattern> patterns;
		String currentElement;
		String currentString;
		
		
		public FuzzPatternXMLParser(List<PlatformErrorMessageMatchPattern> patterns)
        {
	        this.patterns = patterns;
        }

		@Override
	    public void endElement(String uri, String localName, String name) throws SAXException
	    {
			if (currentElement.equals("textsearchstring"))
			{
				currentPattern.addTextPattern(currentString);
			}
			else if (currentElement.equals("titlesearchstring"))
			{
				currentPattern.addTitlePattern(currentString);
			}
			currentElement = "";
	    }
		
		@Override
	    public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException
	    {
			if (name.equalsIgnoreCase("pattern"))
			{
				currentPattern = new PlatformErrorMessageMatchPattern();
				patterns.add(currentPattern);
			}
			currentElement = name.toLowerCase();
			currentString = "";
	    }

		@Override
	    public void characters(char[] ch, int start, int length) throws SAXException
	    {
			String s = new String(ch, start, length);
			currentString += s;

	    }
	}

	public static Map<Integer, String> getMessageHits()
    {
		initialize();
    	return messageHits;
    }

	public static TextOption getRegexErrorPatternsOption()
    {
		if (regexErrorPatternsOption == null)
		{
			regexErrorPatternsOption = new TextOption("Manual regex error patterns", "", "Each line is a seperate regular express to match error messages", true);
		}
    	return regexErrorPatternsOption;
    }
}
