/*
 * TesterQueue.java
 * 
 * Created on September 15, 2007, 9:52 PM
 * 
 * To change this template, choose Tools | Template Manager and open the
 * template in the editor.
 */

package com.grendelscan.tests.queue;

import java.util.HashMap;
import java.util.Iterator;
import java.util.*;
import java.util.Map;
import java.util.TreeMap;

import com.grendelscan.scan.Scan;
import com.grendelscan.scan.abstractQueue.ScanQueue;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testJobs.TestJobStatus;

/**
 * 
 * @author Administrator
 */
public class TesterQueue implements ScanQueue
{
	private TreeMap<Integer, TestJob> queue;
	private Map<Integer, Integer> pendingCount;
	private Map<Integer, Integer> processedCount;
	private Map<Integer, Integer> times;
	private boolean queueLocked = false;
	private int maxQueueSize = 100;
	private int lastJobGroupNumber = 0;
	private TestQueueStorage queueStorage;
	private boolean running = true;
//	private Map<SessionState, Set<TestJob>> testBlocks;
//	private Scan scan;
//	private int loggedOutCompareThreshold = 90; 
	
	public boolean isRunning()
    {
    	return running;
    }

	public void setRunning(boolean running)
    {
    	this.running = running;
    }

	/** Creates a new instance of TesterQueue */
	public TesterQueue(Scan scan)
	{
//		this.scan = scan;
		queue = new TreeMap<Integer, TestJob>();
		pendingCount = new HashMap<Integer, Integer>();
		processedCount = new HashMap<Integer, Integer>();
		times = new HashMap<Integer, Integer>();
		queueStorage = new TestQueueStorage(scan, this);
//		testBlocks = new HashMap<SessionState, Set<TestJob>>();
	}
	
	public synchronized void addTime(TestJob testJob, long time)
	{
		int total = 0;
		int moduleNumber = testJob.getModuleNumber();
		if (times.containsKey(moduleNumber))
		{
			total = times.get(moduleNumber);
		}
		total += time;
		times.put(moduleNumber, total);
	}
	
	public void addTests(Set<TestJob> tests)
	{
		if (tests.size() > 0)
		{
			if (queue.size() < maxQueueSize)
			{
				addToMemoryQueue(tests);
			}
			else
			{
				addToDiskQueue(tests);
			}
		}
	}
	
	private void addToMemoryQueue(Iterable<TestJob> tests)
	{
		if (!queueLocked)
		{
			synchronized (queue)
			{
				for (TestJob test: tests)
				{
					Integer i = pendingCount.get(test.getModuleNumber());
					if (i == null)
					{
						i = 0;
					}
					pendingCount.put(test.getModuleNumber(), ++i);
		            queue.put(test.getId(), test);
				}
			}
		}
	}
	
	private void addToDiskQueue(Set<TestJob> tests)
	{
		lastJobGroupNumber++;
		queueStorage.storeTestGroup(tests, lastJobGroupNumber);
	}
	
	private void loadNextJobGroup()
	{
		addToMemoryQueue(queueStorage.getNextTestGroup());
	}
	
	
	public boolean contains(TestJob job)
	{
		return queue.containsValue(job);
	}
	
	public boolean contains(int jobID)
	{
		return queue.containsKey(jobID);
	}
	
	public void emptyQueue()
	{
		synchronized(queue)
		{
			Iterator<Integer> i = queue.keySet().iterator();
			while (i.hasNext())
			{
				i.next();
				i.remove();
			}
			queueStorage.empty();
		}
	}
	
	public void lockQueue()
	{
		queueLocked = true;
	}
	
	public void unlockQueue()
	{
		queueLocked = false;
	}
	
	public TestJob getNext()
	{
		TestJob job = null;
		if (!queueLocked)
		{
			synchronized (queue)
			{
				if (queue.size() < maxQueueSize)
				{
					loadNextJobGroup();
				}
				if (queue.size() > 0)
				{
//					Map<Integer, TestJob> jobs = new HashMap<Integer, TestJob>();
					for (Integer testJobID: queue.keySet())
					{
						TestJob tempJob = queue.get(testJobID);
//						jobs.put(tempJob.getId(), tempJob);
						if ((tempJob.getStatus() == TestJobStatus.NOT_STARTED) && tempJob.noRemainingDependencies())
						{
							tempJob.setStatus(TestJobStatus.LOCKED);
							job = tempJob;
							break;
						}
					}
//					System.out.println("done");
				}
			}
		}
		return job;
	}
	
	public int queueLength()
	{
		int length = queue.size() + queueStorage.getStoredJobs();
		return length;
	}
	
	public void removeJob(TestJob testJob)
	{
		synchronized (queue)
		{
			Integer i = pendingCount.get(testJob.getModuleNumber());
			pendingCount.put(testJob.getModuleNumber(), --i);
			queue.remove(testJob.getId());
			
			i = processedCount.get(testJob.getModuleNumber());
			if (i == null)
			{
				i = 0;
			}
			i++;
			processedCount.put(testJob.getModuleNumber(), i);
		}
	}

	public TestQueueStorage getQueueStorage()
    {
    	return queueStorage;
    }

	public Map<Integer, Integer> getPendingCount()
    {
    	return pendingCount;
    }

	public Map<Integer, Integer> getProcessedCount()
    {
    	return processedCount;
    }

	public Map<Integer, Integer> getTimes()
    {
    	return times;
    }
}








//	/**
//	 * If enabled and relevant, check to see if the results of the transaction in this 
//	 * test job indicate that the session was logged out.
//	 * 
//	 * If I am only checking against a single string, this is better done in the requester queue,
//	 * before it is even categorized. However, once I add the ability to compare multiple strings
//	 * (basically, looking for a pattern of similar pages), it makes more sense here.
//	 * @param testJob
//	 */
//	private void processCompletedJob(TestJob testJob)
//	{
//		if (scan.getScanSettings().isUseLoggedOutDetection() && 
//				testJob.getTestModule().isSessionSensative() && 
//				testJob instanceof TransactionTestJob)
//		{ 
//			TransactionTestJob tTestJob = (TransactionTestJob) testJob;
//			AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(tTestJob.getTransactionID());
//			if (transaction.getAuthenticationPackage() != null)
//			{
//				if (HtmlNodeUtilities.compareDOMText(transaction.getResponseDOM(), transaction.getAuthenticationPackage().getLoggedOutPageText(), 100, true) > loggedOutCompareThreshold)
//				{
//					Debug.debug("A session was logged out. Rolling back tests.");
//					rollBackTests(transaction);
//				}
//			}
//		}
//	}
//	
//	private void rollBackTests(AbstractHttpTransaction failedTransaction)
//	{
//		Set<TransactionTestJob> reJobs = new HashSet<TransactionTestJob>();
//		AbstractHttpTransaction newLoginTransaction = failedTransaction.getAuthenticationPackage().createLoginTransaction(failedTransaction.getUsername());
//		newLoginTransaction.execute(false);
//		synchronized(queue)
//		{
//			for(String sessionIdName: failedTransaction.getAuthenticationPackage().getSessionIDNames())
//			{
//				SessionState session = failedTransaction.getSessionStates().get(sessionIdName);
//				for(TestJob testJob: testBlocks.get(session))
//				{
//					TransactionTestJob tTestJob = (TransactionTestJob) testJob;
//					AbstractHttpTransaction replacementTransaction;
//					if (reJobs.contains(tTestJob))
//					{
//						replacementTransaction = scan.getTransactionRecord().getTransaction(tTestJob.getId());
//					}
//					else
//					{
//						AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(tTestJob.getTransactionID());
//						replacementTransaction = transaction.cloneTransactionRequest();
//						tTestJob.setTransactionID(replacementTransaction.getId());
//					}
//					
//					SessionState newState = SessionStateUtils.findNewSessionIDInResponse(session.getSessionIDName(), newLoginTransaction);
//					if (newState != null)
//					{
//						replacementTransaction.replaceSessionState(newState);
//					}
//				}
//			}
//		
//			for (TransactionTestJob tTestJob: reJobs)
//			{
//
//				AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(tTestJob.getId());
//				/*
//				 * If the transaction has already been executed, or if an execution succeeds, 
//				 * add the test to the queue
//				 */
//				if (transaction.getResponseWrapper() != null || transaction.execute(false))
//				{
//					queue.put(tTestJob.getId(), tTestJob);
//				}
//			}
//		}
//	}
//	
//	private SessionState getReplacementSessionState(SessionState oldSession, AuthenticationPackage authenticationPackage, String username)
//	{
//		AbstractHttpTransaction loginTransaction = authenticationPackage.createLoginTransaction(username);
//		loginTransaction.execute(false);
//		return loginTransaction.getSessionStates().get(oldSession.getSessionIDName());
//	}
	
	
	
//	From addText
	//			if (scan.getScanSettings().isUseLoggedOutDetection() && 
//			test.getTestModule().isSessionSensative() && 
//			test instanceof TransactionTestJob)
//	{
//		TransactionTestJob tTestJob = (TransactionTestJob) test;
//		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(tTestJob.getTransactionID());
//		if (transaction.getAuthenticationPackage() != null)
//		{
//			for (String sessionName: transaction.getAuthenticationPackage().getSessionIDNames())
//			{
//				SessionState session = transaction.getSessionStates().get(sessionName);
//				if (session != null)
//				{
//					addJobsToBlocks(test, session);
//				}
//			}
//		}
//	}

//private void addJobsToBlocks(TestJob testJob, SessionState session)
//{
//Set<TestJob> jobs = testBlocks.get(session);
//
//if (jobs == null)
//{
//	jobs = new HashSet<TestJob>(20);
//	testBlocks.put(session, jobs);
//}
//jobs.add(testJob);
//}
