/*
 * TesterThread.java
 *
 * Created on September 15, 2007, 9:52 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.grendelscan.tests.queue;
import java.util.Date;

import com.grendelscan.scan.Scan;
import com.grendelscan.scan.ScanPhase;
import com.grendelscan.scan.abstractQueue.AbstractQueueThread;
import com.grendelscan.scan.abstractQueue.QueueThreadGroup;
import com.grendelscan.scan.abstractQueue.QueueThreadState;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.utils.Debug;
/**
 *
 * @author Administrator
 */
public class TesterThread implements Runnable, AbstractQueueThread
{
	private TesterQueue queue;
    private Thread thread;
    private QueueThreadState threadState;
    private Scan scan;
    private String currentModule; 
    
    public String getCurrentModule()
    {
    	return currentModule;
    }

	public QueueThreadState getThreadState()
    {
    	return threadState;
    }

	/** Creates a new instance of TesterThread */
    public TesterThread(int ID, TesterQueue queue, QueueThreadGroup threadGroup, Scan scan) 
    {
    	threadState = QueueThreadState.CREATING;
        this.queue = queue;
    	thread = new Thread(threadGroup, this, "TesterThread-" + ID);
    	this.scan = scan;
    }
    
	public void start()
	{
		thread.start();
	}
            
    private TestJob getNextTest()
    {
        TestJob test = null;
        while (test == null && (scan.isRunning() || queue.isRunning()))
        {
        	if (queue.isRunning())
        	{
        		test = queue.getNext();
        	}
            if (test == null && (scan.isRunning() || queue.isRunning()))
            {
                try
                {
                	synchronized(thread)
                	{
                		thread.wait(250);
                	}
                }
                catch (InterruptedException e)
                {
                }
            }
        }
        return test;
    }

    public void run()
    {
        TestJob test;
        while (scan.isRunning() || queue.isRunning())
        {
        	threadState = QueueThreadState.POLLING;
            test = getNextTest();
            if (test != null && queue.isRunning() && scan.isRunning())
            {
            	threadState = QueueThreadState.PROCESSING;
            	currentModule = test.getModule().getName() + " (" + test.getModuleNumber() + ")";
            	Date startTime = new Date();
            	try
            	{
            		test.runTest();
            	}
            	catch (Exception e)
            	{
            		Debug.errDebug("Exception caught when trying to run test module \"" + test.getModule().getName() + 
            				"\" (module number " + test.getModuleNumber() + "): ", e);
            	}
            	finally
            	{
                	scan.getTesterQueue().removeJob(test);
            	}
            	currentModule = "";
            	Date stopTime = new Date();
            	queue.addTime(test, stopTime.getTime() - startTime.getTime());
            }
        }
    	threadState = QueueThreadState.TERMINATING;
    }
    
	/**
	 * This will properly interrupt the thread and tell it to process no other tasks. 
	 * It does not currently have the ability to abort a task already started. 
	 */
    public void interupt()
    {
		thread.interrupt();
    }

	public final String getName()
    {
	    return thread.getName();
    }
}
