/*
 * TestJob.java
 *
 * Created on September 15, 2007, 10:07 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.grendelscan.tests.testJobs;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import com.grendelscan.scan.Scan;
import com.grendelscan.tests.queue.TesterQueue;
import com.grendelscan.tests.testModules.TestModule;
/**
 *
 * @author David Byrne
 */
public abstract class TestJob implements Serializable 
{
    protected abstract void internalRunTest();
    private static Integer lastID = 0;
    protected transient Scan scan;
    private final int id;
    protected transient TesterQueue queue;
    protected Set<Integer> dependencies;
    protected TestJobStatus status = TestJobStatus.NOT_STARTED;
    protected int moduleNumber;
    
    
    public TestJob(Scan scan, int moduleNumber)
    {
    	this.moduleNumber = moduleNumber;
    	synchronized(lastID)
    	{
    		id = ++lastID;
    	}
    	this.scan = scan;
    	queue = scan.getTesterQueue();
    	dependencies = new HashSet<Integer>();
    }
   
    public TestModule getModule()
    {
    	return scan.getScanSettings().getMasterTestModuleCollection().getTestModule(moduleNumber);
    }
    
    public void setStatus(TestJobStatus status)
    {
    	this.status = status;
    }
    
    public void runTest()
    {
    	status = TestJobStatus.RUNNING;
    	internalRunTest();
    	status = TestJobStatus.COMPLETE;
    }
    
    
    public boolean noRemainingDependencies()
    {
    	boolean noMoreDependencies = true;
    	for (int dependencyID: dependencies)
    	{
    		if (queue.contains(dependencyID))
    		{
    			noMoreDependencies = false;
    			break;
    		}
    	}

    	return noMoreDependencies;
    }
    
    public void addDependency(TestJob dependency)
    {
    	dependencies.add(dependency.getId());
    }
    
	public TestJobStatus getStatus()
    {
    	return status;
    }
	
	public int getId()
    {
    	return id;
    }

	public int getModuleNumber()
    {
    	return moduleNumber;
    }

	public void setScan(Scan scan)
    {
    	this.scan = scan;
    }

	public void setQueue(TesterQueue queue)
    {
    	this.queue = queue;
    }

}
