package com.grendelscan.tests.testModules;


import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.w3c.dom.html2.HTMLElement;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.requester.http.factories.*;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.MultiSelectOptionGroup;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testTypes.ByHtmlElementTest;
import com.grendelscan.utils.CollectionUtils;
import com.grendelscan.utils.*;
import com.grendelscan.utils.URIStringUtils;

public class Module0002 extends TestModule implements ByHtmlElementTest
{
	private Set<String> tagNames;
	private List<ConfigurationOption> configOptions;
	private SelectableOption allDirectoriesOption;
//	private Map<String, SelectableOption> tagOptions;
	private MultiSelectOptionGroup tagOptionsGroup;

    /** Creates a new instance of Module00001 */
    public Module0002() 
    {
//    	tagOptions = new HashMap<String, SelectableOption>(1);
		configOptions = new ArrayList<ConfigurationOption>();
		
		allDirectoriesOption = new SelectableOption("Request all directories", true, 
				"This will request the directory of all the URLS discovered, but the full URL will only be requested if the tag is selected below.");
		configOptions.add(allDirectoriesOption);
		
		SelectableOption tagOption;
		tagOptionsGroup = new MultiSelectOptionGroup("Tag selection", "Select which tags this module should search in.");
		configOptions.add(tagOptionsGroup);
		
		tagOption = new SelectableOption("A", true, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("APPLET", false, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("AREA", true, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("BASE", true, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("EMBED", false, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("FRAME", true, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("IFRAME", true, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("LINK", false, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("SCRIPT", true, "");
		tagOptionsGroup.addOption(tagOption);

		tagOption = new SelectableOption("IMG", false, "");
		tagOptionsGroup.addOption(tagOption);
   }
    
	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }
	
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SPIDER;
	}

	@Override
	public String getDescription()
	{
		return "Can request the SRC or HREF attribute for the A, APPLET, " +
				"AREA, BASE, EMBED, FRAME, IFRAME, IMG, LINK, and SCRIPT " +
				"tags. This module carries some risk, especially with " +
				"the A tag. For example, if there is a link that deletes " +
				"content, this module won't know that it's dangerous. " +
				"The risk can be mitigated by using the URL blacklist " +
				"feature. Note that the HTTP standard states that GET " +
				"should only be used for requests that will not modify " +
				"data. That doesn't mean that all applications comply. " +
				"If you aren't familiar with the application, you should " +
				"probably disable the A tag.";
	}

	public String[] getHtmlElements()
	{
		String tags[];
		if (allDirectoriesOption.isSelected())
		{
			tags = new String[] { "APPLET", "EMBED", "FRAME", "IFRAME", "SCRIPT", "IMG",
					"A", "AREA", "BASE", "LINK"};
		}
		else 
		{
			initTagNames();
			tags = CollectionUtils.toStringArray(tagNames);
		}
		return tags;
	}
	
	private void initTagNames()
	{
		if (tagNames == null)
		{
			tagNames = new HashSet<String>(1);
			for(SelectableOption option: tagOptionsGroup.getAllOptions())
			{
				if (option.isSelected())
				{
					tagNames.add(option.getName());
				}
			}
		}
	}

	@Override
	public String getName()
	{
		return "HTML tag requester";
	}

	@Override
	public void initialize()
	{
		initTagNames();
	}


			
	public void testByHtmlElement(int transactionID, HTMLElement element, String elementType)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	URI uri = getUri(element);
    	if (uri != null)
    	{
 			try
			{
	    		if (tagNames.contains(element.getTagName().toUpperCase()))
	    		{
					HttpGetTransaction request = new HttpGetTransaction(scan, uri, transaction.getId(), TransactionSource.SPIDER);
					scan.getRequesterQueue().addSpiderRequest(request, false, "Module 2");
	    		}
	    		/*
	    		 * We don't need to request the dir if the URI was requested; the 
	    		 * byDir categorizer will take care of it for us
	    		 */
	    		else if (allDirectoriesOption.isSelected() && scan.isRunning())
	    		{
					HttpGetTransaction request = new HttpGetTransaction(scan, URIStringUtils.getDirectoryUri(uri.toASCIIString()), transaction.getId(), TransactionSource.SPIDER);
					scan.getRequesterQueue().addSpiderRequest(request, false, "Module 2 - Directory only");
	    		}
			}
			catch (URISyntaxException e)
			{
				Debug.debug("Problem with a URI (" + uri.toASCIIString() + ") found in module00002: " + e.toString());
			}
    	}
	}
	
//	String[] r = { "A", "AREA", "BASE", "LINK" };
	//
//		@Override
//		public String[] getHtmlElements()
//		{
//			String tags[];
//			if (allDirectoriesOption.isSelected())
//			{
//				tags = new String[] { "APPLET", "EMBED", "FRAME", "IFRAME", "SCRIPT", "IMG" };

	private URI getUri(HTMLElement element)
	{
		String tag = element.getTagName().toUpperCase();
		String rawUri;
		if (tag.equals("APPLET") || tag.equals("EMBED") || tag.equals("FRAME") || 
				tag.equals("IFRAME") || tag.equals("SCRIPT") || tag.equals("IMG"))
		{
			rawUri = element.getAttribute("src");
		}
		else // Assume that it's an HREF-type tag
		{
			rawUri = element.getAttribute("href");
		}
		URI uri = null;
		if (URIStringUtils.isUsableUri(rawUri) || false)
		{
			try
	        {
		        uri = UriFactory.makeAbsoluteUri(rawUri, element.getBaseURI());
	        }
	        catch (URISyntaxException e)
	        {
	        	// Don't really care
	        }
		}
		return uri;
	}

	@Override
    public int getModuleNumber()
    {
	    return 2;
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
