package com.grendelscan.tests.testModules;


import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.w3c.dom.NodeList;
import org.w3c.dom.html2.HTMLElement;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.factories.HttpTransactionFactory;
import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.AbstractStandardHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testTypes.ByHtmlElementTest;
import com.grendelscan.utils.Debug;

public class Module0004 extends TestModule implements ByHtmlElementTest
{
	private String defaultText;
	private List<ConfigurationOption> configOptions;
	private SelectableOption getMethodOption;
	private SelectableOption postMethodOption;

	
	public Module0004()
	{
		configOptions = new ArrayList<ConfigurationOption>(2);
		getMethodOption = new SelectableOption("GET method", true, "Requests forms that use the GET method");
		postMethodOption = new SelectableOption("POST method", false, "Requests forms that use the POST method");
		configOptions.add(getMethodOption);
		configOptions.add(postMethodOption);
		
	}
	
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SPIDER;
	}

	
	public static String formResponsePrefix = "formResponse";

	@Override
	public String getDescription()
	{
		return "Fills out a form with guessed data to continue spidering. The data " +
				"is based on the input element type and the name of the query " +
				"parameter. The values are defined in conf/default_form_values.conf. " +
				"Enable this module ONLY if you are familiar with the application. " +
				"For example, if there is a form that deletes content, this module " +
				"won't know that it's dangerous. The risk can be mitigated by " +
				"using the URL blacklist feature, and by only using the GET method.";
	}

	public String[] getHtmlElements()
	{
		String[] r = { "FORM" };
		return r;
	}

	@Override
	public int getModuleNumber()
	{
		return 4;
	}

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	@Override
	public String getName()
	{
		return "Form baseline";
	}

	@Override
	public void initialize()
	{
		defaultText = ConfigurationManager.getString("default_form_values.default_text");
	}

	public void testByHtmlElement(int transactionID, HTMLElement element, String elementType)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		AbstractHttpTransaction request = buildRequest(transaction, element);
		if (request != null && scan.isRunning())
		{
			scan.getRequesterQueue().addSpiderRequest(request, false, "Module 4");
		}
	}

	private void buildQuery(AbstractStandardHttpTransaction request, AbstractHttpTransaction originalTransaction, HTMLElement element)
	{
		HashSet<String> radioNodes = new HashSet<String>();
		NodeList inputs = element.getElementsByTagName("INPUT");
		for (int index = 0; index < inputs.getLength(); index++)
		{
			Boolean useParameter = false;
			HTMLElement input = (HTMLElement) inputs.item(index);
			
			// Begin initialize attributes
			String name = input.getAttribute("name");
			if ((name == null) || name.equals(""))
			{
				// If there is no name, it can't be used in a submission, so skip it
				continue;
			}

			int length;
			String lengthStr = input.getAttribute("length");
			try
			{
				length = Integer.valueOf(lengthStr);
			}
			catch (Exception e)
			{
				length = 0;
			}

			String type = input.getAttribute("type");
			if (type == null || type.equals(""))
			{
				type = "text";
			}
			String value = input.getAttribute("value");
			
			// End initialize attributes
			
			// Handle based on input type
			if (type.equalsIgnoreCase("radio") && !radioNodes.contains(name))
			{
				// If this is the first radio button of the group, pick it
				radioNodes.add(name);
				useParameter = true;
			}
			// if the value wasn't already found in the tag, and the parameter isn't ignored
			else if (((value == null) || value.equals("")) && !scan.isQueryParameterIgnored(name))
			{
				if (type.equalsIgnoreCase("text") || type.equalsIgnoreCase("textarea"))
				{
					
					value = ConfigurationManager.getString("default_form_values." + name, name + " " + defaultText);
					if (length > 0)
					{
						// We're truncating because this is only for spidering, not for testing
						value = value.substring(0, length);
					}
					useParameter = true;
				}

				else if (type.equalsIgnoreCase("password"))
				{
					value = ConfigurationManager.getString("default_form_values.password", name + "password");
					useParameter = true;
				}

				else if (type.equalsIgnoreCase("checkbox"))
				{
					if (Math.random() < 0.5)
					{
						value = "on";
					}
					useParameter = true;
				}
				
				else if (type.equalsIgnoreCase("hidden"))
				{
					// The server might be expecting a blank hidden field
					useParameter=true;
				}
			} 
			else 
			{
				// If there is already a value present, use it
				useParameter = true;
			}
			if (useParameter)
			{
				request.addQueryParameter(name, value);
			}
		}

	}

	private AbstractHttpTransaction buildRequest(AbstractHttpTransaction originalTransaction, HTMLElement element)
	{
		String action;
		String method = element.getAttribute("METHOD");
		if (method != null && method.equalsIgnoreCase("POST"))
		{
			if (!postMethodOption.isSelected())
			{
				return null;
			}
			method = "POST";
		}
		else
		{
			if (!getMethodOption.isSelected())
			{
				return null;
			}
			method = "GET";
		}
		
		action = element.getAttribute("ACTION");
		if ((action == null) || action.equals(""))
		{
			action = originalTransaction.getUriString();
		}
		

		AbstractStandardHttpTransaction request = null;
		try
		{
			URI actionURI = UriFactory.makeAbsoluteUri(action, originalTransaction.getUriString()); 
			request = HttpTransactionFactory.createTransaction(scan, method, actionURI, originalTransaction.getId(), TransactionSource.SPIDER);
			buildQuery(request, originalTransaction, element);
		}
		catch (URISyntaxException e) 
		{
			Debug.errDebug("URI syntax problem in Module0004 (" + action + "): " + e.toString());
		}

		return request;
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
