package com.grendelscan.tests.testModules;


import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.requester.queue.RequesterQueue;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testModules.settings.SingleSelectOptionGroup;
import com.grendelscan.tests.testTypes.ByDirectoryTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.HttpUtils;

public class Module0005 extends TestModule implements ByDirectoryTest
{
	private List<String> directoryNames;
	private List<String> discoveredNames;
	private RequesterQueue requesterQueue;
	private List<ConfigurationOption> configOptions;

	private SelectableOption smallNameOption, mediumNameOption, largeNameOption, extraLargeNameOption;

	public Module0005()
    {
		SingleSelectOptionGroup nameOptionsGroup = new SingleSelectOptionGroup("Name list size", "Select the name list size to use.");
		configOptions = new ArrayList<ConfigurationOption>();
		configOptions.add(nameOptionsGroup);

		smallNameOption = new SelectableOption("Small (100 directory names)", false, "");
		nameOptionsGroup.addOption(smallNameOption);
		mediumNameOption = new SelectableOption("Medium (300 directory names)", true, "");
		nameOptionsGroup.addOption(mediumNameOption);
		largeNameOption = new SelectableOption("Large (500 directory names)", false, "");
		nameOptionsGroup.addOption(largeNameOption);
		extraLargeNameOption = new SelectableOption("Extra large (819 directory names)", false, "");
		nameOptionsGroup.addOption(extraLargeNameOption);

    }
	
	@Override
    public int[] getPrerequisites()
    {
		int i[] = {19};
	    return i;
    }


	@Override
	public String getDescription()
	{
		return "Looks for common directory names in all directories and sub-directories " +
				"observed during the scan. This can take a while, so only enable " +
				"this module if you're willing to wait";
	}

	@Override
	public int getModuleNumber()
	{
		return 5;
	}

	@Override
	public String getName()
	{
		return "Directory enumerator";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.FILE_ENUMERATION;
	}

	@SuppressWarnings("unchecked")
    @Override
	public void initialize()
	{
		discoveredNames = new ArrayList(1);
		directoryNames = ConfigurationManager.getList("file_enumeration.small_directory_list"); 
		if (mediumNameOption.isSelected() || largeNameOption.isSelected() || extraLargeNameOption.isSelected())
		{
			directoryNames.addAll(ConfigurationManager.getList("file_enumeration.medium_directory_list")); 
		}
		if (largeNameOption.isSelected() || extraLargeNameOption.isSelected())
		{
			directoryNames.addAll(ConfigurationManager.getList("file_enumeration.large_directory_list")); 
		}
		if (extraLargeNameOption.isSelected())
		{
			directoryNames.addAll(ConfigurationManager.getList("file_enumeration.extra_large_directory_list")); 
		}
		requesterQueue = scan.getRequesterQueue(); 

	}

	public void testByDirectory(int transactionID, String directory)
	{
    	AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactionID);
		for(String newDirectory: directoryNames)
		{
			if (!scan.isRunning())
			{
				break;
			}
			String uri = originalTransaction.getHostUriWithoutTrailingSlash() + directory + newDirectory + "/";
			try
			{
				HttpGetTransaction testTransaction = new HttpGetTransaction(scan, uri, originalTransaction.getId(), TransactionSource.ENUMERATION);
				requesterQueue.processRequestNow(testTransaction, "Module 5", false, true, false, true);
				if (HttpUtils.fileExists(testTransaction.getLogicalResponseCode())) 
				{
					scan.getCategorizerQueue().addTransaction(testTransaction);
					discoveredNames.add(testTransaction.getAbsoluteUriString());
				}
			}
			catch (URISyntaxException e)
			{
				Debug.errDebug("URI syntax problem in Module0005 (" + uri + "): " + e.toString());
			}
		}
	}

	private void report()
	{
		String list = "";
		for (String uri: discoveredNames)
		{
			list += HtmlUtils.makeLink(uri) + "<br>";
		}
		String uri = discoveredNames.get(0) + (discoveredNames.size() > 1 ? " and others" : "");
		ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.INFO, uri, "Directories discovered", 
				"Directories were discovered by guessing the name", 
				"The following directories were discovered by guessing the name: <br>" + list, 
				"If these directories were not linked to by the website, they could lead an attacker to unintended areas.", 
				"Review the contents of the directories and remove them if they aren't required.", "");
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	@Override
    public void scanIsComplete()
    {
		if (discoveredNames.size() > 0)
		{	
			report();
		}
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
