package com.grendelscan.tests.testModules;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.UnvalidatedHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.nikto.Nikto;
import com.grendelscan.tests.libraries.nikto.NiktoTest;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.MultiSelectOptionGroup;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testTypes.ByHostTest;
import com.grendelscan.utils.HtmlUtils;

public class Module0018 extends TestModule implements ByHostTest
{
	private HashMap<Character, SelectableOption> categoryOptions;
	private Set<Character> enabledCategories;
	private List<ConfigurationOption> configOptions;
	private Map<String, Set<String>> niktoMessages;
	private static int execCount = 0; 
	
	public Module0018()
	{
		/*
		0 - File Upload
		1 - Interesting File / Seen in logs
		2 - Misconfiguration / Default File
		3 - Information Disclosure
		4 - Injection (XSS/Script/HTML)
		5 - Remote File Retrieval - Inside Web Root
		6 - Denial of Service
		7 - Remote File Retrieval - Server Wide
		8 - Command Execution / Remote Shell
		9 - SQL Injection
		a - Authentication Bypass
		b - Software Identification
		g - Generic (Don't rely on banner)
		x - Reverse Tuning Options (i.e., include all except specified)
	*/	

		SelectableOption categoryOption;
		MultiSelectOptionGroup categoryOptionsGroup = new MultiSelectOptionGroup("Nikto Categories", "Select which categories of Nikto tests should be run.");
		categoryOptions = new HashMap<Character, SelectableOption>();
		
		categoryOption = new SelectableOption("File uploads", true, "Looks for conditions that could allow an arbitrary file to be uploaded to the server.");
		categoryOptions.put('0', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);
		
		categoryOption = new SelectableOption("Misc tests", true, "Looks for \"interesting files\" and sends requests that have been seen in web servers logs as part of unidentified attacks/recon.");
		categoryOptions.put('1', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);
		
		categoryOption = new SelectableOption("Misconfiguration", true, "Tests for misconfiguration of software, and searches for default files that may help with fingerprinting or attacks.");
		categoryOptions.put('2', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("Information disclosure", true, "Searches for information disclosure, such as internal/private IP addresses.");
		categoryOptions.put('3', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("XSS", true, "Tests for Cross Site Scripting (XSS), HTML injection, and similar attacks.");
		categoryOptions.put('4', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("File retrieval - web root", true, "Tests for remote file retrieval inside the web root.");
		categoryOptions.put('5', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("File retrieval - server wide", true, "Tests for remote file retrieval across the entire server.");
		categoryOptions.put('7', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("Denial of Service", true, "Tests for known Denial of Service vulnerabilities. It should not result in a disruption to service.");
		categoryOptions.put('6', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("Command injection", true, "Tests for command injection vulnerabilities that may result in a remote shell.");
		categoryOptions.put('8', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("SQL injection", true, "Tests for known SQL Injection vulnerabilities.");
		categoryOptions.put('9', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("Authentication bypass", true, "Tests for authentication bypass vulnerabilities.");
		categoryOptions.put('a', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("Software identification", true, "Attempts to identify what software is running on the targeted server.");
		categoryOptions.put('b', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);

		categoryOption = new SelectableOption("Generic", true, "Attempts to identify what software is running on the targeted server.");
		categoryOptions.put('g', categoryOption);
		categoryOptionsGroup.addOption(categoryOption);
	
		configOptions = new ArrayList<ConfigurationOption>();
		configOptions.add(categoryOptionsGroup);
	}
	@Override
	public String getDescription()
	{
		return "Runs the tests from the primary Nikto database (db_tests) against " +
				"all servers discovered (and allowed) in the scan. Note that some " +
				"Nikto tests are prone to false positives. This tool has enhanced " +
				"file-not-found detection (when \"automatic response code " +
				"overrides\" is enabled). That will help reduce some false " +
				"positives, but, as with all findings, Nikto results should be " +
				"manually confirmed.\n" +
				"\n" +
				Nikto.getNiktoCredits();
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.NIKTO;
	}

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }
	@Override
	public int getModuleNumber()
	{
		return 18;
	}

	@Override
	public String getName()
	{
		return "General Nikto Tests";
	}

	@Override
	public void initialize()
	{
		niktoMessages = new HashMap<String, Set<String>>();
		enabledCategories = new HashSet<Character>();
		for (char optionCode: categoryOptions.keySet())
		{
			if (categoryOptions.get(optionCode).isSelected())
			{
				enabledCategories.add(optionCode);
			}
		}
	}
	
	private boolean isTestEnabled(String categoryCodes)
	{
		boolean enabled = false;
		char chars[] = new char[categoryCodes.length()];
		categoryCodes.getChars(0, categoryCodes.length(), chars, 0);
		for (char c: chars)
		{
			if (enabledCategories.contains(c))
			{
				enabled = true;
				break;
			}
		}
		return enabled;
	}

	public void testByServer(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		Nikto.initializeIndexPHP(transaction);
		Set<NiktoTest> tests = Nikto.getNiktoTests();
		for (NiktoTest test: tests)
		{
			if (!scan.isRunning())
			{
				break;
			}
			
			if (isTestEnabled(test.getCategories()))
			{
				runTest(transaction, test);
			}
			
		}
		generateReport(transaction);
	}
	
	private void runTest(AbstractHttpTransaction transaction, NiktoTest test)
	{
		String host = transaction.getHostUriWithoutTrailingSlash();
		for (String testUri: test.constructTestUris(scan, transaction.getHostUriWithoutTrailingSlash()))
		{
			if (!scan.isRunning())
			{
				break;
			}
			UnvalidatedHttpTransaction testTransaction = new UnvalidatedHttpTransaction(scan, 0, test.getMethod(), 
					testUri, transaction.getHost(), transaction.getPort(), transaction.isSecure(), 
					TransactionSource.NIKTO, Nikto.getMajorHttpVersion(), Nikto.getMinorHttpVersion(), null);
			testTransaction.execute("Module " + this.getModuleNumber(), false, true);
			if (test.matches(testTransaction))
			{
				// We only need to analyze the transaction if it succeeds
				//scan.getCategorizerQueue().addTransaction(testTransaction);
				if (!niktoMessages.containsKey(host))
				{
					Set<String> messages = new HashSet<String>();
					niktoMessages.put(host, messages);
				}
				String message = "OSVDB-";
				if (test.getOsvdbNumber() > 0)
				{
					message += test.getOsvdbNumber();
				}
				else
				{
					message += "unknown";
				}
				message += ": " + testTransaction.getMethod() + " "  + testTransaction.getUriString() + " : " + test.getMessage();
				testTransaction.writeToDisk();
				message += "<br>The transaction used for testing can be viewed " + HtmlUtils.makeLink(testTransaction.getSavedUrl(), "here") + ".<br><br>";
				niktoMessages.get(host).add(message);
			}
		}
	}
	
	private void generateReport(AbstractHttpTransaction transaction)
	{
		String host = transaction.getHostUriWithoutTrailingSlash();
		if (niktoMessages.containsKey(host))
		{
			String briefDescription = "The Nikto database found some matches";
			String longDescription = "The Nikto database has found matches on this server.<br><br>";
			for (String message: niktoMessages.get(host))
			{
				longDescription += message + "<br>";
			}
			String impact = "The security impact varies between findings.";
			String recomendations = "Verify the accuracy of the findings, and investigate to determine " +
					"the best course of action. Usually, that involves installing software updates or " +
					"correcting a configuration mistake.";
			
			ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.MEDIUM, host, 
					"Nikto findings", briefDescription, longDescription, impact, recomendations, "");
			scan.getReportCollector().addEvent(event);
		}
	}
	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
