package com.grendelscan.tests.testModules;

import java.util.HashSet;
import java.util.Set;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.TokenTesting.TokenContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextType;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextTypeUtils;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.testTypes.ByOutputContextTest;
import com.grendelscan.utils.*;
public class Module0024 extends TestModule implements ByOutputContextTest
{
	private Set<TokenContext> contexts;
	
	@Override
	public String getDescription()
	{
		return "Each query parameter will be individually seeded with a token. This module will " +
				"report on what tokens are used in output, even if it is in a future request. " +
				"The tokens are the same as what is used for XSS and CRLF injection testing. " +
				"Parameters can be removed from scope by identifying it as a \"forbidden " +
				"parameter\" in the scan setup.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.ARCHITECTURE;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 24;
	}
	
	@Override
	public String getName()
	{
		return "Input / Output flows";
	}
	
	@Override
	public void initialize()
	{
		contexts = new HashSet<TokenContext>();
	}
	
	public TokenContextType[] getDesiredContexts()
	{
		return TokenContextTypeUtils.getAllContexts();
	}
	
	@Override
    public int[] getPrerequisites()
    {
	    return new int[] {20};
    }
	
	public void testByOutputContext(TokenContext context)
	{
		contexts.add(context);
	}

	@Override
    public void scanIsComplete()
    {
		if (contexts.size() == 0)
		{
			return;
		}
		String table = "<table>\n";
		table += "<tr style=\"font-weight: bold;\"><td>Input URL</td><td>Input parameter name</td><td></tr>\n" + 
			"<tr style=\"font-weight: bold;\"><td>Output URL</td><td>Output context</td></tr>\n" +
			"<tr style=\"font-weight: bold;\"><td>&nbsp;</td></tr>\n";
		for (TokenContext context: contexts)
		{
			AbstractHttpTransaction inputTransaction = TokenTesting.getTokenTest(context.getToken());
			AbstractHttpTransaction outputTransaction = scan.getTransactionRecord().getTransaction(context.getOutputTransactionID()); 
			
			inputTransaction.writeToDisk();
			outputTransaction.writeToDisk();
			
			String outputText;
			if (inputTransaction.getId() == outputTransaction.getId())
			{
				outputText = "Same as input";
			}
			else
			{
				outputText = HtmlUtils.makeLink(outputTransaction.getSavedUrl(), outputTransaction.getBaseUri());
			}
			
			table += "<tr><td>" + 
				HtmlUtils.makeLink(inputTransaction.getSavedUrl(), inputTransaction.getBaseUri()) + "</td><td>" + 
				context.getOriginalInputParameterName() + "</td><td></tr>\n" + 
				"<tr><td>" + outputText + "</td><td>" + 
				context.getContextType().getDescription() + "</td></tr>\n" +
				"<tr><td>&nbsp;</td></tr>\n";
		}
		table += "</table>\n";
		String shortDescript = "Input / output data flows were documented";
		String longDescription = "Random tokens were submitted into all query parameters. The table " +
				"below lists cases were tokens were observed in output from the server. The links are " +
				"to the actual transaction.<br>\n<br>\n" + table;
		ReportableEvent event =
	        new ReportableEvent(null, getName(), EventSeverity.INFO, "Multiple",
	                "Input / output flows", shortDescript,
	                longDescription, "These data flows could be vulnerable to attacks such as XSS or CRLF injection", 
	                "Further testing may be desirable", "");
		scan.getReportCollector().addEvent(event);
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
