package com.grendelscan.tests.testModules;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.tests.testTypes.ByHttpResponseCodeTest;
import com.grendelscan.scan.reporting.*;
import java.io.*;
import java.util.*;
import java.lang.*;

import com.grendelscan.tests.testModules.settings.*;
import com.grendelscan.utils.*;
public class Module0029 extends TestModule implements ByHttpResponseCodeTest
{
	Set<String> urls;
	FileNameOption outputFileOption;
	SelectableOption recordQueriesOption;
	SelectableOption recordAllHTMLFilesOption;
	List<ConfigurationOption> configOptions;
	
	@Override
    public void scanIsComplete()
    {
		String urlsArray[] = urls.toArray(new String[0]);
		Arrays.sort(urlsArray);
		String reportString = "";
		for (String url: urlsArray)
		{
			reportString += url + "\n";
		}
		String filename;
		if (outputFileOption.getValue().contains(File.separator))
		{
			filename = outputFileOption.getValue();
		}
		else
		{
			filename = scan.getScanSettings().getOutputDirectory() + File.separator + outputFileOption.getValue();
		}
		if (!FileUtils.writeToFile(filename, reportString))
		{
			reportString = "Writing to the original file name failed\n\n" + reportString;
			FileUtils.writeToFile(scan.getScanSettings().getOutputDirectory() + File.separator + "web-site-map", reportString);
		}
		
		genReport();
    }
	
	private void genReport()
	{

		String title = "URL list";
		String shortDesc = "A list of requested URLs was generated";
		String longDesc = "A list of all requested URLs was generated and saved to " + HtmlUtils.makeLink(outputFileOption.getValue()) +
			". In total, " + urls.size() + " URLs were observed.";
		ReportableEvent event = new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.INFO, "", 
				title, shortDesc, longDesc, "", "", "");
		scan.getReportCollector().addEvent(event);
	}

	
	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	public Module0029()
	{
		configOptions = new ArrayList<ConfigurationOption>(3);
		outputFileOption = new FileNameOption("Output file name","web-site-map", "The output filename for the website map.", false);
		configOptions.add(outputFileOption);

		recordQueriesOption = new SelectableOption("Record URL queries", false, "If selected, all unique URL query strings will also be recorded.");
		configOptions.add(recordQueriesOption);

		recordAllHTMLFilesOption = new SelectableOption("Record all files", false, "If not selected, only the URLs to web files (HTML, XHTML, XML, JavaScript, CSS, and text) will be recorded. This will omit images, audio, vido, pdfs, etc");
		configOptions.add(recordAllHTMLFilesOption);
	
	}
	
	@Override
	public String getDescription()
	{
		return "Records a list of all URLs successfully requested. This does not include attack requests.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.ARCHITECTURE;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 29;
	}
	
	@Override
	public String getName()
	{
		return "Website map";
	}
	
	@Override
	public void initialize()
	{
		urls = new HashSet<String>(100);
	}
	
	public String[] getResponseCodes()
	{
	    return new String[] {"200", "301", "302", "303", "304", "307", "401", "403", "500", "504"};
	}
	
	public void testByHttpResponseCode(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	if (recordAllHTMLFilesOption.isSelected() || MimeUtils.isWebMimeType(transaction.getResponseMimeType()))
    	{
	    	if (recordQueriesOption.isSelected())
	    	{
	    		urls.add(transaction.getAbsoluteUriString());
	    	}
	    	else
	    	{
	    		urls.add(transaction.getBaseUri());
	    	}
    	}
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	 
}
