package com.grendelscan.tests.testModules;

import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testTypes.ByHostTest;
import com.grendelscan.requester.http.transactions.*;
import com.grendelscan.tests.libraries.TokenTesting.*;
import com.grendelscan.utils.*;
import com.grendelscan.requester.*;

import java.util.*;
public class Module0032 extends TestModule implements ByHostTest
{
	Set<Integer> vulnerableHosts;
	@Override
	public String getDescription()
	{
		return "Tests for cross-site tracing (XST), which is an attack that can steal cookies using the " +
				"TRACE or TRACK debug method.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.WEB_SERVER_CONFIGURATION;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 32;
	}
	
	@Override
	public String getName()
	{
		return "XST";
	}
	
	@Override
	public void initialize()
	{
		vulnerableHosts = new HashSet<Integer>(1);
	}
	
	public void testByServer(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		int id = 0;
    	if ((id = testMethod(transaction, "TRACE")) > 0 || (id = testMethod(transaction, "TRACK")) > 0)
		{
    		Debug.discoveryMessage("XST discovered on " + transaction.getHostUri());
			vulnerableHosts.add(id);
		}
	}
	
	private void report()
	{
		if (vulnerableHosts.size() > 0)
		{
			String title = "HTTP debug method enabled";
			String briefDescription = "The TRACE/TRACK method was enabled.";
			String longDescription = "The TRACE/TRACK method was enabled on the servers listed below:<br><br>";
			
			String firstHost = "";
			for (int id: vulnerableHosts)
			{
		    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(id);
		    	transaction.writeToDisk();
				firstHost = transaction.getHostUri();
				longDescription += "&nbsp;&nbsp;&nbsp;&nbsp;" + HtmlUtils.makeLink(transaction.getSavedUrl(), transaction.getHostUri()) + "<br>";
			}

			String impact = "An attack known as Cross Site Tracing (XST) leverages this method to steal cookies. " +
					"Once a session key in a cookie is obtained by an attacker, he can hijack a legitimate users session.";
			String recomendations = "Disable HTTP TRACE/TRACK on all web servers: <ul>" +
					"<li>IIS: http://www.microsoft.com/technet/prodtechnol/WindowsServer2003/Library/IIS/d779ee4e-5cd1-4159-b098-66c10c5a3314.mspx?mfr=true</li>" +
					"<li>Apache: http://httpd.apache.org/docs/2.0/mod/core.html#traceenable</li></ul>";
			String references = "http://www.owasp.org/index.php/Testing_for_HTTP_Methods_and_XST";
			
			ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.LOW, firstHost + (vulnerableHosts.size() > 1 ? " and others" : ""), 
					title, briefDescription, longDescription, impact, recomendations, references);
			scan.getReportCollector().addEvent(event);
		}
	}
	
	private int testMethod(AbstractHttpTransaction transaction, String method)
	{
		int id = 0;
		UnvalidatedHttpTransaction test = new UnvalidatedHttpTransaction(scan, 0, method, transaction.getHostUri(), transaction.getHost(), transaction.getPort(), false, TransactionSource.TEST, null);
		String tokenName = TokenTesting.generateToken();
		String tokenValue = TokenTesting.generateToken();
		
		test.addRequestHeader(tokenName, tokenValue);
		test.execute("Module " + getModuleNumber());
		if (test.getResponseBodyAsString().contains(tokenName + ": " + tokenValue))
		{
			id = test.getId();
			test.writeToDisk();
		}
		return id;
	}

	@Override
    public void scanIsComplete()
    {
		report();
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
