package com.grendelscan.tests.testModules;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.platformErrorMessages.PlatformErrorMessages;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testTypes.AllTransactionsTest;
import java.util.*;
import com.grendelscan.utils.*;
 
public class Module0036 extends TestModule implements AllTransactionsTest
{
	private List<ConfigurationOption> configOptions;

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	@Override
	public String getDescription()
	{
		return "Searches all responses for platform error messages (.Net, PHP, Cold Fusion, MySQL, Oracle, etc). " +
				"Only the first error message in each response will be reported, so you may want to manually " +
				"review the entire response.\n" +
				"\n" +
				"An optional list of regular expressions that define platform error messages can be provided. " +
				"The expression is matched against any part of the response, so \"error\" will work (you don't " +
				"need \".*error.*\"). The patterns are case insensitive and will be tested against both the raw response body, and the " +
				"combined text elements of the HTML. This option is useful if you know that there are unusual " +
				"messages in the targeted application. This setting is shared with the \"Platform error " +
				"messages\" module.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.INFORMATION_LEAKAGE;
	}
	
	public Module0036()
    {
		configOptions = new ArrayList<ConfigurationOption>();
		configOptions.add(PlatformErrorMessages.getRegexErrorPatternsOption());
    }

	@Override
	public int getModuleNumber()
	{
		return 36;
	}
	
	@Override
	public String getName()
	{
		return "Platform error messages";
	}
	
	@Override
	public void initialize()
	{
	}
	
	public void testAllTransactions(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);

		String matchPattern = PlatformErrorMessages.isErrorMatch(transaction, true);
		if (!matchPattern.equals(""))
		{
			transaction.writeToDisk();
		}
	}

	@Override
    public void scanIsComplete()
    {
		if (PlatformErrorMessages.getMessageHits().size() > 0)
		{
			String title = "Platform error messages found";
			String shortDesc = "Some platform error messages were found in a response";
			String longDesc = "Some platform error messages were found in a response. The transactions" +
					" are listed in the table below:<br><br>" +
					"\n<table><tr style=\"font-weight: bold\"><td>Transaction</td><td>Match pattern</td></tr>\n";
			for(int transactionID: PlatformErrorMessages.getMessageHits().keySet())
			{
				AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
				longDesc += "<tr><td>" + HtmlUtils.makeLink(transaction.getSavedUrl(), String.valueOf(transactionID)) + "</td><td>" +
				HtmlUtils.escapeHTML(PlatformErrorMessages.getMessageHits().get(transactionID)) + "</td></tr>\n";
			}
			longDesc += "</table>\n";
			
			String impact = "The impact will vary depending on the nature of the error.";
			String recomendations = "Each result should be individually investigated. The error message " +
					"may provide a hint about the internal structure of the application, or attacks that " +
					"can be launched.";
			ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.INFO, 
					"Multiple", title, shortDesc, longDesc, impact, recomendations, "");
			scan.getReportCollector().addEvent(event);
		}
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	
}
