package com.grendelscan.utils;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.grendelscan.GUI.ScanStatusWindow;
import com.grendelscan.scan.*;
import com.grendelscan.*;

public class Debug
{
	static private ScanStatusWindow scanStatusWindow = null;
	static private OutputStream errorOutfile;
	static private PrintStream errorPrintStream;
	public synchronized static void initializeLibrary(Scan scan)
	{
		try
		{
			String errorFilename = scan.getScanSettings().getOutputDirectory() + File.separator + ConfigurationManager.getString("logging.error_log_file");
			errorOutfile = new FileOutputStream(errorFilename);
			errorPrintStream = new PrintStream(errorOutfile);
			String header = GrendelScan.versionText + "\n\n";
			header += "Java vendor: " + System.getProperty("java.vm.vendor") + "\n";
			header += "Java VM: " + System.getProperty("java.vm.name") + " " + System.getProperty("java.runtime.version") + "\n";
			header += "OS: " + System.getProperty("os.name") + " " + System.getProperty("os.version") + "\n";
            errorOutfile.write(header.getBytes());
            errorOutfile.flush();
		}
		catch (Exception e)
		{
			Debug.errDebug("Problem opening file for writing errorLog: " + e.toString(), e);
		}
		
	}

	public synchronized static void debug(String message)
	{
		if (scanStatusWindow != null)
		{
			scanStatusWindow.appendStatusText(message + "\n");
		}
	}

	public static void discoveryMessage(String message)
	{
		Debug.debug("***** " + message);
	}
	
	public synchronized static void errDebug(String message, Throwable e)
	{
		System.err.println(message);
		if (scanStatusWindow != null)
		{
			scanStatusWindow.appendStatusText(message + "\n");
			if (errorOutfile != null)
			{
				try
                {
					Date date = new Date();
					SimpleDateFormat s = new SimpleDateFormat("HH:mm:ss, MM/dd/yyy");
	                errorOutfile.write(("\n\n" + s.format(date) + "\n" + message + "\n").getBytes());
	                errorOutfile.flush();
                }
                catch (IOException e2)
                {
                	System.err.println("Error writing to error log: " + e2.toString());
                	e2.printStackTrace();
                }
			}
		}
		if (e != null)
		{
			e.printStackTrace();
			if (errorPrintStream != null)
			{
				e.printStackTrace(errorPrintStream);
			}
		}
	}

	public synchronized static void errDebug(String message)
	{
		errDebug(message, null);
	}

	public static void setScanStatusWindow(ScanStatusWindow scanStatusWindow)
    {
    	Debug.scanStatusWindow = scanStatusWindow;
    }
}
