package com.grendelscan.utils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MimeUtils
{

	/**
     * Anything that could be remotely web-related. HTML, XHTML, XML,
     * CSS, text, javascript, vbscript, and json
     * @param mimeType
     * @return
     */
    public static boolean isWebMimeType(String mimeType)
    {
    	boolean html = false;
    	String m = mimeType.toLowerCase();
    	if (
    			m.equals("text/html") 
    			|| m.equals("application/xhtml+xml") 
    			|| m.equals("application/xml") 
    			|| m.equals("text/xml") 
    			|| m.equals("application/javascript") 
    			|| m.equals("application/json") 
    			|| m.equals("application/x-javascript") 
    			|| m.equals("text/ecmascript")
    			|| m.equals("text/javascript") 
    			|| m.equals("application/vbscript") 
    			|| m.equals("text/vbscript")
    			|| m.equals("text/css") 
    			|| m.equals("text/plain") 
    			)
    	{
    		html = true;
    	}
    	
    	return html;
    }

    /**
     * Also returns true for XML types, since XHTML can be sent as an XML MIME type
     * @param mimeType
     * @return
     */
	public static boolean isHtmlMimeType(String mimeType)
    {
    	boolean html = false;
    	String m = mimeType.toLowerCase();
    	if (m.equals("text/html") 
    			|| m.equals("application/xhtml+xml") 
    			|| m.equals("application/xml")
    			|| m.equals("text/ecmascript")
    			|| m.equals("text/xml") 
    			)
    	{
    		html = true;
    	}
    	
    	return html;
    }

	
	public static boolean isJavaScriptMimeType(String mimeType)
    {
    	boolean javaScript = false;
    	String m = mimeType.toLowerCase();
    	if (m.equals("application/javascript") 
    			|| m.equals("application/json") 
    			|| m.equals("application/x-javascript")
    			|| m.equals("text/ecmascript")
    			|| m.equals("text/javascript") 
    			)
    	{
    		javaScript = true;
    	}
    	
    	return javaScript;
    }

	
	public static boolean isImageMimeType(String mimeType)
	{
	   	boolean image = false;
    	String m = mimeType.toLowerCase();
    	if (m.startsWith("image/"))
    	{
    		image = true;
    	}
    	
    	return image;
	}
	

	public static boolean isTextMimeType(String mimeType)
	{
	   	boolean text = false;
    	String m = mimeType.toLowerCase();
    	if (m.startsWith("text/")
    		|| m.startsWith("message/")
    		|| m.startsWith("multipart/")
    		|| isWebMimeType(m)
    		)
    	{
    		text = true;
    	}
    	
    	return text;
	}
	

	/**
     * Also returns XML types, since XHTML can be sent as an XML MIME type
     * @param mimeType
     * @return {"text/html", "application/xhtml+xml", "application/xml", "text/ecmascript", "text/xml"}
     */
	public static String[] getHtmlMimeTypes()
	{
		return new String[] {"text/html", "application/xhtml+xml", "application/xml", "text/ecmascript", "text/xml"};
	}
	
	/**
	 * 
	 * @return {"application/javascript", "application/json", "application/x-javascript", "text/ecmascript", "text/javascript"}
	 */
	public static String[] getJavaScriptMimeTypes()
	{
		return new String[] {"application/javascript", "application/json", "application/x-javascript", "text/ecmascript", "text/javascript"};
	}
	
	public static String getFileExtension(String mimeType)
	{
		mimeType = mimeType.toLowerCase();
		String extension  = "";
		if (mimeType.equals("text/html"))
		{
			extension = ".html";
		}
		else if (mimeType.equals("text/html"))
		{
			extension = ".html";
		}
		else if (mimeType.equals("application/xhtml+xml"))
		{
			extension = ".xhtml";
		}
		else if (mimeType.equals("application/xml") || mimeType.equals("text/xml"))
		{
			extension = ".xml";
		}
		else if (mimeType.equals("text/css"))
		{
			extension = ".css";
		}
		else if (mimeType.equals("text/plain"))
		{
			extension = ".txt";
		}
		else if (mimeType.equals("application/javascript") 
				|| mimeType.equals("application/x-javascript") 
				|| mimeType.equals("text/javascript") 
				|| mimeType.equals("text/ecmascript") 
				|| mimeType.equals("application/json"))
		{
			extension = ".js";
		}
		else if (mimeType.equals("application/vbscript"))
		{
			extension = ".vbs";
		}
		else if (mimeType.equals("audio/wav") || mimeType.equals("audio/x-wav"))
		{
			extension = ".wav";
		}
		else if (mimeType.equals("image/jpeg") || mimeType.equals("image/pjpeg"))
		{
			extension = ".jpg";
		}
		else if (mimeType.equals("image/gif"))
		{
			extension = ".gif";
		}
		else if (mimeType.equals("image/png"))
		{
			extension = ".png";
		}
		else if (mimeType.equals("audio/mpeg") || mimeType.equals("video/mpeg") || mimeType.equals("video/x-mpeg"))
		{
			extension = ".mpg";
		}
		else if (mimeType.equals("audio/mpeg3") || mimeType.equals("audio/x-mpeg-3") || mimeType.equals("audio/mpeg3"))
		{
			extension = ".mp3";
		}
		else if (mimeType.equals("application/pdf"))
		{
			extension = ".pdf";
		}
		else if (mimeType.equals("video/quicktime"))
		{
			extension = ".mov";
		}
		else if (mimeType.equals("application/x-troff-msvideo") 
				|| mimeType.equals("video/avi") 
				|| mimeType.equals("video/msvideo") 
				|| mimeType.equals("video/x-msvideo"))
		{
			extension = ".avi";
		}
		else if (mimeType.equals("image/bmp") || mimeType.equals("image/x-windows-bmp"))
		{
			extension = ".bmp";
		}
		return extension;
		
	}
	
	public static String getMimeFromContentType(String contentType)
	{
		String mimeType = "";
		if (contentType != null)
		{
			Pattern mimePattern = Pattern.compile("^([^;]+)");
			Matcher matcher = mimePattern.matcher(contentType);
			if (matcher.find())
			{
				mimeType = matcher.group();
			}
		}
		return mimeType;

	}
	
}
