package com.grendelscan.utils;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class StringUtils
{
	public static final int FORMAT_ALL_ASCII = 3;
	public static final int FORMAT_ALL_CASE_ALPHA = 8;
	public static final int FORMAT_ALL_CASE_ALPHANUMERIC = 9;
	public static final int FORMAT_BASE64 = 2;
	public static final int FORMAT_DECIMAL = 4;
	public static final int FORMAT_HEXADECIMAL = 0;
	public static final int FORMAT_HTML_DECIMAL_ENCODED = 12;
	public static final int FORMAT_HTML_HEX_ENCODED = 11;
	public static final int FORMAT_LOW_ASCII = 1;
	public static final int FORMAT_LOWER_CASE_ALPHA = 7;
	public static final int FORMAT_PLAIN_TEXT = 5;
	public static final int FORMAT_UPPER_CASE_ALPHA = 6;
	public static final int FORMAT_URL_ENCODED = 10;
	public static final int FORMAT_DECIMAL_COMMA_SEPERATED = 13;
	public static final int FORMAT_HTML_DOUBLE_PADDED_HEX_ENCODED = 14;
	private static Random random = new Random();
	private static Map<String, Character> htmlCodes = new HashMap<String, Character>();
	
	
	
	static
	{
		htmlCodes.put("amp", new Character('&'));
		htmlCodes.put("lt", new Character('<'));
		htmlCodes.put("gt", new Character('>'));
		htmlCodes.put("quot", new Character('"'));
		htmlCodes.put("nbsp", new Character((char) 160));

		htmlCodes.put("lsquo", new Character('`'));
		htmlCodes.put("rsquo", new Character(''));

		htmlCodes.put("frasl", new Character((char) 47));
		htmlCodes.put("ndash", new Character((char) 8211));
		htmlCodes.put("mdash", new Character((char) 8212));
		htmlCodes.put("iexcl", new Character((char) 161));
		htmlCodes.put("cent", new Character((char) 162));
		htmlCodes.put("pound", new Character((char) 163));
		htmlCodes.put("curren", new Character((char) 164));
		htmlCodes.put("yen", new Character((char) 165));
		htmlCodes.put("brvbar", new Character((char) 166));
		htmlCodes.put("brkbar", new Character((char) 166));
		htmlCodes.put("sect", new Character((char) 167));
		htmlCodes.put("uml", new Character((char) 168));
		htmlCodes.put("die", new Character((char) 168));
		htmlCodes.put("copy", new Character((char) 169));
		htmlCodes.put("ordf", new Character((char) 170));
		htmlCodes.put("laquo", new Character((char) 171));
		htmlCodes.put("not", new Character((char) 172));
		htmlCodes.put("shy", new Character((char) 173));
		htmlCodes.put("reg", new Character((char) 174));
		htmlCodes.put("macr", new Character((char) 175));
		htmlCodes.put("hibar", new Character((char) 175));
		htmlCodes.put("deg", new Character((char) 176));
		htmlCodes.put("plusmn", new Character((char) 177));
		htmlCodes.put("sup2", new Character((char) 178));
		htmlCodes.put("sup3", new Character((char) 179));
		htmlCodes.put("acute", new Character((char) 180));
		htmlCodes.put("micro", new Character((char) 181));
		htmlCodes.put("para", new Character((char) 182));
		htmlCodes.put("middot", new Character((char) 183));
		htmlCodes.put("cedil", new Character((char) 184));
		htmlCodes.put("sup1", new Character((char) 185));
		htmlCodes.put("ordm", new Character((char) 186));
		htmlCodes.put("raquo", new Character((char) 187));
		htmlCodes.put("frac14", new Character((char) 188));
		htmlCodes.put("frac12", new Character((char) 189));
		htmlCodes.put("frac34", new Character((char) 190));
		htmlCodes.put("iquest", new Character((char) 191));
		htmlCodes.put("Agrave", new Character((char) 192));
		htmlCodes.put("Aacute", new Character((char) 193));
		htmlCodes.put("Acirc", new Character((char) 194));
		htmlCodes.put("Atilde", new Character((char) 195));
		htmlCodes.put("Auml", new Character((char) 196));
		htmlCodes.put("Aring", new Character((char) 197));
		htmlCodes.put("AElig", new Character((char) 198));
		htmlCodes.put("Ccedil", new Character((char) 199));
		htmlCodes.put("Egrave", new Character((char) 200));
		htmlCodes.put("Eacute", new Character((char) 201));
		htmlCodes.put("Ecirc", new Character((char) 202));
		htmlCodes.put("Euml", new Character((char) 203));
		htmlCodes.put("Igrave", new Character((char) 204));
		htmlCodes.put("Iacute", new Character((char) 205));
		htmlCodes.put("Icirc", new Character((char) 206));
		htmlCodes.put("Iuml", new Character((char) 207));
		htmlCodes.put("ETH", new Character((char) 208));
		htmlCodes.put("Ntilde", new Character((char) 209));
		htmlCodes.put("Ograve", new Character((char) 210));
		htmlCodes.put("Oacute", new Character((char) 211));
		htmlCodes.put("Ocirc", new Character((char) 212));
		htmlCodes.put("Otilde", new Character((char) 213));
		htmlCodes.put("Ouml", new Character((char) 214));
		htmlCodes.put("times", new Character((char) 215));
		htmlCodes.put("Oslash", new Character((char) 216));
		htmlCodes.put("Ugrave", new Character((char) 217));
		htmlCodes.put("Uacute", new Character((char) 218));
		htmlCodes.put("Ucirc", new Character((char) 219));
		htmlCodes.put("Uuml", new Character((char) 220));
		htmlCodes.put("Yacute", new Character((char) 221));
		htmlCodes.put("THORN", new Character((char) 222));
		htmlCodes.put("szlig", new Character((char) 223));
		htmlCodes.put("agrave", new Character((char) 224));
		htmlCodes.put("aacute", new Character((char) 225));
		htmlCodes.put("acirc", new Character((char) 226));
		htmlCodes.put("atilde", new Character((char) 227));
		htmlCodes.put("auml", new Character((char) 228));
		htmlCodes.put("aring", new Character((char) 229));
		htmlCodes.put("aelig", new Character((char) 230));
		htmlCodes.put("ccedil", new Character((char) 231));
		htmlCodes.put("egrave", new Character((char) 232));
		htmlCodes.put("eacute", new Character((char) 233));
		htmlCodes.put("ecirc", new Character((char) 234));
		htmlCodes.put("euml", new Character((char) 235));
		htmlCodes.put("igrave", new Character((char) 236));
		htmlCodes.put("iacute", new Character((char) 237));
		htmlCodes.put("icirc", new Character((char) 238));
		htmlCodes.put("iuml", new Character((char) 239));
		htmlCodes.put("eth", new Character((char) 240));
		htmlCodes.put("ntilde", new Character((char) 241));
		htmlCodes.put("ograve", new Character((char) 242));
		htmlCodes.put("oacute", new Character((char) 243));
		htmlCodes.put("ocirc", new Character((char) 244));
		htmlCodes.put("otilde", new Character((char) 245));
		htmlCodes.put("ouml", new Character((char) 246));
		htmlCodes.put("divide", new Character((char) 247));
		htmlCodes.put("oslash", new Character((char) 248));
		htmlCodes.put("ugrave", new Character((char) 249));
		htmlCodes.put("uacute", new Character((char) 250));
		htmlCodes.put("ucirc", new Character((char) 251));
		htmlCodes.put("uuml", new Character((char) 252));
		htmlCodes.put("yacute", new Character((char) 253));
		htmlCodes.put("thorn", new Character((char) 254));
		htmlCodes.put("yuml", new Character((char) 255));
	}
	
	
	/**
	 * Only supports ASCII right now. Don't try unicode. Converts all
	 * characters, even ones that don't need to be
	 * 
	 * Supports FORMAT_HTML_HEX_ENCODED
	 * 
	 * @param string
	 * @param format
	 * @return
	 */
	public static String completeEncode(String string, int format)
	{
		Formatter formatter = new Formatter();
		String result = "";
		for (char character: string.toCharArray())
		{
			int numericValue = Character.valueOf(character);
			switch (format)
			{
				case FORMAT_HTML_HEX_ENCODED:
					formatter.format("&%02X;", numericValue);
					break;
				case FORMAT_HTML_DOUBLE_PADDED_HEX_ENCODED:
					formatter.format("&0000%02X;", numericValue);
					break;
				case FORMAT_HTML_DECIMAL_ENCODED:
					formatter.format("&%02d", numericValue);
					break;
				case FORMAT_URL_ENCODED:
					formatter.format("%%%02X", numericValue);
					break;
				case FORMAT_HEXADECIMAL:
					formatter.format("%02X", numericValue);
					break;
				case FORMAT_DECIMAL_COMMA_SEPERATED:
					formatter.format("%02d,", numericValue);
					break;
				default:
					result += character;
			}
		}
		if (result.equals(""))
		{
			result = formatter.toString();
		}
		if (format == FORMAT_DECIMAL_COMMA_SEPERATED)
		{
			result = result.replaceFirst(",$", "");
		}
		return result;
	}

	/**
	 * Only supports ASCII right now. Don't try unicode. Only converts
	 * non-alphanumeric characters
	 * 
	 * Supports FORMAT_HTML_HEX_ENCODED
	 * 
	 * @param string
	 * @param format
	 * @return
	 */
	public static String nonAlphanumericEncode(String string, int format)
	{
		Formatter formatter = new Formatter();
		String result = "";
		for (char character: string.toCharArray())
		{
			if (((character >= 'a') && (character <= 'z')) ||
					((character >= 'A') && (character <= 'Z')) ||
					((character >= '0') && (character <= '9')))
			{
				int numericValue = Character.getNumericValue(character);
				switch (format)
				{
					case FORMAT_HTML_HEX_ENCODED:
						result += formatter.format("&%02x;", numericValue);
						break;
					case FORMAT_HTML_DECIMAL_ENCODED:
						result += formatter.format("&%02d", numericValue);
						break;
					case FORMAT_URL_ENCODED:
						result += formatter.format("%%%02x", numericValue);
						break;
					case FORMAT_HEXADECIMAL:
						result += formatter.format("%02x", numericValue);
						break;
					
					default:
						result += character;
				}
			}
			else
			{
				result += character;
			}
			
		}
		return result;
	}

	/*
	 * public static final int FORMAT_HEXADECIMAL = 0; public static final int
	 * FORMAT_LOW_ASCII = 1; public static final int FORMAT_BASE64 = 2; public
	 * static final int FORMAT_ALL_ASCII = 3; public static final int
	 * FORMAT_DECIMAL = 4; public static final int FORMAT_PLAIN_TEXT = 5; public
	 * static final int FORMAT_UPPER_CASE_ALPHA = 6; public static final int
	 * FORMAT_LOWER_CASE_ALPHA = 7; public static final int
	 * FORMAT_ALL_CASE_ALPHA = 8; public static final int
	 * FORMAT_ALL_CASE_ALPHANUMERIC = 9; public static final int
	 * FORMAT_URL_ENCODED = 10; public static final int FORMAT_HTML_ENCODED =
	 * 11;
	 * 
	 * 
	 */
	public static byte generateRandomByte(int stringFormat)
	{

		switch (stringFormat)
		{
			case FORMAT_DECIMAL:
			{
				int ranges[][] = { { '0', '9' } };
				return (byte) rand(ranges);
			}
			case FORMAT_UPPER_CASE_ALPHA:
			{
				int ranges[][] = { { 'A', 'Z' } };
				return (byte) rand(ranges);
			}
			case FORMAT_LOWER_CASE_ALPHA:
			{
				int ranges[][] = { { 'a', 'z' } };
				return (byte) rand(ranges);
			}
			case FORMAT_ALL_CASE_ALPHA:
			{
				int ranges[][] = { { 'a', 'z' }, { 'A', 'Z' } };
				return (byte) rand(ranges);
			}
			default:
				int ranges[][] = { { 0, 255 } };
				return (byte) rand(ranges);
		}
	}

	public static String generateRandomString(int stringFormat, int length)
	{
		byte rawString[] = new byte[length];
		for (int index = 0; index < length; index++)
		{
			rawString[index] = generateRandomByte(stringFormat);
		}

		return new String(rawString);
	}

	public static double getBitsPerByte(int format)
	{
		double bits;
		switch (format)
		{
			case FORMAT_HEXADECIMAL:
				bits = 4;
				break;
			case FORMAT_LOW_ASCII:
				bits = 7;
				break;
			case FORMAT_BASE64:
				bits = 6;
				break;
			case FORMAT_ALL_ASCII:
				bits = 8;
				break;
			case FORMAT_DECIMAL:
				bits = 3.3219;
				break;
			case FORMAT_PLAIN_TEXT:
				bits = 6.5545;
				break;
			default:
				bits = 0;
		}
		return bits;
	}

	/**
	 * Note that it can be difficult to tell the difference between a plain text
	 * word and a base64 string. If there is a possibility of confusion, check
	 * the result. This will guess base64 before alphanumeric. Also note that
	 * this doesn't check for validity of base64 format. This is because padding
	 * may be omitted by some applications when the value is sent to the client.
	 * 
	 * Only FORMAT_ALL_ASCII, FORMAT_DECIMAL, FORMAT_HEXADECIMAL, FORMAT_BASE64,
	 * FORMAT_PLAIN_TEXT, FORMAT_LOW_ASCII, AND FORMAT_ALL_ASCII are supported.
	 * 
	 * @param target
	 * @return
	 */
	public static int getStringFormat(String target)
	{
		int format = FORMAT_ALL_ASCII;

		Pattern pattern = Pattern.compile("^[0-9]+$");
		if (target.matches("^[0-9]+$"))
		{
			format = FORMAT_DECIMAL;
		}
		// This isn't a mistake I want the cases to be mutually exclusive. I.e.
		// all upper or all lower
		else if (target.matches("^[0-9a-f]+$") || target.matches("^[0-9A-F]+$"))
		{
			format = FORMAT_HEXADECIMAL;
		}
		else if (target.matches("^[0-9a-zA-Z=+/]+$"))
		{
			format = FORMAT_BASE64;
		}
		else if (target.matches("^[\\x20-\\x7d]+$"))
		{
			format = FORMAT_PLAIN_TEXT;
		}
		else if (target.matches("^[\\x00-\\x7f]+$"))
		{
			format = FORMAT_LOW_ASCII;
		}
		else
		{
			format = FORMAT_ALL_ASCII;
		}

		return format;
	}

	public static int rand(int[][] ranges)
	{
		int returnValue = 0;
		int totalValues = 0;
		for (int range[]: ranges)
		{
			int high = range[1];
			int low = range[0];
			totalValues += (high - low) + 1;
		}

		int randomValue = random.nextInt(totalValues);

		totalValues = 0;
		int previousTotalValues;
		for (int range[]: ranges)
		{
			int high = range[1];
			int low = range[0];
			previousTotalValues = totalValues;
			totalValues += (high - low) + 1;
			if (randomValue <= totalValues)
			{
				int offset = randomValue - previousTotalValues;
				returnValue = low + offset;
				break;
			}
		}
		return returnValue;
	}

	/**
	 * This really needs to be improved. Way too slow.
	 */ 
	public static int scoreStringDifference(String stringA, String stringB, int perfectScore)
	{
		int score = 0;
		double maxLength;
		if (stringA.equals(stringB))
		{
			return perfectScore;
		}
		
		int levenshteinDistance = org.apache.commons.lang.StringUtils.getLevenshteinDistance(stringA, stringB);

		if (stringA.length() > stringB.length())
		{
			maxLength = stringA.length();
		}
		else
		{
			maxLength = stringB.length();
		}

		if (maxLength > 0)
		{
			score = (int) Math.round(((maxLength - levenshteinDistance) / maxLength) * perfectScore);
		}

		return score;
	}

	public static int scoreStringDifferenceIgnoreCase(String stringA, String stringB, int perfectScore)
	{
		if (stringA == null)
		{
			stringA = "";
		}
		if (stringB == null)
		{
			stringB = "";
		}
		return scoreStringDifference(stringA.toLowerCase(), stringB.toLowerCase(), perfectScore);
	}

	public static String normalizeWhiteSpace(String input)
	{
		return input.replaceAll("[\\x00-\\x20]++", " ");
	}

	/**
	 * Case insensitive
	 * 
	 * @param text
	 * @param regexs
	 * @return
	 */
	public static boolean testMultipleRegexs(String text, Iterable<String> regexs)
	{
		boolean found = false;
		for(String regex: regexs)
		{
			Pattern pattern = Pattern.compile(regex, Pattern.CASE_INSENSITIVE);
			if (pattern.matcher(text).find())
			{
				found = true;
				break;
			}
		}
		return found;
	}
	
	public static String join(String[] strings, String joinString)
	{
		String joined = "";
		for (String string: strings)
		{
			joined += string.concat(joinString);
		}
		return joined;
	}
	
	
	/**
	 * Based on the entity parser in Lobo
	 * @param rawText
	 * @return
	 */
	public static String decodeHtml(String rawText)
	{
		int startIdx = 0;
		StringBuffer sb = null;

		
		for(;;) 
		{
			int ampIdx = rawText.indexOf("&", startIdx);
			if(ampIdx == -1) 
			{
				if(sb == null) 
				{
					return rawText;
				}
				else 
				{
					sb.append(rawText.substring(startIdx));
					return sb.toString();
				}
			}
			if(sb == null) 
			{
				sb = new StringBuffer();
			}
			sb.append(rawText.substring(startIdx, ampIdx));
			int colonIdx = rawText.indexOf(";", ampIdx);
			if(colonIdx == -1) 
			{
				sb.append('&');
				startIdx = ampIdx+1;
				continue;
			}
			String spec = rawText.substring(ampIdx+1, colonIdx);
			if(spec.startsWith("#")) 
			{
				String number = spec.substring(1).toLowerCase();
				int decimal;
				try 
				{
					if(number.startsWith("x")) 
					{
						decimal = Integer.parseInt(number.substring(1), 16);
					}
					else 
					{
						decimal = Integer.parseInt(number);
					}
					sb.append((char) decimal);
				} 
				catch(NumberFormatException nfe) 
				{
					sb.append(spec);
				}
			}
			else 
			{
				int chInt = getEntityChar(spec);
				if(chInt == -1) 
				{
					sb.append('&');
					sb.append(spec);
					sb.append(';');
				}
				else 
				{
					sb.append((char) chInt);
				}
			}
			startIdx = colonIdx+1;
		}
	}

	private static final int getEntityChar(String spec) 
	{
		Character c = (Character) htmlCodes.get(spec);
		if(c == null) 
		{
			String specTL = spec.toLowerCase();
			c = (Character) htmlCodes.get(specTL);
			if (c == null) 
			{
				return -1;
			}
		}
		return (int) c.charValue();
	}
	
	public static String unquote(String string)
	{
		String unquoted = string;
		if (string != null && !string.equals(""))
		{
			char first = string.charAt(0);
			char last = string.charAt(string.length() - 1);
			
			if (first == last && (first == '"' || first == '\''))
			{
				unquoted = string.substring(1, string.length() - 1);
			}
			else
			{
				unquoted = string;
			}
		}
		return unquoted;
	}
	
	public static boolean notEmpty(String string)
	{
		if (string != null && !string.equals(""))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * This will return the actual value if the string is a
	 * number, otherwise it will return the base 256 value 
	 * @param tempString
	 * @return
	 */
	public static double getNumericValue(String string)
	{
		double value = 0;
		String tempString = string.toLowerCase();
		if (tempString.matches("[0-9\\.]+"))
		{
			value = Double.valueOf(tempString);
		}
		else
		{
			int power = 0;
			// loop over the string starting at the last character
			for (int index = tempString.length() - 1; index >= 0; index--, power++)
			{
				value += Math.pow(256, power) * tempString.charAt(index); 
			}
		}
		
		return value;
	}

	static Pattern perlMetaCharacterPattern = Pattern.compile("\\\\([\\$@\"&%/])");

	/**
	 * This will apply Perl rules to unescape strings in that format. For example
	 * \$ is only found in Perl, or related languages. This is mostly for Nikto
	 * databases. Note that it will also unescape some things that are usually
	 * specific to regexes, such as \/, but not all regex metacharacters.
	 * @param perlString
	 * @return
	 */
	public static String unescapePerlStrings(String perlString)
	{
		Matcher m = perlMetaCharacterPattern.matcher(perlString);
		return m.replaceAll("$1");
	}
	
	public static String md5Hash(String source)
	{
		String result = null;
		MessageDigest md = null;
		try
		{
			md = MessageDigest.getInstance("MD5");
			result = new String(md.digest(source.getBytes()));
		}
		catch (NoSuchAlgorithmException e)
		{
			Debug.errDebug("Something weird with the MD5 hash call in md5Hash: " + e.toString(), e);
		}
		
		return result;
	}

}
