/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Jan 14, 2006
 */
package org.cobra_grendel.html.domimpl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import org.cobra_grendel.html.FormInput;
import org.cobra_grendel.html.HtmlRendererContext;
import org.cobra_grendel.html.js.Executor;
import org.mozilla.javascript.Function;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.html2.HTMLCollection;
import org.w3c.dom.html2.HTMLFormElement;

public class HTMLFormElementImpl extends HTMLAbstractUIElement implements HTMLFormElement
{
	private class InputFilter implements NodeFilter
	{
		/*
		 * (non-Javadoc)
		 * 
		 * @see org.xamjwg.html.domimpl.NodeFilter#accept(org.w3c.dom.Node)
		 */
		public boolean accept(Node node)
		{
			return HTMLFormElementImpl.isInput(node);
		}
	}
	
	private Function onsubmit;
	
	public HTMLFormElementImpl()
	{
		super("FORM");
	}
	
	public HTMLFormElementImpl(String name)
	{
		super(name);
	}
	
	static boolean isInput(Node node)
	{
		String name = node.getNodeName().toLowerCase();
		return name.equals("input") || name.equals("textarea") || name.equals("select");
	}
	
	public String getAcceptCharset()
	{
		return getAttribute("acceptCharset");
	}
	
	public String getAction()
	{
		return getAttribute("action");
	}
	
	public HTMLCollection getElements()
	{
		return new DescendentHTMLCollection(this, new InputFilter());
	}
	
	public String getEnctype()
	{
		return getAttribute("enctype");
	}
	
	public int getLength()
	{
		return new DescendentHTMLCollection(this, new InputFilter()).getLength();
	}
	
	public String getMethod()
	{
		String method = getAttribute("method");
		if (method == null)
		{
			method = "GET";
		}
		return method;
	}
	
	public String getName()
	{
		return getAttribute("name");
	}
	
	public Function getOnsubmit()
	{
		return getEventFunction(onsubmit, "onsubmit");
	}
	
	public String getTarget()
	{
		return getAttribute("target");
	}
	
	public Object item(final int index)
	{
		try
		{
			visit(new NodeVisitor()
			{
				private int current = 0;
				
				public void visit(Node node)
				{
					if (HTMLFormElementImpl.isInput(node))
					{
						if (current == index)
						{
							throw new StopVisitorException(node);
						}
						current++;
					}
				}
			});
		}
		catch (StopVisitorException sve)
		{
			return sve.getTag();
		}
		return null;
	}
	
	public Object namedItem(final String name)
	{
		try
		{
			// TODO: This could use document.namedItem.
			visit(new NodeVisitor()
			{
				public void visit(Node node)
				{
					if (HTMLFormElementImpl.isInput(node))
					{
						if (name.equals(((Element) node).getAttribute("name")))
						{
							throw new StopVisitorException(node);
						}
					}
				}
			});
		}
		catch (StopVisitorException sve)
		{
			return sve.getTag();
		}
		return null;
	}
	
	public void reset()
	{
		visit(new NodeVisitor()
		{
			public void visit(Node node)
			{
				if (node instanceof HTMLBaseInputElement)
				{
					((HTMLBaseInputElement) node).resetInput();
				}
			}
		});
	}
	
	public void setAcceptCharset(String acceptCharset)
	{
		setAttribute("acceptCharset", acceptCharset);
	}
	
	public void setAction(String action)
	{
		setAttribute("action", action);
	}
	
	public void setEnctype(String enctype)
	{
		setAttribute("enctype", enctype);
	}
	
	public void setMethod(String method)
	{
		setAttribute("method", method);
	}
	
	public void setName(String name)
	{
		setAttribute("name", name);
	}
	
	public void setOnsubmit(Function value)
	{
		onsubmit = value;
	}
	
	public void setTarget(String target)
	{
		setAttribute("target", target);
	}
	
	public void submit()
	{
		this.submit(null);
	}
	
	/**
	 * This method should be called when form submission is done by a submit
	 * button.
	 * 
	 * @param extraFormInputs
	 *            Any additional form inputs that need to be submitted, e.g. the
	 *            submit button parameter.
	 */
	public final void submit(final FormInput[] extraFormInputs)
	{
		// Disabled for now (too dangerous)
		
		
//		Function onsubmit = getOnsubmit();
//		if (onsubmit != null)
//		{
//			// TODO: onsubmit event object?
//			if (!Executor.executeFunction(this, onsubmit, null))
//			{
//				return;
//			}
//		}
//		HtmlRendererContext context = getHtmlRendererContext();
//		if (context != null)
//		{
//			final ArrayList formInputs = new ArrayList();
//			if (extraFormInputs != null)
//			{
//				for (int i = 0; i < extraFormInputs.length; i++)
//				{
//					formInputs.add(extraFormInputs[i]);
//				}
//			}
//			visit(new NodeVisitor()
//			{
//				public void visit(Node node)
//				{
//					if (node instanceof HTMLElementImpl)
//					{
//						FormInput[] fis = ((HTMLElementImpl) node).getFormInputs();
//						if (fis != null)
//						{
//							for (int i = 0; i < fis.length; i++)
//							{
//								FormInput fi = fis[i];
//								if (fi.getName() == null)
//								{
//									throw new IllegalStateException("Form input does not have a name: " + node);
//								}
//								formInputs.add(fi);
//							}
//						}
//					}
//				}
//			});
//			FormInput[] fia = (FormInput[]) formInputs.toArray(FormInput.EMPTY_ARRAY);
//			String href = getAction();
//			if (href == null)
//			{
//				href = getBaseURI();
//			}
//			try
//			{
//				URL url = getFullURL(href);
//				context.submitForm(getMethod(), url, getTarget(), getEnctype(), fia);
//			}
//			catch (MalformedURLException mfu)
//			{
//				this.warn("submit()", mfu);
//			}
//		}
	}
}
