/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Jan 28, 2006
 */
package org.cobra_grendel.html.domimpl;

import org.cobra_grendel.html.BrowserFrame;
import org.cobra_grendel.html.js.Window;
import org.w3c.dom.Document;
import org.w3c.dom.html2.HTMLFrameElement;

public class HTMLFrameElementImpl extends HTMLElementImpl implements HTMLFrameElement, FrameNode
{
	private volatile BrowserFrame browserFrame;
	
	private boolean noResize;
	
	public HTMLFrameElementImpl(String name)
	{
		super(name);
	}
	
	public HTMLFrameElementImpl(String name, boolean noStyleSheet)
	{
		super(name, noStyleSheet);
	}
	
	public BrowserFrame getBrowserFrame()
	{
		return browserFrame;
	}
	
	public Document getContentDocument()
	{
		BrowserFrame frame = browserFrame;
		if (frame == null)
		{
			// Not loaded yet
			return null;
		}
		return frame.getContentDocument();
	}
	
	public Window getContentWindow()
	{
		BrowserFrame frame = browserFrame;
		if (frame == null)
		{
			// Not loaded yet
			return null;
		}
		return Window.getWindow(frame.getHtmlRendererContext());
	}
	
	public String getFrameBorder()
	{
		return getAttribute("frameBorder");
	}
	
	public String getLongDesc()
	{
		return getAttribute("longdesc");
	}
	
	public String getMarginHeight()
	{
		return getAttribute("marginHeight");
	}
	
	public String getMarginWidth()
	{
		return getAttribute("marginWidth");
	}
	
	public String getName()
	{
		return getAttribute("name");
	}
	
	public boolean getNoResize()
	{
		return noResize;
	}
	
	public String getScrolling()
	{
		return getAttribute("scrolling");
	}
	
	public String getSrc()
	{
		return getAttribute("src");
	}
	
	public void setBrowserFrame(BrowserFrame frame)
	{
		browserFrame = frame;
	}
	
	public void setFrameBorder(String frameBorder)
	{
		setAttribute("frameBorder", frameBorder);
	}
	
	public void setLongDesc(String longDesc)
	{
		setAttribute("longdesc", longDesc);
	}
	
	public void setMarginHeight(String marginHeight)
	{
		setAttribute("marginHeight", marginHeight);
	}
	
	public void setMarginWidth(String marginWidth)
	{
		setAttribute("marginWidth", marginWidth);
	}
	
	public void setName(String name)
	{
		setAttribute("name", name);
	}
	
	public void setNoResize(boolean noResize)
	{
		this.noResize = noResize;
	}
	
	public void setScrolling(String scrolling)
	{
		setAttribute("scrolling", scrolling);
	}
	
	public void setSrc(String src)
	{
		setAttribute("src", src);
	}
	
}
