package org.cobra_grendel.html.domimpl;

import org.cobra_grendel.html.BrowserFrame;
import org.cobra_grendel.html.js.Window;
import org.w3c.dom.Document;
import org.w3c.dom.html2.HTMLIFrameElement;

public class HTMLIFrameElementImpl extends HTMLAbstractUIElement implements HTMLIFrameElement, FrameNode
{
	private volatile BrowserFrame browserFrame;
	
	public HTMLIFrameElementImpl(String name)
	{
		super(name);
	}
	
	public String getAlign()
	{
		return getAttribute("align");
	}
	
	public BrowserFrame getBrowserFrame()
	{
		return browserFrame;
	}
	
	public Document getContentDocument()
	{
		// TODO: Domain-based security
		BrowserFrame frame = browserFrame;
		if (frame == null)
		{
			// Not loaded yet
			return null;
		}
		return frame.getContentDocument();
	}
	
	public Window getContentWindow()
	{
		BrowserFrame frame = browserFrame;
		if (frame == null)
		{
			// Not loaded yet
			return null;
		}
		return Window.getWindow(frame.getHtmlRendererContext());
	}
	
	public String getFrameBorder()
	{
		return getAttribute("frameborder");
	}
	
	public String getHeight()
	{
		return getAttribute("height");
	}
	
	public String getLongDesc()
	{
		return getAttribute("longdesc");
	}
	
	public String getMarginHeight()
	{
		return getAttribute("marginheight");
	}
	
	public String getMarginWidth()
	{
		return getAttribute("marginwidth");
	}
	
	public String getName()
	{
		return getAttribute("name");
	}
	
	public String getScrolling()
	{
		return getAttribute("scrolling");
	}
	
	public String getSrc()
	{
		return getAttribute("src");
	}
	
	public String getWidth()
	{
		return getAttribute("width");
	}
	
	public void setAlign(String align)
	{
		setAttribute("align", align);
	}
	
	public void setBrowserFrame(BrowserFrame frame)
	{
		browserFrame = frame;
		if (frame != null)
		{
			String src = getAttribute("src");
			if (src != null)
			{
				try
				{
					frame.loadURL(getFullURL(src));
				}
				catch (java.net.MalformedURLException mfu)
				{
					this.warn("setBrowserFrame(): Unable to navigate to src.", mfu);
				}
			}
		}
	}
	
	public void setFrameBorder(String frameBorder)
	{
		setAttribute("frameborder", frameBorder);
	}
	
	public void setHeight(String height)
	{
		setAttribute("height", height);
	}
	
	public void setLongDesc(String longDesc)
	{
		setAttribute("longdesc", longDesc);
	}
	
	public void setMarginHeight(String marginHeight)
	{
		setAttribute("marginHeight", marginHeight);
	}
	
	public void setMarginWidth(String marginWidth)
	{
		setAttribute("marginWidth", marginWidth);
	}
	
	public void setName(String name)
	{
		setAttribute("name", name);
	}
	
	public void setScrolling(String scrolling)
	{
		setAttribute("scrolling", scrolling);
	}
	
	public void setSrc(String src)
	{
		setAttribute("src", src);
	}
	
	public void setWidth(String width)
	{
		setAttribute("width", width);
	}
	
	@Override
	protected void assignAttributeField(String normalName, String value)
	{
		if ("src".equals(normalName))
		{
			BrowserFrame frame = browserFrame;
			if (frame != null)
			{
				try
				{
					frame.loadURL(getFullURL(value));
				}
				catch (java.net.MalformedURLException mfu)
				{
					this.warn("assignAttributeField(): Unable to navigate to src.", mfu);
				}
			}
		}
		else
		{
			super.assignAttributeField(normalName, value);
		}
	}
}
