/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Nov 19, 2005
 */
package org.cobra_grendel.html.domimpl;

import java.util.ArrayList;

import org.cobra_grendel.html.js.Executor;
import org.mozilla.javascript.Function;
import org.w3c.dom.html2.HTMLImageElement;

public class HTMLImageElementImpl extends HTMLAbstractUIElement implements HTMLImageElement
{
	private class LocalImageListener implements ImageListener
	{
		private final String expectedImgSrc;
		
		public LocalImageListener(String imgSrc)
		{
			expectedImgSrc = imgSrc;
		}
		
		public void imageLoaded(ImageEvent event)
		{
			dispatchEvent(expectedImgSrc, event);
		}
	}
	
	private java.awt.Image image = null;
	
	private String imageSrc;
	
	private final ArrayList listeners = new ArrayList(1);
	
	private Function onload;
	
	public HTMLImageElementImpl()
	{
		super("IMG");
	}
	
	public HTMLImageElementImpl(String name)
	{
		super(name);
	}
	
	/**
	 * Adds a listener of image loading events. The listener gets called right
	 * away if there's already an image.
	 * 
	 * @param listener
	 */
	public void addImageListener(ImageListener listener)
	{
		ArrayList l = listeners;
		java.awt.Image currentImage;
		synchronized (l)
		{
			currentImage = image;
			l.add(listener);
		}
		if (currentImage != null)
		{
			// Call listener right away if there's already an
			// image; holding no locks.
			listener.imageLoaded(new ImageEvent(this, currentImage));
			// Should not call onload handler here. That's taken
			// care of otherwise.
		}
	}
	
	public String getAlign()
	{
		return getAttribute("align");
	}
	
	public String getAlt()
	{
		return getAttribute("alt");
	}
	
	public String getBorder()
	{
		return getAttribute("border");
	}
	
	public int getHeight()
	{
		UINode r = uiNode;
		return r == null ? 0 : r.getBounds().height;
	}
	
	public int getHspace()
	{
		return getAttributeAsInt("hspace", 0);
	}
	
	public final java.awt.Image getImage()
	{
		synchronized (listeners)
		{
			return image;
		}
	}
	
	public boolean getIsMap()
	{
		return getAttributeAsBoolean("isMap");
	}
	
	public String getLongDesc()
	{
		return getAttribute("longDesc");
	}
	
	public String getName()
	{
		return getAttribute("name");
	}
	
	public Function getOnload()
	{
		return getEventFunction(onload, "onload");
	}
	
	public String getSrc()
	{
		return getAttribute("src");
	}
	
	public String getUseMap()
	{
		return getAttribute("useMap");
	}
	
	public int getVspace()
	{
		return getAttributeAsInt("vspace", 0);
	}
	
	public int getWidth()
	{
		UINode r = uiNode;
		return r == null ? 0 : r.getBounds().width;
	}
	
	public void removeImageListener(ImageListener listener)
	{
		ArrayList l = listeners;
		synchronized (l)
		{
			l.remove(l);
		}
	}
	
	public void setAlign(String align)
	{
		setAttribute("align", align);
	}
	
	public void setAlt(String alt)
	{
		setAttribute("alt", alt);
	}
	
	public void setBorder(String border)
	{
		setAttribute("border", border);
	}
	
	// /* (non-Javadoc)
	// * @see org.xamjwg.html.renderer.RenderableContext#getHeightLength()
	// */
	// public HtmlLength getHeightLength() {
	// return this.heightLength;
	// }
	//
	// /* (non-Javadoc)
	// * @see org.xamjwg.html.renderer.RenderableContext#getWidthLength()
	// */
	// public HtmlLength getWidthLength() {
	// return this.widthLength;
	// }
	
	// /* (non-Javadoc)
	// * @see org.xamjwg.html.renderer.RenderableContext#getAlignmentX()
	// */
	// public float getAlignmentX() {
	// return 0.5f;
	// }
	//
	// /* (non-Javadoc)
	// * @see org.xamjwg.html.renderer.RenderableContext#getAlignmentY()
	// */
	// public float getAlignmentY() {
	// return this.alignmentY;
	// }
	
	// private HtmlLength widthLength;
	// private HtmlLength heightLength;
	// private float alignmentY = 1.0f;
	
	// /* (non-Javadoc)
	// * @see
	// org.xamjwg.html.domimpl.ElementImpl#assignAttributeField(java.lang.String,
	// java.lang.String)
	// */
	// protected void assignAttributeField(String normalName, String value) {
	// super.assignAttributeField(normalName, value);
	// if("width".equals(normalName)) {
	// try {
	// this.widthLength = new HtmlLength(value);
	// } catch(Exception err) {
	// this.warn("Bad width spec: " + value, err);
	// this.widthLength = null;
	// }
	// }
	// else if("height".equals(normalName)) {
	// try {
	// this.heightLength = new HtmlLength(value);
	// } catch(Exception err) {
	// this.warn("Bad height spec: " + value, err);
	// this.heightLength = null;
	// }
	// }
	// }
	//	
	// private final void assignAlignment(String value) {
	// if(value.equalsIgnoreCase("middle")) {
	// this.alignmentY = 0.5f;
	// }
	// else if(value.equalsIgnoreCase("top")) {
	// this.alignmentY = 0.0f;
	// }
	// else if(value.equalsIgnoreCase("bottom")) {
	// this.alignmentY = 1.0f;
	// }
	// else {
	// this.alignmentY = 1.0f;
	// }
	// }
	
	public void setHeight(int height)
	{
		setAttribute("height", String.valueOf(height));
	}
	
	public void setHspace(int hspace)
	{
		setAttribute("hspace", String.valueOf("hspace"));
	}
	
	public void setIsMap(boolean isMap)
	{
		setAttribute("isMap", isMap ? "isMap" : null);
	}
	
	public void setLongDesc(String longDesc)
	{
		setAttribute("longDesc", longDesc);
	}
	
	public void setName(String name)
	{
		setAttribute("name", name);
	}
	
	public void setOnload(Function onload)
	{
		this.onload = onload;
	}
	
	/**
	 * Sets the image URI and starts to load the image. Note that an
	 * HtmlRendererContext should be available to the HTML document for images
	 * to be loaded.
	 */
	public void setSrc(String src)
	{
		setAttribute("src", src);
	}
	
	public void setUseMap(String useMap)
	{
		setAttribute("useMap", useMap);
	}
	
	public void setVspace(int vspace)
	{
		setAttribute("vspace", String.valueOf(vspace));
	}
	
	public void setWidth(int width)
	{
		setAttribute("width", String.valueOf(width));
	}
	
	private void dispatchEvent(String expectedImgSrc, ImageEvent event)
	{
		ArrayList l = listeners;
		ImageListener[] listenerArray;
		synchronized (l)
		{
			if (!expectedImgSrc.equals(imageSrc))
			{
				return;
			}
			image = event.image;
			// Get array of listeners while holding lock.
			listenerArray = (ImageListener[]) l.toArray(ImageListener.EMPTY_ARRAY);
		}
		int llength = listenerArray.length;
		for (int i = 0; i < llength; i++)
		{
			// Inform listener, holding no lock.
			listenerArray[i].imageLoaded(event);
		}
		Function onload = getOnload();
		if (onload != null)
		{
			// TODO: onload event object?
			Executor.executeFunction(HTMLImageElementImpl.this, onload, null);
		}
	}
	
	private void loadImage(String src)
	{
		HTMLDocumentImpl document = (HTMLDocumentImpl) this.document;
		if (document != null)
		{
			synchronized (listeners)
			{
				imageSrc = src;
				image = null;
			}
			if (src != null)
			{
				document.loadImage(src, new LocalImageListener(src));
			}
		}
	}
	
	@Override
	protected void assignAttributeField(String normalName, String value)
	{
		super.assignAttributeField(normalName, value);
		if ("src".equals(normalName))
		{
			loadImage(value);
		}
	}
}
