package org.cobra_grendel.html.domimpl;

import org.cobra_grendel.html.FormInput;
import org.w3c.dom.DOMException;
import org.w3c.dom.html2.HTMLElement;
import org.w3c.dom.html2.HTMLOptionsCollection;
import org.w3c.dom.html2.HTMLSelectElement;

public class HTMLSelectElementImpl extends HTMLBaseInputElement implements HTMLSelectElement
{
	private HTMLOptionsCollection options;
	
	public HTMLSelectElementImpl(String name)
	{
		super(name);
	}
	
	public void add(HTMLElement element, HTMLElement before) throws DOMException
	{
		insertBefore(element, before);
	}
	
	public int getLength()
	{
		return getOptions().getLength();
	}
	
	public boolean getMultiple()
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			return ic.getMultiple();
		}
		else
		{
			return false;
		}
	}
	
	public HTMLOptionsCollection getOptions()
	{
		synchronized (this)
		{
			if (options == null)
			{
				options = new HTMLOptionsCollectionImpl(this);
			}
			return options;
		}
	}
	
	public int getSelectedIndex()
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			return ic.getSelectedIndex();
		}
		else
		{
			return -1;
		}
	}
	
	public int getSize()
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			return ic.getVisibleSize();
		}
		else
		{
			return 0;
		}
	}
	
	public String getType()
	{
		return getMultiple() ? "select-multiple" : "select-one";
	}
	
	public void remove(int index)
	{
		try
		{
			removeChild(getOptions().item(index));
		}
		catch (DOMException de)
		{
			this.warn("remove(): Unable to remove option at index " + index + ".", de);
		}
	}
	
	@Override
	public void resetInput()
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			ic.resetInput();
		}
	}
	
	public void setLength(int length) throws DOMException
	{
		getOptions().setLength(length);
	}
	
	public void setMultiple(boolean multiple)
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			ic.setMultiple(multiple);
		}
	}
	
	public void setSelectedIndex(int selectedIndex)
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			ic.setSelectedIndex(selectedIndex);
		}
	}
	
	public void setSize(int size)
	{
		InputContext ic = inputContext;
		if (ic != null)
		{
			ic.setVisibleSize(size);
		}
	}
	
	@Override
	protected FormInput[] getFormInputs()
	{
		// Needs to be overriden for forms to submit.
		String name = getName();
		if (name == null)
		{
			return null;
		}
		return new FormInput[] { new FormInput(name, getValue()) };
	}
}
