/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Dec 4, 2005
 */
package org.cobra_grendel.html.domimpl;

import java.util.ArrayList;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.html2.HTMLCollection;
import org.w3c.dom.html2.HTMLElement;
import org.w3c.dom.html2.HTMLTableRowElement;

public class HTMLTableRowElementImpl extends HTMLElementImpl implements HTMLTableRowElement
{
	public HTMLTableRowElementImpl()
	{
		super("TR", true);
	}
	
	public HTMLTableRowElementImpl(String name)
	{
		super(name, true);
	}
	
	public void deleteCell(int index) throws DOMException
	{
		synchronized (treeLock)
		{
			ArrayList nl = nodeList;
			if (nl != null)
			{
				int size = nl.size();
				int trcount = 0;
				for (int i = 0; i < size; i++)
				{
					Node node = (Node) nl.get(i);
					if (node instanceof org.w3c.dom.html2.HTMLTableCellElement)
					{
						if (trcount == index)
						{
							removeChildAt(index);
						}
						trcount++;
					}
				}
			}
		}
		throw new DOMException(DOMException.INDEX_SIZE_ERR, "Index out of range");
	}
	
	public String getAlign()
	{
		return getAttribute("align");
	}
	
	public String getBgColor()
	{
		return getAttribute("bgcolor");
	}
	
	public HTMLCollection getCells()
	{
		NodeFilter filter = new NodeFilter()
		{
			public boolean accept(Node node)
			{
				return node instanceof HTMLTableCellElementImpl;
			}
		};
		return new DescendentHTMLCollection(this, filter);
	}
	
	public String getCh()
	{
		return getAttribute("ch");
	}
	
	public String getChOff()
	{
		return getAttribute("choff");
	}
	
	public int getRowIndex()
	{
		NodeImpl parent = (NodeImpl) getParentNode();
		if (parent == null)
		{
			return -1;
		}
		try
		{
			parent.visit(new NodeVisitor()
			{
				private int count = 0;
				
				public void visit(Node node)
				{
					if (node instanceof HTMLTableRowElementImpl)
					{
						if (HTMLTableRowElementImpl.this == node)
						{
							throw new StopVisitorException(new Integer(count));
						}
						count++;
					}
				}
			});
		}
		catch (StopVisitorException sve)
		{
			return ((Integer) sve.getTag()).intValue();
		}
		return -1;
	}
	
	public int getSectionRowIndex()
	{
		// TODO Auto-generated method stub
		return 0;
	}
	
	public String getVAlign()
	{
		return getAttribute("valign");
	}
	
	public HTMLElement insertCell(int index) throws DOMException
	{
		org.w3c.dom.Document doc = document;
		if (doc == null)
		{
			throw new DOMException(DOMException.WRONG_DOCUMENT_ERR, "Orphan element");
		}
		HTMLElement cellElement = (HTMLElement) doc.createElement("TD");
		synchronized (treeLock)
		{
			if (index == -1)
			{
				appendChild(cellElement);
				return cellElement;
			}
			ArrayList nl = nodeList;
			if (nl != null)
			{
				int size = nl.size();
				int trcount = 0;
				for (int i = 0; i < size; i++)
				{
					Node node = (Node) nl.get(i);
					if (node instanceof org.w3c.dom.html2.HTMLTableCellElement)
					{
						if (trcount == index)
						{
							insertAt(cellElement, i);
							return cellElement;
						}
						trcount++;
					}
				}
			}
			else
			{
				appendChild(cellElement);
				return cellElement;
			}
		}
		throw new DOMException(DOMException.INDEX_SIZE_ERR, "Index out of range");
	}
	
	public void setAlign(String align)
	{
		setAttribute("align", align);
	}
	
	public void setBgColor(String bgColor)
	{
		setAttribute("bgcolor", bgColor);
	}
	
	public void setCh(String ch)
	{
		setAttribute("ch", ch);
	}
	
	public void setChOff(String chOff)
	{
		setAttribute("choff", chOff);
	}
	
	public void setVAlign(String vAlign)
	{
		setAttribute("valign", vAlign);
	}
}
