/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Oct 22, 2005
 */
package org.cobra_grendel.html.test;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.cobra_grendel.html.BrowserFrame;
import org.cobra_grendel.html.FormInput;
import org.cobra_grendel.html.HtmlObject;
import org.cobra_grendel.html.HtmlParserContext;
import org.cobra_grendel.html.HtmlRendererContext;
import org.cobra_grendel.html.UserAgentContext;
import org.cobra_grendel.html.domimpl.FrameNode;
import org.cobra_grendel.html.domimpl.HTMLDocumentImpl;
import org.cobra_grendel.html.parser.DocumentBuilderImpl;
import org.cobra_grendel.html.parser.InputSourceImpl;
import org.cobra_grendel.util.io.RecordedInputStream;
import org.w3c.dom.html2.HTMLCollection;
import org.w3c.dom.html2.HTMLElement;
import org.w3c.dom.html2.HTMLLinkElement;

/**
 * The <code>SimpleHtmlRendererContext</code> class implements the
 * {@link org.cobra_grendel.html.HtmlRendererContext} interface. Note that this
 * class provides simple implementations of most methods, which should be
 * overridden to provide real-world functionality.
 */
public class SimpleHtmlRendererContext implements HtmlRendererContext
{
	private static final Logger logger = Logger.getLogger(SimpleHtmlRendererContext.class.getName());
	private static final Set mediaNames = new HashSet();
	
	private UserAgentContext bcontext = null;
	
	private volatile HtmlRendererContext opener;
	
	private final HtmlRendererContext parentRcontext;
	
	private volatile String sourceCode;
	
	static
	{
		// Media names claimed by this context.
		Set mn = mediaNames;
		mn.add("screen");
		mn.add("tv");
		mn.add("tty");
		mn.add("all");
	}
	
	/**
	 * Constructs a SimpleHtmlRendererContext.
	 * 
	 * @param contextComponent
	 *            The component that will render HTML.
	 */
	public SimpleHtmlRendererContext()
	{
		super();
		parentRcontext = null;
	}
	
	/**
	 * Constructs a simple <code>HtmlRendererContext</code> without a parent.
	 * This constructor should not be used to create the context of a frame with
	 * a parent.
	 * 
	 * @param contextComponent
	 * @param pcontext
	 * @deprecated HtmlParserContext is no longer used in this class.
	 */
	@Deprecated
	public SimpleHtmlRendererContext(HtmlParserContext pcontext)
	{
		this(pcontext, null);
	}
	
	/**
	 * Constructs a SimpleHtmlRendererContext.
	 * 
	 * @param contextComponent
	 *            The component that will render HTML.
	 * @param pcontext
	 *            A parser context.
	 * @param parentRcontext
	 *            The parent's renderer context. This is <code>null</code> for
	 *            the root renderer context. Normally ony frame renderer
	 *            contexts would have parents.
	 * @deprecated HtmlParserContext is no longer used in this class.
	 */
	@Deprecated
	public SimpleHtmlRendererContext(HtmlParserContext pcontext, HtmlRendererContext parentRcontext)
	{
		super();
		this.parentRcontext = parentRcontext;
	}
	
	/**
	 * Constructs a SimpleHtmlRendererContext.
	 * 
	 * @param contextComponent
	 *            The component that will render HTML.
	 * @param parentRcontext
	 *            The parent's renderer context. This is <code>null</code> for
	 *            the root renderer context. Normally ony frame renderer
	 *            contexts would have parents.
	 */
	public SimpleHtmlRendererContext(HtmlRendererContext parentRcontext)
	{
		super();
		this.parentRcontext = parentRcontext;
	}
	
	public void alert(String message)
	{
	}
	
	public void back()
	{
	}
	
	public void blur()
	{
		this.warn("back(): Not overridden");
	}
	
	public void close()
	{
		this.warn("close(): Not overridden");
	}
	
	// Methods useful to Window below:
	
	public boolean confirm(String message)
	{
		return true;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlContext#createBrowserFrame()
	 */
	public BrowserFrame createBrowserFrame()
	{
		return null;
	}
	
	public void error(String message)
	{
		if (logger.isLoggable(Level.SEVERE))
		{
			logger.log(Level.SEVERE, message);
		}
	}
	
	public void error(String message, Throwable throwable)
	{
		if (logger.isLoggable(Level.SEVERE))
		{
			logger.log(Level.SEVERE, message, throwable);
		}
	}
	
	public void focus()
	{
		this.warn("focus(): Not overridden");
	}
	
	public String getDefaultStatus()
	{
		this.warn("getDefaultStatus(): Not overridden");
		return "";
	}
	
	/**
	 * This needs to be fixed for the scanner
	 */
	public HTMLCollection getFrames()
	{
		/*
		 * Object rootNode = this.htmlPanel.getRootNode();
		 * 
		 * if(rootNode instanceof HTMLDocumentImpl) { return ((HTMLDocumentImpl)
		 * rootNode).getFrames(); } else { return null; }
		 */
		return null;
	}
	
	public HtmlObject getHtmlObject(HTMLElement element)
	{
		HtmlObject result;
		if ("OBJECT".equalsIgnoreCase(element.getTagName()))
		{
			result = null;
		}
		else
		{
			result = new SimpleHtmlObject(element);
		}
		this.warn("getHtmlObject(): Not overridden; returning " + result + " for " + element + ".");
		return result;
	}
	
	public int getLength()
	{
		this.warn("getLength(): Not overridden");
		return 0;
	}
	
	public String getName()
	{
		this.warn("getName(): Not overridden");
		return "";
	}
	
	public HtmlRendererContext getOpener()
	{
		return opener;
	}
	
	public HtmlRendererContext getParent()
	{
		return parentRcontext;
	}
	
	public String getSourceCode()
	{
		return sourceCode;
	}
	
	public String getStatus()
	{
		this.warn("getStatus(): Not overridden");
		return "";
	}
	
	public HtmlRendererContext getTop()
	{
		HtmlRendererContext ancestor = parentRcontext;
		if (ancestor == null)
		{
			return this;
		}
		return ancestor.getTop();
	}
	
	public UserAgentContext getUserAgentContext()
	{
		this.warn("getUserAgentContext(): Not overridden; returning simple one.");
		synchronized (this)
		{
			if (bcontext == null)
			{
				bcontext = new SimpleUserAgentContext();
			}
			return bcontext;
		}
	}
	
	public boolean isClosed()
	{
		this.warn("isClosed(): Not overridden");
		return false;
	}
	
	public boolean isMedia(String mediaName)
	{
		return mediaNames.contains(mediaName.toLowerCase());
	}
	
	/**
	 * Should be overridden to return true if the link has been visited.
	 */
	public boolean isVisitedLink(HTMLLinkElement link)
	{
		// TODO
		return false;
	}
	
	/**
	 * Implements simple navigation with incremental rendering, and target
	 * processing, including frame lookup. Should be overridden to allow for
	 * more robust browser navigation.
	 * <p>
	 * <b>Notes:</b>
	 * <ul>
	 * <li>Encoding ISO-8859-1 assumed always.
	 * <li>Caching is not implemented.
	 * <li>Cookies are not implemented.
	 * <li>Incremental rendering is not optimized for ignorable document change
	 * notifications.
	 * <li>Other HTTP features are not implemented.
	 * </ul>
	 */
	public void navigate(final URL href, String target)
	{
		// This method implements simple incremental rendering.
		if (target != null)
		{
			HtmlRendererContext topCtx = getTop();
			HTMLCollection frames = topCtx.getFrames();
			if (frames != null)
			{
				org.w3c.dom.Node frame = frames.namedItem(target);
				if (frame instanceof FrameNode)
				{
					BrowserFrame bframe = ((FrameNode) frame).getBrowserFrame();
					if (bframe == null)
					{
						throw new IllegalStateException("Frame node without a BrowserFrame instance: " + frame);
					}
					if (bframe.getHtmlRendererContext() != this)
					{
						bframe.loadURL(href);
						return;
					}
				}
			}
			target = target.trim().toLowerCase();
			if ("_top".equals(target))
			{
				getTop().navigate(href, null);
				return;
			}
			else if ("_parent".equals(target))
			{
				HtmlRendererContext parent = getParent();
				if (parent != null)
				{
					parent.navigate(href, null);
					return;
				}
			}
			else if ("_blank".equals(target))
			{
				this.open(href.toExternalForm(), "cobra.blank", "", false);
				return;
			}
			else
			{
				// fall through
			}
		}
		
		URL urlForLoading;
		if (href.getProtocol().equals("file"))
		{
			// Remove query so it works.
			try
			{
				urlForLoading = new URL(href.getProtocol(), href.getHost(), href.getPort(), href.getPath());
			}
			catch (java.net.MalformedURLException throwable)
			{
				this.warn("malformed", throwable);
				urlForLoading = href;
			}
		}
		else
		{
			urlForLoading = href;
		}
		final URL finalURLForLoading = urlForLoading;
		// Make request asynchronously.
		new Thread()
		{
			@Override
			public void run()
			{
				try
				{
					URL uri = href;
					logger.info("process(): Loading URI=[" + uri + "].");
					long time0 = System.currentTimeMillis();
					// Using potentially different URL for loading.
					URLConnection connection = finalURLForLoading.openConnection();
					connection.setRequestProperty("User-Agent", getUserAgentContext().getUserAgent());
					connection.setRequestProperty("Cookie", "");
					if (connection instanceof HttpURLConnection)
					{
						HttpURLConnection hc = (HttpURLConnection) connection;
						hc.setInstanceFollowRedirects(true);
						int responseCode = hc.getResponseCode();
						logger.info("process(): HTTP response code: " + responseCode);
					}
					InputStream in = connection.getInputStream();
					try
					{
						sourceCode = null;
						long time1 = System.currentTimeMillis();
						RecordedInputStream rin = new RecordedInputStream(in);
						InputStream bin = new BufferedInputStream(rin, 8192);
						HtmlParserContext pcontext = createParserContext(uri);
						DocumentBuilderImpl builder = new DocumentBuilderImpl(pcontext, SimpleHtmlRendererContext.this);
						String actualURI = uri.toExternalForm();
						// Only create document, don't parse.
						HTMLDocumentImpl document =
						        (HTMLDocumentImpl) builder.createDocument(new InputSourceImpl(bin, actualURI,
						                "ISO-8859-1"));
						// Set document in HtmlPanel. Safe to call outside GUI
						// thread.
						// SimpleHtmlRendererContext.this.htmlPanel.setDocument(document,
						// SimpleHtmlRendererContext.this);
						// Now start loading.
						document.load();
						long time2 = System.currentTimeMillis();
						logger.info("Parsed URI=[" + uri + "]: Parse elapsed: " + (time2 - time1)
						        + " ms. Connection elapsed: " + (time1 - time0) + " ms.");
						sourceCode = rin.getString("ISO-8859-1");
					}
					finally
					{
						in.close();
					}
				}
				catch (Exception err)
				{
					SimpleHtmlRendererContext.this.error("navigate(): Error loading or parsing request.", err);
				}
			}
		}.start();
	}
	
	public HtmlRendererContext open(java.net.URL url, String windowName, String windowFeatures, boolean replace)
	{
		return null;
	}
	
	public HtmlRendererContext open(String url, String windowName, String windowFeatures, boolean replace)
	{
		return null;
	}
	
	public String prompt(String message, String inputDefault)
	{
		return "";
	}
	
	/**
	 * Implements reload as navigation to current URL. Override to implement a
	 * more robust reloading mechanism.
	 */
	public void reload()
	{
	}
	
	public void scroll(int x, int y)
	{
		this.warn("scroll(): Not overridden");
	}
	
	public void setDefaultStatus(String message)
	{
		this.warn("setDefaultStatus(): Not overridden.");
	}
	
	public void setOpener(HtmlRendererContext opener)
	{
		this.opener = opener;
	}
	
	public void setStatus(String message)
	{
		this.warn("setStatus(): Not overridden");
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlContext#submitForm(java.lang.String,
	 *      java.lang.String, java.lang.String, java.lang.String,
	 *      org.xamjwg.html.FormInput[])
	 */
	public void submitForm(String method, java.net.URL action, String target, String enctype, FormInput[] formInputs)
	{
		StringBuffer sb = new StringBuffer();
		String lineBreak = System.getProperty("line.separator");
		if (formInputs != null)
		{
			for (int i = 0; i < formInputs.length; i++)
			{
				sb.append("INPUT: " + formInputs[i].toString());
				sb.append(lineBreak);
			}
		}
		this.warn("submitForm(): Not overridden; method=" + method + "; action=" + action + "; target=" + target
		        + "; enctype=" + enctype + lineBreak + sb);
	}
	
	public void warn(String message)
	{
		if (logger.isLoggable(Level.WARNING))
		{
			logger.log(Level.WARNING, message);
		}
	}
	
	public void warn(String message, Throwable throwable)
	{
		if (logger.isLoggable(Level.WARNING))
		{
			logger.log(Level.WARNING, message, throwable);
		}
	}
	
	/**
	 * This method is called by the local navigate() implementation and creates
	 * a {@link SimpleHtmlParserContext}. Override this method if you need to
	 * use the local navigate() implementation with an overridden parser
	 * context.
	 */
	protected HtmlParserContext createParserContext(java.net.URL url)
	{
		return new SimpleHtmlParserContext();
	}
}
