package org.cobra_grendel.html.test;

import java.security.Policy;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.cobra_grendel.html.HttpRequest;
import org.cobra_grendel.html.UserAgentContext;

/**
 * Simple implementation of {@link org.cobra_grendel.html.UserAgentContext}.
 */
public class SimpleUserAgentContext implements UserAgentContext
{
	private static final Logger logger = Logger.getLogger(SimpleUserAgentContext.class.getName());
	
	/**
	 * Creates a {@link org.cobra_grendel.html.test.SimpleHttpRequest} instance.
	 * Override if a custom mechanism to make requests is needed.
	 */
	public HttpRequest createHttpRequest()
	{
		return new SimpleHttpRequest(this);
	}
	
	public void error(String message)
	{
		if (logger.isLoggable(Level.SEVERE))
		{
			logger.log(Level.SEVERE, message);
		}
	}
	
	public void error(String message, Throwable throwable)
	{
		if (logger.isLoggable(Level.SEVERE))
		{
			logger.log(Level.SEVERE, message, throwable);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getAppCodeName()
	 */
	public String getAppCodeName()
	{
		return "Cobra";
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getAppMinorVersion()
	 */
	public String getAppMinorVersion()
	{
		return "0";
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getAppName()
	 */
	public String getAppName()
	{
		return "Browser";
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getAppVersion()
	 */
	public String getAppVersion()
	{
		return "1";
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getBrowserLanguage()
	 */
	public String getBrowserLanguage()
	{
		return "EN";
	}
	
	public String getCookie(java.net.URL url)
	{
		this.warn("getCookie(): Method not overridden; returning null.");
		return "";
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getPlatform()
	 */
	public String getPlatform()
	{
		return System.getProperty("os.name");
	}
	
	/**
	 * Returns -1. Override to provide a different Rhino optimization level.
	 */
	public int getScriptingOptimizationLevel()
	{
		return -1;
	}
	
	/**
	 * Returns <code>null</code>. This method must be overridden if
	 * JavaScript code is untrusted.
	 */
	public Policy getSecurityPolicy()
	{
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#getUserAgent()
	 */
	public String getUserAgent()
	{
		return "Mozilla/4.0 (compatible; MSIE 6.0;) Cobra/Simple";
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.xamjwg.html.HtmlParserContext#isCookieEnabled()
	 */
	public boolean isCookieEnabled()
	{
		this.warn("isCookieEnabled(): Not overridden - returning false");
		return false;
	}
	
	/**
	 * Returns <code>true</code>. Implementations wishing to disable
	 * JavaScript may override this method.
	 */
	public boolean isScriptingEnabled()
	{
		return true;
	}
	
	public void setCookie(java.net.URL url, String cookieSpec)
	{
		this.warn("setCookie(): Method not overridden.");
	}
	
	public void warn(String message)
	{
		if (logger.isLoggable(Level.WARNING))
		{
			logger.log(Level.WARNING, message);
		}
	}
	
	public void warn(String message, Throwable throwable)
	{
		if (logger.isLoggable(Level.WARNING))
		{
			logger.log(Level.WARNING, message, throwable);
		}
	}
}
