/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Jun 9, 2005
 */
package org.cobra_grendel.util;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;

/**
 * @author J. H. S.
 */
public class CollectionUtilities
{
	/**
	 * 
	 */
	private CollectionUtilities()
	{
		super();
	}
	
	public static Enumeration getIteratorEnumeration(final Iterator i)
	{
		return new Enumeration()
		{
			public boolean hasMoreElements()
			{
				return i.hasNext();
			}
			
			public Object nextElement()
			{
				return i.next();
			}
		};
	}
	
	public static Iterator iteratorUnion(final Iterator[] iterators)
	{
		return new Iterator()
		{
			private Iterator current = iterators.length > 0 ? iterators[0] : null;
			private int iteratorIndex = 0;
			
			public boolean hasNext()
			{
				for (;;)
				{
					if (current == null)
					{
						return false;
					}
					if (current.hasNext())
					{
						return true;
					}
					iteratorIndex++;
					current = iteratorIndex >= iterators.length ? null : iterators[iteratorIndex];
				}
			}
			
			public Object next()
			{
				for (;;)
				{
					if (current == null)
					{
						throw new NoSuchElementException();
					}
					try
					{
						return current.next();
					}
					catch (NoSuchElementException nse)
					{
						iteratorIndex++;
						current = iteratorIndex >= iterators.length ? null : iterators[iteratorIndex];
					}
				}
			}
			
			public void remove()
			{
				if (current == null)
				{
					throw new NoSuchElementException();
				}
				current.remove();
			}
		};
	}
	
	public static Collection reverse(Collection collection)
	{
		LinkedList newCollection = new LinkedList();
		Iterator i = collection.iterator();
		while (i.hasNext())
		{
			newCollection.addFirst(i.next());
		}
		return newCollection;
	}
}
