/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Mar 19, 2005
 */
package org.cobra_grendel.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EventListener;
import java.util.EventObject;

/**
 * @author J. H. S.
 */
public abstract class EventDispatch2
{
	private static final EventListener[] EMPTY_ARRAY = new EventListener[0];
	private Collection listeners;
	
	public EventDispatch2()
	{
	}
	
	public final void addListener(EventListener listener)
	{
		synchronized (this)
		{
			if (listeners == null)
			{
				listeners = createListenerCollection();
			}
			listeners.add(listener);
		}
	}
	
	public Collection createListenerCollection()
	{
		return new ArrayList();
	}
	
	public final boolean fireEvent(EventObject event)
	{
		EventListener[] larray;
		synchronized (this)
		{
			Collection listeners = this.listeners;
			if ((listeners == null) || (listeners.size() == 0))
			{
				return false;
			}
			larray = (EventListener[]) this.listeners.toArray(EMPTY_ARRAY);
		}
		int length = larray.length;
		for (int i = 0; i < length; i++)
		{
			// Call holding no locks
			dispatchEvent(larray[i], event);
		}
		return true;
	}
	
	public final void removeListener(EventListener listener)
	{
		synchronized (this)
		{
			if (listeners != null)
			{
				listeners.remove(listener);
			}
		}
	}
	
	protected abstract void dispatchEvent(EventListener listener, EventObject event);
}
