package org.cobra_grendel.util;

/**
 * A task that can be used in a thread or thread pool. The caller can wait for
 * the task to finish by joining it.
 */
public abstract class JoinableTask implements SimpleThreadPoolTask
{
	private boolean done = false;
	
	public void cancel()
	{
		forceDone();
	}
	
	public final void forceDone()
	{
		synchronized (this)
		{
			done = true;
			notifyAll();
		}
	}
	
	public void join() throws InterruptedException
	{
		synchronized (this)
		{
			while (!done)
			{
				this.wait();
			}
		}
	}
	
	public final void run()
	{
		try
		{
			execute();
		}
		finally
		{
			synchronized (this)
			{
				done = true;
				notifyAll();
			}
		}
	}
	
	protected abstract void execute();
}
